/* cloac_lib.c for  UCL CLOAC module
   extracted from clkcntl.c GFM 981108 
   changes for linux BJG/PWP/ 29.05.02
   additions for ATLAS RCC interface (SBCs) JCH 24.06.02 */

#include <stdio.h>

#ifdef NIVXI
#include <nivxi.h>
#endif

#ifdef RAID
#include "../include/rd13lib.h"
#endif

#ifdef QB_linux
#define WND_SIZE 0x1000
#include "../QB/InitVME.h"
#endif

#ifdef RCC
#include <rcc_error.h>
#include <vme_rcc.h>
#endif

#include "cloac_lib.h"

unsigned long unix_address;

/* cc_load_command.
 * Load the contents of the input argument into the command register.
 */

int cc_load_command(unsigned short value)
{
    volatile unsigned short *ip;

    ip = (unsigned short *)(unix_address+COMMAND);

    *ip = value;
    return 0;
}


/* cc_load_delay.
 * Load the contents of the input argument into the delay register.
 */

int cc_load_delay(unsigned short value)
{
    volatile unsigned short *ip;

    ip = (unsigned short *)(unix_address+DELAY);

    *ip = value;
    return 0;
}


/* cc_load_enable.
 * Load the contents of the input argument into the enable register.
 */

int cc_load_enable(unsigned short value)
{


    volatile unsigned short *ip;

    ip = (unsigned short *)(unix_address+ENABLE);

    *ip = value;
    return 0;
}

/* cc_load_frequency.
 * Load the contents of the input argument into the frequency register.
 */

int cc_load_frequency(unsigned short value)
{
    volatile unsigned short *ip;

    ip = (unsigned short *)(unix_address+FREQUENCY);

    *ip = value;
    return 0;
}


/* cc_load_string.
 * Load the contents of the input argument into the string register.
 */

int cc_load_string(unsigned short value)
{
    volatile unsigned short *ip;

    ip = (unsigned short *)(unix_address+STRING);

    *ip = value;
    return 0;
}


/* cc_load_window.
 * Load the contents of the input argument into the window register.
 */

int cc_load_window(unsigned short value)
{
    volatile unsigned short *ip;

    ip = (unsigned short *)(unix_address+WINDOW);

    *ip = value;
    return 0;
}


/* cc_read_command.
 * Read the contents of command register into the output argument.
 */

int cc_read_command(unsigned short *value)
{
    volatile unsigned short *ip;

    ip = (unsigned short *)(unix_address+COMMAND);

    *value = *ip;
    return 0;
}


/* cc_read_delay.
 * Read the contents of delay register into the output argument.
 */

int cc_read_delay(unsigned short *value)
{
    volatile unsigned short *ip;

    ip = (unsigned short *)(unix_address+DELAY);

    *value = *ip;
    return 0;
}


/* cc_read_enable.
 * Read the contents of enable register into the output argument.
 */

int cc_read_enable(unsigned short *value)
{
    volatile unsigned short *ip;

    ip = (unsigned short *)(unix_address+ENABLE);

    *value = *ip;
    return 0;
}


/* cc_read_frequency.
 * Read the contents of frequency register into the output argument.
 */

int cc_read_frequency(unsigned short *value)
{
    volatile unsigned short *ip;

    ip = (unsigned short *)(unix_address+FREQUENCY);

    *value = *ip;
    return 0;
}


/* cc_read_state.
 * Read the contents of "busy state" register into the output argument.
 */

int cc_read_state(unsigned short *value)
{
    volatile unsigned short *ip;

    ip = (unsigned short *)(unix_address+STATE);

    *value = *ip;
    return 0;
}


/* cc_read_window.
 * Read the contents of window register into the output argument.
 */

int cc_read_window(unsigned short *value)
{
    volatile unsigned short *ip;

    ip = (unsigned short *)(unix_address+WINDOW);

    *value = *ip;
    return 0;
}


/* cc_vmemap.
 * Map from VME address space to unix address space.
 */

int cc_vmemap(unsigned long address)
{
  char *addr;
#if defined(NIVXI)
  short ret;
  UINT32 w,wstart,wend;
#elif defined(RCC)
  VME_ErrorCode_t status;
  VME_MasterMap_t master_map;
  u_int virtual_address;
  int handle;
#else
  int status,am;
#endif

#ifdef RAID
    am = 0x39;         /*A24 access*/
    status = RD13_VmeMap((char *)address,am,0x100,&addr,0);
    if(status != 0) {
       printf("VME mapping failed, status = %d\n",status);
       return status;
    }
    unix_address = (unsigned long) addr;
#endif

#ifdef NIVXI
  InitVXIlibrary();		
  /*MapVXIAddressSize(0x80000);*/  /* for standard module test system,
                                   allow 512k general purpose A24 window */

  MapVXIAddressSize(0x180000);	  /* for standard module test system,
                                   allow 1.5MB general purpose A24 window */

  /* MapVXIAddressSize(0x800000);  for testbeam, allow 8MB A24 window */

  /* VXI-style access parameter, 2 for A24, 3 for A32 */
  addr = (char *)MapVXIAddress(2,address,5000,&w,&ret);
  if(ret != 0) {
     printf("cc_vmemap: NIVXI VME mapping failed, status = %d for address %x\n",ret,address);
     return (int)ret;
  }
  printf("cc_vmemap: MapVXIAddress ret %d win %d addr %8x for vme %8x\n", 
		ret, w, addr, address);

  ret = GetWindowRange(w, &wstart, &wend);
  printf("cloac_vmemap: WindowRange start %lx end %lx\n", wstart, wend);

  unix_address = (unsigned long) addr;
#endif

#ifdef QB_linux
  InitVME();
  Initqb(&address ,WND_SIZE,VME_A24);
  unix_address = (unsigned long) address;
#endif

#ifdef RCC
  status = VME_Open();
/*set up the structure for the map call */
  master_map.vmebus_address = address;
  master_map.window_size = 0x10000;
  master_map.address_modifier = VME_A24;
  master_map.options = 0;
  status = VME_MasterMap(&master_map,&handle);
  if(status != VME_SUCCESS) {
    VME_ErrorPrint(status);
    return (int)status;
  }
  status = VME_MasterMapVirtualAddress(handle,&virtual_address);
  printf("cloac_vmemap: VME address %x mapped to %x, handle %d\n",
    address, virtual_address,handle);
  unix_address = (unsigned long)virtual_address;
#endif

   return 0;
}
