  /* cloac_menu.c: simple CLOAC menu functions
   * GFM 981108
   * version PWP 27.06.00
   * version Carlos May 2002 - remove impossible <0 tests on unsigned shorts
   */

#include <stdio.h>
#include <stdlib.h>

#include "../stlib/abc.h"
#include "../stlib/abc_config.h"

#include "cloac_lib.h"
#include "cloac_menu.h"

/* #define CLOAC_DEBUG */

int cc_init(){

  /* Issue CLOAC over-all reset:  must come first;  
     load_command(0) must also precede any new command  */
  unsigned short value;

  cc_read_enable(&value);

  if(!(value&1)){
    /* internal clock already enabled -
     * do not reset CLOAC to avoid clock interruption
     * PWP 12.11.99 */

    cc_load_command(0);
    cc_load_command(VOARST);
    cc_load_command(0);
  }

  /* Enable internal clock: must come before  */  
  cc_load_enable(1);


  /* Load default constants */
  cc_load_window(0x0fff);
  cc_load_delay(128);
  cc_load_frequency(0);

 
  /* Load default fast command bit patterns */
  cc_load_command(0);

  cc_load_string(TRIG);
  cc_load_command(LDTRIG);
  cc_load_command(0);

  cc_load_string(SRST);
  cc_load_command(LDSRST);
  cc_load_command(0);

  cc_load_string(BCRST);
  cc_load_command(LDBCRST);
  cc_load_command(0);

  cc_load_string(CAL1);
  cc_load_command(LDCAL1);
  cc_load_command(0);

  cc_load_string(CAL2);
  cc_load_command(LDCAL2);
  cc_load_command(0);

  if (value&1) return(0);
  else return(1);        /* return 1 if CLOAC reset to warn user to reset BiLED phase */

}

int cc_load_cal(){
  /* 
     Load CAL2 register with string  111111110000
                                     aaaaaacccccc
     aaaaaa = 111111 => Address all chips
     cccccc = 110000 => Issue calibrate strobe 
  */

  cc_load_command(0);
  cc_load_string(CAL2);
  cc_load_command(LDCAL2);
  cc_load_command(0);

  return 0;
}

int cc_load_cal_special(unsigned short cal1, unsigned short cal2){
  /*
     Load the CAL1 and CAL2 registers with arbitrary bit patterns
  */

  printf("cc_load_cal_special: changing CAL1 to %x and CAL2 to %x\n", cal1, cal2);

  cc_load_command(0);
  cc_load_string(cal1);
  cc_load_command(LDCAL1);
  cc_load_command(0);

  cc_load_command(0);
  cc_load_string(cal2);
  cc_load_command(LDCAL2);
  cc_load_command(0);

  return 0;
}

int cc_load_pulse(){
  /* 
     Load CAL2 register with string  111111100000
                                     aaaaaacccccc
     aaaaaa = 111111 => Address all chips
     cccccc = 100000 => Pulse input register 
  */

  cc_load_command(0);
  cc_load_string(PULSE);
  cc_load_command(LDCAL2);
  cc_load_command(0);

  return 0;
}


int cc_slowcommand(unsigned short *cb, int length){
  int i,nw,nb,len64;
  unsigned short in,out;

  len64 = length / 4;
  if (len64 < 1) return(-1);

  cc_load_command(0);

  for (nw=0; nw<len64; nw++) {
    for (nb=0; nb<4; nb++) {
      in = cb[(nw*4)+nb];
      out = 0x0000;
      for (i=0; i<16; i++) out = out | (((in >> (15-i)) & 0x0001) << i);

      cc_load_string(out);
      cc_load_command(LDSLOWCOM);
      cc_load_command(0);
    }
    cc_load_command(RSTSLOWCOUNT);
    cc_load_command(0);
    cc_load_command(VSLOWCOM);
    cc_load_command(0);
  }

  return 0;
}


int cc_menu() {
  char cm[128];
  char c;
  unsigned short v = 0;
  int run = 1;

  while(run) {
    printf("CLOAC: e(nable) d(isable) c(ommand) b(urst) t(rigfreq) r(esetfreq) (dela)y\n");
    printf("       (ca)l(burst) p(load pulse) P(load cal) w(inlength) W(indelay)\n");  
    printf("       m(onitor state) x\n");
    scanf("%1s %hd", cm, &v);
    c = cm[0];
    if (c == 'x') run = 0;
    else if (c == 'e') cc_menu_enable(v);
    else if (c == 'd') cc_menu_disable(v);
    else if (c == 'c') cc_menu_command(v);
    else if (c == 'b') cc_menu_burst(v);
    else if (c == 'l') cc_menu_calburst(v);
    else if (c == 't') cc_menu_trfreq(v);
    else if (c == 'r') cc_menu_srfreq(v);
    else if (c == 'p') cc_load_pulse();
    else if (c == 'P') cc_load_cal();
    else if (c == 'y') cc_menu_delay(v); 
    else if (c == 'm') cc_menu_monitor_state((int)v);
    else if (c == 'w') cc_menu_winlength(v);
    else if (c == 'W') cc_menu_windelay(v);
    else {
      printf("CLOAC: ERROR: unknown command %c\n", c);
    }
  }
  return(0);
}

int cc_menu_enable(unsigned short v){
  unsigned short prev,next,cur;

  if ( v>15 ){
#ifdef CLOAC_DEBUG
    printf("CLOAC: ERROR: enable argument %d outside valid range\n", v);
#endif
    return(-1);
  } else {
    cc_read_enable(&prev);
    next = prev | (1<<v);
    cc_load_enable(next);
    cc_read_enable(&cur);
#ifdef CLOAC_DEBUG
    printf("CLOAC: enable reg was %4x, should be %4x, and seems to be %4x\n",prev,next,cur);
#endif
  }
  return(0);
}

int cc_menu_disable(unsigned short v){
  unsigned short prev,next,cur;

  if ( v>15 ){
#ifdef CLOAC_DEBUG
    printf("CLOAC: ERROR: disable argument %d outside valid range\n", v);
#endif
    return(-1);
  } else {
    cc_read_enable(&prev);
    next = prev & ~(1<<v);
    cc_load_enable(next);
    cc_read_enable(&cur);
#ifdef CLOAC_DEBUG
    printf("CLOAC: enable reg was %4x, should be %4x, and seems to be %4x\n",prev,next,cur);
#endif
  }
  return(0);
}

int cc_menu_command(unsigned short v) {
  if ( v>15 ){
#ifdef CLOAC_DEBUG
    printf("CLOAC: ERROR: command argument %d outside valid range\n", v);
#endif
    return(-1);
  } else {
    if ((v==6)||(v==7)) 
      cc_load_command((unsigned short)(0x1<<v));   /* ### how to unset ? */
    else {
      cc_load_command((unsigned short)(0x1<<v));
      cc_load_command(0);
    }
  }
  return(0);
}

int cc_menu_trfreq(unsigned short v){
  unsigned short prev,next,cur;
  
  if (v > 31 ){
#ifdef CLOAC_DEBUG
    printf("CLOAC: ERROR: trfreq argument %d outside valid range\n", v);
#endif
    return -3;
  } else {
    cc_read_frequency(&prev);
    next = (prev & 0x1f00) | v;
    cc_load_frequency(next);

    cc_read_frequency(&cur);
#ifdef CLOAC_DEBUG
    printf("CLOAC: freq reg was %4x, should be %4x, and seems to be %4x\n", prev,next,cur);
#endif
  }
  return 0;
}

int cc_menu_srfreq(unsigned short v){
  unsigned short prev,next,cur;
 
  if (v > 31 ){
#ifdef CLOAC_DEBUG
    printf("CLOAC: ERROR: srfreq argument %d outside valid range\n", v);
#endif
    return -3;
  } else {
    cc_read_frequency(&prev);
    next = (prev & 0x001f) | (v<<8);
    cc_load_frequency(next);

    cc_read_frequency(&cur);
#ifdef CLOAC_DEBUG
    printf("CLOAC: freq reg was %4x, should be %4x, and seems to be %4x\n", prev,next,cur);
#endif
  }
  return 0;
}

int cc_menu_burst(unsigned short v){

  int i = 0;
  unsigned short busy_reg = 0x0080;

  /* Some CLOAC modules do not always respond first time to the
   * command "set ldvburst" - hence I changed this section
   * of the code to loop a few times until CLOAC does respond
   * PWP 08.12.00
   */

   while((i<10)&&(busy_reg&0x0080)){ 
    cc_load_command(0x0000);  /* clear command register */
    cc_load_command(0x0040);  /* enable vburst */

    cc_load_string(v);        /* put value in string reg */
    cc_load_command(0x8040);  /* keep vburst; set ldvburst */

    cc_read_state(&busy_reg);
    i++;
  }
  printf("Vburst took %d goes to start\n",i);

  cc_load_command(0x0040);  /* keep vburst; reset ldburst */
  cc_menu_monitor_state(1);

  return(0);
}

int cc_menu_calburst(unsigned short v){
  
  cc_load_command(0x0040);  /* enable bursts; stop triggers now */

  cc_load_string(v);        /* put value in string reg */

  cc_load_command(0x8040);  /* keep vburst; set ldvburst */ 
  cc_load_command(0x0048);  /* keep vburst; reset ldburst; set vcal */
  return(0);
}

int cc_menu_sburst(unsigned short v){  /* burst of single triggers */
  int i;

  if (v < 1) return -1;
  
  for (i=0; i<v; i++) {
    cc_load_command(0);
    cc_load_command(0x0001);
    cc_load_command(0);
    /* ## insert software delay */
  }   
  return(0);
}

int cc_menu_delay(unsigned short v){
  unsigned short prev,next,cur;
  
  if (v > 143 ){
#ifdef CLOAC_DEBUG
    printf("CLOAC: ERROR: delay argument %d outside valid range\n", v);
#endif
    return -3;
  } else {
    cc_read_delay(&prev);
    next = (prev & 0x1f00) | v;
    cc_load_delay(next);

    cc_read_delay(&cur);
#ifdef CLOAC_DEBUG
    printf("CLOAC: delay reg was %4x, should be %4x, and seems to be %4x\n", prev,next,cur);
#endif
  }
  return(0);
}

int cc_menu_monitor_state(int v){
  int cnt,slp;
  unsigned short enable_reg, command_reg, busy_reg;

  cnt = 0;

  while (cnt++ < v){

    cc_read_enable(&enable_reg);
    cc_read_command(&command_reg);
    cc_read_state(&busy_reg);

    printf("CLOAC REGS:  enable %4x  command %4x  busy %4x\n",
      enable_reg, command_reg, busy_reg);

    slp = 1000000;
    while (slp--);
  }
  
  return 0;
}

int cc_menu_vbusy(unsigned short v){
  unsigned short prev,next,cur;

  cc_read_command(&prev);

  if(v==0) next = (prev & 0xff7f);
  else     next = (prev | 0x0080);

  cc_load_command(next);
  cc_read_command(&cur);
#ifdef CLOAC_DEBUG
    printf("CLOAC:  command reg was %4x, should be %4x, and seems to be %4x\n", prev,next,cur);
#endif

  return 0;
}

int cc_menu_winlength(unsigned short v){
  unsigned short prev,next,cur;
  
  if (v > 31 ){
#ifdef CLOAC_DEBUG
    printf("CLOAC: ERROR: window length argument %d outside valid range 0 - 31\n", v);
#endif
    return -3;
  } else {
    cc_read_window(&prev);
    next = (prev & 0x1f00) | v;   // preserve window delay in the high order byte
    cc_load_window(next);

    cc_read_window(&cur);
#ifdef CLOAC_DEBUG    
    printf("CLOAC: window reg was %4x, should be %4x, and seems to be %4x\n", prev,next,cur);
#endif
  }
  return(0);
}

int cc_menu_windelay(unsigned short v){
  unsigned short prev,next,cur;
  
  if (v > 31){
#ifdef CLOAC_DEBUG
    printf("CLOAC: ERROR: window delay argument %d outside valid range 0 - 31\n", v);
#endif
    return -3;
  } else {
    cc_read_window(&prev);
    next = (prev & 0x001f) | (v<<8);   // preserve window length in the low order byte
    cc_load_window(next);

    cc_read_window(&cur);
#ifdef CLOAC_DEBUG
    printf("CLOAC: delay reg was %4x, should be %4x, and seems to be %4x\n", prev,next,cur);
#endif
  }
  return(0);
}
