//
//
//  TKwikFit.h
//
//  Version 0.1 
//  PWP 26.10.00
//


#if _MSC_VER >= 1000
#pragma once
#endif

#include "TObject.h"
#include "TH1.h"
#include "TH2.h"
#include "TF1.h"
#include "TGraphErrors.h"
#include "TFitter.h"

#include "../stlib/sct_scans.h"
#include "../stdll/TModuleDefine.h"
#include "../stdll/functions.h"

class TKwikFit : public TObject {

  public:

    TKwikFit(short new_id);
    ~TKwikFit();

    void TKwikFit::ResetData();
    void TKwikFit::SetModuleName(char* module_name); 

    void TKwikFit::LoadScan(TH2F* h_scan0, TH2F* h_scan1, TH1F* h_nevent);
    void TKwikFit::FitScan(short fit_type, short cal_mode);   // Fit error function or complementary error function
    void TKwikFit::PrintScan();                               // Show scan error and fit details


    char longname[80];
    char name[32];

    short id;
    short link0;
    short link1;


    short scan[ST_MAX_SCANPOINTS][2][768];
    float scan_points[ST_MAX_SCANPOINTS];
    int   scan_trigs[ST_MAX_SCANPOINTS];

    short scan_max_points;
    float scan_start;
    float scan_stop;
    float scan_step;
    float scan_lo;
    float scan_hi;

    short use_ch_maxima;
    short scan_fitted;

    TH2F *h_scan0;
    TH2F *h_scan1;

    TH1S *h_code0; 
    TH1S *h_code1;
    TH1F *h_mean0;
    TH1F *h_mean1;
    TH1F *h_sigma0;
    TH1F *h_sigma1;
    TH1F *h_chisq0;
    TH1F *h_chisq1;
    TH1F *h_fom0;
    TH1F *h_fom1;

    float chip_mean[2][6];
    float chip_sigma[2][6];
    float chip_chisq[2][6];
    float chip_fom[2][6];
    float chip_prob[2][6];
    short chip_badfits[2][6];
    short chip_goodfits[2][6];

    float side_mean[2];
    float side_sigma[2];
    float side_chisq[2];
    float side_fom[2];
    float side_prob[2];
    short side_badfits[2];
    short side_goodfits[2];

    float overall_mean;
    float overall_sigma;
    float overall_chisq;
    float overall_fom;
    float overall_prob;
    short overall_badfits;
    short overall_goodfits;

    short likelihood;
    short use_range;

  private:

    int burst;
    int normalised;
 
    ClassDef(TKwikFit,1)
};

  void ErfLikelihoodFit(Int_t &npar, Double_t *gin,
                        Double_t &f, Double_t *par, Int_t iflag);
  void ErfcLikelihoodFit(Int_t &npar, Double_t *gin,
                        Double_t &f, Double_t *par, Int_t iflag);
  void NErfLikelihoodFit(Int_t &npar, Double_t *gin,
                        Double_t &f, Double_t *par, Int_t iflag);
  void NErfcLikelihoodFit(Int_t &npar, Double_t *gin,
                        Double_t &f, Double_t *par, Int_t iflag);

