/* 
    lvtest.c :  sctlv test program

    GFM 990504
    amended PWP 26.05.99 and 27.05.02
*/


#include <stdlib.h>
#include <stdio.h>

#ifdef RAID
  #include <posix/time.h>
#endif

#include "sctlv_lib.h"
#include "../stlib/sct_hardware.h"

void sctlv_menu(short id);

int main(){
  int code;
  short i,id;
  unsigned long address;

  printf("Attempting vmemap of SCTLV (Low voltage power supply modules)\n");
  address = SCTLV_CRATE_ADDRESS;
  for(i=0;i<SCTLV_NUMBER;i++){
    code = sctlv_vmemap(address,&id);   /* Now map the individual modules PWP 17.01.01 */
    address += SCTLV_SPACING;
  }
  
  sctlv_menu(0);
 
  return 0;  
}

void sctlv_menu(short id) {
  char cm[128];
  char c;
  int v = 0;
  float fv = 0.0;
  float fi = 0.0;
  int run = 1;
  short i;

  printf("\n");

  while(run) {
    printf("\n");
    printf("SCTLV: 1(on) 0 (off) t(restart channel) d(set defaults) s(print status)\n");
    printf("       r(module hard reset)  a(alter select)  b(brief status all channels)\n");
    printf("       v(set Vdd) c(set Vcc) i(set_id) x(exit)\n");
    scanf("%1s", cm);
    c = cm[0];
    if (c == 'x') run = 0;
    else if (c == '1'){ 
      sctlv_show_error(sctlv_set_power(id,1));
      sctlv_show_error(sctlv_print_status(id));
    }else if (c == '0'){
      sctlv_show_error(sctlv_set_power(id,0));
      sctlv_show_error(sctlv_print_status(id));
    }else if (c == 's'){
      sctlv_show_error(sctlv_print_status(id));
    }else if (c == 'b'){
      for(i=0;i<SCTLV_NUMBER*2;i++){
        sctlv_show_error(sctlv_brief_status(i));
      }
    }else if (c == 'r'){ 
      sctlv_show_error(sctlv_do_reset(id));
    }else if (c == 'd'){ 
      sctlv_show_error(sctlv_set_defaults(id)); 
      sctlv_show_error(sctlv_print_status(id));
    }else if (c == 'v'){
      printf("SCTLV: enter new Vdd: ");
      scanf("%f", &fv);
      printf("SCTLV: enter new Idd: ");
      scanf("%f", &fi);
      sctlv_show_error(sctlv_set_vdd(id, fv, fi));
      sctlv_show_error(sctlv_print_status(id));
    }else if (c == 'c'){
      printf("SCTLV: enter new Vcc: ");
      scanf("%f", &fv);
      printf("SCTLV: enter new Icc: ");
      scanf("%f", &fi);
      sctlv_show_error(sctlv_set_vcc(id, fv, fi));
      sctlv_show_error(sctlv_print_status(id));
    }else if (c == 't'){
      sctlv_show_error(sctlv_board_restart(id));
      sctlv_show_error(sctlv_print_status(id));
    }else if (c == 'a'){
      printf("SCTLV: enter Select (0/1): ");
      scanf("%d", &v);
      sctlv_show_error(sctlv_set_select(id, (short)v));
      sctlv_show_error(sctlv_print_status(id));
    }else if (c == 'i'){ 
      printf("SCTLV: enter new id: "); 
      scanf("%d", &v); 
      id = v; 
      printf(" ... id set to %d\n", id);
      sctlv_show_error(sctlv_print_status(id));
    }else{
      printf("SCTLV: ERROR: unknown command %c\n", c);
    }
  }
  printf("\n");
  sctlv_set_power(id,0);
}

