#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <signal.h>
#include <gtk/gtk.h>
#include "../stlib/sct_hardware.h"

#include "Beer.xpm"

#define GTK_MESSAGEBOX(obj)          GTK_CHECK_CAST (obj, gtk_messagebox_get_type (), GtkMessagebox)
#define GTK_MESSAGEBOX_CLASS(klass)  GTK_CHECK_CLASS_CAST (klass, gtk_messagebox_get_type (), GtkMessageboxClass)
#define IS_MESSAGEBOX(obj)       GTK_CHECK_TYPE (obj, gtk_messagebox_get_type ())

typedef struct _GtkMessagebox GtkMessagebox;
typedef struct _GtkMessageboxClass GtkMessageboxClass;

struct _GtkMessagebox {
      GtkDialog dlg;
      int rc;
};

struct _GtkMessageboxClass {
      GtkDialogClass parent_class;
};
typedef enum {
   MSG_BOX_NONE = 0,
   MSG_BOX_OK = 0x01,
   MSG_BOX_CANCEL = 0x02,
   MSG_BOX_YES = 0x04,
   MSG_BOX_NO = 0x08,
   MSG_BOX_ABORT = 0x10,
   MSG_BOX_RETRY = 0x20,
   MSG_BOX_IGNORE = 0x40
} GtkMessageboxButton;

guint gtk_messagebox_get_type (void);
GtkWidget *gtk_messagebox_new ();
GtkMessageboxButton gtk_messagebox_exec (GtkMessagebox *);

static GtkDialog *parent_class = NULL;

void messagebox_class_init (GtkMessageboxClass * klass)
{
    GtkObjectClass *obj_class;
    GtkDialogClass *dlg_class;

    obj_class = (GtkObjectClass *) klass;
    dlg_class = (GtkDialogClass *) klass;
    parent_class = gtk_type_class (gtk_dialog_get_type ());
}
guint gtk_messagebox_get_type ()
{
   static guint msg_type = 0;

   if (!msg_type) {
      GtkTypeInfo msg_info = {
	 "GtkMessagebox",
	 sizeof (GtkMessagebox),
	 sizeof (GtkMessageboxClass),
	 (GtkClassInitFunc) messagebox_class_init,
	 (GtkObjectInitFunc) 0,
	 0,			/* (GtkArgSetFunc) */
	 0,			/* (GtkArgGetFunc) */
      };

      msg_type = gtk_type_unique (gtk_dialog_get_type (), &msg_info);
   }

   return msg_type;
}


void messagebox_done (GtkMessagebox * mb, GtkMessageboxButton btype)
{
    mb->rc = btype;
    gtk_main_quit ();
}

void messagebox_button (GtkWidget * w, gpointer ptr)
{
    GtkMessageboxButton btype = (GtkMessageboxButton) ptr;
    GtkMessagebox *mb = GTK_MESSAGEBOX (gtk_object_get_data (GTK_OBJECT (w), "mbox"));
    messagebox_done (mb, btype);
}



GtkWidget *gtk_messagebox_new ()
{
    GtkMessagebox *mbox = gtk_type_new (gtk_messagebox_get_type ());
    GtkDialog *dlg = GTK_DIALOG (mbox);
    GtkWidget *hbox;
    GtkWidget *pixmap;
    GdkColormap *colormap;
    GdkPixmap *gdkpixmap;
    GdkBitmap *mask;

    GtkWidget *lblApplication;
    GtkWidget *hbuttonbox;
    GtkWidget *btn;

    gtk_container_set_border_width (GTK_CONTAINER (mbox), 10);
    gtk_window_set_title (GTK_WINDOW (mbox),"Stop SCTDAQ scan");
    GTK_WINDOW (mbox)->type = GTK_WINDOW_DIALOG;
    gtk_window_set_modal (GTK_WINDOW (mbox), TRUE);
    gtk_window_set_policy (GTK_WINDOW (mbox), TRUE, TRUE, FALSE);


    gtk_widget_show (dlg->vbox);
    gtk_box_set_spacing (GTK_BOX (dlg->vbox), 15);
    hbox = gtk_hbox_new (FALSE, 0);
    gtk_widget_show (hbox);
    gtk_box_pack_start (GTK_BOX (dlg->vbox), hbox, TRUE, TRUE, 0);

    colormap = gtk_widget_get_colormap (GTK_WIDGET (mbox));
    gdkpixmap = gdk_pixmap_colormap_create_from_xpm_d (
       NULL, colormap, &mask, NULL, (gchar **) Beer_xpm);

    if (gdkpixmap == NULL)
	g_warning ("Couldn't create pixmap\n");
    else {
	pixmap = gtk_pixmap_new (gdkpixmap, mask);
	gtk_widget_show (pixmap);
	gtk_box_pack_start (GTK_BOX (hbox), pixmap, FALSE, TRUE, 0);
	gtk_misc_set_alignment (GTK_MISC (pixmap), 0.1, 0.25);
	gdk_pixmap_unref (gdkpixmap);
	gdk_bitmap_unref (mask);
    }

    
    lblApplication = gtk_label_new ("What about a break ?");
    gtk_widget_show (lblApplication);
    gtk_box_pack_start (GTK_BOX (hbox), lblApplication, FALSE, FALSE, 0);


    gtk_container_set_border_width (GTK_CONTAINER (dlg->action_area), 0);
    gtk_widget_show (dlg->action_area);

    hbuttonbox = gtk_hbutton_box_new ();
    gtk_widget_show (hbuttonbox);
    gtk_box_pack_start (GTK_BOX (dlg->action_area), hbuttonbox, TRUE, TRUE, 0);

    btn = gtk_button_new_with_label ("Stop");
    gtk_widget_show (btn);
    gtk_object_set_data (GTK_OBJECT (btn), "mbox", mbox);
    gtk_signal_connect (GTK_OBJECT (btn), 
			"clicked", 
			GTK_SIGNAL_FUNC (messagebox_button), 
			(gpointer) MSG_BOX_OK);
    gtk_container_add (GTK_CONTAINER (hbuttonbox), btn);
    GTK_WIDGET_SET_FLAGS (btn, GTK_CAN_DEFAULT);

    btn = gtk_button_new_with_label ("Abort");
    gtk_widget_show (btn);
    gtk_object_set_data (GTK_OBJECT (btn), "mbox", mbox);
    gtk_signal_connect (GTK_OBJECT (btn), 
			"clicked", 
			GTK_SIGNAL_FUNC (messagebox_button), 
			(gpointer) MSG_BOX_CANCEL);
    gtk_container_add (GTK_CONTAINER (hbuttonbox), btn);
    GTK_WIDGET_SET_FLAGS (btn, GTK_CAN_DEFAULT);

    mbox->rc = MSG_BOX_NONE;
    return GTK_WIDGET (mbox);
}

GtkMessageboxButton gtk_messagebox_exec (GtkMessagebox * mb)
{
   GtkMessageboxButton rc;
   mb->rc = MSG_BOX_NONE;
   gtk_widget_show (GTK_WIDGET (mb));
   gtk_main ();
   rc = mb->rc;
   return rc;
}

int delete_event( GtkWidget *w, GdkEvent  *e, gpointer p)
{
   return TRUE; 
}

GtkMessagebox *mb; 
void quit(int x) {
    messagebox_done(mb,MSG_BOX_ABORT);
} 

int GNUstop(int argc, char **argv) {
    GtkMessageboxButton rc;
    FILE *fp;
    char s_start[128], s_stop[128];


    signal (SIGINT,quit);
    sprintf(s_start,"%sstart.txt",ST_ETC);
    sprintf(s_stop, "%sstop.txt", ST_ETC);

    remove(s_stop);
    fp = fopen(s_start,"w");
    fprintf(fp,"RUNNING\n");
    fclose(fp);


    gtk_init (&argc, &argv);
    mb = GTK_MESSAGEBOX(gtk_messagebox_new());
    gtk_signal_connect (GTK_OBJECT (mb), "delete_event",
			GTK_SIGNAL_FUNC (delete_event), NULL);
    
    while (1) {
	// we are supposed to be killed by sctdaq
	rc = gtk_messagebox_exec(mb);
	if ( rc == MSG_BOX_OK ) {
	    // Stop scan
	    remove(s_start);
	    fp = fopen(s_stop,"w");
	    fprintf(fp,"STOP\n");
	    fclose(fp);
	}
	else if ( rc == MSG_BOX_CANCEL ) {
	    // Abort sequence
	    remove(s_start);
	    fp=fopen(s_stop,"w");
	    fprintf(fp,"ABORT\n");
	    fclose(fp);      
	}
	else if ( rc == MSG_BOX_ABORT ) {
	    gtk_widget_destroy(GTK_WIDGET(mb));
	    break;
	}
    }
    return 0;
}


int main(int argc, char **argv) {
    int pid;
    pid = fork();
    if ( pid == 0 ) {
	if ( GNUstop(argc,argv) ) {
	    printf("Error starting the server\n");
	    exit(1);
	}
	return 0;
    }
    else 
	return 0;    
}
