#include <parameters.h>
#define MAXNDAT 30
#define MAXNSETS 6

void CompInjSets(Int_t NSETS, Int_t *NDAT, Int_t *run, Int_t *scans, Char_t *alllabels, 
		 Char_t *genlabel, Int_t plot=0){

  Int_t i,j,k,chip,goodchip, nread=1, nmod, chan,link,offset;
  Float_t noise, refnse[MAX_MODULES][MAX_CHIPS*MAX_LINKS],avg[MAXNSETS][MAX_MODULES][MAX_CHIPS*MAX_LINKS];
  Float_t spread[MAXNSETS][MAX_MODULES][MAX_CHIPS*MAX_LINKS],diff[144],maxnse=0.,tmp;
  Char_t fname[128],mname[30][MAX_MODULES],text[128],line[1024],label[MAXNDAT][64];
  Float_t allnse[MAX_LINKS*MAX_CHIPS], id[MAX_LINKS*MAX_CHIPS];
  Float_t xe[MAX_LINKS*MAX_CHIPS], ye[MAX_LINKS*MAX_CHIPS];
  TGraphErrors *grm;
  TPostScript *ps;

  FILE *output;
  TFile *strun;
  TH1F* h_sigma;

  // check array sizes
  if(NSETS>MAXNSETS){
    printf("Too many data sets, maximum is %d\n",MAXNSETS);
    return;
  }
  for(k=0;k<NSETS;k++){
    if(NDAT[k]>MAXNDAT){
      printf("Too many data points in set %d, maximum is %d\n",k,MAXNDAT);
      return;
    }
  }

  // pick labels into individual parts
  j=0;
  k=0;
  for(i=0;i<strlen(alllabels);i++){
    if(alllabels[i]==':') {
      label[j][k]='\0';
      j++;
      k=0;
    }
    else{
      label[j][k]=alllabels[i];
      k++;
    }
  }
  label[j][k]='\0';
  j++;
  if(j!=NSETS){
    printf("Found %d data sets, but only %d labels\n",NSETS,j);
    return;
  }

  // initialise arrays
  for(k=0;k<NSETS;k++){
    for(j=0;j<MAX_MODULES;j++){
      for(link=0;link<MAX_LINKS;link++){
	for(chip=0;chip<MAX_CHIPS;chip++){
	  avg[k][j][chip+MAX_CHIPS*link]=0.;
	  spread[k][j][chip+MAX_CHIPS*link]=0.;
	}
      }
    }
  }

  // read from data file
  offset=0;
  for(k=0;k<NSETS;k++){
    printf("Reading set %d with %d data points\n",k,NDAT[k]);
    for(i=0;i<NDAT[k];i++){
      sprintf(fname,"%sstrun%d_%d.root",DATA_DIRECTORY,run[offset+i],scans[offset+i]);
      strun = new TFile(fname);
      TObjString* time  = (TObjString*)strun->Get("Time");
      TObjString* place = (TObjString*)strun->Get("Place");
      for(j=0;j<MAX_MODULES;j++){
	sprintf(text,"ModuleName;%d",j+1);
	TObjString* name = (TObjString*)strun->Get(text);
	if(name){
	  if(strcmp(name->GetName(),"missing")){
	    sprintf(mname[j],"%s",name->GetName());
	  }
	  else
	    break;
	}
	else
	  break;
	for(link=0;link<MAX_LINKS;link++){
	  sprintf(text,"h_sigma%d;%d",link,j+1);
	  h_sigma= (TH1F*)strun->Get(text);
	  for(chip=0;chip<MAX_CHIPS;chip++){
	    noise=0.;
	    for(chan=(chip*128);chan<((chip+1)*128);chan++){
	      noise += h_sigma->GetBinContent(chan);
	    }
	    if(i==0) 
	      refnse[j][chip+MAX_CHIPS*link] = noise*6240/55./128.;
	    else{
	      noise *=6240/55./128.;
	      noise = pow(noise,2) - pow(refnse[j][chip+MAX_CHIPS*link],2);
	      if(noise>0) 
		noise = sqrt(noise);
	      else 
		noise = 0.;
	      avg[k][j][chip+MAX_CHIPS*link]    += noise;
	      spread[k][j][chip+MAX_CHIPS*link] += noise*noise;
	    }
	  }
	}
      }
      strun->Close();
      if(i==0){
	if(k>0 && j!= nmod){
	  printf("Different number of modules in the data sets\n");
	  return;
	} else{
	  nmod = j;
	  printf("found %d modules\n",nmod);
	}
      }
    }
    
    // normalise and find maximum noise difference
    for(chip=0;chip<MAX_LINKS*MAX_CHIPS;chip++){
      for(j=0;j<nmod;j++){
	avg[k][j][chip] /= (Float_t) NDAT[k];
	spread[k][j][chip] /= (Float_t) NDAT[k];
	tmp = sqrt((spread[k][j][chip]-pow(avg[k][j][chip],2))/(Float_t)(NDAT[k]-1));
	spread[k][j][chip] = tmp;
	if(avg[k][j][chip]>maxnse)
	  maxnse = avg[k][j][chip];
      }
    }
    offset += NDAT[k];
  }

  maxnse*=1.1;
  //  if(maxnse<100) maxnse = 100.;

  // open canvas
  TCanvas* cinj= new TCanvas ("cinj","Noise Injection",1100,750);
  
  // get time/place info
  
  sprintf(fname,"%sstrun%d_%d.root",DATA_DIRECTORY,run[0],scans[0]);

  // plot
  
  if(plot){
    sprintf(fname,"%sNinjComp_%d_%d.ps",PS_DIRECTORY,run[0],scans[0]);
    ps = new TPostScript(fname,112); 
    ps->NewPage();
  }

  cinj->cd();
  cinj->Clear();
  pad_text   = new TPad("pad_text", "Text",        0.00,0.90,1.00,1.00);
  pad_plot   = new TPad("pad_plot", "Plots",       0.00,0.00,1.00,0.90);
  pad_text->Draw();
  pad_plot->Draw();
  cinj->Update();

  pad_text->cd();
  TPaveText *t1 = new TPaveText(0.0,0.0,1.0,1.0);
  t1->SetFillColor(10);
  t1->SetTextAlign(13);
  t1->SetTextSize(0.3);
  t1->SetTextColor(4);
  sprintf(line,"ATLAS SCT: pick-up with noise injection");
  if(time)  sprintf(line,"%s  -  %s",line,time->GetName());
  if(place) sprintf(line,"%s  -  %s",line,place->GetName());
  t1->AddText(line);
  sprintf(line,"Input-noise at 2fC from threshold scans - difference to zero-injection");
  t1->AddText(line);
  if(genlabel) t1->AddText(genlabel);
  t1->Draw();
  cinj->Update();
  
  pad_plot->cd();

  // draw drawing area
  pad_plot->Range(-2*nmod,-.12*maxnse,nmod*17,1.12*maxnse);
  TLegend *leg = new TLegend(.73,1.-.07*(Float_t)(NSETS-1),1.,1.);
  leg->SetTextFont(62);
  leg->SetTextSize(0.03);
  TGaxis *xax = new TGaxis(0,0,nmod*12,0,0,nmod,6*100+nmod,"U");
  xax->SetName("xaxis");
  xax->SetTitle("Module");
  xax->Draw();
  TGaxis *yax = new TGaxis(0,0,0,maxnse,0,maxnse,808,"");
  yax->SetName("yaxis");
  yax->SetTitle("Quadratic difference in input-noise (ENC)");
  yax->Draw();
  for(i=0;i<=nmod;i++){
    l = new TLine(i*12,0,i*12,maxnse);
    l->Draw();
  }
  for(i=0;i<nmod;i++){
    TText t;
    t.SetTextSize(0.04);
    t.DrawText((Double_t)(i*12+1),-.05*maxnse,mname[i]);
  }
  l = new TLine(0,maxnse,nmod*12,maxnse);
  l->Draw();
  l = new TLine(0,0,nmod*12,0);
  l->Draw();

  // draw graph
  for(k=0;k<NSETS;k++){
    for(i=0;i<nmod;i++){
      goodchip=0;
      for(chip=0;chip<12;chip++){
	if(refnse[i][chip]>1000){
	  id[goodchip]=0.5+(Float_t)chip + 12*i;
	  allnse[goodchip] = avg[k][i][chip];
	  xe[goodchip]=0.;
	  ye[goodchip] = spread[k][i][chip];
	  goodchip++;
	}
      }
      grm = new TGraphErrors(goodchip,id,allnse,xe,ye);
      grm->SetTitle("");
      grm->SetMarkerSize(1);
      grm->SetMarkerStyle(20);
      grm->SetMarkerColor((Color_t) k+1);
      grm->SetLineColor((Color_t) k+1);
      if(i==0) leg->AddEntry(grm,label[k],"p");
      grm->Draw("PL");
    }
  }

  leg->Draw();
  cinj->Update();

  if(plot) ps->Close();

  return;
}

