void cmn_chmx2(Int_t runno, Int_t scanno, Float_t qthr) {

#include "../macros/parameters.h"
#define MAX_MOD 12

  // define colour palette
  
  const Int_t colNum=50;
  Int_t palette[2*colNum];
  for(Int_t i=0;i<colNum;i++){
    if(! gROOT->GetColor(100+i)){
      //TColor *color = new TColor (100+i,1-(i/50.0),1-(i/50.0),1-(i/50.0),"");
      TColor *color = new TColor (100+i,1,(i/50.0),(i/50.0),"");
    }else{
      TColor *color = gROOT->GetColor(100+i);
      //color->SetRGB(1-(i/50.0),1-(i/50.0),1-(i/50.0));
      color->SetRGB(1,(i/50.0),(i/50.0));
    }
    palette[i]=100+i;
  }
  for(Int_t i=0;i<colNum;i++){
    if(! gROOT->GetColor(100+i+colNum)){
      TColor *color = new TColor (100+i+colNum,1-(i/50.0),1-(i/50.0),1,"");
    }else{
      TColor *color = gROOT->GetColor(100+i+colNum);
      color->SetRGB(1-(i/50.0),1-(i/50.0),1);
    }
    palette[i+colNum]=100+i+colNum;
  }
  gStyle->SetPalette(2*colNum,palette);

  Char_t line[1024],filename[258],mod_name[32],mname[MAX_MOD][50];
  Int_t mod, nevt;

  // read the module names

  sprintf(filename,"%sstrun%d_%d.root",DATA_DIRECTORY,runno,scanno);
  TFile* f = new TFile(filename);

  for(Int_t mod=0;mod<MAX_MOD;mod++){
    

    sprintf(mod_name,"ModuleName;%d",mod+1);

    if(mod==0){
      TObjString* time  = (TObjString*)f->Get("Time");
      TObjString* place = (TObjString*)f->Get("Place");
      TTree* tr = (TTree*) f->Get("EvTree_m0");
      nevt = tr->GetEntries();
    }
    
    TObjString* name = (TObjString*)f->Get(mod_name);    
    if(name){
      sprintf(mname[mod],"%s",name->GetName());
    }
    else{
      printf("asked for non-existing module\n");
      return;
    }


  }

  f->Close();
  f->Delete();

  //set up the canvas

  TCanvas *can = new TCanvas("c","c",1100,750);
  can->cd();
  can->Clear();
  
  // open PS output file
  sprintf(filename,"D:\\sctvar\\ps\\CorrelNoise_%d_%d.ps",runno,scanno);
  ps = new TPostScript(filename,112);

  ps->NewPage();
  pad_t = new TPad("pad_t","Text",        0.00,0.90,1.00,1.00,33);
  pad_p = new TPad("pad_p","Projections",   0.00,0.00,1.00,0.90,33);
  pad_t->Draw();
  pad_p->Draw();
   
  TPaveText *t1 = new TPaveText(0.0,0.0,1.0,1.0);
  t1->SetFillColor(10);
  t1->SetTextAlign(13);
  t1->SetTextSize(0.3);
  t1->SetTextColor(4);

  sprintf(line,"ATLAS SCT Correlated Noise");
  if(time)  sprintf(line,"%s  -  %s",line,time->GetName());
  if(place) sprintf(line,"%s  -  %s",line,place->GetName());
  t1->AddText(line);

  sprintf(line,"Run %d   Scan %d  -  %d Events at Qthr = %.1ffC", runno, scanno, nevt,qthr);
  t1->AddText(line);
  pad_t->cd();
  t1->Draw();
  can->Update();

  // read histo
  sprintf(filename,"%scmn_%d_%d.root",DATA_DIRECTORY,runno,scanno);
  TFile* f = new TFile(filename);

  TH2F* histo = (TH2F*)f->Get("histo");
  
  // draw

  pad_p->cd();
  //pad_p->Range(-20,-15,MAX_MOD*12+15,MAX_MOD*12+15);

  histo->GetXaxis()->SetLabelColor(33);
  histo->GetXaxis()->SetAxisColor(33);
  histo->GetXaxis()->SetTickLength(0.0001);

  histo->GetYaxis()->SetLabelColor(33);
  histo->GetYaxis()->SetAxisColor(33);
  histo->GetYaxis()->SetTickLength(0.0001);

  histo->SetMaximum(1.0);
  histo->SetMinimum(-1.0);

  histo->Draw("COLZ");

  TGaxis *xax = new TGaxis(-0.5,-0.5,MAX_MOD*12-0.5,-0.5,0,MAX_MOD,2*100+MAX_MOD,"-U");
  xax->SetName("xaxis");
  xax->Draw();
  TGaxis *yax = new TGaxis(-0.5,-0.5,-0.5,MAX_MOD*12-0.5,0,MAX_MOD,2*100+MAX_MOD,"+U");
  yax->SetName("yaxis");
  yax->Draw();
  
  TGaxis *xtopax = new TGaxis(-0.5,MAX_MOD*12-0.5,MAX_MOD*12-0.5,MAX_MOD*12-0.5,0,MAX_MOD,2*100+MAX_MOD,"+U");
  xtopax->SetName("xtopaxis");
  xtopax->Draw();
  TGaxis *ytopax = new TGaxis(MAX_MOD*12-0.5,-0.5,MAX_MOD*12-0.5,MAX_MOD*12-0.5,0,MAX_MOD,2*100+MAX_MOD,"+U");
  ytopax->SetName("ytopaxis");
  ytopax->Draw();

  for(Int_t i=0;i<MAX_MOD;i++){
    TText t;
    t.SetTextSize(0.035);
    t.DrawText((Double_t)(i*12+1),-10.,mname[i]);
    t.DrawText(-15.,(Double_t)(i*12+3),mname[i]);
  }
  
  can->Update();

  ps->Close();
  //f->Close();
  return;
}
