void cn_chip_matrix(Int_t runno, Int_t scanno, Float_t qthr) {

  gROOT->Reset();
  
#include "../macros/systest/Evtree.h"
#include "../macros/parameters.h"

  Char_t line[1024],filename[70],module0[30],module1[30], filestore[50],mod_name[50],mname[MAX_MODULES][50];
  Int_t nmod, totchips, nevt, chip, chip2, nchips=MAX_CHIPS*MAX_LINKS, mod;

  Float_t meansqs[MAX_MODULES*MAX_CHIPS*MAX_LINKS];
  Float_t hpe[MAX_MODULES*MAX_CHIPS*MAX_LINKS] ,hpe2[MAX_MODULES*MAX_CHIPS*MAX_LINKS];
 
  Float_t mean[MAX_MODULES*MAX_CHIPS*MAX_LINKS],sd[MAX_MODULES*MAX_CHIPS*MAX_LINKS];
  Float_t meansqd[MAX_MODULES*MAX_CHIPS*MAX_LINKS];
  Float_t meanprod[MAX_MODULES*MAX_CHIPS*MAX_LINKS][MAX_MODULES*MAX_CHIPS*MAX_LINKS];
  Float_t correl[MAX_MODULES*MAX_CHIPS*MAX_LINKS][MAX_MODULES*MAX_CHIPS*MAX_LINKS]; 

  Double_t sumsqd[MAX_MODULES*MAX_CHIPS*MAX_LINKS],sum[MAX_MODULES*MAX_CHIPS*MAX_LINKS];
  Double_t prod[MAX_MODULES*MAX_CHIPS*MAX_LINKS][MAX_MODULES*MAX_CHIPS*MAX_LINKS];
 
  Evtree* m[MAX_MODULES];
  TTree* tree[MAX_MODULES];

  TDatime *dt = new TDatime;
  printf("starting program at %d\n",dt->GetTime());

  //open the file

  TFile* f;
  sprintf(filename,"d://sctvar//data//strun%d_%d.root",runno,scanno);
  f = new TFile(filename);
  
  //loop over modules (over the trees) and open everything

  nmod = 0;
  for(Int_t mod=0;mod<MAX_MODULES;mod++){
  //for(mod=0;mod<1;mod++){
 
    if(mod==0){
      TObjString* time  = (TObjString*)f->Get("Time");
      TObjString* place = (TObjString*)f->Get("Place");
    }

    sprintf(mod_name,"ModuleName;%d",mod+1);
    TObjString* name = (TObjString*)f->Get(mod_name);
    if(name){
      sprintf(mname[mod],"%s",name->GetName());
      if(!strcmp(mname[mod],"missing")) break;
      nmod++;
    }
    else
      break;

    sprintf(module0,"EvTree_m%d",mod);
    tree[mod] = (TTree*)gDirectory->Get(module0);
    m[mod] = new Evtree(tree[mod]);
   
    if(mod==0) 
      nevt = tree[mod]->GetEntries();
    else
      if(nevt!=tree[mod]->GetEntries()){
	printf("trees have different number of events -> stop\n");
	return;
      }

  }
  printf("found %d modules with trees with %d entries\n",nmod,nevt);
  //nevt=1000;

  // define histogram

  totchips = nmod*nchips;
  TH2F *histo = new TH2F("histo","correlated noise chip by chip",totchips,0,totchips,totchips,0,totchips);
  histo->SetStats(kFALSE);

  //now loop over the events and the chips

  for(chip=0;chip<totchips;chip++){
    sum[chip]=0.0;
    sumsqd[chip]=0.0;
    for(chip2=0;chip2<totchips;chip2++){
      prod[chip][chip2]=0.0;
    }
  }

  //start the event looop

  for(Int_t ev=0;ev<nevt;ev++){
    
    //laod arrays
    
    for(mod=0;mod<nmod;mod++){
      m[mod]->GetEntry(ev);
    }
 
    //get number of hits per chip in this event, hpe[chip]
  
    mod=-1;
    for(chip=0;chip<totchips;chip++){
  
      //if(chip%nchips==0) mod++;
      mod = chip/nchips;
     
      hpe[chip]=0.0;
      for(Int_t strip=0;strip<MAX_TARGETS;strip++){
	if((chip-mod*nchips)<MAX_CHIPS){
	  if(m[mod]->strm0[strip+((chip-mod*nchips)*MAX_TARGETS)])           hpe[chip]+=1.;
	}else{
	  if(m[mod]->strm1[strip+((chip-mod*nchips-MAX_CHIPS)*MAX_TARGETS)]) hpe[chip]+=1.0;
	}
      }	
    }
    
  
    for(chip=0;chip<totchips;chip++){
    
      sum[chip]=sum[chip]+(Double_t)hpe[chip];
      sumsqd[chip]=sumsqd[chip]+(Double_t)(hpe[chip])**2.0;
      
      if(hpe[chip]){

	for(chip2=0;chip2<totchips;chip2++){
	  
	  prod[chip][chip2]=prod[chip][chip2]+((Double_t)hpe[chip]*(Double_t)hpe[chip2]);
	}
      }
    }
    if((ev+1)%100==0) printf("finished event %d \n",ev+1);  
    
  }

  //now have arrays sum, sumsqd and prod

  //do the maths on the arrays
  
  for(chip=0;chip<totchips;chip++){
  
    //find mean for chips
    
    mean[chip]=(Float_t)sum[chip]/(Float_t)nevt;
  
    //find mean of squares for chip

    meansqs[chip]=(Float_t)sumsqd[chip]/(Float_t)nevt;

    //find mean sqd

    meansqd[chip]=(mean[chip])**2;

    for(chip2=0;chip2<totchips;chip2++){

      //find mean of products

      meanprod[chip][chip2]=prod[chip][chip2]/(Float_t)nevt;
    }
  }
  
  for(chip=0;chip<totchips;chip++){
    for(chip2=0;chip2<totchips;chip2++){
     
      if((((meansqs[chip]-meansqd[chip])**0.5)*((meansqs[chip2]-meansqd[chip2])**0.5))==0){
	correl[chip][chip2]=0.0;
      }else{      
	correl[chip][chip2] = (meanprod[chip][chip2]-(mean[chip]*mean[chip2]))/
	                        (((meansqs[chip]-meansqd[chip])**0.5)*((meansqs[chip2]-meansqd[chip2])**0.5));
      }

      histo->Fill(chip,chip2,correl[chip][chip2]);  
    }
  }
  
  TDatime *dt2 = new TDatime;
  printf("finished analysis at %d\n",dt2->GetTime());

  // define colour palette
  
  const Int_t colNum=50;
  Int_t palette[2*colNum];
  for(Int_t i=0;i<colNum;i++){
    if(! gROOT->GetColor(100+i)){
      TColor *color = new TColor (100+i,1,(i/50.0),(i/50.0),"");
    }else{
      TColor *color = gROOT->GetColor(100+i);
      color->SetRGB(1,(i/50.0),(i/50.0));
    }
    palette[i]=100+i;
  }
  for(Int_t i=0;i<colNum;i++){
    if(! gROOT->GetColor(100+i+colNum)){
      TColor *color = new TColor (100+i+colNum,1-(i/50.0),1-(i/50.0),1,"");
    }else{
      TColor *color = gROOT->GetColor(100+i+colNum);
      color->SetRGB(1-(i/50.0),1-(i/50.0),1);
    }
    palette[i+colNum]=100+i+colNum;
  }
  gStyle->SetPalette(2*colNum,palette);

  //set up the canvas

  TCanvas *can = new TCanvas("c","c",1100,750);
  can->cd();
  can->Clear();
  
  // open PS output file
  sprintf(filename,"D:\\sctvar\\ps\\CorrelNoise_%d_%d.ps",runno,scanno);
  ps = new TPostScript(filename,112);

  ps->NewPage();
  pad_t = new TPad("pad_t","Text",        0.00,0.90,1.00,1.00,33);
  pad_p = new TPad("pad_p","Projections",   0.00,0.00,1.00,0.90,33);
  pad_t->Draw();
  pad_p->Draw();
   
  TPaveText *t1 = new TPaveText(0.0,0.0,1.0,1.0);
  t1->SetFillColor(10);
  t1->SetTextAlign(13);
  t1->SetTextSize(0.3);
  t1->SetTextColor(4);

  sprintf(line,"ATLAS SCT Correlated Noise");
  if(time)  sprintf(line,"%s  -  %s",line,time->GetName());
  if(place) sprintf(line,"%s  -  %s",line,place->GetName());
  t1->AddText(line);

  sprintf(line,"Run %d   Scan %d  -  %d Events at Qthr = %.1ffC", runno, scanno, nevt, qthr);
  t1->AddText(line);
  pad_t->cd();
  t1->Draw();
  can->Update();

  // draw

  pad_p->cd();

  histo->GetXaxis()->SetLabelColor(33);
  histo->GetXaxis()->SetAxisColor(33);
  histo->GetXaxis()->SetTickLength(0.0001);

  histo->GetYaxis()->SetLabelColor(33);
  histo->GetYaxis()->SetAxisColor(33);
  histo->GetYaxis()->SetTickLength(0.0001);

  histo->SetMaximum(1.0);
  histo->SetMinimum(-1.0);

  histo->Draw("COLZ");

  TGaxis *xax = new TGaxis(0,0,totchips,0,0,nmod,2*100+nmod,"-U");
  xax->SetName("xaxis");
  xax->Draw();
  TGaxis *yax = new TGaxis(0,0,0,totchips,0,nmod,2*100+nmod,"+U");
  yax->SetName("yaxis");
  yax->Draw();
  
  TGaxis *xtopax = new TGaxis(0,totchips,totchips,totchips,0,nmod,2*100+nmod,"+U");
  xtopax->SetName("xtopaxis");
  xtopax->Draw();
  TGaxis *ytopax = new TGaxis(totchips,0,totchips,totchips,0,nmod,2*100+nmod,"+U");
  ytopax->SetName("ytopaxis");
  ytopax->Draw();

  for(Int_t i=0;i<nmod;i++){
    TText t;
    t.SetTextSize(0.035);
    t.DrawText((Double_t)(i*nchips+1),-10.,mname[i]);
    t.DrawText(-15.,(Double_t)(i*nchips+3),mname[i]);
  }
  
  can->Update();

  ps->Close();

  return;
}
