#include "../macros/parameters.h"

cn_plot(Int_t runno, Int_t scanno, Float_t qthr, Bool_t zoom=kFALSE, Float_t zoom_scale=0.1){

  Int_t nmod, totchips,nevt,nchips=MAX_CHIPS*MAX_LINKS;
  Char_t fname[128],line[1024],mname[MAX_MODULES][50];
  sprintf(fname,"%scnmx_%d_%d.root",RESULTS_DIRECTORY,runno,scanno);
  TFile* f = new TFile(fname);
  if(zoom){
    TH2F* histo_zoom = (TH2F*)f->Get("cmn_histo");
    TH2F* histo = histo_zoom->Clone();
    histo->Reset();
    Int_t nx=histo_zoom->GetNbinsX();
    Int_t ny=histo_zoom->GetNbinsY();
    for(Int_t i=0;i<nx;i++){
      for(Int_t j=0;j<ny;j++){
	Float_t content = histo_zoom->GetBinContent(histo_zoom->GetBin(i+1,j+1));
	if(content>zoom_scale)  {
	  printf("bin %d,%d is pos. saturated\n",i,j);
	  content=zoom_scale;
	}
	if(content<-zoom_scale){
	  content=-zoom_scale;
	  printf("bin %d,%d is neg. saturated\n",i,j);
	}
	histo->SetCellContent(i+1,j+1,content);	
      }
    }
  }
  else
    TH2F* histo = (TH2F*)f->Get("cmn_histo");

  nmod = 0;
  for(Int_t mod=0;mod<MAX_MODULES;mod++){

    if(mod==0){
      TObjString* time  = (TObjString*)f->Get("Time");
      TObjString* place = (TObjString*)f->Get("Place");
      TObjString* nevo  = (TObjString*)f->Get("Events");
      sscanf(nevo->GetName(),"%d", &nevt);
    }

    sprintf(fname,"ModuleName;%d",mod+1);
    TObjString* name = (TObjString*)f->Get(fname);
    if(name){
      sprintf(mname[mod],"%s",name->GetName());
      if(!strcmp(mname[mod],"missing")) break;
      nmod++;
    }
    else
      break;
  }
  totchips = nmod*MAX_CHIPS*MAX_LINKS;

  // define colour palette
  
  const Int_t colNum=50;
  Int_t palette[2*colNum];
  for(Int_t i=0;i<colNum;i++){
    if(! gROOT->GetColor(100+i)){
      TColor *color = new TColor (100+i,1,(i/50.0),(i/50.0),"");
    }else{
      TColor *color = gROOT->GetColor(100+i);
      color->SetRGB(1,(i/50.0),(i/50.0));
    }
    palette[i]=100+i;
  }
  for(Int_t i=0;i<colNum;i++){
    if(! gROOT->GetColor(100+i+colNum)){
      TColor *color = new TColor (100+i+colNum,1-(i/50.0),1-(i/50.0),1,"");
    }else{
      TColor *color = gROOT->GetColor(100+i+colNum);
      color->SetRGB(1-(i/50.0),1-(i/50.0),1);
    }
    palette[i+colNum]=100+i+colNum;
  }
  gStyle->SetPalette(2*colNum,palette);

  //set up the canvas

  TCanvas *can = new TCanvas("c","c",1100,750);
  can->cd();
  can->Clear();
  
  pad_t = new TPad("pad_t","Text",        0.00,0.90,1.00,1.00,33);
  pad_p = new TPad("pad_p","Projections",   0.00,0.00,1.00,0.90,33);
  pad_t->Draw();
  pad_p->Draw();
   
  TPaveText *t1 = new TPaveText(0.0,0.0,1.0,1.0);
  t1->SetFillColor(10);
  t1->SetTextAlign(13);
  t1->SetTextSize(0.3);
  t1->SetTextColor(4);

  sprintf(line,"ATLAS SCT Correlated Noise");
  if(time)  sprintf(line,"%s  -  %s",line,time->GetName());
  if(place) sprintf(line,"%s  -  %s",line,place->GetName());
  t1->AddText(line);

  sprintf(line,"Run %d   Scan %d  -  %d Events at Qthr = %.1ffC", runno, scanno, nevt, qthr);
  t1->AddText(line);
  pad_t->cd();
  t1->Draw();
  can->Update();

  // draw

  pad_p->cd();

  histo->GetXaxis()->SetLabelColor(33);
  histo->GetXaxis()->SetAxisColor(33);
  histo->GetXaxis()->SetTickLength(0.0001);

  histo->GetYaxis()->SetLabelColor(33);
  histo->GetYaxis()->SetAxisColor(33);
  histo->GetYaxis()->SetTickLength(0.0001);

  if(zoom){
    histo->SetMaximum(zoom_scale*(1.+1.e-5));
    histo->SetMinimum(-zoom_scale*(1.+1.e-5));
    histo->SetTitle("correlated noise strip by strip - chip summary - zoomed scale");
  }else{
    histo->SetMaximum(1);
    histo->SetMinimum(-1);
    histo->SetTitle("correlated noise strip by strip - chip summary");
  }

  histo->Draw("COLZ");

  TGaxis *xax = new TGaxis(0,0,totchips,0,0,nmod,2*100+nmod,"-U");
  xax->SetName("xaxis");
  xax->Draw();
  TGaxis *yax = new TGaxis(0,0,0,totchips,0,nmod,2*100+nmod,"+U");
  yax->SetName("yaxis");
  yax->Draw();
  
  TGaxis *xtopax = new TGaxis(0,totchips,totchips,totchips,0,nmod,2*100+nmod,"+U");
  xtopax->SetName("xtopaxis");
  xtopax->Draw();
  TGaxis *ytopax = new TGaxis(totchips,0,totchips,totchips,0,nmod,2*100+nmod,"+U");
  ytopax->SetName("ytopaxis");
  ytopax->Draw();

  for(Int_t i=0;i<nmod;i++){
    TText t;
    t.SetTextSize(0.035);
    t.DrawText((Double_t)(i*nchips+1),-10.,mname[i]);
    t.DrawText(-15.,(Double_t)(i*nchips+3),mname[i]);
  }
  
  can->Update();

  //f->Close();

  return;
}
