void cn_strip_fullmatrix(Int_t runno, Int_t scanno, Float_t qthr, Int_t mod, Int_t chip1=0, Int_t chip2=0) {

  gROOT->Reset();
  
#include "../macros/systest/Evtree.h"
#include "../macros/parameters.h"

  Char_t line[1024],filename[70],module0[30],module1[30], filestore[50],mod_name[50],mname[MAX_MODULES][50];
  Int_t nmod, totstrips, nevt, strip2, nstrips=MAX_CHANS*MAX_LINKS, strip, mod, mod2, strip3, strip4;

  Bool_t hita, hitb;

  Float_t chipcorrel, stripcorrel;

  Double_t denom, prod[MAX_TARGETS][MAX_TARGETS], suma[MAX_TARGETS], sumb[MAX_TARGETS];
 
  Evtree* m;
  TTree* tree;

  TDatime *dt = new TDatime;
  printf("starting program at %d\n",dt->GetTime());

  // define histogram

  //totstrips = nmod*nstrips;
  totstrips=MAX_TARGETS;
  TH2F *histo = new TH2F("histo","correlated noise",
			 totstrips,chip1*totstrips,(chip1+1)*totstrips,totstrips,chip2*totstrips,(chip2+1)*totstrips);
  histo->SetStats(kFALSE);
 
  //open the file

  TFile* f;
  sprintf(filename,"d://sctvar//data//strun%d_%d.root",runno,scanno);
  f = new TFile(filename);
  
  // find module
 
  TObjString* time  = (TObjString*)f->Get("Time");
  TObjString* place = (TObjString*)f->Get("Place");

  sprintf(mod_name,"ModuleName;%d",mod+1);
  TObjString* name = (TObjString*)f->Get(mod_name);
  if(name){
    sprintf(mname[mod],"%s",name->GetName());
    if(!strcmp(mname[mod],"missing")) break;
    nmod++;
  }
  else{
    printf("cannot find requested module\n");
    return;
  }
    
  sprintf(module0,"EvTree_m%d",mod);
  tree = (TTree*)gDirectory->Get(module0);
  m = new Evtree(tree);
   
  nevt = tree->GetEntries();
  printf("found tree with %d entries\n",nevt);
  //nevt=1000;

  //reset array
  for(strip=0;strip<totstrips;strip++){
    suma[strip]=0.;
    sumb[strip]=0.;
    for(strip2=0;strip2<totstrips;strip2++)
      prod[strip][strip2]=0.0;
  }

  //start the event looop
  for(Int_t ev=0;ev<nevt;ev++){
    //load arrays
    m->GetEntry(ev);
    if(ev%100==0) printf("processing event %d\n",ev);

    //get hits per strip in this event
  
    for(strip=0;strip<totstrips;strip++){
    
      strip3 = strip+chip1*totstrips;
      strip4 = strip+chip2*totstrips;
      
      if(strip3<MAX_CHANS)
	hita = m->strm0[strip3];
      else			     
	hita = m->strm1[strip3-MAX_CHANS];
      
      if(hita) suma[strip]++;

      if(strip4<MAX_CHANS)
	hitb = m->strm0[strip4];
      else	  
	hitb = m->strm1[strip4-MAX_CHANS];

      if(hitb) sumb[strip]++;
      
      if(hita){

	for(strip2=0;strip2<totstrips;strip2++){	
	  
	  strip4 = strip2+chip2*totstrips;
	  
	  if(strip4<MAX_CHANS)
	    hitb = m->strm0[strip4];
	  else	  
	    hitb = m->strm1[strip4-MAX_CHANS];
	  
	  if(hitb)
	    prod[strip][strip2]++;
	}
      }

    }
  }
  f->Close();
  
  //find mean
  for(strip=0;strip<totstrips;strip++){
    suma[strip]/=(Double_t)nevt;
    sumb[strip]/=(Double_t)nevt;
    for(strip2=0;strip2<totstrips;strip2++)
      prod[strip][strip2]/=(Double_t)nevt;
  }

  //do the maths
  
  for(strip=0;strip<totstrips;strip++){
    for(strip2=0;strip2<totstrips;strip2++){
      denom = TMath::Sqrt(suma[strip]-suma[strip]*suma[strip])*
                   TMath::Sqrt(sumb[strip2]-sumb[strip2]*sumb[strip2]);
      if(denom)
	stripcorrel = (Float_t)((prod[strip][strip2]-suma[strip]*sumb[strip2])/denom);
      else{
	//printf("warning: zero denominator in element %d %d\n",strip,strip2);
	stripcorrel = 0.;
      }
      //printf("c_%d_%d = %f\n",strip,strip2,stripcorrel);
      //histo->Fill(strip,strip2,stripcorrel);
      histo->SetCellContent(strip+1,strip2+1,stripcorrel);
    }
  }
  
  TDatime *dt2 = new TDatime;
  printf("finished analysis at %d\n",dt2->GetTime());

  // define colour palette
  
  const Int_t colNum=50;
  Int_t palette[2*colNum];
  for(Int_t i=0;i<colNum;i++){
    if(! gROOT->GetColor(100+i)){
      TColor *color = new TColor (100+i,1,(i/50.0),(i/50.0),"");
    }else{
      TColor *color = gROOT->GetColor(100+i);
      color->SetRGB(1,(i/50.0),(i/50.0));
    }
    palette[i]=100+i;
  }
  for(Int_t i=0;i<colNum;i++){
    if(! gROOT->GetColor(100+i+colNum)){
      TColor *color = new TColor (100+i+colNum,1-(i/50.0),1-(i/50.0),1,"");
    }else{
      TColor *color = gROOT->GetColor(100+i+colNum);
      color->SetRGB(1-(i/50.0),1-(i/50.0),1);
    }
    palette[i+colNum]=100+i+colNum;
  }
  gStyle->SetPalette(2*colNum,palette);

  //set up the canvas

  TCanvas *can = new TCanvas("c","c",900,1000);
  can->cd();
  can->Clear();
  
  // open PS output file
  sprintf(filename,"D:\\sctvar\\ps\\CorrelNoise_%d_%d.ps",runno,scanno);
  ps = new TPostScript(filename,112);

  ps->NewPage();
  pad_t = new TPad("pad_t","Text",        0.00,0.90,1.00,1.00,33);
  pad_p = new TPad("pad_p","Projections",   0.00,0.00,1.00,0.90,33);
  pad_t->Draw();
  pad_p->Draw();
   
  TPaveText *t1 = new TPaveText(0.0,0.0,1.0,1.0);
  t1->SetFillColor(10);
  t1->SetTextAlign(13);
  t1->SetTextSize(0.3);
  t1->SetTextColor(4);

  sprintf(line,"ATLAS SCT Correlated Noise");
  if(time)  sprintf(line,"%s  -  %s",line,time->GetName());
  if(place) sprintf(line,"%s  -  %s",line,place->GetName());
  t1->AddText(line);

  sprintf(line,"Run %d   Scan %d  Module %s -  %d Events at Qthr = %.1ffC", runno, scanno, mname[mod], nevt, qthr);
  t1->AddText(line);
  pad_t->cd();
  t1->Draw();
  can->Update();

  // draw

  pad_p->cd();

  histo->SetMaximum(1.0);
  histo->SetMinimum(-1.0);

  histo->Draw("COLZ");
  
  can->Update();

  ps->Close();

  return;
}
