void stan_all_scurves(short run, short scan, short moi, short loi, short start_ch, short nch){
  char f_name[64], h_name[32], h_title[32], out_fname[64];
  char labelt[10],s[256],scan_name[128],unit[12];
  float scan_start, scan_stop, scan_step;
  int  chip, i, j, n;
  short scan_type, scan_max_points;

  // Revised Version 0.2 PWP 11.03.00
  // Revised Version 0.3 PWP 08.08.01 - use parameters.h
  // Revised version 0.31 PWP 12.04.02 - now knows about command delay scans
  // Revised version 0.32 BJG 30.04.02 - fix some memory leaks
  // Revised version 0.33 PWP 29.05.02 - put (some of) them back again!

  #ifdef DATA_DIRECTORY
    // do not reload parameters.h
  #else
    // must load parameters.h
  #include "../macros/parameters.h"
  #endif

  printf("stan_all_scurves called for module %d link %d\n",moi,loi);

  // Load other files needed by this one
  gROOT->LoadMacro("stan_util.cpp");

// Options which change macro output
  int postscript = 1;

// Global style variations

  stan_styles();
  gStyle->SetCanvasColor(0);
  gStyle->SetMarkerStyle(21);
  gStyle->SetMarkerSize(0.3);
  gStyle->SetMarkerColor(1);
  gStyle->SetLineColor(2);
  gStyle->SetOptStat(0);

// Open file, or obtain pointer to histogram in memory

  TObjString *time = 0;
  TObjString *place = 0;
  TObjString *mname = 0;
  
  if(run==-1){  

    if(loi) TH2F *h_scan = (TH2F*)e->m[moi]->h_scan1;
    else    TH2F *h_scan = (TH2F*)e->m[moi]->h_scan0;
    TH1F *h_nevent = (TH1F*)e->h_scan_evcnt;

    run = e->runnum;
    scan = e->scan_count;

    n = (h_scan->GetMaximum());
    if (n<1.0) n =1.0;

    scan_type = e->scan_type;
    sprintf(scan_name,"%s",e->scan_name);
    scan_max_points = e->scan_max_points; 
   
    if(e->scan_count==0){
      printf("  NO DATA in memory!!  exiting\n\n");
      return;
    }
    printf("  Using Data in memory\n");

    TDatime *dt = new TDatime;
    time = new TObjString(dt->AsString());
    place =0; 
    mname =0;

    delete dt;

    sprintf(unit,"");
 
  } else {

    sprintf(f_name,"%sstrun%d_%d.root",DATA_DIRECTORY,run,scan);

    if(loi) sprintf(h_name,"h_scan1;%d",moi+1);
    else    sprintf(h_name,"h_scan0;%d",moi+1);
    printf("  Using histogram %s from ROOT Datafile %s\n",h_name,f_name);

    TFile *strun = new TFile(f_name);
    TH2F *h_scan= (TH2F*)strun->Get(h_name);
    TH1F *h_nevent= (TH1F*)strun->Get("h_scan_evcnt");

    sprintf(h_name,"ModuleName;%d",moi+1); // PWP bugfix 07.04.00
    time  = (TObjString*)strun->Get("Time");
    place = (TObjString*)strun->Get("Place");
    mname = (TObjString*)strun->Get(h_name);

    n = (h_scan->GetMaximum());
    if (n<1.0) n =1.0;

    sprintf(scan_name,"Unknown");

    TString title = h_scan->GetTitle();
    title.ToUpper();
    if (title.Contains("THRESHOLD")){
      scan_type = 1;
      sprintf(scan_name,"THRESHOLD (mV)");
      sprintf(unit,"mV");
    }
    if (title.Contains("VCAL")){
      scan_type = 2;
      sprintf(scan_name,"VCAL (mV)");
      sprintf(unit,"mV");
    }
    if (title.Contains("STROBE")){
      scan_type = 3;
      sprintf(scan_name,"Strobe Delay");
      sprintf(unit,"");
    }
    if (title.Contains("PREAMP")){
      scan_type = 4;
      sprintf(scan_name,"Preamp Current (uA)");
      sprintf(unit,"uA");
    }
    if (title.Contains("SHAPER")){
      scan_type = 5;
      sprintf(scan_name,"Shaper Current (uA)");
      sprintf(unit,"uA");
    }
    if (title.Contains("COMMAND")){
      scan_type = 5;
      sprintf(scan_name,"Command Delay(BCO)");
      sprintf(unit,"BCO");
    }
  }

// titles for graphs - might as well do this now.

  float x[128];
  float xe[128];
  float y[768][128];
  float ye[768][128];

  TAxis * axis = h_scan->GetYaxis();
  scan_max_points = axis->GetNbins();
  scan_start = axis->GetBinCenter(1);
  scan_stop  = axis->GetBinCenter(scan_max_points);
  scan_step  = axis->GetBinWidth(1);

  printf("scan_start %f, stop %f, npoints %f\n",scan_start, scan_stop, scan_max_points);

  for(i=0;i<scan_max_points; i++){
    x[i] = axis->GetBinCenter(i+1);
    xe[i] = 1;
  }

  for(i=0;i<768;i++){
    for(j=0;j<scan_max_points; j++){
      y[i][j] = h_scan->GetBinContent((770*(j+1))+i);    //PWP bugfix  10.03.00
      ye[i][j] = 1; // TODO - proper binomial error stats
    }
  }

  if(postscript){
    sprintf(out_fname,"%sstscrun%d_%d_%d_%d.ps",PS_DIRECTORY,run,scan,moi,loi);

    TPostScript *ps = new TPostScript(out_fname,112);
    printf("Generating postscript file %s\n",out_fname);
  }

// New canvas for scurves

  sprintf(s,"%s - scurves Module %d Link %d", scan_name, moi, loi);
  TCanvas* csc=new TCanvas ("csc",s,1100,750);
  pad_t0 = new TPad("pad_t0","title",        0.00,0.95,1.00,1.000,33);
  pad_c0 = new TPad("pad_c0","Scurves CAL0", 0.00,0.00,1.00,0.950,33);

  pad_c0->Divide(8,4,0,0,10);

  pad_t0->Draw();
  pad_c0->Draw();

  TPaveText *ttitle = new TPaveText(0.01,0.05,0.99,0.95);
  ttitle->SetFillColor(10);  
  ttitle->SetTextAlign(13);
  ttitle->SetTextSize(0.4);
  ttitle->SetTextColor(4);

  sprintf(s,"ATLAS SCT Module Test");
  if(time) { sprintf(s,"%s  -  %s",s,time->GetName()); delete time; }
  if(place) { sprintf(s,"%s  -  %s",s,place->GetName()); delete place; }
  if(mname) { sprintf(s,"%s  -  Module %s",s,mname->GetName()); delete mname; }
  ttitle->AddText(s);
  sprintf(s,"Run %d Scan %d Module %d Link %d - %s from %3.2f%s to %3.2f%s in %3.2f%s steps, total %d points",
    run,scan,moi,loi,scan_name,scan_start,unit,scan_stop,unit,scan_step,unit,scan_max_points);
  ttitle->AddText(s);

  TGraph* gr[768];
  TH2F*   hr[768];

  for(i=0;i<nch;i++){

    if((i%32)==0){
      if(postscript) ps->NewPage();
      pad_t0->cd();
      ttitle->Draw();
    } 

    pad_c0->cd((i%32)+1);

    sprintf(h_name,"h_sc%d",i);
    sprintf(h_title,"Channel %d",i+start_ch);
    hr[i] = new TH2F(h_name,h_title,2,scan_start,scan_stop,2,-(n*0.1),(n*1.1));
    hr[i]->Draw();  
    hr[i]->SetXTitle(scan_name);  

    gr[i] = new TGraph(scan_max_points,x,y[i+start_ch]);
    gr[i]->SetTitle(h_title);
    gr[i]->Draw("LP");

    if((i%32)==31) csc->Update();
  }

  if(postscript) {
    ps->Close();
    delete ps;
  }
}

void stan_all_scurves(short moi, short loi){

  stan_all_scurves(-1,0,moi,loi,0,768);
}

void stan_all_scurves(short run, short scan, short moi, short loi){

  stan_all_scurves(run,scan,moi,loi,0,768);
}

void stan_all_scurves(){

  printf("Possible Uses of stan_all_scurves:\n\n");
  printf(".x stan_all_scurves.cpp(module,link)\n");
  printf(".x stan_all_scurves.cpp(run,scan,module,link)\n");
  printf(".x stan_all_scurves.cpp(run,scan,module,link,start_ch,n_ch)\n");

}


