void stan_compare(int run1,int scan1,int moi1,int loi1,int run2,int scan2,
             int moi2,int loi2, int lo_ch, int hi_ch, int log){

  char f_name1[64],f_name2[64], h_name1[32],h_name2[32],s[128];

  // Revised Version 0.2 PWP 08.08.01 - use parameters.h

  #ifdef DATA_DIRECTORY
    // do not reload parameters.h
  #else
    // must load parameters.h
  #include "../macros/parameters.h"
  #endif

  // Load other files needed by this one
  gROOT->LoadMacro("stan_util.cpp");

  // Global style variations
  stan_styles();
  gStyle.SetOptStat(0);

  sprintf(f_name1,"%sstrun%d_%d.root",DATA_DIRECTORY,run1,scan1);

  if(loi1)sprintf(h_name1,"h_scan1;%d",moi1+1);
  else    sprintf(h_name1,"h_scan0;%d",moi1+1);
  printf("  Histogram1: %s from ROOT Datafile %s\n",h_name1,f_name1);

  TFile *strun1 = new TFile(f_name1);
  TH2F  *h1= (TH2F*)strun1->Get(h_name1);
  TAxis *yaxis1 = h1->GetYaxis();
  Text_t *ytitle1 = yaxis1->GetTitle(); 

  sprintf(f_name2,"%sstrun%d_%d.root",DATA_DIRECTORY,run2,scan2);

  if(loi2)sprintf(h_name2,"h_scan1;%d",moi2+1);
  else    sprintf(h_name2,"h_scan0;%d",moi2+1);
  printf("  Histogram1: %s from ROOT Datafile %s\n",h_name2,f_name2);

  TFile *strun2 = new TFile(f_name2);
  TH2F  *h2= (TH2F*)strun2->Get(h_name2);
  TAxis *yaxis2 = h2->GetYaxis();
  Text_t *ytitle2 = yaxis2->GetTitle(); 


  TCanvas* c1=new TCanvas ("c1","stan_compare",1100,750);
  pad_t = new TPad("pad_t","Text",           0.01,0.91,0.99,0.99,33);
  pad_s1 = new TPad("pad_s1","Scan 1",       0.01,0.45,0.60,0.90,33);
  pad_s2 = new TPad("pad_s2","Scan 2",       0.01,0.01,0.60,0.45,33);
  pad_p1 = new TPad("pad_p1","Projection 1", 0.60,0.45,0.99,0.90,33);
  pad_p2 = new TPad("pad_p2","Projection 2", 0.60,0.01,0.99,0.45,33);

  pad_t->Draw();
  pad_s1->Draw();
  pad_s2->Draw();
  pad_p1->Draw();
  pad_p2->Draw();

  if (log){
    pad_s1->SetLogz();
    pad_s2->SetLogz();
    pad_p1->SetLogy();
    pad_p2->SetLogy();
  }

  TPaveText *t1 = new TPaveText(0.0,0.0,1.0,1.0);
  t1->SetFillColor(10);
  t1->SetTextAlign(13);
  t1->SetTextSize(0.3);
  t1->SetTextColor(4);
  if(log) t1->AddText("ATLAS SCT Scan Comparison - log scale");
  else    t1->AddText("ATLAS SCT Scan Comparison - linear scale");

  sprintf(s,"Run %d   Scan %d   Module %d   Stream %d (Projection of ch %d to ch %d) vs", run1, scan1, moi1, loi1, lo_ch, hi_ch);
  t1->AddText(s);
  sprintf(s,"Run %d   Scan %d   Module %d   Stream %d (Projection of ch %d to ch %d)", run2, scan2, moi2, loi2, lo_ch, hi_ch);
  t1->AddText(s);
  pad_t->cd();
  t1->Draw();

  pad_s1->cd();
  if(log>1)h1->SetMaximum(1);
  h1->Draw("COLZ");
  pad_s2->cd();
  if(log>1)h2->SetMaximum(1);
  h2->Draw("COLZ");

  float s_factor=((float)1.0/(float)((hi_ch-lo_ch)+1));
  printf("scale factor= %3.2E\n",s_factor);

  pad_p1->cd();
  h1->ProjectionY("h1p",lo_ch,hi_ch);
  h1p->Scale(s_factor);
  h1p->SetXTitle(ytitle1);
  if(log) h1p->SetYTitle("Log(occupancy)");
  else    h1p->SetYTitle("Occupancy");
  if(log>1) h1p->SetMaximum(1);
  h1p->Draw();

  pad_p2->cd();
  h2->ProjectionY("h2p",lo_ch,hi_ch);
  h2p->Scale(s_factor);
  h2p->SetXTitle(ytitle2);
  if(log) h2p->SetYTitle("Log(occupancy)");
  else    h2p->SetYTitle("Occupancy");
  if(log>1) h2p->SetMaximum(1);
  h2p->Draw();

  // To plot the sum of the two projections
  // instead of h2p, uncomment the next three lines:
  // h_ptr1 = (TH1*) gDirectory->GetList()->FindObject(h1p);
  // h2p->Add(h_ptr1,1);
  // h2p->SetTitle("Sum of Projections");

  c1->Update();
}

void stan_compare(int run1,int scan1,int moi1,int loi1,int run2,int scan2,int moi2,int loi2, int log){

  stan_compare(run1, scan1, moi1, loi1, run2, scan2, moi2, loi2, 0, 768, log);
}

void stan_compare(int run1,int scan1,int moi1,int loi1,int run2,int scan2,int moi2,int loi2){
  // assume linear scale if called without additional arguments

  stan_compare(run1, scan1, moi1, loi1, run2, scan2, moi2, loi2, 0, 768, 0);

}

void stan_compare(){

  printf("Possible Uses of stan_compare:\n\n");
  printf(".x stan_compare.cpp(run1,scan1,moi1,loi1,run2,scan2,moi2,loi2)\n");
  printf(".x stan_compare.cpp(run1,scan1,moi1,loi1,run2,scan2,moi2,loi2,log)\n");
  printf(".x stan_compare.cpp(run1,scan1,moi1,loi1,run2,scan2,moi2,loi2,lo_ch,hi_ch,log)\n");

}

