Int_t get_val(char* line, Int_t index, double* val){

  // convert next element of data row "line" to type double
  // (CINT sscanf is limited to <43 elements, hence this approach)

  char substring[20];

  sscanf(line+=index,"%s ",substring);
  sscanf(substring,"%lf",val);
  return 1 + strlen(substring);
}

void stan_fitrc(int run, int sno, int moi, int loi, int chipset, int fit_type, 
                double range_lo, double range_hi, int ascii, int postscript){

  // Routine to plot and fit the response curve for a module
  // Supports three fit options:
  //   1 - Second Order Polynomial
  //   2 - "Grillo" function
  //   3 - Exponential fit

  // Version 0.3 P.W.Phillips, 24.11.99
  // includes bugfix reported by Nobu, affecting gain and input noise
  // from polynomial fit

  // Version 0.4 PWP 31.05.00
  // make range parameters arguments
  // Version 0.41 BJG 30.04.02   Fix some Memory leaks

  char  f_name[64], scan_name[32];
  char  h_name1[32], h_name2[32], h_name3[32], h_name4[32];
  char  h_title1[32],h_title2[32],h_title3[32],h_title4[32];
  char  m_title[32],s_title[32],out_fname[128], s[256], unit[12], line[1024];
  int   i,j,k,n,x,y, index;
  double var;

  // Locality options now moved to stan_util.cpp

  // Options which change macro output
  int draw_fit_projections = 0;

  printf("stan_fitrc called for module %d link %d\n",moi,loi);

  // Load other files needed by this one
  gROOT->LoadMacro("stan_util.cpp");

  // Global style variations
  stan_styles();
  gStyle.SetOptStat(0);
  gStyle.SetOptFit(10);


  // Read file "stan_rc.dat"
  // which gives scan numbers and charges
  // of up to MAX_SCANS runs 

  #define MAX_SCANS 20;

  Float_t scan[MAX_SCANS];
  Float_t q[MAX_SCANS];
  Float_t qe[MAX_SCANS];

  if (RAL) sprintf(f_name,"M:\\sctdaq\\analysis\\stan_rc.dat");
  else sprintf(f_name,"C:\\sctdaq\\analysis\\stan_rc.dat");
  FILE* fitfile = fopen(f_name,"r");
  if(fitfile==NULL){
    printf("Could not open scan/charge info file %s\n",f_name);
    return;
  } else {
    printf("File %s opened for reading\n",f_name);

    int n_points=0;

    fgets(line,1024,fitfile);
    while((fgets(line,1024,fitfile) != NULL)&&(n_points<MAX_SCANS)){
      index = 0;
      index+=get_val(line,index,&var);
      scan[n_points]=(float)var;
      index+=get_val(line,index,&var);
      q[n_points]=(float)var;
      qe[n_points]=0.1;      // ?????

      printf("Point %d Scan %d Charge %3.2f\n",n_points,scan[n_points],q[n_points]);
      n_points++;
    }
    fclose(fitfile);
  }

  // new section 24.11.99
  // load the second of the root files to obtain timestamp info

  if(RAL) sprintf(f_name,"M:\\sctvar\\data\\strun%d_%d.root",run,sno+1);
  else    sprintf(f_name,"D:\\sctvar\\data\\strun%d_%d.root",run,sno+1);
  
  printf("Using Time/Place/Module name information from %s\n",f_name); 
  TFile *strun = new TFile(f_name);

  sprintf(h_name1,"ModuleName;%d",moi+1);
  TObjString *time  = (TObjString*)strun->Get("Time");
  TObjString *place = (TObjString*)strun->Get("Place");
  TObjString *mname = (TObjString*)strun->Get(h_name1);

  // load data

  Float_t vt50[768][MAX_SCANS];
  Float_t vt50_err[768][MAX_SCANS];
  Float_t sigma[768][MAX_SCANS];
  Float_t sigma_err[768][MAX_SCANS];

  for(j=0;j<n_points;j++){
    if (RAL) sprintf(f_name,"M:\\sctvar\\data\\strun%d_%d_%d_%d.fit",run,(sno+j),moi,loi);
    else sprintf(f_name,"D:\\sctvar\\data\\strun%d_%d_%d_%d.fit",run,(sno+j),moi,loi);
    FILE* fitfile = fopen(f_name,"r");

    if(fitfile==NULL){
      printf("Could not open fit file %s\n",f_name);
      return;
    } else {
      printf("File %s opened for reading\n",f_name);

      i=0;

      fgets(line,1024,fitfile);
      while(fgets(line,1024,fitfile) != NULL){
        index = 10;
        index+=get_val(line,index,&var);
        vt50[i][j]=(float)var;
        index+=get_val(line,index,&var);
        vt50_err[i][j]=(float)var;
        index+=get_val(line,index,&var);
        sigma[i][j]=(float)var;
        index+=get_val(line,index,&var);
        sigma_err[i][j]=(float)var;
        i++;
      }
    fclose(fitfile);
    }
  }

  // Chip projections - something a bit different here.
  // Create and fill 6 * npoints histograms for the vt50 values
  // fit mean and sigma - use these values to fill chip projection histograms.

  Float_t chip_vt50[6][MAX_SCANS];
  Float_t chip_vt50_err[6][MAX_SCANS];
  Float_t chip_gain[6][MAX_SCANS];
  Float_t chip_gain_err[6][MAX_SCANS];
  Float_t chip_sigma[6][MAX_SCANS];
  Float_t chip_sigma_err[6][MAX_SCANS];
  Float_t chip_innse[6][MAX_SCANS];
  Float_t chip_innse_err[6][MAX_SCANS];

  Float_t chip_p0[6];
  Float_t chip_p1[6];
  Float_t chip_p2[6];

  TH1F* hvp[6][MAX_SCANS];
  TH1F* hsp[6][MAX_SCANS];
  TH1F* h_ptr;

  for(i=0;i<6;i++){ // loop over chip
    for(j=0;j<n_points;j++){ // loop over scan

      sprintf(h_name1,"hvp%d",((i*n_points)+j));
      sprintf(h_name2,"hsp%d",((i*n_points)+j));
      sprintf(h_title1,"vt50, chip %d, charge %2.2ffC",i,q[j]);
      sprintf(h_title2,"sigma, chip %d, charge %2.2ffC",i,q[j]);

      // delete projections if they already exist
      if(h_ptr = (TH1F*) gDirectory->GetList()->FindObject(h_name1)) delete h_ptr;
      if(h_ptr = (TH1F*) gDirectory->GetList()->FindObject(h_name2)) delete h_ptr;

  //    TH1F *h_ptr = new TH1F(h_name1,h_title1,320,0,640);
      TH1F *h_ptr = new TH1F(h_name1,h_title1,300,40,640);
      hvp[i][j] = (TH1F*) gDirectory->GetList()->FindObject(h_name1);
      
      TH1F *h_ptr = new TH1F(h_name2,h_title2,100,1,20);  // deliberately discount 0cm strips
      hsp[i][j] = (TH1F*) gDirectory->GetList()->FindObject(h_name2);

      for(k=(128*i);k<(128*(i+1));k++){
        hvp[i][j]->Fill(vt50[k][j]);
        hsp[i][j]->Fill(sigma[k][j]);
      }

      TF1 *f1 = new TF1("f1","gaus",0,640);
      hvp[i][j]->Fit("f1","0Q");
      chip_vt50[i][j]=f1->GetParameter(1);
      chip_vt50_err[i][j]=f1->GetParameter(2);
      if (chip_vt50_err[i][j]<1) chip_vt50_err[i][j]=50;

      TF1 *f2 = new TF1("f2","gaus",1,20);
      f2->SetParLimits(1,1,20);
      hsp[i][j]->Fit("f2","0Q");
      chip_sigma[i][j]=f2->GetParameter(1);
      chip_sigma_err[i][j]=f2->GetParameter(2);

      printf("chip %d charge %3.2f vt50 %3.2f sigma %3.2f noise %3.2f sigma %3.2f\n",
              i,q[j],chip_vt50[i][j],chip_vt50_err[i][j],chip_sigma[i][j],chip_sigma_err[i][j]);
    }
  }


// Before we draw the main canvas, open the postscript file
// if the relevant option has been selected 

  if(postscript){
    if(RAL) sprintf(out_fname,"M:\\sctvar\\data\\strc%d_%d_%d_%d.ps",run,sno,moi,loi);
    else    sprintf(out_fname,"D:\\sctvar\\data\\strc%d_%d_%d_%d.ps",run,sno,moi,loi);

    TPostScript *ps = new TPostScript(out_fname,112);
    ps->NewPage();
  }


  TPaveText *t1 = new TPaveText(0.0,0.0,1.0,1.0);
  t1->SetFillColor(10);
  t1->SetTextAlign(13);
  t1->SetTextSize(0.3);
  t1->SetTextColor(4);
  sprintf(s,"ATLAS SCT Module Response Curve");
  if(time) {
    sprintf(s,"%s  -  %s",s,time->GetName());
    delete time;
  }
  if(place) {
    sprintf(s,"%s  -  %s",s,place->GetName());
    delete place;
  }
  if(mname) {
    sprintf(s,"%s  -  Module %s",s,mname->GetName());
    delete mname;
  }
  switch(chipset){
    case 0:
      sprintf(s,"%s  -  ABCD2T",s);
      break;
    case 1:
      sprintf(s,"%s  -  ABCD2NT",s);
      break;
    case 2:
      sprintf(s,"%s  -  ABC/CAFE",s);
      break;
  }
  switch(fit_type){
    case 4:
      sprintf(s,"%s  -  Linear fit",s);
      break;
    case 3:
      sprintf(s,"%s  -  Exponential fit",s);
      break;
    case 2:
      sprintf(s,"%s  -  Grillo fit",s);
      break;
    default:
      sprintf(s,"%s  -  Polynomial fit",s);
      break;
  }
  t1->AddText(s);
  sprintf(s,"Run %d   Scans %d - %d   Charges %2.2f - %2.2f fC   Module %d   Stream %d", 
          run, sno, sno+scan[n_points-1], q[0], q[n_points-1], moi, loi);
  t1->AddText(s);


// Open and draw the first canvas

  TCanvas* c1=new TCanvas ("c1","stan_fitrc",1100,750);
  pad_t1   = new TPad("pad_t",   "Text",            0.01,0.91,0.99,0.99,33);
  pad_rc   = new TPad("pad_rc",  "Response Curves", 0.01,0.45,0.50,0.90,33);
  pad_gain = new TPad("pad_gain","Gain",            0.51,0.45,0.99,0.90,33);
  pad_onse = new TPad("pad_onse","Output Noise",    0.01,0.01,0.50,0.45,33);
  pad_inse = new TPad("pad_inse","Input Noise",     0.51,0.01,0.99,0.45,33);

  pad_t1->Draw();
  pad_rc->Draw();
  pad_rc->Divide(3,2);
  pad_gain->Draw();
  pad_gain->Divide(3,2);
  pad_onse->Draw();
  pad_onse->Divide(3,2);
  pad_inse->Draw();
  pad_inse->Divide(3,2);

  c1->Update();

  TH2F* hrc[6];
  TH2F* hgn[6];
  TH2F* hon[6];
  TH2F* hin[6];
  TH2F* h_ptr2f;

  for(i=0;i<6;i++){ // loop over chip

    sprintf(h_name1,"hrc%d",(i));
    sprintf(h_name2,"hon%d",(i));
    sprintf(h_name3,"hgain%d",(i));
    sprintf(h_name4,"hon%d",(i));
    sprintf(h_title1,"Response Curve chip %d",i);
    sprintf(h_title2,"Gain chip %d",i);
    sprintf(h_title3,"Output Noise chip %d",i);
    sprintf(h_title4,"Input Noise chip %d",i);

    // delete projections if they already exist
    if(h_ptr2f = (TH2F*) gDirectory->GetList()->FindObject(h_name1)) delete h_ptr2f;
    if(h_ptr2f = (TH2F*) gDirectory->GetList()->FindObject(h_name2)) delete h_ptr2f;
    if(h_ptr2f = (TH2F*) gDirectory->GetList()->FindObject(h_name3)) delete h_ptr2f;
    if(h_ptr2f = (TH2F*) gDirectory->GetList()->FindObject(h_name4)) delete h_ptr2f;

    TH2F *h_ptr2f = new TH2F(h_name1,h_title1,2,0,8,2,0,640);
    hrc[i] = (TH2F*) gDirectory->GetList()->FindObject(h_name1);
    if (chipset==0) TH2F *h_ptr2f = new TH2F(h_name2,h_title2,2,0,8,2,0,100);
    else            TH2F *h_ptr2f = new TH2F(h_name2,h_title2,2,0,8,2,0,200);
    hgn[i] = (TH2F*) gDirectory->GetList()->FindObject(h_name2);
    TH2F *h_ptr2f = new TH2F(h_name3,h_title3,2,0,8,2,0,20);
    hon[i] = (TH2F*) gDirectory->GetList()->FindObject(h_name3);
    TH2F *h_ptr2f = new TH2F(h_name4,h_title4,2,0,8,2,0,1800);
    hin[i] = (TH2F*) gDirectory->GetList()->FindObject(h_name4);

    pad_rc->cd(i+1);
    hrc[i]->SetXTitle("Q (fC)");
    hrc[i]->SetYTitle("Response (mV)");
    hrc[i]->Draw();

    grc = new TGraphErrors(n_points,q,chip_vt50[i],qe,chip_vt50_err[i]);
    grc->SetMarkerStyle(6);
    grc->SetMarkerColor(kBlack);
    grc->Draw("ep");
    switch(fit_type){
      case 4:   // linear 
        TF1 *f3 = new TF1("f3","pol1",range_lo,range_hi);
        f3->SetParLimits(2,0,0);
        break;
      case 3:   // exponential
        TF1 *f3 = new TF1("f3",st_exp,range_lo,range_hi,3);
        f3->SetParameters(70,170,-10);
        break;
      case 2:   // grillo
        TF1 *f3 = new TF1("f3",grillo,range_lo,range_hi,3);
        f3->SetParameters(57,48,339);
        break;
      default:  // polynomial
        TF1 *f3 = new TF1("f3",pol2,range_lo,range_hi);
        break;
    }
    grc->Fit("f3","QR");
    f3->SetRange(0,8);
    f3->Draw("same");

    chip_p0[i] = f3->GetParameter(0);
    chip_p1[i] = f3->GetParameter(1);
    chip_p2[i] = f3->GetParameter(2);
    printf("Chip %d fit: p0 %3.2E p1 %3.2E p2 %3.2E \n", i, chip_p0[i],chip_p1[i],chip_p2[i]);
     
    pad_onse->cd(i+1);
    hon[i]->SetXTitle("Q (fC)");
    hon[i]->SetYTitle("Noise (mV)");
    hon[i]->Draw();

    gon = new TGraphErrors(n_points,q,chip_sigma[i],qe,chip_sigma_err[i]);
    gon->SetMarkerStyle(6);
    gon->SetMarkerColor(kBlack);
    gon->Draw("ep");
    
    // Calculate gain and input noise 

    for(j=0;j<n_points;j++){
      switch(fit_type){
        case 3:   // exponential
          chip_gain[i][j]= (chip_p0[i]*exp(-q[j]/chip_p1[i]))/
            (chip_p1[i] * pow( (1.0+(exp(-q[j]/chip_p1[i]))),2));
          chip_gain_err[i][j]= 0;
          break;
        case 2:   // grillo
          Float_t d1 = 1.0 +  pow( (chip_p2[i]/(chip_p1[i]*q[j])),2);
          Float_t d2 = pow(chip_p1[i],2) * pow(q[j],3) * pow(d1,1.5);
          chip_gain[i][j]=pow(chip_p2[i],3)/d2;
          chip_gain_err[i][j]= 0;
          break;
        default:  // polynomial
          chip_gain[i][j]= chip_p1[i]+(q[j]*2*chip_p2[i]);
          chip_gain_err[i][j]= 0;
          break;
      }
      printf("  gain = %3.2f at %3.2f fC\n",chip_gain[i][j],q[j]); 
      if(chip_gain[i][j]==0) chip_gain[i][j]=-1;    // PWP bugfix 07.04.00
      chip_innse[i][j]=6250.0 * chip_sigma[i][j]/chip_gain[i][j];
      chip_innse_err[i][j]= 6250.0 * chip_sigma_err[i][j]/chip_gain[i][j];
    }
 
    pad_gain->cd(i+1);
    hgn[i]->SetXTitle("Q (fC)");
    hgn[i]->SetYTitle("Gain (mV/fC)");
    hgn[i]->Draw();

    ggn = new TGraphErrors(n_points,q,chip_gain[i],qe,chip_gain_err[i]);
    ggn->SetMarkerStyle(6);
    ggn->SetMarkerColor(kBlack);
    ggn->Draw("ep");

    pad_inse->cd(i+1);
    hin[i]->SetXTitle("Q (fC)");
    hin[i]->SetYTitle("Noise (ENC)");
    hin[i]->Draw();

    gin = new TGraphErrors(n_points,q,chip_innse[i],qe,chip_innse_err[i]);
    gin->SetMarkerStyle(6);
    gin->SetMarkerColor(kBlack);
    gin->Draw("ep");
  }

  pad_t1->cd();
  t1->Draw();
  c1->Update();

  if(postscript) {
    ps->Close();
    delete ps;  
  }

  // print fit parameters to screen

  printf("chip method function p0 p1 p2\n");
  for(i=0;i<6;i++){
    Int_t chip = i;
    if(loi) chip+=6;
    printf("%d %d %d %3.2f %3.2f %3.2f\n",
            chip,1,fit_type,chip_p0[i],chip_p1[i],chip_p2[i]);
  } 

  if(ascii){
    if(RAL) sprintf(f_name,"M:\\sctvar\\data\\strc%d_%d_%d_%d.rcdat",run,sno,moi,loi);
    else    sprintf(f_name,"D:\\sctvar\\data\\strc%d_%d_%d_%d.rcdat",run,sno,moi,loi);

    FILE* rcfile = fopen(f_name,"w");
    if(rcfile==NULL){
      printf("Could not open file for response curve functional fit ascii output\n");
    } else {
      fprintf(rcfile,"chip method function p0 p1 p2\n");
      for(i=0;i<6;i++){
        Int_t chip = i;
        if(loi) chip+=6;
        fprintf(rcfile,"%d %d %d %3.2f %3.2f %3.2f\n",
                chip,1,fit_type,chip_p0[i],chip_p1[i],chip_p2[i]);
      } 
    }
  fclose(rcfile);
  }

  return;
}

void stan_fitrc(int run, int sno, int moi, int loi, int chipset, int fit_type, double range_lo, double range_hi){

  stan_fitrc(run,sno,moi,loi, chipset, fit_type, range_lo, range_hi, 1, 1);
}

void stan_fitrc(int run, int sno, int moi, int loi, int fit_type, double range_lo, double range_hi){

  stan_fitrc(run,sno,moi,loi, 0, fit_type, range_lo, range_hi, 1, 1);
}

void stan_fitrc(int run, int sno, int moi, int loi, int chipset, int fit_type){

  stan_fitrc(run,sno,moi,loi, chipset, fit_type, 0.0, 8.0, 1,1);
}

void stan_fitrc(int run, int sno, int moi, int loi, int chipset){

  stan_fitrc(run,sno,moi,loi, chipset, 1, 0.0, 8.0, 1,1);
}

void stan_fitrc(int run, int sno, int moi, int loi){

  stan_fitrc(run,sno,moi,loi, 0, 1, 0.0, 8.0, 1,1);
}

void stan_fitrc(){

  printf("Possible Uses of stan_fitrc:\n\n");
  printf(".x stan_fitrc.cpp(run,start_scan,module,link)\n");
  printf(".x stan_fitrc.cpp(run,start_scan,module,link,chipset)\n");
  printf(".x stan_fitrc.cpp(run,start_scan,module,link,chipset,fit_type)\n");
  printf(".x stan_fitrc.cpp(run,start_scan,module,link,fit_type,range_lo,range_hi)\n");
  printf(".x stan_fitrc.cpp(run,start_scan,module,link,chipset,fit_type,range_lo,range_hi)\n");
  printf(".x stan_fitrc.cpp(run,start_scan,module,link,chipset,fit_type,range_lo,range_hi,ascii,postscript)\n\n");
  printf("Default chipset is 0 - ABCDT, use 1 - ABCDNT1  2 - ABC/CAFE\n");
  printf("Default fit type is 1 - polynomial, use 2 - Grillo 3 - Exponential 4 - Linear\n\n");

}

