Int_t get_val(char* line, Int_t index, double* val){

  // convert next element of data row "line" to type double
  // (CINT sscanf is limited to <43 elements, hence this approach)

  char substring[20];

  sscanf(line+=index,"%s ",substring);
  sscanf(substring,"%lf",val);
  return 1 + strlen(substring);
}

void stan_fittw(int run, int sno, int moi, int loi, int chipset, int fit_type, int ascii, int postscript){

  // Routine to evaluate the timewalk of a module
  // Supports three fit options:
  //   1 - Second Order Polynomial
  //   2 - "Grillo" function
  //   3 - Exponential fit

  // Version 0.1 P.W.Phillips, 14.12.99
  // Version 0.11 BJG, 30.04.02   Fix some memory leaks

  char  f_name[64], scan_name[32];
  char  h_name1[32], h_name2[32], h_name3[32], h_name4[32];
  char  h_title1[32],h_title2[32],h_title3[32],h_title4[32];
  char  m_title[32],s_title[32],out_fname[128], s[256], unit[12], line[1024];
  int   i,j,k,n,x,y, index;
  double var;

  // Locality options now moved to stan_util.cpp

  // Options which change macro output
  int draw_fit_projections = 0;

  printf("stan_fittw called for module %d link %d\n",moi,loi);

  // Load other files needed by this one
  gROOT->LoadMacro("stan_util.cpp");

  // Global style variations
  stan_styles();
  gStyle.SetOptStat(0);
  gStyle.SetOptFit(10);


  // Read file "stan_tw.dat"
  // which gives scan numbers and charges
  // of up to 16 runs 


  Float_t scan[16];
  Float_t q[16];
  Float_t qe[16];

  if (RAL) sprintf(f_name,"M:\\sctdaq\\analysis\\stan_tw.dat");
  else sprintf(f_name,"C:\\sctdaq\\analysis\\stan_tw.dat");
  FILE* fitfile = fopen(f_name,"r");
  if(fitfile==NULL){
    printf("Could not open scan/charge info file %s\n",f_name);
    return;
  } else {
    printf("File %s opened for reading\n",f_name);

    int n_points=0;

    fgets(line,1024,fitfile);
    while((fgets(line,1024,fitfile) != NULL)&&(n_points<16)){
      index = 0;
      index+=get_val(line,index,&var);
      scan[n_points]=(float)var;
      index+=get_val(line,index,&var);
      q[n_points]=(float)var;
      qe[n_points]=0.1;      // ?????

      printf("Point %d Scan %d Charge %3.2f\n",n_points,scan[n_points],q[n_points]);
      n_points++;
    }
    fclose(fitfile);
  }

  // new section 24.11.99
  // load the first of the root files to obtain timestamp info

  if(RAL) sprintf(f_name,"M:\\sctvar\\data\\strun%d_%d.root",run,sno);
  else    sprintf(f_name,"D:\\sctvar\\data\\strun%d_%d.root",run,sno);
  
  printf("Using Time/Place/Module name information from %s\n",f_name); 
  TFile *strun = new TFile(f_name);

  sprintf(h_name1,"ModuleName;%d",moi+1);
  TObjString *time  = (TObjString*)strun->Get("Time");
  TObjString *place = (TObjString*)strun->Get("Place");
  TObjString *mname = (TObjString*)strun->Get(h_name1);

  // load data

  Float_t vt50[768][16];
  Float_t vt50_err[768][16];
  Float_t sigma[768][16];
  Float_t sigma_err[768][16];

  for(j=0;j<n_points;j++){
    if (RAL) sprintf(f_name,"M:\\sctvar\\data\\strun%d_%d_%d_%d.fit",run,(sno+j),moi,loi);
    else sprintf(f_name,"D:\\sctvar\\data\\strun%d_%d_%d_%d.fit",run,(sno+j),moi,loi);
    FILE* fitfile = fopen(f_name,"r");

    if(fitfile==NULL){
      printf("Could not open fit file %s\n",f_name);
      return;
    } else {
      printf("File %s opened for reading\n",f_name);

      i=0;

      fgets(line,1024,fitfile);
      while(fgets(line,1024,fitfile) != NULL){
        index = 10;
        index+=get_val(line,index,&var);
        vt50[i][j]=(float)var;
        index+=get_val(line,index,&var);
        vt50_err[i][j]=(float)var;
        index+=get_val(line,index,&var);
        sigma[i][j]=(float)var;
        index+=get_val(line,index,&var);
        sigma_err[i][j]=(float)var;
        i++;
      }
    fclose(fitfile);
    }
  }

  // Chip projections - something a bit different here.
  // Create and fill 6 * npoints histograms for the vt50 values
  // fit mean and sigma - use these values to fill chip projection histograms.

  Float_t chip_vt50[6][16];
  Float_t chip_vt50_err[6][16];
  Float_t chip_sigma[6][16];
  Float_t chip_sigma_err[6][16];
  Float_t chip_edge1[6];
  Float_t chip_edge2[6];
  Float_t chip_tw[6];
  Float_t chip_cal[6];

  Float_t chip_p0[6];
  Float_t chip_p1[6];
  Float_t chip_p2[6];

  TH1F* hvp[6][16];
  TH1F* hsp[6][16];
  TH1F* h_ptr;

  for(i=0;i<6;i++){ // loop over chip
    for(j=0;j<n_points;j++){ // loop over scan

      sprintf(h_name1,"hvp%d",((i*n_points)+j));
      sprintf(h_name2,"hsp%d",((i*n_points)+j));
      sprintf(h_title1,"mean, chip %d, charge %2.2ffC",i,q[j]);
      sprintf(h_title2,"sigma, chip %d, charge %2.2ffC",i,q[j]);

      // delete projections if they already exist
      if(h_ptr = (TH1F*) gDirectory->GetList()->FindObject(h_name1)) delete h_ptr;
      if(h_ptr = (TH1F*) gDirectory->GetList()->FindObject(h_name2)) delete h_ptr;

      TH1F *h_ptr = new TH1F(h_name1,h_title1,128,1,64);
      hvp[i][j] = (TH1F*) gDirectory->GetList()->FindObject(h_name1);
      
      TH1F *h_ptr = new TH1F(h_name2,h_title2,128,1,10);
      hsp[i][j] = (TH1F*) gDirectory->GetList()->FindObject(h_name2);

      for(k=(128*i);k<(128*(i+1));k++){
        hvp[i][j]->Fill(vt50[k][j]);
        hsp[i][j]->Fill(sigma[k][j]);
      }

      TF1 *f1 = new TF1("f1","gaus",1,64);
      hvp[i][j]->Fit("f1","0Q");
      chip_vt50[i][j]=f1->GetParameter(1);
      chip_vt50_err[i][j]=f1->GetParameter(2);
      if (chip_vt50_err[i][j]<0.1) chip_vt50_err[i][j]=32;

      TF1 *f2 = new TF1("f2","gaus",1,10);
      f2->SetParLimits(1,0,10);
      hsp[i][j]->Fit("f2","0Q");
      chip_sigma[i][j]=f2->GetParameter(1);
      chip_sigma_err[i][j]=f2->GetParameter(2);

      printf("chip %d charge %3.2f vt50 %3.2f sigma %3.2f noise %3.2f sigma %3.2f\n",
              i,q[j],chip_vt50[i][j],chip_vt50_err[i][j],chip_sigma[i][j],chip_sigma_err[i][j]);
    }
  }


// Before we draw the main canvas, open the postscript file
// if the relevant option has been selected 

  if(postscript){
    if(RAL) sprintf(out_fname,"M:\\sctvar\\data\\sttw%d_%d_%d_%d.ps",run,sno,moi,loi);
    else    sprintf(out_fname,"D:\\sctvar\\data\\sttw%d_%d_%d_%d.ps",run,sno,moi,loi);

    TPostScript *ps = new TPostScript(out_fname,112);
    ps->NewPage();
  }


  TPaveText *t1 = new TPaveText(0.0,0.0,1.0,1.0);
  t1->SetFillColor(10);
  t1->SetTextAlign(13);
  t1->SetTextSize(0.3);
  t1->SetTextColor(4);
  sprintf(s,"ATLAS SCT Module Timewalk");
  if(time) { sprintf(s,"%s  -  %s",s,time->GetName()); delete time; }
  if(place) { sprintf(s,"%s  -  %s",s,place->GetName()); delete place; }
  if(mname) { sprintf(s,"%s  -  Module %s",s,mname->GetName()); delete mname; } 
  switch(chipset){
    case 0:
      sprintf(s,"%s  -  ABCD2T",s);
      break;
    case 1:
      sprintf(s,"%s  -  ABCD2NT",s);
      break;
    case 2:
      sprintf(s,"%s  -  ABC/CAFE",s);
      break;
  }
  switch(fit_type){
    case 3:
      sprintf(s,"%s  -  Exponential fit",s);
      break;
    case 2:
      sprintf(s,"%s  -  Grillo fit",s);
      break;
    default:
      sprintf(s,"%s  -  Polynomial fit",s);
      break;
  }
  t1->AddText(s);
  sprintf(s,"Run %d   Scans %d - %d   Charges %2.2f - %2.2f fC   Module %d   Stream %d", 
          run, sno, sno+scan[n_points-1], q[0], q[n_points-1], moi, loi);
  t1->AddText(s);


// Open and draw the first canvas

  TCanvas* c1=new TCanvas ("c1","stan_fittw",1100,750);
  pad_t1   = new TPad("pad_t1",  "Text",     0.01,0.91,0.99,0.99,33);
  pad_tw   = new TPad("pad_tw",  "Timewalk", 0.01,0.01,0.99,0.90,33);
//  pad_tw   = new TPad("pad_tw",  "Timewalk", 0.01,0.01,0.50,0.90,33);
//  pad_onse = new TPad("pad_onse","Noise",    0.50,0.01,0.99,0.90,33);

  pad_t1->Draw();
  pad_tw->Draw();
  pad_tw->Divide(2,3);
//  pad_onse->Draw();
//  pad_onse->Divide(2,3);

  c1->Update();

  TH2F* htw[6];
  TH2F* hon[6];
  TH2F* h_ptr2f;

  for(i=0;i<6;i++){ // loop over chip

    sprintf(h_name1,"htw%d",(i));
    sprintf(h_name2,"hon%d",(i));
    sprintf(h_title1,"Timewalk chip %d",i);
    sprintf(h_title2,"Sigma chip %d",i);

    // delete projections if they already exist
    if(h_ptr2f = (TH2F*) gDirectory->GetList()->FindObject(h_name1)) delete h_ptr2f;
    if(h_ptr2f = (TH2F*) gDirectory->GetList()->FindObject(h_name2)) delete h_ptr2f;

    TH2F *h_ptr2f = new TH2F(h_name1,h_title1,2,0,12,2,0,64);
    htw[i] = (TH2F*) gDirectory->GetList()->FindObject(h_name1);
    TH2F *h_ptr2f = new TH2F(h_name2,h_title3,2,0,12,2,0,20);
    hon[i] = (TH2F*) gDirectory->GetList()->FindObject(h_name2);

    pad_tw->cd(i+1);
    htw[i]->SetXTitle("Q (fC)");
    htw[i]->SetYTitle("Strobe Delay (units)");
    htw[i]->Draw();

    // last data point is for calibration, hence not included in timewalk graph
    gtw = new TGraphErrors(n_points-1,q,chip_vt50[i],qe,chip_vt50_err[i]);
    gtw->SetMarkerStyle(6); 
    gtw->SetLineColor(kRed);
    gtw->SetMarkerColor(kBlack);
    gtw->Draw("ep");

    switch(fit_type){
      case 3:   // exponential
        TF1 *f3 = new TF1("f3",st_exp,0.1,12,3);
         f3->SetParameters(70,1,-10);
        break;
      case 2:   // grillo
        TF1 *f3 = new TF1("f3",grillo,0.1,12,3);
         f3->SetParameters(-10,45,60);
        break;
      default:  // polynomial
        TF1 *f3 = new TF1("f3",pol2,0,12);
         f3->SetParameters(35,5,-1);
        break;
    }
    gtw->Fit("f3","QR");
    chip_p0[i] = f3->GetParameter(0);
    chip_p1[i] = f3->GetParameter(1);
    chip_p2[i] = f3->GetParameter(2);

    chip_edge1[i] = f3->Eval(1.25);
    chip_edge2[i] = f3->Eval(10);
    chip_tw[i]  = chip_edge2[i] - chip_edge1[i];
    chip_cal[i] = chip_vt50[i][n_points-2] - chip_vt50[i][n_points-1];
    if (chip_cal[i]==0.0) chip_cal[i]=1;

    printf("Chip %d fit: p0 %3.2E p1 %3.2E p2 %3.2E \n", i, chip_p0[i],chip_p1[i],chip_p2[i]);
    printf("        edge1 %3.2E edge2 %3.2E tw %3.2E \n", chip_edge1[i],chip_edge2[i],chip_tw[i]);

    TText * t = new TText();
    sprintf(h_title1,"Calibration: 25 ns = %2.1f units",chip_cal[i]);
    t->DrawText(4,30,h_title1);
    sprintf(h_title1,"Timewalk (1.25 to 10fC):");
    t->DrawText(4,20,h_title1);
    sprintf(h_title1,"%2.1f units => %2.1f nS",chip_tw[i],(chip_tw[i]*25.0)/chip_cal[i]);
    t->DrawText(4,10,h_title1);
     
//    pad_onse->cd(i+1);
//    hon[i]->SetXTitle("Q (fC)");//
//    hon[i]->SetYTitle("Sigma (Units)");
//    hon[i]->Draw();

    // last data point is for calibration, hence not included in sigma graph
//    gon = new TGraphErrors(n_points-1,q,chip_sigma[i],qe,chip_sigma_err[i]);
//    gon->SetMarkerStyle(6);
//    gon->SetMarkerColor(kBlack);
//    gon->Draw("ep");
    
  }

  pad_t1->cd();
  t1->Draw();
  c1->Update();

  if(postscript) {
    ps->Close();
    delete ps;
  }

  // print fit parameters to screen

  printf("chip method function p0 p1 p2\n");
  for(i=0;i<6;i++){
    Int_t chip = i;
    if(loi) chip+=6;
    printf("%d %d %d %3.2f %3.2f %3.2f\n",
            chip,1,fit_type,chip_p0[i],chip_p1[i],chip_p2[i]);
  } 

  if(ascii){
    if(RAL) sprintf(f_name,"M:\\sctvar\\data\\sttw%d_%d_%d_%d.twdat",run,sno,moi,loi);
    else    sprintf(f_name,"D:\\sctvar\\data\\sttw%d_%d_%d_%d.twdat",run,sno,moi,loi);

    FILE* twfile = fopen(f_name,"w");
    if(twfile==NULL){
      printf("Could not open file for response curve functional fit ascii output\n");
    } else {
      fprintf(twfile,"chip method function p0 p1 p2\n");
      for(i=0;i<6;i++){
        Int_t chip = i;
        if(loi) chip+=6;
        fprintf(twfile,"%d %d %d %3.2f %3.2f %3.2f\n",
                chip,1,fit_type,chip_p0[i],chip_p1[i],chip_p2[i]);
      } 
    }
  fclose(twfile);
  }

  return;
}

void stan_fittw(int run, int sno, int moi, int loi, int chipset, int fit_type, int ascii){

  stan_fittw(run,sno,moi,loi, chipset, fit_type, ascii, 1);
}

void stan_fittw(int run, int sno, int moi, int loi, int chipset, int fit_type){

  stan_fittw(run,sno,moi,loi, chipset, fit_type, 1, 1);
}

void stan_fittw(int run, int sno, int moi, int loi, int chipset){

  stan_fittw(run,sno,moi,loi, chipset, 3, 1, 1);
}

void stan_fittw(int run, int sno, int moi, int loi){

  stan_fittw(run,sno,moi,loi, 0, 3, 1, 1);
}

void stan_fittw(){

  printf("Possible Uses of stan_fittw:\n\n");
  printf(".x stan_fittw.cpp(run,start_scan,module,link)\n");
  printf(".x stan_fittw.cpp(run,start_scan,module,link,chipset)\n");
  printf(".x stan_fittw.cpp(run,start_scan,module,link,chipset,fit_type)\n");
  printf(".x stan_fittw.cpp(run,start_scan,module,link,chipset,fit_type,ascii)\n");
  printf(".x stan_fittw.cpp(run,start_scan,module,link,chipset,fit_type,ascii,postscript)\n\n");
  printf("Default chipset is 0 - ABCDT, use 1 - ABCDNT1  2 - ABC/CAFE\n");
  printf("Default fit type is 3 - Exponential, use 2 - Grillo  1 - polynomial\n\n");

}

