Int_t get_val(char* line, Int_t index, double* val){

  // convert next element of data row "line" to type double
  // (CINT sscanf is limited to <43 elements, hence this approach)

  char substring[20];

  sscanf(line+=index,"%s ",substring);
  sscanf(substring,"%lf",val);
  return 1 + strlen(substring);
}

void stan_fittwbychan(int run, int sno, int moi, int loi, int chipset, int fit_type, int ascii, int postscript){

  // Routine to evaluate the timewalk of a module
  // Supports three fit options:
  //   1 - Second Order Polynomial
  //   2 - "Grillo" function
  //   3 - Exponential fit

  // Version 0.1 P.W.Phillips, 14.12.99
  // Version 0.11 BJG, 30.04.02   Fix some memory leaks

  char  f_name[64], scan_name[32];
  char  h_name1[32], h_name2[32], h_name3[32], h_name4[32];
  char  h_title1[32],h_title2[32],h_title3[32],h_title4[32];
  char  m_title[32],s_title[32],out_fname[128], s[256], unit[12], line[1024];
  int   i,j,k,n,x,y, index;
  double var;

  // Locality options now moved to stan_util.cpp

  // Options which change macro output
  int draw_fit_projections = 0;

  printf("stan_fittw called for module %d link %d\n",moi,loi);

  // Load other files needed by this one
  gROOT->LoadMacro("stan_util.cpp");

  // Global style variations
  stan_styles();
  gStyle.SetOptStat(0);
  gStyle.SetOptFit(10);


  // Read file "stan_tw.dat"
  // which gives scan numbers and charges
  // of up to 16 runs 


  Float_t scan[16];
  Float_t q[16];
  Float_t qe[16];

  if (RAL) sprintf(f_name,"M:\\sctdaq\\analysis\\stan_tw.dat");
  else sprintf(f_name,"C:\\sctdaq\\analysis\\stan_tw.dat");
  FILE* fitfile = fopen(f_name,"r");
  if(fitfile==NULL){
    printf("Could not open scan/charge info file %s\n",f_name);
    return;
  } else {
    printf("File %s opened for reading\n",f_name);

    int n_points=0;

    fgets(line,1024,fitfile);
    while((fgets(line,1024,fitfile) != NULL)&&(n_points<16)){
      index = 0;
      index+=get_val(line,index,&var);
      scan[n_points]=(float)var;
      index+=get_val(line,index,&var);
      q[n_points]=(float)var;
      qe[n_points]=0.1;      // ?????

      printf("Point %d Scan %d Charge %3.2f\n",n_points,scan[n_points],q[n_points]);
      n_points++;
    }
    fclose(fitfile);
  }

  // new section 24.11.99
  // load the first of the root files to obtain timestamp info

  if(RAL) sprintf(f_name,"M:\\sctvar\\data\\strun%d_%d.root",run,sno);
  else    sprintf(f_name,"D:\\sctvar\\data\\strun%d_%d.root",run,sno);
  
  printf("Using Time/Place/Module name information from %s\n",f_name); 
  TFile *strun = new TFile(f_name);

  sprintf(h_name1,"ModuleName;%d",moi+1);
  TObjString *time  = (TObjString*)strun->Get("Time");
  TObjString *place = (TObjString*)strun->Get("Place");
  TObjString *mname = (TObjString*)strun->Get(h_name1);

  // load data

  Float_t vt50[768][16];
  Float_t vt50_err[768][16];
  Float_t sigma[768][16];
  Float_t sigma_err[768][16];

  for(j=0;j<n_points;j++){
    if (RAL) sprintf(f_name,"M:\\sctvar\\data\\strun%d_%d_%d_%d.fit",run,(sno+j),moi,loi);
    else sprintf(f_name,"D:\\sctvar\\data\\strun%d_%d_%d_%d.fit",run,(sno+j),moi,loi);
    FILE* fitfile = fopen(f_name,"r");

    if(fitfile==NULL){
      printf("Could not open fit file %s\n",f_name);
      return;
    } else {
      printf("File %s opened for reading\n",f_name);

      i=0;

      fgets(line,1024,fitfile);
      while(fgets(line,1024,fitfile) != NULL){
        index = 10;
        index+=get_val(line,index,&var);
        vt50[i][j]=(float)var;
        index+=get_val(line,index,&var);
        vt50_err[i][j]=(float)var;
        index+=get_val(line,index,&var);
        sigma[i][j]=(float)var;
        index+=get_val(line,index,&var);
        sigma_err[i][j]=(float)var;
        i++;
      }
    fclose(fitfile);
    }
  }

  // Chip projections - something a bit different here.
  // Create and fill 6 * npoints histograms for the vt50 values
  // fit mean and sigma - use these values to fill chip projection histograms.

  Float_t chip_vt50[6][16];
  Float_t chip_vt50_err[6][16];
  Float_t chip_sigma[6][16];
  Float_t chip_sigma_err[6][16];
  Float_t chip_edge1[6];
  Float_t chip_edge2[6];
  Float_t chip_tw[6];
  Float_t chip_cal[6];

  Float_t chip_p0[6];
  Float_t chip_p1[6];
  Float_t chip_p2[6];

  TH1F* hvp[6][16];
  TH1F* hsp[6][16];
  TH1F* h_ptr;

// Before we draw the main canvas, open the postscript file
// if the relevant option has been selected 

  if(postscript){
    if(RAL) sprintf(out_fname,"M:\\sctvar\\data\\sttw%d_%d_%d_%d.ps",run,sno,moi,loi);
    else    sprintf(out_fname,"D:\\sctvar\\data\\sttw%d_%d_%d_%d.ps",run,sno,moi,loi);

    TPostScript *ps = new TPostScript(out_fname,112);
    ps->NewPage();
  }


  TPaveText *t1 = new TPaveText(0.0,0.0,1.0,1.0);
  t1->SetFillColor(10);
  t1->SetTextAlign(13);
  t1->SetTextSize(0.3);
  t1->SetTextColor(4);
  sprintf(s,"ATLAS SCT Module Timewalk");
  if(time) { sprintf(s,"%s  -  %s",s,time->GetName()); delete time; }
  if(place) { sprintf(s,"%s  -  %s",s,place->GetName()); delete place; }
  if(mname) { sprintf(s,"%s  -  Module %s",s,mname->GetName()); delete mname; }
  switch(chipset){
    case 0:
      sprintf(s,"%s  -  ABCD2T",s);
      break;
    case 1:
      sprintf(s,"%s  -  ABCD2NT",s);
      break;
    case 2:
      sprintf(s,"%s  -  ABC/CAFE",s);
      break;
  }
  switch(fit_type){
    case 3:
      sprintf(s,"%s  -  Exponential fit",s);
      break;
    case 2:
      sprintf(s,"%s  -  Grillo fit",s);
      break;
    default:
      sprintf(s,"%s  -  Polynomial fit",s);
      break;
  }
  t1->AddText(s);
  sprintf(s,"Run %d   Scans %d - %d   Charges %2.2f - %2.2f fC   Module %d   Stream %d", 
          run, sno, sno+scan[n_points-1], q[0], q[n_points-1], moi, loi);
  t1->AddText(s);


// Open and draw the first canvas

  TCanvas* c1=new TCanvas ("c1","stan_fittw",1100,750);
  pad_t1   = new TPad("pad_t1",  "Text",       0.01,0.91,0.99,0.99,33);
  pad_tw   = new TPad("pad_tw",  "Timewalk",   0.01,0.01,0.80,0.90,33);
  pad_twp  = new TPad("pad_twp", "Projections",0.80,0.01,0.99,0.90,33);

  pad_t1->Draw();
  pad_tw->Draw();
  pad_tw->Divide(1,3);
  pad_twp->Draw();
  pad_twp->Divide(1,3);

  c1->Update();

  Float_t chan_cal[768];
  Float_t chan_re[768];
  Float_t chan_tw[768];

  TGraphErrors* gtw[768];

  TH1* hp_cal[7];
  TH1* hp_re[7];
  TH1* hp_tw[7];

  TH1F *h_cal = new TH1F("h_cal","Calibration Factor (ns/unit)",   768,0.5,768.5);
  TH1F *h_re  = new TH1F("h_re", "Rising Edge of pulse at 10.0fC", 768,0.5,768.5);
  TH1F *h_tw  = new TH1F("h_tw", "Timewalk 1.25 to 10fC (ns)",     768,0.5,768.5);

  h_cal->SetMinimum(1.0);
  h_cal->SetXTitle("Channel");
  h_cal->SetYTitle("ns per bin");
  h_re->SetMinimum(40);
  h_re->SetXTitle("Channel");
  h_re->SetYTitle("Strobe Delay units");
  h_tw->SetMinimum(10);
  h_tw->SetMaximum(30);
  h_tw->SetXTitle("Channel");
  h_tw->SetYTitle("ns");

  for(i=0;i<7;i++){

    sprintf(h_name1,"hp_cal%d",i);
    sprintf(h_name2,"hp_re%d",i);
    sprintf(h_name3,"hp_tw%d",i);

    sprintf(h_title1,"Calibration Factor");
    if(i) sprintf(h_title1,"%s, chip %d",h_title1,i-1);
    sprintf(h_title2,"Rising Edge at 10fC");
    if(i) sprintf(h_title2,"%s, chip %d",h_title2,i-1);
    sprintf(h_title3,"Timewalk 1.25 - 10fC");
    if(i) sprintf(h_title3,"%s, chip %d",h_title3,i-1);

    // Delete projection histograms if they already exist
    if(h_ptr = (TH1F*) gDirectory->GetList()->FindObject(h_name1)) delete h_ptr;
    if(h_ptr = (TH1F*) gDirectory->GetList()->FindObject(h_name2)) delete h_ptr;
    if(h_ptr = (TH1F*) gDirectory->GetList()->FindObject(h_name3)) delete h_ptr;

    // Create the histograms 
    hp_cal[i]= new TH1F(h_name1,h_title1,200,1.0,1.2); 
    hp_re[i] = new TH1F(h_name2,h_title2,64,32,63); 
    hp_tw[i] = new TH1F(h_name3,h_title3,80,10,30); 
    hp_cal[i]->SetXTitle("ns per bin");
    hp_re[i]->SetXTitle("Strobe Delay (units)");
    hp_tw[i]->SetXTitle("Timewalk (ns)");
  }

  for(i=0;i<768;i++){ // loop over channels

    gtw[i] = new TGraphErrors(n_points-1,q,vt50[i],qe,vt50_err[i]);

    switch(fit_type){
      case 3:   // exponential
        TF1 *f3 = new TF1("f3",st_exp,0.1,12,3);
         f3->SetParameters(70,1,-10);
        break;
      case 2:   // grillo
        TF1 *f3 = new TF1("f3",grillo,0.1,12,3);
         f3->SetParameters(-10,45,60);
        break;
      default:  // polynomial
        TF1 *f3 = new TF1("f3",pol2,0,12);
         f3->SetParameters(35,5,-1);
        break;
    }

    gtw[i]->Fit("f3","QR");

    chan_cal[i] = 25.0 / (vt50[i][n_points-2] - vt50[i][n_points-1]);
    if (chan_cal[i]==0.0) chan_cal[i]=1;

    chan_re[i] = f3->Eval(10);
    chan_tw[i] = chan_cal[i]* ( chan_re[i] - f3->Eval(1.25));

    h_cal->Fill(i,chan_cal[i]); 
    h_re->Fill(i,chan_re[i]); 
    h_tw->Fill(i,chan_tw[i]); 

    hp_cal[0]->Fill(chan_cal[i]); 
    hp_re[0]->Fill(chan_re[i]); 
    hp_tw[0]->Fill(chan_tw[i]); 

    hp_cal[(i/128)+1]->Fill(chan_cal[i]); 
    hp_re[(i/128)+1]->Fill(chan_re[i]); 
    hp_tw[(i/128)+1]->Fill(chan_tw[i]); 

  }


  pad_tw->cd(1);
  h_re->Draw();
  pad_tw->cd(2);
  h_tw->Draw();
  pad_tw->cd(3);
  h_cal->Draw();
  c1->Update();

  pad_twp->cd(1);
  hp_re[0]->Fit("gaus");
  c1->Update();
  pad_twp->cd(2);
  hp_tw[0]->Fit("gaus");
  c1->Update();
  pad_twp->cd(3);
  hp_cal[0]->Fit("gaus");
  c1->Update();


  pad_t1->cd();
  t1->Draw();
  c1->Update();

// Open and draw the second canvas

  if(postscript) ps->NewPage();

  TCanvas* c2=new TCanvas ("c2","stan_fittw",1100,750);
  pad_t2   = new TPad("pad_t2",  "Text",       0.01,0.91,0.99,0.99,33);
  pad_twp2  = new TPad("pad_twp2", "Projections",0.01,0.01,0.99,0.90,33);

  pad_t2->Draw();
  pad_twp2->Draw();
  pad_twp2->Divide(6,3);
  c2->Update();

  for(i=1;i<7;i++){
    pad_twp2->cd(i);
    hp_re[i]->Fit("gaus");
    pad_twp2->cd(i+6);
    hp_tw[i]->Fit("gaus");
    pad_twp2->cd(i+12);
    hp_cal[i]->Fit("gaus");
  }

  pad_t2->cd();
  t1->Draw();
  c2->Update();

// Open and draw the third canvas

  if(postscript) ps->NewPage();

  TCanvas* c3=new TCanvas ("c3","stan_fittw",1100,750);
  pad_t3   = new TPad("pad_t2",  "Text",     0.01,0.91,0.99,0.99,33);
  pad_tw2  = new TPad("pad_tw2", "Timewalk", 0.01,0.01,0.99,0.90,33);

  pad_t3->Draw();
  pad_tw2->Draw();
  pad_tw2->Divide(6,3);
  c3->Update();

  TH2F* htw[18];

  for(i=0;i<18;i++){

    sprintf(h_name1,"htw%d",i);
    sprintf(h_title1,"Timewalk channel %d",i*42);

    htw[i] = new TH2F(h_name1,h_title1,2,0,12,2,0,64);

    pad_tw2->cd(i+1);
    htw[i]->SetXTitle("Q (fC)");
    htw[i]->SetYTitle("Strobe Delay (units)");
    htw[i]->Draw();
    gtw[i*42]->Draw();
  }

  pad_t3->cd();
  t1->Draw();
  c3->Update();

  if(postscript) {
    ps->Close();
    delete ps;
  }

  return;
}

void stan_fittwbychan(int run, int sno, int moi, int loi, int chipset, int fit_type, int ascii){

  stan_fittwbychan(run,sno,moi,loi, chipset, fit_type, ascii, 1);
}

void stan_fittwbychan(int run, int sno, int moi, int loi, int chipset, int fit_type){

  stan_fittwbychan(run,sno,moi,loi, chipset, fit_type, 1, 1);
}

void stan_fittwbychan(int run, int sno, int moi, int loi, int chipset){

  stan_fittwbychan(run,sno,moi,loi, chipset, 2, 1, 1);
}

void stan_fittwbychan(int run, int sno, int moi, int loi){

  stan_fittwbychan(run,sno,moi,loi, 0, 2, 1, 1);
}

void stan_fittwbychan(){

  printf("Possible Uses of stan_fittwbychan:\n\n");
  printf(".x stan_fittwbychan.cpp(run,start_scan,module,link)\n");
  printf(".x stan_fittwbychan.cpp(run,start_scan,module,link,chipset)\n");
  printf(".x stan_fittwbychan.cpp(run,start_scan,module,link,chipset,fit_type)\n");
  printf(".x stan_fittwbychan.cpp(run,start_scan,module,link,chipset,fit_type,ascii)\n");
  printf(".x stan_fittwbychan.cpp(run,start_scan,module,link,chipset,fit_type,ascii,postscript)\n\n");
  printf("Default chipset is 0 - ABCDT, use 1 - ABCDNT1  2 - ABC/CAFE\n");
  printf("Default fit type is 3 - Exponential, use 2 - Grillo  1 - polynomial\n\n");

}

