//
// stan_projection.cpp
// PWP 08.09.99
//

#define MAX_SCAN_DIMS 10;


Int_t st_findfirstevent(TNtuple* STNT, const Text_t *selection, Int_t firstentry)
{
  // Loop on NTuple from "firstentry" to end;
  // return first entry which satisfies the selection criteria
  // Based on TTree:Scan

  TTreeFormula *select;
  Int_t entry,entryNumber;
  Int_t nentries = (Int_t) STNT->GetEntries();

  // Compile selection expression
  select = 0;
  if (strlen(selection)) {
    select = new TTreeFormula("Selection",selection,STNT);
    if (!select){
      return(0);
    }
    if (!select->GetNdim()) {
      delete select; 
      return(0);
    }
  }

  // loop on all selected entries

  Int_t found = 0;
  entry = firstentry;
  while((!found)&&(entry<nentries)) {
    entryNumber = STNT->GetEntryNumber(entry);
    if (entryNumber < 0) break;
    STNT->LoadTree(entryNumber);
    select->GetNdata();
    found = (int) select->EvalInstance(0);
    entry++;
  }

  // delete temporary object
  delete select;
  if (found) return(entry);
  else return(-1);
}



Int_t st_get_vars(TNtuple* STNT, Int_t entry, Float_t var_settings[MAX_SCAN_DIMS], Float_t* nevt){

  // Based on TTree:Show

  STNT->GetEntry(entry);
  TObjArray *leaves  = STNT->GetListOfLeaves();
  Int_t nleaves = leaves->GetEntriesFast();

  Int_t j = 0;
  for (Int_t i=(nleaves-5);i>1;i--) {
    TLeaf *leaf = (TLeaf*)leaves->UncheckedAt(i);
    TBranch *branch = leaf->GetBranch();
    if (branch->GetListOfBranches()->GetEntriesFast() > 0) continue;

    // only handle floats for now, and only first element
    if (leaf->IsA() == TLeafF::Class()) {
      Float_t *value = (Float_t*)leaf->GetValuePointer();
      if(j<MAX_SCAN_DIMS) var_settings[j]=value[0];
    } else {
      if(j<MAX_SCAN_DIMS) var_settings[j]=-1;
    }
    j++;
  }

  // Now for nevt - always nleaves -2;
  TLeaf *leaf = (TLeaf*)leaves->UncheckedAt(nleaves-2);
  TBranch *branch = leaf->GetBranch();
  if (branch->GetListOfBranches()->GetEntriesFast() > 0) continue;

  // only handle floats for now, and only first element
  if (leaf->IsA() == TLeafF::Class()) {
    Float_t *value = (Float_t*)leaf->GetValuePointer();
    *nevt=value[0];
  }

  return j;
}


Int_t st_get_vartypes(TNtuple* STNT, Int_t var_type[MAX_SCAN_DIMS]){

  // Count the number of scan variables present in the ROOT NTUple
  // convert variable names back to ST variable identifier numbers

  TObjArray *leaves  = STNT->GetListOfLeaves();
  Int_t nleaves = leaves->GetEntriesFast();

  Int_t j = 0;  
  for(Int_t i=(nleaves-5);i>1;i--){
    TLeaf *leaf = (TLeaf*)leaves->UncheckedAt(i);
    stub = leaf->GetName();
    if(j<MAX_SCAN_DIMS) printf("  %s, var_type %d\n",stub, var_type[j] = match_stub(stub));
    j++;
  }
  printf("Found %d scan dimensions\n",j);

  return j;
}



int stan_projection(char* in_nt, int sno, int mid, Int_t var_type[MAX_SCAN_DIMS], 
                    Float_t var_settings[12][MAX_SCAN_DIMS], Float_t * nevt){

  int i,j;
  char c_txt[32],m_txt[32],s_txt[32];
  char projection[32],stub[16];
  TCut chip, module, scan, strobed;

  // Define cuts

  if(sno>0) sprintf(s_txt,"sno==%d",sno); else sprintf(s_txt,"");
  if(mid>-1)sprintf(m_txt,"mid==%d",mid); else sprintf(m_txt,"");

  module  = new TCut(m_txt);
  scan    = new TCut(s_txt);
  strobed = new TCut("(ch%4)==calmode");

  // Find the desired NTuple Object and establish
  // what were the scanned variables

  STNT = (TNtuple*)gDirectory->Get(in_nt);
  if (STNT==0){
    printf("st_project_scan called for invalid NTuple name - exiting\n");
    return -1;
  }

  // Load other files needed by this one
 // gROOT->LoadMacro("stan_util.cpp");

  int n_dims = st_get_vartypes(STNT, var_type);

  for(i=0;i<n_dims;i++) printf("Scan dimension %d vartype = %d\n",i,var_type[i]);

  // Determine what were the scanned variables,
  // and what were the settings for each chip

  Int_t firstevent = 0;
  Int_t firstfoundevent = 0;
  printf("Determining higher order scan settings for each chip\n");
  for (i=0;i<12;i++){
    sprintf(c_txt,"(ch>%d)&&(ch<%d)",(i*128),((i+1)*128));
    chip  = new TCut(c_txt);

    firstevent = st_findfirstevent(STNT,chip+module+scan,firstevent);
    if(firstevent>=0){
      if(i==0) firstfoundevent=firstevent; 
      st_get_vars(STNT, firstevent, var_settings[i], nevt);
    }else{
      printf("No data for chip %d!\n",i);
      firstevent = 0;
    }
  }


  // Form the desired projection.  Sadly ROOT v2.22 will not 
  // fill the histogram which we need prior to use of fitslicesy: 
  // we must create and fill the histogram manually

  match_var(var_type[0],stub);
  sprintf(projection,"nhit:%s:ch",stub);
  printf("Selected Projection is %s\n",projection);
  printf("Selected cuts are %s and %s\n",m_txt, s_txt);

  // Use TTree::Draw to form the projection.  To reduce the time taken 
  // to form the projection,  set the start event as determined above
  // and limit the number of rows to be processed to 400000 
  // (64 points * 1536 channels * 4 calmodes = 393216)

  STNT->Draw(projection,module+scan+strobed,"",400000,firstfoundevent);

  Int_t * entries  = STNT->GetSelectedRows();
  Float_t * hit = STNT->GetV1();
  Float_t * var = STNT->GetV2();
  Float_t * ch  = STNT->GetV3();

  if(entries==0) {
    printf("No entries selected - returning\n");
    return -2;
  }
  printf("%d entries selected\n",entries);

  // find bounds and step size prior to histogram creation.
  // NB. step size must be constant if we persist with this method.

  Int_t scan_points =0;
  Float_t last_var = -9999;
  Float_t scan_lo =   9999;
  Float_t scan_hi =  -9999; 
  Float_t scan_step;
  Float_t * var2 = var;

  for(i=0;i<entries; i++){
    if(*var2<scan_lo) scan_lo = *var2;
    if(*var2>scan_hi) scan_hi = *var2;
    if(*var2!=last_var){
      if(scan_points==1){
        scan_step = ((*var2-last_var)/2);
        if (scan_step<0) scan_step = scan_step * -1;
      }
      last_var = *var2;
      scan_points++;
    }
    var2++;
  }

  printf("Scan %s from %3.2f to %3.2f, %d steps of %3.2f each\n",
         stub, scan_lo, scan_hi, scan_points, scan_step);

  // create and fill the histogram

  TH2F* h_scan = new TH2F("h_scan",projection, 1536,+0.5,1536.5, 
                     scan_points, scan_lo-(scan_step/2), scan_hi+(scan_step/2));

  for(i=0;i<entries; i++) h_scan->Fill(ch[i], var[i], hit[i]);

  // h_scan->Draw("COLZ");

  return n_dims;
}  

stan_projection(Int_t l_sno, Int_t l_mid){

  Float_t l_nevt;
  Float_t l_var_settings[12][MAX_SCAN_DIMS];
  Int_t   l_var_type[MAX_SCAN_DIMS];

  stan_projection("STNT1",l_sno, l_mid, l_var_type, l_var_settings, &l_nevt);
}

