void BypassTest(Int_t do_restart) { 

  //
  // A number of possible bypass configurations, and the associated 
  // changes to the enabled command and data streams have been
  // implemented as ConfigureVariable(ST_BYPASS,var):
  //
  // n   M0 S1 S2 S3 S4 E5 M8 S9 S10 S11 S12 E13   L0 L1   M0 M1
  // 0   M  E  -  -  -  -  M  E   -   -   -   -    1  1    1  1
  // 1   M  S  E  -  -  -  M  S   E   -   -   -    1  1    1  1
  // 2   M  S  S  E  -  -  M  S   S   E   -   -    1  1    1  1
  // 3   M  S  S  S  E  -  M  S   S   S   E   -    1  1    1  1
  // 4   M  S  S  S  S  E  M  S   S   S   S   E    1  1    1  1
  // 5   M  X  E  -  -  -  M  X   E   -   -   -    1  1    1  1
  // 6   M  S  X  E  -  -  M  S   X   E   -   -    1  1    1  1
  // 7   M  S  S  X  E  -  M  S   S   X   E   -    1  1    1  1
  // 8   M  S  S  S  X  E  M  S   S   S   X   E    1  1    1  1
  // 9   M  S  S  S  S  S  X  S   S   S   S   E    2  0    1  0
  // 10  X  S  S  S  S  E  M  S   S   S   S   S    0  2    0  1
  // 11  M  S  S  S  S  S  S  S   S   S   S   E    2  0    1  0
  // 12  S  S  S  S  S  E  M  S   S   S   S   S    0  2    0  1
  // 13  M  S  S  S  S  X  S  S   S   S   S   E    2  1    1  0
  // 12  S  S  S  S  S  E  M  S   S   S   S   X    0  2    0  1
  //
  // hence these configurations may be tested simply by 
  // executing a scan of variable type 17
  //
  // version 0.1 PWP 02.11.00
  // version 0.2 PWP 14.11.00 - include analysis
  // version 0.3 LE  21.11.00 - resetting values after scan
  // version 0.4 PWP 23.11.00 - replace maxtrigs by ntrigs (oops!!)
  // version 0.5 LE  16.02.00 - conforming tests to Module QA document 3.0

  Int_t last = 10;  // for barrel hybrid or module
  //Int_t last = 14;  // for forward hybrid or module

  // load the analysis macro 
  gROOT->ProcessLine(".L BypassAnalysis.cpp");

  //Store settings before the scan
  Int_t trtype   = e->burst_trtype;
  Int_t ntrigs   = e->burst_ntrigs;
  Int_t cal_loop = e->do_cal_loop;
  Int_t do_fits  = e->do_fits;


  // Configure the scan
  e->StOutScanReset();
  e->ascii_nt=0;

  e->do_cal_loop=0;       // cal loop OFF
  e->do_fits=0;           // integrated fitting OFF

  e->burst_trtype=0;      // L1A
  e->burst_ntrigs=100;

  e->ConfigureVariable(11,1);  // X1X compression 
  e->ConfigureVariable(13,0);  // EDGE DETECT OFF
  e->ConfigureVariable(7,3);   // No masked channels
  e->ConfigureVariable(14,1);  // Route Mask to Pipeline      


  e->ConfigureScan(17,0,last,1);  // 17 - bypass configuration

  st_scan(11,-1);


  // Reset values after scan
  e->burst_trtype = trtype;   
  e->burst_ntrigs = ntrigs;
  e->do_cal_loop  = cal_loop;
  e->do_fits      = do_fits;      
  e->ConfigureVariable(17,4);  // 12 chips through 2 links

  // NB. During production testing, the "Restart" should be
  // skipped to save time, it takes much longer than the scan
  // itself!!  The mask and trim files need only be loaded
  // prior to the analogue tests PWP 23.11.00

  if(do_restart){
    e->Restart();              // Reload mask and trim files.
  }

  //
  // Call the analysis macro.
  // Arguments are:
  //  run, scan, module, n_configs_tested
  //

  BypassAnalysis(e->runnum, e->scannum, -1, last+1);
} 

void BypassTest(){

  // By default, mask and trim files are reloaded
  // after the scan. PWP 23.11.00

  BypassTest(1);
}


