void CharacterisationTest(Int_t standalone){

  //
  // Perform all tests as listed in Characterisation sequence
  //
  // Version 0.1 PWP 15.11.00
  // Version 0.2 PWP 15.11.00
  // Version 0.3 PWP 23.11.00
  // Version 0.4 PWP 10.12.00
  // Version 0.5 PWP 18.05.00
  // Version 0.6 PWP 19.09.01
  // Version 0.7 PWP 20.12.01
  // Version 0.8 BJG 30.04.02 - Fix some memory leaks
  // Version 0.9 PWP 24.09.02 - generate config files at end of test
  //

  // Clear abort flag to start
  e->abort=0;

  TStopwatch* patek_phillipe = new TStopwatch();

  // Detector IV curve:
  //   it is left up to the user to do this

  if((e->abort==0)&&(standalone==0)){
    gROOT->ProcessLine(".L HardReset.cpp");
    HardReset();
  }

  if(e->abort==0){
    e->Restart();
    gROOT->ProcessLine(".L FullBypassTest.cpp");
    FullBypassTest();
  }

  if(e->abort==0){
    gROOT->ProcessLine(".L RedundancyTest.cpp");
    RedundancyTest(0);
  }

  if(e->abort==0){
    gROOT->ProcessLine(".L PipelineTest.cpp");
    PipelineTest(0);
  }

  if(e->abort==0){
    e->Restart();
    gROOT->ProcessLine(".L StrobeDelay.cpp");
    StrobeDelay(1);
  }

  if(e->abort==0){
    gROOT->ProcessLine(".L ThreePointGain.cpp");
    ThreePointGain();
  }

  if(e->abort==0){
    gROOT->ProcessLine(".L TrimRange.cpp");
    TrimRange(10,2);
    e->ExecuteConfigs(); // load new TrimRange and TrimDAC settings
  }

  if(e->abort==0){
    gROOT->ProcessLine(".L ResponseCurve.cpp");
    ResponseCurve();
  }

  if(e->abort==0){
    gROOT->ProcessLine(".L No.cpp");
    No();
  }

  if(e->abort==0){
    gROOT->ProcessLine(".L Timewalk.cpp");
    Timewalk(1);
  }

  if(e->abort==0){
    e->WriteRODConfigs();
  }

  printf("CharacterisationTest completed\n");
  patek_phillipe->Print();
  delete patek_phillipe;
}

void CharacterisationTest(){
  CharacterisationTest(0);
}

void CharacterisationReplay(Int_t run, Int_t start_scan){

  // Reanalyse the data for a full characterisation run
  // PWP 18.12.01

  TStopwatch* patek_phillipe = new TStopwatch();

  gROOT->ProcessLine(".L FullBypassAnalysis.cpp");
  FullBypassAnalysis(run,start_scan,-1);

  gROOT->ProcessLine(".L RedundancyAnalysis.cpp");
  RedundancyAnalysis(run,start_scan+6,-1);

  gROOT->ProcessLine(".L PipelineAnalysis.cpp");
  PipelineAnalysis(run,start_scan+7,-1);

  gROOT->ProcessLine(".L StrobeDelayPlot.cpp");
  StrobeDelayPlot(run,start_scan+10,-1);

  gROOT->ProcessLine(".L ResponseCurvePlot.cpp");
  ThreePointGainPlot(run,start_scan+11,4);

  gROOT->ProcessLine(".L TrimRangePlot.cpp");
  TrimRangePlot(run,start_scan+14,2,4,1.0);

  ResponseCurvePlot(run,start_scan+42,3);

  gROOT->ProcessLine(".L NoPlot.cpp");
  NoPlot(run,start_scan+52,1,0,1);

  gROOT->ProcessLine(".L TimewalkPlot.cpp");
  TimewalkPlot(run,start_scan+54);

  printf("CharacterisationReplay completed\n");
  patek_phillipe->Print();
  delete patek_phillipe;
}

