void CurrentScan(int start_num) {
  // Performs a set of threshold scans to extract gain and noise for
  // the FE currents set in c:\sctdaq\user\currents.dat

  int imod;
  char file[50],line[50];
  int scan=2;

  // Store values before scan

  Float_t in_charge[6][12];
  Float_t in_preamp[6][12];
  Float_t in_shaper[6][12];

  for (module=0; module<6; module++){
    if(e->m[module]->present){
      for(chip=0;chip<12;chip++){
	in_charge[module][chip] = e->RetrieveVariable(module,chip,2);
	in_preamp[module][chip] = e->RetrieveVariable(module,chip,4);
	in_shaper[module][chip] = e->RetrieveVariable(module,chip,5);
      }
    }
  }

  // Arrays with containing current matrix
  int nchg;
  int npre;
  int nshpr;
  float charge[9];
  float preamp[9];
  float shaper[9];

  // Open file withcurrent configuration data
  sprintf(file,"C:\\sctdaq\\macros\\currents.dat");
  FILE* conffile = fopen(file,"r");
  if(conffile==NULL){
    printf("Could not open current/charge info file %s\n",file);
    return;
  } else {
    printf("File %s opened for reading\n",file);
  }

    // Read input charges.
    fgets(line,200,conffile);
    fgets(line,200,conffile);
    sscanf(line,"%d %f %f %f %f %f %f %f %f %f",&nchg,&charge[0],&charge[1],&charge[2],
	   &charge[3],&charge[4],&charge[5],&charge[6],&charge[7],&charge[8]);

    // Read input preamp.
    fgets(line,200,conffile);
    fgets(line,200,conffile);
    sscanf(line,"%d %f %f %f %f %f %f %f %f %f",&npre,&preamp[0],&preamp[1],&preamp[2],
	   &preamp[3],&preamp[4],&preamp[5],&preamp[6],&preamp[7],&preamp[8]);

    // Read input shaper.
    fgets(line,200,conffile);
    fgets(line,200,conffile);
    sscanf(line,"%d %f %f %f %f %f %f %f %f %f",&nshpr,&shaper[0],&shaper[1],&shaper[2],
	   &shaper[3],&shaper[4],&shaper[5],&shaper[6],&shaper[7],&shaper[8]);


  printf("Read %d charges: ",nchg);
  for(int i=0;i<nchg;i++) {
    printf("%3.2f ",charge[i]);
  }
  printf("fC\n");

  printf("Read %d preamps: ",npre);
  for(int i=0;i<npre;i++) {
    printf("%3.1f ",preamp[i]);
  }
  printf("uA\n");

  printf("Read %d shapers: ",nshpr);
  for(int i=0;i<nshpr;i++) {
    printf("%3.1f ",shaper[i]);
  }
  printf("uA\n");

   //Set scan range  (Threshold scan)
   e->ConfigureScan(1,0,400,2.5);
   e->ShowScan();

   for(int ipre=0;ipre<npre;ipre++) {
      //Set preamplifier current
      e->ConfigureVariable(4,preamp[ipre]);

      for(int ishap=0;ishap<nshpr;ishap++) {
	 //Set shaper current
	 e->ConfigureVariable(5,shaper[ishap]);

	 for(int ichg=0;ichg<nchg;ichg++) {
	    //Set calibration pulse
	    e->ConfigureVariable(2,charge[ichg]*10);
   
	    //Send values to chip
	    e->ExecuteConfigs();
	    
	    //Check that the values are set
	    //  st_dialog_show_module_config();
            //  printf("type a number to continue (int) ?",imod);
	    //  scanf("%d",&imod);

	    //Run scan
	    printf("\nScan %d for FE %3.1f/%3.1f, charge %3.1f\n",(scan-1),preamp[ipre],shaper[ishap],charge[ichg]);
	    if(scan>start_num) { 
	      st_scan(11,-1);
	    }
	    scan++;
	 }
      }
   }

   // Restore values after scan
   for (module=0; module<6; module++){
     if(e->m[module]->present){
       for(chip=0;chip<12;chip++){
	 e->ConfigureVariable(module,chip,2,in_charge[module][chip]);
	 e->ConfigureVariable(module,chip,4,in_preamp[module][chip]);
	 e->ConfigureVariable(module,chip,5,in_shaper[module][chip]);
       }
     }
   }
}


void CurrentScan() {

  CurrentScan(1);
}

























