
// DBUpload.cpp
//
// routines pertinent to the DB upload
// of hybrid and module test results
//
// version 0.1 BJG/PWP 19.12.01
// version 0.2 BJG 09-10/02 - Add a couple of special cases for conversion of old files
//                            ModIVScan -> DetModIV
//                            Add extra '.'s for response curve
// version 0.3 BJG 4/11/02 - Change tabs to spaces

  #ifdef DATA_DIRECTORY
    // do not reload parameters.h
  #else
    // must load parameters.h
  #include <parameters.h>
  #endif

Int_t SplitOutput(Char_t* f_stub) {
  // Objective: Split module_date file into module_testnum

  Char_t in_name[64]; 
  Char_t out_name[64]; 

  FILE* infile = NULL;
  FILE* outfile = NULL;
  Char_t line[256];

  Int_t n_errors = 0;
  Int_t n_test = 0;

  // Vars for parsing scan_info
  Int_t inScanInfo = 0;
  Int_t scanState = 0;
  Int_t numPoints;
  Int_t pointsRead;

  Int_t inLTT = 0;

  sprintf(in_name,"%s%s.txt",RESULTS_DIRECTORY,f_stub);
  infile = fopen(in_name,"r");

  if(infile==NULL){
    printf("Could not open result infile %s\n",in_name);
  } else {
    printf("File %s opened to read results\n",in_name);

    // Find a test
    while(fgets(line,256,infile)!=NULL){
      if(strncmp(line,"%NewTest",8)==0){
        n_test ++;
        printf("found test %d\n",n_test);

        if(outfile) { // file already open
          fclose(outfile);
          outfile = NULL; 
        } 

        // Create output file name
        sprintf(out_name, "%s%s_%d.txt", UPLOAD_DIRECTORY, f_stub, n_test);
        outfile = fopen(out_name, "w");

        if(outfile==NULL) {
          printf("Could not open output file %s\n", out_name);
          n_errors++;
        } else {
          printf("Opened file %s\n", out_name);
          fprintf(outfile, 
                  "# Test #%d from file %s\n", 
                  n_test, in_name);
        }
      }
      if(outfile) {
        if(strncmp(line,"%",1)==0){
          inScanInfo = 0;
          inLTT = 0;

          if(strncmp(line,"%SCAN_INFO",10)==0){
            inScanInfo = 1;
            scanState = 0;
            pointsRead = 0;
          } else if(strncmp(line, "%LongTermTest", 13) == 0) {
            inLTT = 1;
          } else if(strncmp(line, "%ModIVScan", 10) == 0) {
            strcpy(line, "%DetModIV");
          } else {
            if(inScanInfo && pointsRead < 16) {
              // Add some dots at the end
              fprintf(outfile, "# Need %d more points\n", 16-pointsRead);
              for(int i=0; i < (16 - pointsRead); i++)
                fprintf(outfile, ". ");
              fprintf(outfile, "\n");
            }
          }
        }

        // Correct scan_info table
        if(inScanInfo) {
          if(strncmp(line, "#", 1) != 0) {
            float dummy;

            switch(scanState) {
            case 0: case 1:
              scanState ++;
              break;
            case 2:
              // n_points
              sscanf(line, "%d", &numPoints);
              scanState ++;
              break;
            case 3:
              // points

              char linecopy[256];
              strcpy(linecopy, line);

              char *token;

              if(token = strtok(linecopy, "\t ")) {
                pointsRead ++;
                while(token = strtok(NULL, "\t ")) {
                  pointsRead ++;
                }
              }
              printf("SCAN_INFO: Found %d points so far out of %d\n", pointsRead, numPoints);

              if(pointsRead > 16 || pointsRead > numPoints) {
                printf("Too many points in SCAN_INFO table %d (will be printed anyway)\n", pointsRead);
                n_errors ++;
              }

              break;
            }
          }
          if(inLTT) {
            if(strncmp(line, "TEST", 4) == 0) {
              fprintf(outfile, "%");
              inLTT = 0;
            }
          }
        }

        // Replace tabs by spaces
        for(int i=0; i<strlen(line); i++) {
          if(line[i] == '\t') {
            line[i] = ' ';
          }
        }

        // A file is open - write the present line to it
        fprintf(outfile,"%s", line);
      }
    }

    if(outfile) { // file already open
      fclose(outfile);
      outfile = NULL; 
    } 

    printf("Finished processing file\n");
    printf("A total of %d tests were found\n",n_test);
    printf("A total of %d errors were reported\n",n_errors);
  }

  if(outfile!=NULL) 
    fclose(outfile);
  if(infile!=NULL)
    fclose(infile);

  return n_test;
}  

void UploadFiles(Char_t* f_stub, Int_t n_tests){
  // Upload the files to the database
  // using Java application from Geneva

  Char_t line[256];
  Int_t i;

  for(i=1;i<=n_tests;i++){
    // upload each file
    sprintf(line, "java UploadTestData %s%s_%d.txt",
                  UPLOAD_DIRECTORY, f_stub, i);
    printf("%s\n",line);
    // gSystem->Exec(line);
  }

}
Bool_t CheckPresent(Char_t* f_stub){
  // Is the line f_stub in the log file?

  FILE *file;
  Char_t line[256];

  sprintf(line,"%suploaded.log",UPLOAD_DIRECTORY);
  file = fopen(line, "r");

  Int_t length = strlen(f_stub);
  Bool_t found = (Bool_t) 0;

  if(file == NULL) {
    // Couldn't open the file
    found = (Bool_t) 0;
  } else {
    // Search through the lines
    while(fgets(line,160,file)!=NULL){
      if(strncmp(line,f_stub,length)==0){
        found = (Bool_t) 1;
        break;
      }
    }

    fclose(file);
  }

  return found;
}

void AppendUpload(Char_t* f_stub){
  // add selected results file to the list of
  // processed files

  FILE *file;
  Char_t line[256];

  sprintf(line,"%suploaded.log",UPLOAD_DIRECTORY);
  file = fopen(line, "a");

  if(file == NULL) {
    fprintf(stderr, "Can't open upload record file\n");
  } else {
    fprintf(file, "%s\n", f_stub);

    fclose(file);
  }
}

void DeleteFiles(Char_t* f_stub){
  // delete all upload files coming from results
  // file f_stub after they have been uploaded

  Char_t line[256];

  sprintf(line,"del %s%s*.txt",UPLOAD_DIRECTORY,f_stub);
  printf("%s\n",line);
  gSystem->Exec(line);
}

void DBUpload(Char_t* f_stub){
  // Upload results from the specified file to the database

  Int_t n_tests = 0;

  // Check against list of uploaded files
  if(CheckPresent(f_stub) == (Bool_t) 0) {
    // Break the results file down 
    // into its consituent tests
    n_tests = SplitOutput(f_stub);

    if(n_tests>0){
      // Upload files to the database
      UploadFiles(f_stub,n_tests);

      // Delete files after upload
      DeleteFiles(f_stub);

      // Add file to list of uploaded files.
      AppendUpload(f_stub);
    }
  } else {
    printf("Data from results file %s%s.txt\n",RESULTS_DIRECTORY,f_stub);
    printf("has already been uploaded\n");
  }
}



