void PVTest(Float_t startVdd, Float_t stopVdd, Float_t stepVdd, Int_t do_restart) {

  //
  // Test both clock/command options
  // also send alternating patterns to the mask register
  //
  // version 0.2 PWP 13.11.00
  // version 0.3 PWP 14.11.00
  // version 0.4 LE  21.11.00 - resetting values after scan.
  // version 0.5 PWP 23.11.00 - cosmetics
  //


  // load the analysis macro 
  gROOT->ProcessLine(".L PVAnalysis.cpp");

  // Store settings before scan
  Int_t cal_loop = e->do_cal_loop;
  Int_t do_fits  = e->do_fits;    
  Int_t trtype   = e->burst_trtype;
  Int_t ntrigs   = e->burst_ntrigs;
  Int_t debug_level = e->debug_level;
  Int_t do_preburst = e->do_preburst;


  // Configure the scan
  e->StOutScanReset();
  e->ascii_nt=0;

  e->do_cal_loop=0;        // cal loop OFF
  e->do_fits=0;            // integrated fitting OFF
  e->do_mask_test=1;       // alternating mask patterns ON 
  e->config_frequency = 1; // send full configuration before each event

  e->burst_trtype=31;      // SOFT RESET + (com_delay BCOs) + L1A
  e->burst_ntrigs=100;
  e->debug_level=5;
  e->do_preburst=1;

  e->ConfigureVariable(11,1);    // X1X compression 
  e->ConfigureVariable(13,0);    // EDGE DETECT OFF
  e->ConfigureVariable(14,1);    // Route Mask to Pipeline      
  e->ConfigureVariable(25,129);  // com delay      


  e->ConfigureScan(33,startVdd,stopVdd,stepVdd);

  st_scan(11,-1);

 
  // Reset values after scan
  e->do_cal_loop  = cal_loop;
  e->do_fits      = do_fits;    
  e->burst_trtype = trtype;
  e->burst_ntrigs = ntrigs;
  e->debug_level=debug_level;
  e->do_preburst=do_preburst;

  e->do_mask_test=0;       // alternating mask patterns OFF 
  e->config_frequency = 0; // do not send configuration before each event

  e->ConfigureVariable(25,200);  // Reset default command delay      
  e->ConfigureVariable(38,0);    // set SELECT back to 0


  // NB. During production testing, the "Restart" should be
  // skipped to save time, it takes much longer than the scan
  // itself!!  The mask and trim files need only be loaded
  // prior to the analogue tests PWP 23.11.00

  if(do_restart){
    e->Restart();              // Reload mask and trim files.
  }

  //
  // Call the analysis macro.
  // Arguments are:
  //  run, scan, module
  //

  if(startVdd<stopVdd) {
    PVAnalysis(e->runnum, e->scannum,-1,startVdd,stopVdd,TMath::Abs(stepVdd));
  } else {
    PVAnalysis(e->runnum, e->scannum,-1,startVdd,stopVdd,-TMath::Abs(stepVdd));
  }
}

void PVTest(){

  PVTest(5.0,3.3,0.1,0);

}














