void PipelineAnalysis(Int_t run, Int_t scan, Int_t moi){

  // analysis of PipelineTest
  // version 0.1 PWP 13.11.00
  // version 0.2 PWP 15.11.00  bugfix
  // version 0.3 PWP 25.05.01 - Use parameter.h and Toolbox.cpp
  // version 0.4 LE  06.08.01 - Look for stuck cells and cells over, print bad channel list.
  // version 0.5 PWP 08.08.01 - open both files once and once only; analyse data in one loop
  // version 0.51 PWP 21.08.01 - database compliance
  // version 0.52 PWP 12.09.01 - further database compliance
  // version 0.53 PWP 12.09.01 - small bugfix to defect identification loop;  record DEAD or
  //                             STUCK channels as opposed to cells when all 12 cells defective
  // version 0.6 PWP 15.11.01 - compliance with new DB format
  // version 0.61 PWP 12.04.02 - also list cell number for DEADCELL and STUCKCELL defects.
  //                             This information is prefaced by the "#" character so as not to
  //                             interfere with DB compatibility
  // version 0.62 BJG 30.04.02 - minor syntax errors

  #ifdef DATA_DIRECTORY
    // do not reload parameters.h
  #else
    // must load parameters.h
  #include <parameters.h>
  #endif

  Int_t module, link, chip, chan, i;
  char f_name[128], h_name[64];

  Short_t chan_dead[MAX_LINKS][MAX_CHANS];
  Short_t chan_over[MAX_LINKS][MAX_CHANS];
  Short_t chan_stuck[MAX_LINKS][MAX_CHANS];

  Bool_t cell_dead[MAX_LINKS][MAX_CHANS][MAX_PIPELINE];
  Bool_t cell_stuck[MAX_LINKS][MAX_CHANS][MAX_PIPELINE];

  Short_t chip_bad[MAX_LINKS][MAX_CHIPS];
  Short_t chip_dead[MAX_LINKS][MAX_CHIPS];
  Short_t chip_over[MAX_LINKS][MAX_CHIPS];
  Short_t chip_stuck[MAX_LINKS][MAX_CHIPS];
  Short_t chip_good[MAX_LINKS][MAX_CHIPS];
  Short_t chip_status[MAX_LINKS][MAX_CHIPS];
  Short_t module_status, n_bad_chips, n_good_chips;

  // Load other files needed by this one
  gROOT->LoadMacro("stan_util.cpp");
  gROOT->LoadMacro("ToolBox.cpp");


  //
  // Open both data files
  //

  sprintf(f_name,"%sstrun%d_%d.root",DATA_DIRECTORY,run,scan);
  TFile *strun1 = new TFile(f_name);
    
  if(strun1==0){
    printf("Failed to open first file, %s - EXITING!!\n",f_name);
    return;
  }

  sprintf(f_name,"%sstrun%d_%d.root",DATA_DIRECTORY,run,scan+1);
  TFile *strun2 = new TFile(f_name);
    
  if(strun2==0){
    printf("Failed to open second file, %s - EXITING!!\n",f_name);
    return;
  }


  //
  //  loop over modules
  //

  Int_t start = 0;
  Int_t end   = 0;

  if(moi==-1){
    start = 0;
    end   = MAX_MODULES;

  } else {
    start = moi;
    end   = moi+1;
  }

  for (module=start; module<end; module++){

    //
    // Reset all arrays
    //
    
    module_status = -1;  // So far no data seen from the module.
    n_bad_chips   = 0;   // no chips have yet failed
    n_good_chips  = 0;   // no chips have yet passed

    for(link=0;link<MAX_LINKS;link++){
      for(chip=0;chip<MAX_CHIPS;chip++){
        chip_bad[link][chip]    = 0;
        chip_good[link][chip]   = 0;
        chip_status[link][chip] = 0;
        chip_dead[link][chip]   = 0;
        chip_over[link][chip]   = 0;
        chip_stuck[link][chip]  = 0;
      }
      for(chan=0;chan<MAX_CHANS;chan++){
        chan_dead[link][chan]  = 0;
        chan_over[link][chan]  = 0;
        chan_stuck[link][chan] = 0;
	
	for(i=0;i<MAX_PIPELINE;i++){
	  cell_dead[link][chan][i]  = kFALSE;
	  cell_stuck[link][chan][i]  = kFALSE;
	}
      }
    }



    //
    // First run, find dead channels or channels with more
    // than 100% occupancy.
    //


    for(link=0;link<MAX_LINKS;link++){

      // find histograms

      sprintf(h_name,"h_scan%d;%d",link,module+1);
      TH2F *h_scana= (TH2F*)strun1->Get(h_name);
      TH2F *h_scanb= (TH2F*)strun2->Get(h_name);

      if((h_scana!=0)&&(h_scanb!=0)){
        if(h_scana->GetMean()>0){ // (hscanb should be empty)
          printf("Analysing data for module %d link %d\n",
          module, link);

          // Data exists for module on at least one side
          module_status = 0;  // but maybe not passed.

          // Categorise each channel
 
          for(chan=0;chan<MAX_CHANS;chan++){
            chip = chan/128;

            for(i=0;i<MAX_PIPELINE;i++){
              Float_t occa = h_scana->GetBinContent((770*(i+1))+chan);
              Float_t occb = h_scanb->GetBinContent((770*(i+1))+chan);

              if(occa==0){
                chan_dead[link][chan]++;
                chip_dead[link][chip]++;
		cell_dead[link][chan][i]  = kTRUE;
              }
	      
              if(occa>MAX_TRIGGERS){
                chan_over[link][chan]++;
                chip_over[link][chip]++;
              }
	      
              if(occb>0){
                chan_stuck[link][chan]++;
                chip_stuck[link][chip]++;
		cell_stuck[link][chan][i] = kTRUE;
              }

            } // end for cell
          } // end for(chan)
	} // end if(h_scana->GetMean()>0)
      } // end if(h_scana&&hscanb)
    } // end for(link)

    
    //
    // Second stage of analysis: how many chips have passed?
    //

    if(module_status>-1){  // data exists for module

      for(link=0;link<MAX_LINKS;link++){

	// Determine number of good channels per chip
	for(chan=0;chan<MAX_CHANS;chan++){
	  chip = chan/128;
	  
	  if( (chan_dead[link][chan]==0) && (chan_over[link][chan]==0) && (chan_stuck[link][chan]==0) ){
	    chip_good[link][chip]++;
	  }else{
	    chip_bad[link][chip]++;
	  }
	} // end for(chan)

	
	for(chip=0;chip<MAX_CHIPS;chip++){
	  if( (chip_good[link][chip]>=126) && (chip_dead[link][chip]<24) &&
	      (chip_stuck[link][chip]<24)  && (chip_over[link][chip]<24) ){

	    chip_status[link][chip]=1; // PASS
	    n_good_chips++;

	  }else{
	    chip_status[link][chip]=0; // FAIL
	    n_bad_chips++;
	  }
	}

      } // end for(link)
    } // end if(module_status)

    //
    // Third stage of analysis: has the module passed?
    //

    if((n_bad_chips+n_good_chips)==0){
      module_status = -1;  // NO DATA
      
    }else if(n_good_chips==12){
      module_status =  1; // PASS
      
    }else{
      module_status =  0; // FAIL
    }

    
    if(module_status>-1){ // Do not display or save results if no data!!
      

      //
      // Results to screen
      //


      printf("          pass good  bad  stuck over dead\n");

      for(link=0;link<MAX_LINKS;link++){
        for(chip=0;chip<MAX_CHIPS;chip++){
          Int_t chip2 = (link*6)+chip;

          if(chip_status[link][chip]){
            printf("  chip %2d PASS  ", chip2);
          }else{
            printf("  chip %2d FAIL  ", chip2);
          }

          printf("%3d  %3d  %3d  %3d  %3d\n",
		 chip_good[link][chip],
		 chip_bad[link][chip],
		 chip_stuck[link][chip],
		 chip_over[link][chip],
		 chip_dead[link][chip]);

        } // end for(chip)
      } // end for(link)


      //
      // Results to file
      //

      FILE* outfile = OpenResults("PipelineTest", strun1, module, run, scan, module_status,0); 

      if(outfile){

        fprintf(outfile,"%%PipelineTest\n");
        fprintf(outfile,"#\n");

        fprintf(outfile,"#NGOOD\n");

        fprintf(outfile,"#M0 S1  S2  S3  S4  E5\n");
        for(chip=0;chip<MAX_CHIPS;chip++){
          fprintf(outfile,"%03d ", chip_good[0][chip]);
        }
        fprintf(outfile,"\n#\n");

        fprintf(outfile,"#M8 S9  S10 S11 S12 E13\n");
        for(chip=0;chip<MAX_CHIPS;chip++){
          fprintf(outfile,"%03d ", chip_good[1][chip]);
        }
        fprintf(outfile,"\n#\n");


        // First list defects which can affect groups of channels

        Int_t first = -1;
        Int_t last = -1;
        Int_t n_defects=0;
        for(link=0;link<MAX_LINKS;link++){
          for(chip=0;chip<MAX_CHIPS;chip++){

            first = -1;
            last  = -1;

            for(chan=(chip*128);chan<((chip+1)*128);chan++){
              if(chan_dead[link][chan]==MAX_PIPELINE){
                if(first==-1){
                  first = (link*MAX_CHANS)+chan;
                }
                last = (link*MAX_CHANS)+chan;
              }else if(first>-1){
                ListDefect(outfile,"DEAD",first,last);
                n_defects++;
                first=-1;
                last=-1;
              }
            }

            if(first>-1){
              ListDefect(outfile,"DEAD",first,last);
              n_defects++;
            }

            first = -1;
            last  = -1;

            for(chan=(chip*128);chan<((chip+1)*128);chan++){
              if(chan_stuck[link][chan]==MAX_PIPELINE){
                if(first==-1){
                  first = (link*MAX_CHANS)+chan;
                }
                last = (link*MAX_CHANS)+chan;
              }else if(first>-1){
                ListDefect(outfile,"STUCK",first,last);
                n_defects++;
                first=-1;
                last=-1;
              }
            }

            if(first>-1){
              ListDefect(outfile,"STUCK",first,last);
              n_defects++;
            }


          } // end for(chip)
        } // end for(link)


        // Now list defects which can affect only single channels

        for(link=0;link<MAX_LINKS;link++){
          for(chan=0;chan<MAX_CHANS;chan++){

            if((chan_dead[link][chan]>0)&&(chan_dead[link][chan]<MAX_PIPELINE)){

              for(i=0;i<MAX_PIPELINE;i++){
                if(cell_dead[link][chan][i]){
                  fprintf(outfile,"#deadcell chan %d cell %d\n",(link*MAX_CHANS+chan),i);
                  ListDefect(outfile,"DEADCELL",(link*MAX_CHANS+chan),(link*MAX_CHANS+chan));
                  n_defects++;
                }
              }
            }
	    
	    if((chan_stuck[link][chan])&&(chan_stuck[link][chan]<MAX_PIPELINE)){
              for(i=0;i<MAX_PIPELINE;i++){
                if(cell_stuck[link][chan][i]){
                  fprintf(outfile,"#stuckcell chan %d cell %d\n",(link*MAX_CHANS+chan),i);
                  ListDefect(outfile,"STUCKCELL",(link*MAX_CHANS+chan),(link*MAX_CHANS+chan));
                  n_defects++;
		}
	      }
	    }
	  }
	}


        if(n_defects==0){
          fprintf(outfile,"#No defects found!\n");
        }else{
          fprintf(outfile,"#%d defects found\n",n_defects);
        }

        fclose(outfile);

      } // end if(outfile)
    }
  } // end(module)


  strun1->Close();
  strun2->Close();
}


void PipelineAnalysis(Int_t run, Int_t scan){

  PipelineAnalysis(run,scan,-1);
}

