void UploadRC(Int_t module, Int_t chip, Int_t method, Int_t function,
              Float_t p0, Float_t p1, Float_t p2){

  e->ConfigureRC(module,chip,method,function,p0,p1,p2);
}

void ResponseCurvePlot(Int_t run, Int_t n_points, Int_t special_point,
                       Int_t* scan, Float_t* charge, Short_t fit_type, 
                       Short_t dowait, Short_t postscript, Short_t online){

  // version 0.5  PWP 12.10.00
  // version 0.6  LE  18.11.00 Plotting with TGraphErrors, calculated as RMS.
  // version 0.7  PWP 21.11.00 changes to suit 10 point scan
  // version 0.71 PWP 21.11.00 bugfixes 
  // version 0.72 PWP 24.11.00 raise upper noise cut
  // version 0.8  PWP 08.12.00 add code to upload RC fit to ST
  // version 0.81 PWP 25.05.01 use parameter.h and Toolbox.cpp
  // version 0.82 LE  02.06.01 Flag bad chips, reduce output of bad channels, scale histos,
  //                           make output more compliant with ET documentation. RMS noise cut.
  // version 0.83 PWP 05.07.01 Add new form of channel output to print ALL bad channels
  //                           EXCEPT partbonded or unbonded channels of partbonded or 
  //                           unbonded chips.  (This new case is now the default.)
  // version 0.84 PWP 04.10.01 Generate extra file containing gain, offset, noise data for each channel
  // version 0.85 PWP 30.01.02 fix memory leaks PWP after BJG
  // version 0.86 PWP 21.03.02 add web link to postscript plot if WEB_KEY defined
  //                           place leading zero in PS file timestamp
  // version 0.87 BJG 30.04.02 Put DUT type on plots
  //                           Fix some variable types
  //                           Fix some memory leaks
  // version 0.90 PWP 23.09.02 Important changes to the bad channel identification code.
  //                           1) The identification of noisy channels is based upon a cut
  //                              noise > mean_chip_noise + (3 * rms_chip_noise).  However in
  //                              previous versions the calculation of the mean and rms values
  //                              from which the test was made included only channels upto and 
  //                              including the channel being tested.  This has now been 
  //                              revised to include all functional channels of the chip.
  //                           2) Channels with anomalous gain are now identified as follows:
  //                              hi_gain - gain > 1.25 * mean_chip_gain
  //                              lo_gain - gain < 0.75 * mean_chip_gain
  //                              (This is in agreement with the gain cuts used during chip testing.)

  #ifdef DATA_DIRECTORY
    // do not reload parameters.h
  #else
    // must load parameters.h
  #include <parameters.h>
  #endif

  #define MAX_RCSCANS   10

  #define LOST         0
  #define DODGY        1
  #define DEAD         2
  #define STUCK        3
  #define INEFFICIENT  4
  #define UNBONDED     5
  #define LO_GAIN      6
  #define HI_GAIN      7
  #define LO_OFFSET    8
  #define HI_OFFSET    9
  #define PARTBONDED  10
  #define HI_NOISE    11
  #define MAX_CODES   12

  #define MIN_GAIN         0  // These cuts now used only for chip categorisation:
  #define MAX_GAIN       100  // channel gain cuts now 0.75 and 1.25 * chip mean gain
                              // PWP 23.09.02

//#define MIN_OFFSET    -100  // doesn't work
  Float_t MIN_OFFSET  = -100;
  #define MAX_OFFSET     120
  #define BONDED_NOISE   750
  #define MIN_INNSE     1100
  #define MAX_INNSE     1800  // replaced by a rms cut (currently 3 * rms)
                              // for the bad channel list.

  /* The multiple levels of printout which were previously 
   * avaliable have been removed. Only DB compatible printout 
   * is now available. PWP 14.12.01
   */

  Int_t module, link, chip, chan, i;
  char f_name[128], h_name[64], h_title[64], line[1024], comment[20];
  Short_t the_code, chip_code;

  char mname[MAX_MODULES][32];
  char devtype[MAX_MODULES][32];
  char rc_name[32];

  Short_t code[MAX_MODULES][MAX_LINKS][MAX_CHANS][MAX_RCSCANS];
  Float_t mean[MAX_MODULES][MAX_LINKS][MAX_CHANS][MAX_RCSCANS];
  Float_t sigma[MAX_MODULES][MAX_LINKS][MAX_CHANS][MAX_RCSCANS];

  Short_t ngood[MAX_MODULES][MAX_LINKS][MAX_CHANS];
  Short_t module_status[MAX_MODULES][MAX_LINKS];

  Float_t ch_mean[MAX_MODULES][MAX_LINKS][MAX_CHIPS][MAX_RCSCANS];
  Float_t ch_sigma[MAX_MODULES][MAX_LINKS][MAX_CHIPS][MAX_RCSCANS];
  Float_t ch_mean_err[MAX_MODULES][MAX_LINKS][MAX_CHIPS][MAX_RCSCANS];
  Float_t ch_sigma_err[MAX_MODULES][MAX_LINKS][MAX_CHIPS][MAX_RCSCANS];
  Float_t ch_ngood[MAX_MODULES][MAX_LINKS][MAX_CHIPS][MAX_RCSCANS];

  Short_t ch_code[MAX_MODULES][MAX_LINKS][MAX_CHIPS];

  Float_t ch_gain[MAX_MODULES][MAX_LINKS][MAX_CHIPS][MAX_RCSCANS];
  Float_t ch_offset[MAX_MODULES][MAX_LINKS][MAX_CHIPS];
  Float_t ch_innse[MAX_MODULES][MAX_LINKS][MAX_CHIPS][MAX_RCSCANS];
  Float_t ch_innse_err[MAX_MODULES][MAX_LINKS][MAX_CHIPS][MAX_RCSCANS];
  Double_t ch_rcfit[MAX_MODULES][MAX_LINKS][MAX_CHIPS][3];

  Float_t gain_mean[MAX_MODULES][MAX_LINKS][MAX_CHIPS+1];
  Float_t gain_rms[MAX_MODULES][MAX_LINKS][MAX_CHIPS+1];
  Float_t vt50_mean[MAX_MODULES][MAX_LINKS][MAX_CHIPS+1];
  Float_t vt50_rms[MAX_MODULES][MAX_LINKS][MAX_CHIPS+1];
  Float_t offset_mean[MAX_MODULES][MAX_LINKS][MAX_CHIPS+1];
  Float_t offset_rms[MAX_MODULES][MAX_LINKS][MAX_CHIPS+1];
  Float_t innse_mean[MAX_MODULES][MAX_LINKS][MAX_CHIPS+1];
  Float_t innse_rms[MAX_MODULES][MAX_LINKS][MAX_CHIPS+1];


  Short_t ch_bad[MAX_MODULES][MAX_LINKS][MAX_CHIPS][MAX_CODES];
  Short_t lk_bad[MAX_MODULES][MAX_LINKS+1][MAX_CODES];

  Float_t charge_err[MAX_RCSCANS];

  for(Int_t i=0;i<MAX_RCSCANS;i++) charge_err[i] = 0.2; 


  // Load other files needed by this one
  gROOT->LoadMacro("stan_util.cpp");
  gROOT->LoadMacro("ToolBox.cpp");
  stan_styles();
  gStyle->SetOptFit(10);

  // 
  // 1. Ensure that the arrays are cleared to start
  //

  for(module=0;module<MAX_MODULES;module++){
    for(link=0;link<MAX_LINKS;link++){
      for(chan=0;chan<MAX_CHANS;chan++){
        for(i=0;i<MAX_RCSCANS;i++){
          code[module][link][chan][i]=0;
          mean[module][link][chan][i]=0;
          sigma[module][link][chan][i]=0;
        }
        ngood[module][link][chan]=0;
      }
      for(chip=0;chip<MAX_CHIPS;chip++){
	ch_code[module][link][chip]=0;
	gain_mean[module][link][chip]=0;
	gain_rms[module][link][chip]=0;
	vt50_mean[module][link][chip]=0;
	vt50_rms[module][link][chip]=0;
	offset_mean[module][link][chip]=0;
	offset_rms[module][link][chip]=0;
	innse_mean[module][link][chip]=0;
	innse_rms[module][link][chip]=0;
        for(i=0;i<MAX_RCSCANS;i++){
          ch_mean[module][link][chip][i]=0;
          ch_sigma[module][link][chip][i]=0;
          ch_mean_err[module][link][chip][i]=0;
          ch_sigma_err[module][link][chip][i]=0;
          ch_ngood[module][link][chip][i]=0;

          ch_gain[module][link][chip][i]=0;
          ch_innse[module][link][chip][i]=0;
          ch_innse_err[module][link][chip][i]=0;
        }
        for(i=0;i<MAX_CODES;i++){
          ch_bad[module][link][chip][i]=0;
        }
        ch_rcfit[module][link][chip][0]=0;
        ch_rcfit[module][link][chip][1]=0;
        ch_rcfit[module][link][chip][2]=0;
        ch_offset[module][link][chip]=0;
      }
      module_status[module][link] = 0;
    }
    for(link=0;link<MAX_LINKS+1;link++){
      for(i=0;i<MAX_CODES;i++){
        lk_bad[module][link][i]=0;
      }
    }
    sprintf(mname[module]," ");
  }
  

  // 
  // 2. Load the Fitted Scan Data
  //

  // 2.1 Find today's date - needed for file output

  TDatime *dt = new TDatime;
  Int_t date = dt->GetDate();
  Int_t the_time = dt->GetTime();
  delete dt;

  // 2.2 Some simple validity tests and helpful (!) messages

  if(n_points<0){
    printf("You requested that %d scans be processed.\n");
    printf("You must be having a laugh!!\n");
    printf("Now calm down and try again!!!\n");
  }

  if(n_points>MAX_RCSCANS){
    printf("WARNING: requested number of scans (%d) exceeds maximum\n",n_points,MAX_RCSCANS);
    printf("         only %d scans will be processed\n");
    n_points = MAX_RCSCANS;
  }

  if((special_point>n_points)||(special_point<0)){
    printf("WARNING: requested special scan (%d) out of range\n",special_point,n_points);
    printf("         scan %d will be used instead\n",(int)(n_points/2));
    special_point = (int)(n_points/2);
  }

  TObjString* time  = 0;
  TObjString* place = 0;
  TFile *strun = 0;

  // 2.3 Loop to load files

  for(i=0;i<n_points;i++){

    // 2.3.1 Open file

    sprintf(f_name,"%sstrun%d_%d.root",DATA_DIRECTORY,run,scan[i]);
    strun = new TFile(f_name);

    if(strun==0){
      printf("Failed to open file %s - EXITING!!\n",f_name);
      return;
    }

    // 2.3.2 Find histograms and fill arrays

    for (module=0; module<MAX_MODULES; module++){
      if(i==0){
        if(time) delete time;
        if(place) delete place;

        time  = (TObjString*)strun->Get("Time");
        place = (TObjString*)strun->Get("Place");

        TObjString *name = 0;
        sprintf(h_title,"ModuleName;%d",module+1);
        name = (TObjString*)strun->Get(h_title);
        if(name) {
          sprintf(mname[module],"%s",name->GetName());
          delete name;
        }

        TObjString *type = 0;
        sprintf(h_title,"DUT;%d",module+1);
        type = (TObjString*)strun->Get(h_title);
        if(type) {
          sprintf(devtype[module], "%s", type->GetName());
          delete type;
        }
      }

      for(link=0;link<MAX_LINKS;link++){

        sprintf(h_name,"h_code%d;%d",link,module+1);
        TH1S *h_code= (TH1S*)strun->Get(h_name);
        sprintf(h_name,"h_mean%d;%d",link,module+1);
        TH1F *h_mean= (TH1F*)strun->Get(h_name);
        sprintf(h_name,"h_sigma%d;%d",link,module+1);
        TH1F *h_sigma= (TH1F*)strun->Get(h_name);

        if(h_mean){ // fitted data exists
          if(h_mean->GetMean()>0){
            printf("Filling arrays for scan %d charge %f module %d link %d\n",
              scan[i], charge[i], module, link);
            module_status[module][link] = 1;
 
            for(chan=0;chan<768;chan++){
              code[module][link][chan][i] = h_code->GetBinContent(chan);
              mean[module][link][chan][i] = h_mean->GetBinContent(chan);
              sigma[module][link][chan][i]= h_sigma->GetBinContent(chan);
            } 
          }
        }
      } // end for(link)  
    }

    if(i<(n_points-1)){
      strun->Close();
      delete strun;
    }
  } // end for(module)


  //
  // 3. Calculate the arithmetic mean vt50 and outnse values for each chip
  //    and form the response curve graphs
  //

  TGraphErrors* gr_rc2[MAX_MODULES][MAX_LINKS][MAX_CHIPS];
  TGraph* gr_gain[MAX_MODULES][MAX_LINKS][MAX_CHIPS];
  TGraphErrors* gr_outnse[MAX_MODULES][MAX_LINKS][MAX_CHIPS];
  TGraphErrors* gr_innse[MAX_MODULES][MAX_LINKS][MAX_CHIPS];

  // initialise pointers to be zero

  for(module=0;module<MAX_MODULES;module++){
    for(link=0;link<MAX_LINKS;link++){
      for(chip=0;chip<MAX_CHIPS;chip++){
        gr_rc2[module][link][chip]=0;
        gr_gain[module][link][chip]=0;
        gr_outnse[module][link][chip]=0;
        gr_innse[module][link][chip]=0;
      }
    }
  }

  printf("Calculating Chip Averages....\n");

  for(module=0;module<MAX_MODULES;module++){
    if(module_status>0){
      for(link=0;link<MAX_LINKS;link++){
        for(chip=0;chip<MAX_CHIPS;chip++){

          // 3.1 Calculation of averages

          for(i=0;i<n_points;i++){

	    // Sum to calculate mean
            for(chan=(chip*128);chan<((chip+1)*128);chan++){

              if((code[module][link][chan][i]&BAD_TEST)==0){ // if (fit is good)
                ngood[module][link][chan]++;

                ch_ngood[module][link][chip][i]++;
                ch_mean[module][link][chip][i]+=mean[module][link][chan][i];
                ch_sigma[module][link][chip][i]+=sigma[module][link][chan][i];
              }
            }
	    // Calculate average
            if(ch_ngood[module][link][chip][i]>0){
              ch_mean[module][link][chip][i]/=ch_ngood[module][link][chip][i];
              ch_sigma[module][link][chip][i]/=ch_ngood[module][link][chip][i];
            }

	    // Sum to calculate RMS
            for(chan=(chip*128);chan<((chip+1)*128);chan++){

              if((code[module][link][chan][i]&BAD_TEST)==0){ // if (fit is good)

                ch_mean_err[module][link][chip][i]+=
		  (float)TMath::Power((ch_mean[module][link][chip][i]-mean[module][link][chan][i]),2);

                ch_sigma_err[module][link][chip][i]+=
		  (float)TMath::Power((ch_sigma[module][link][chip][i]-sigma[module][link][chan][i]),2);
              }
            }
	    // Calculate RMS
            if(ch_ngood[module][link][chip][i]>0){
              ch_mean_err[module][link][chip][i]=
		(float)TMath::Sqrt(ch_mean_err[module][link][chip][i]/ch_ngood[module][link][chip][i]);
	      ch_sigma_err[module][link][chip][i]=
		(float)TMath::Sqrt(ch_sigma_err[module][link][chip][i]/ch_ngood[module][link][chip][i]);
            }
          } // end for(i<n_points)

          // 3.2 Formation of response curve graphs
        
          gr_rc2[module][link][chip]= new TGraphErrors(n_points,charge,ch_mean[module][link][chip],
						       charge_err,ch_mean_err[module][link][chip]);
          gr_rc2[module][link][chip]->SetMarkerColor(kBlue);
          gr_outnse[module][link][chip]= new TGraphErrors(n_points,charge,ch_sigma[module][link][chip],
							  charge_err,ch_sigma_err[module][link][chip]);
          gr_outnse[module][link][chip]->SetMarkerColor(kBlue);
        }
      }
    }
  }
  printf(" done!\n");


  //
  // 4. Fit a response curve to the set of vt50 data for each channel
  //


  TH1F* h_vt50[MAX_MODULES][MAX_LINKS];
  TH1F* h_gain[MAX_MODULES][MAX_LINKS];
  TH1F* h_offset[MAX_MODULES][MAX_LINKS];
  TH1F* h_innse[MAX_MODULES][MAX_LINKS];

  TH1F* hp_vt50[MAX_MODULES][MAX_LINKS][MAX_CHIPS];
  TH1F* hp_gain[MAX_MODULES][MAX_LINKS][MAX_CHIPS];
  TH1F* hp_offset[MAX_MODULES][MAX_LINKS][MAX_CHIPS];
  TH1F* hp_innse[MAX_MODULES][MAX_LINKS][MAX_CHIPS];

  // initialise pointers to be zero

  for(module=0;module<MAX_MODULES;module++){
    for(link=0;link<MAX_LINKS;link++){
      for(chip=0;chip<MAX_CHIPS;chip++){
        hp_vt50[module][link][chip]=0;
        hp_gain[module][link][chip]=0;
        hp_offset[module][link][chip]=0;
        hp_innse[module][link][chip]=0;
      }

      h_vt50[module][link]=0;
      h_gain[module][link]=0;
      h_offset[module][link]=0;
      h_innse[module][link]=0;
    }
  }


  // 4.1 Select the fit function

  TF1* rc[MAX_MODULES][MAX_LINKS][MAX_CHANS];

  for(module=0;module<MAX_MODULES;module++){
    for(link=0;link<MAX_LINKS;link++){
      for(chip=0;chip<MAX_CHIPS;chip++){
        sprintf(h_name,"rc_%d_%d_%d",module,link,chip);
        switch(fit_type){
          case 4:   // linear 
            rc[module][link][chip] = new TF1(h_name,"pol1",0,8);
            sprintf(rc_name,"linear fit");
            break;
          case 3:   // exponential
            rc[module][link][chip] = new TF1(h_name,st_exp,0,8,3);
            sprintf(rc_name,"exponential fit");
            break;
          case 2:   // grillo
            rc[module][link][chip] = new TF1(h_name,grillo,0,8,3);
            sprintf(rc_name,"grillo fit");
            break;
          default:  // polynomial
            rc[module][link][chip] = new TF1(h_name,"pol2",0,8);
            sprintf(rc_name,"Second order polynomial fit");
            break;
        }
      }
    }
  }

  // 4.2 Fit a response curve to each channel

  for (module=0; module<MAX_MODULES; module++){
    for(link=0;link<MAX_LINKS;link++){

      // 4.2.1 Create histograms to hold gain, offset and input noise

      sprintf(h_name, "h_vt50_%d_%d",module,link);
      sprintf(h_title,"Vt50 at %2.2ffC",charge[special_point]);
      h_vt50[module][link] = new TH1F(h_name,h_title,768,0,768);
      h_vt50[module][link]->SetXTitle("Channel Number");
      h_vt50[module][link]->SetYTitle("Vt50 (mV");

      sprintf(h_name, "h_gain_%d_%d",module,link);
      sprintf(h_title,"Gain at %2.2ffC",charge[special_point]);
      h_gain[module][link] = new TH1F(h_name,h_title,768,0,768);
      h_gain[module][link]->SetXTitle("Channel Number");
      h_gain[module][link]->SetYTitle("Gain (mV/fC)");

      sprintf(h_name, "h_offset_%d_%d",module,link);
      sprintf(h_title,"Extrapolated Offset");
      h_offset[module][link] = new TH1F(h_name,h_title,768,0,768);
      h_offset[module][link]->SetXTitle("Channel Number");
      h_offset[module][link]->SetYTitle("Extrapolated Offset (mV)");

      sprintf(h_name, "h_innse_%d_%d",module,link);
      sprintf(h_title,"Input Noise at %2.2ffC",charge[special_point]);
      h_innse[module][link] = new TH1F(h_name,h_title,768,0,768);
      h_innse[module][link]->SetXTitle("Channel Number");
      h_innse[module][link]->SetYTitle("Input Noise (ENC)");

      for(chip=0;chip<6;chip++){
        Float_t xmin = ch_mean[module][link][chip][special_point]-50;
        Float_t xmax = ch_mean[module][link][chip][special_point]+50;

        sprintf(h_name, "h_vt50_%d_%d_%d",module,link,chip);
        sprintf(h_title,"Chip %d Vt50 at %2.2ffC",chip,charge[special_point]);
        hp_vt50[module][link][chip] = new TH1F(h_name,h_title,200,xmin,xmax);
        hp_vt50[module][link][chip]->SetXTitle("Vt50 (mV)");
        hp_vt50[module][link][chip]->SetYTitle("n(Channels)");
        hp_vt50[module][link][chip]->SetLineColor(kBlue);
        hp_vt50[module][link][chip]->SetMarkerColor(kBlue);

        sprintf(h_name, "h_gain_%d_%d_%d",module,link,chip);
        sprintf(h_title,"Chip %d Gain at %2.2ffC",chip,charge[special_point]);
        hp_gain[module][link][chip] = new TH1F(h_name,h_title,200,0,100);
        hp_gain[module][link][chip]->SetXTitle("Gain (mV/fC)");
        hp_gain[module][link][chip]->SetYTitle("n(Channels)");
        hp_gain[module][link][chip]->SetLineColor(kBlue);
        hp_gain[module][link][chip]->SetMarkerColor(kBlue);

        sprintf(h_name, "h_offset_%d_%d_%d",module,link,chip);
        sprintf(h_title,"Chip %d Extrapolated Offset",chip);
        hp_offset[module][link][chip] = new TH1F(h_name,h_title,100,0,100);
        hp_offset[module][link][chip]->SetXTitle("Extrapolated Offset (mV)");
        hp_offset[module][link][chip]->SetYTitle("n(Channels)");
        hp_offset[module][link][chip]->SetLineColor(kBlue);
        hp_offset[module][link][chip]->SetMarkerColor(kBlue);

        sprintf(h_name, "h_innse_%d_%d_%d",module,link,chip);
        sprintf(h_title,"Chip %d Input Noise at %2.2ffC",chip,charge[special_point]);
        hp_innse[module][link][chip] = new TH1F(h_name,h_title,75,0,2500);
        hp_innse[module][link][chip]->SetXTitle("Input Noise (ENC)");
        hp_innse[module][link][chip]->SetYTitle("n(Channels)");
        hp_innse[module][link][chip]->SetLineColor(kBlue);
        hp_innse[module][link][chip]->SetMarkerColor(kBlue);
      }


      if(module_status[module][link]>0){
        printf("  module %d link %d\n",module,link);

        // 4.2.2 Loop over channels of each module for which we have data

        for(chan=0;chan<MAX_CHANS;chan++){
          chip=chan/128;

          // Revised PWP 12.10.00
          // Exclude bad vt50 points (fit errors) from the RC

          Float_t lx[MAX_RCSCANS];
          Float_t ly[MAX_RCSCANS];

          Int_t lp = 0;
          for(i=0;i<n_points;i++){
            if((code[module][link][chan][i]&BAD_TEST)==0){
              lx[lp]=charge[i];
              ly[lp]=mean[module][link][chan][i];
              lp++;
            }
          }

          // If enough vt50 points are good, fit the response curve

          if(lp>1){ // at least 2 good points

            // Set initial guesses

            switch(fit_type){
              case 3:   // exponential
                rc[module][link][0]->SetParameters(900,6,-400);
                break;
              case 2:   // grillo
                rc[module][link][0]->SetParameters(70,40,600);
                break;
              default:  // polynomial
                rc[module][link][0]->SetParameters(70,50,0);
                break;
            }

            sprintf(h_name,"rc_%d_%d_%d",module,link,0);
            TGraph* gr_rc = new TGraph(lp,lx,ly);
            gr_rc->Fit(h_name,"NRQ");

            Double_t the_rc[3];
            the_rc[0]=(double)rc[module][link][0]->GetParameter(0);
            the_rc[1]=(double)rc[module][link][0]->GetParameter(1);
            the_rc[2]=(double)rc[module][link][0]->GetParameter(2); 

            // Calculate gain and input noise as f(charge)

            Float_t the_gain = 0;
            Float_t the_innse = 0;
            Float_t the_offset = rc[module][link][0]->Eval(0);

            switch(fit_type){
              case 3:   // exponential
                if(the_rc[1]>0){
                  the_gain = (the_rc[0]*exp(-charge[special_point]/the_rc[1]))/
                    (the_rc[1] * pow( (1.0+(exp(-charge[special_point]/the_rc[1]))),2));
                }
                break;
              case 2:   // grillo
                Float_t d1 = 1.0 +  pow( (the_rc[2]/(the_rc[1]*charge[special_point])),2);
                Float_t d2 = pow(the_rc[1],2) * pow(charge[special_point],3) * pow(d1,1.5);
                if(d2>0) the_gain= pow(the_rc[2],3)/d2;
                break;
              default:  // polynomial
                the_gain =  the_rc[1]+((double)charge[special_point]*(double)2.0*(double)the_rc[2]);
                break;
            }

            if(the_gain>0){
              the_innse = 6250 * sigma[module][link][chan][special_point]/the_gain;
            } else {
              the_innse = 0;
            }

            h_vt50[module][link]->Fill(chan,mean[module][link][chan][special_point]);
            hp_vt50[module][link][chip]->Fill(mean[module][link][chan][special_point]);
            h_gain[module][link]->Fill(chan,the_gain);
            hp_gain[module][link][chip]->Fill(the_gain);
            h_offset[module][link]->Fill(chan,the_offset);
            hp_offset[module][link][chip]->Fill(the_offset);
            h_innse[module][link]->Fill(chan,the_innse);
            hp_innse[module][link][chip]->Fill(the_innse);

            gr_rc->Delete();
          } // end if(lp>1)
        } // end for(channel)

        // The bad channel identification code was previously part of the above channel loop.
        // This meant that rms cuts were not constant across each chip, and depended upon the
        // number of channels of that chip which had been processed!
        // I have now corrected this.  PWP 23.09.02

        Float_t logain_cut = 0;
        Float_t higain_cut = 0;
        Float_t innse_cut = 0;

        for(chip=0;chip<6;chip++){
          // Extract mean and RMS for gain, noise and offset.

          gain_mean[module][link][chip] = hp_gain[module][link][chip]->GetMean();
          gain_rms[module][link][chip]  = hp_gain[module][link][chip]->GetRMS();
          vt50_mean[module][link][chip] = hp_vt50[module][link][chip]->GetMean();
          vt50_rms[module][link][chip]  = hp_vt50[module][link][chip]->GetRMS();
          offset_mean[module][link][chip] = hp_offset[module][link][chip]->GetMean();
          offset_rms[module][link][chip]  = hp_offset[module][link][chip]->GetRMS();
          innse_mean[module][link][chip] = hp_innse[module][link][chip]->GetMean();
          innse_rms[module][link][chip]  = hp_innse[module][link][chip]->GetRMS();

          logain_cut = 0.75 * gain_mean[module][link][chip];
          higain_cut = 1.25 * gain_mean[module][link][chip];

          innse_cut = innse_mean[module][link][chip] + 3 * innse_rms[module][link][chip];

          for(chan=(chip*128);chan<((chip+1)*128);chan++){

            // Identify bad channels

            the_code   = code[module][link][chan][special_point];
            the_gain   = h_gain[module][link]->GetBinContent(chan+1),
            the_offset = h_offset[module][link]->GetBinContent(chan+1),
            the_innse  = h_innse[module][link]->GetBinContent(chan+1),

            if(the_gain<logain_cut)        the_code+=CH_LO_GAIN;
            else if(the_gain>higain_cut)   the_code+=CH_HI_GAIN;

            if(the_offset<MIN_OFFSET)      the_code+=CH_LO_OFFSET;
            else if(the_offset>MAX_OFFSET) the_code+=CH_HI_OFFSET;

            if(the_innse<BONDED_NOISE)   the_code+=CH_UNBONDED;
            else if(the_innse<MIN_INNSE) the_code+=CH_PARTBONDED;
            else if(the_innse>innse_cut) the_code+=CH_HI_NOISE;

            code[module][link][chan][special_point] = the_code;
          } // end for(chan)

        } // end for(chip)

      } // end if(module_status)
    } // end for(link)  
  } // end for(module)


  //
  // 5. Bad channel totals
  //

  for(module=0;module<MAX_MODULES;module++){
    for(link=0;link<MAX_LINKS;link++){
      for(chan=0;chan<MAX_CHANS;chan++){

        chip = chan/128;
        the_code=code[module][link][chan][special_point];
        if(the_code>0){
          if (the_code&CH_LOST)        ch_bad[module][link][chip][LOST]++;
          if (the_code&CH_DODGY)       ch_bad[module][link][chip][DODGY]++;

          if (the_code&CH_DEAD)        ch_bad[module][link][chip][DEAD]++;
          if (the_code&CH_STUCK)       ch_bad[module][link][chip][STUCK]++;
          if (the_code&CH_INEFFICIENT) ch_bad[module][link][chip][INEFFICIENT]++;
          if (the_code&CH_UNBONDED)    ch_bad[module][link][chip][UNBONDED]++;
          if (the_code&CH_LO_GAIN)     ch_bad[module][link][chip][LO_GAIN]++;
          if (the_code&CH_HI_GAIN)     ch_bad[module][link][chip][HI_GAIN]++;
          if (the_code&CH_LO_OFFSET)   ch_bad[module][link][chip][LO_OFFSET]++;
          if (the_code&CH_HI_OFFSET)   ch_bad[module][link][chip][HI_OFFSET]++;
          if (the_code&CH_PARTBONDED)  ch_bad[module][link][chip][PARTBONDED]++;
          if (the_code&CH_HI_NOISE)    ch_bad[module][link][chip][HI_NOISE]++;
        }
      }
    }

    for(link=0;link<MAX_LINKS;link++){
      for(chip=0;chip<MAX_CHIPS;chip++){
        for(i=0;i<MAX_CODES;i++){
          lk_bad[module][link][i]+= ch_bad[module][link][chip][i];
          lk_bad[module][2][i]+= ch_bad[module][link][chip][i];
        }
      }
    }
  }


  //
  // 6. Plot the Results
  //

  Int_t ps_page = 0;
  Int_t screen_page = 0;

  TCanvas* c2= new TCanvas ("c2","RC",1100,750);

  TH2F* hf_rc[MAX_LINKS][MAX_CHIPS];
  TH2F* hf_gain[MAX_LINKS][MAX_CHIPS];
  TH2F* hf_outnse[MAX_LINKS][MAX_CHIPS];
  TH2F* hf_innse[MAX_LINKS][MAX_CHIPS];

  TPostScript* ps;

  // Create the frame histograms once and once only

  for(link=0;link<MAX_LINKS;link++){
    for(chip=0;chip<6;chip++){
      sprintf(h_name, "hf_rc_%d_%d",link,chip);
      sprintf(h_title,"Chip %d Response Curve",chip);
      hf_rc[link][chip] = new TH2F(h_name,h_title,2,0,8,2,0,640);
      hf_rc[link][chip]->SetXTitle("Charge(fC)");
      hf_rc[link][chip]->SetYTitle("vt50(mV)");
      hf_rc[link][chip]->SetTitleOffset(1.5,"X");
      hf_rc[link][chip]->SetTitleOffset(1.5,"Y");

      sprintf(h_name, "hf_gain_%d_%d",link,chip);
      sprintf(h_title,"Chip %d Gain",chip);
      hf_gain[link][chip] = new TH2F(h_name,h_title,2,0,8,2,0,100);
      hf_gain[link][chip]->SetXTitle("Charge(fC)");
      hf_gain[link][chip]->SetYTitle("Gain(mV/fC)");
      hf_gain[link][chip]->SetTitleOffset(1.5,"X");
      hf_gain[link][chip]->SetTitleOffset(1.5,"Y");

      sprintf(h_name, "hf_outnse_%d_%d",link,chip);
      sprintf(h_title,"Chip %d Output Noise",chip);
      hf_outnse[link][chip] = new TH2F(h_name,h_title,2,0,8,2,0,30);
      hf_outnse[link][chip]->SetXTitle("Charge(fC)");
      hf_outnse[link][chip]->SetYTitle("Output Noise(mV)");
      hf_outnse[link][chip]->SetTitleOffset(1.5,"X");
      hf_outnse[link][chip]->SetTitleOffset(1.5,"Y");

      sprintf(h_name, "hf_innse_%d_%d",link,chip);
      sprintf(h_title,"Chip %d Input Noise",chip);
      hf_innse[link][chip] = new TH2F(h_name,h_title,2,0,8,2,0,2500);
      hf_innse[link][chip]->SetXTitle("Charge(fC)");
      hf_innse[link][chip]->SetYTitle("Input Noise(ENC)");
      hf_innse[link][chip]->SetTitleOffset(1.5,"X");
      hf_innse[link][chip]->SetTitleOffset(1.5,"Y");
    }
  }

  for(module=0;module<MAX_MODULES;module++){  // loop over modules
    if(module_status[module][0]>0){
      if(postscript){
        ps_page=0;
        sprintf(f_name,"%s%s_RCPlot_%d_%06d.ps",PS_DIRECTORY,mname[module],date,the_time);

        ps = new TPostScript(f_name,112);
      }

      for(link=0;link<MAX_LINKS;link++){   // loop over links


        if((screen_page>0)&&(dowait)){
          printf("Enter a string to continue\n");
          scanf("%s",line);
        }

        ps_page++;
        screen_page++;
        if(ps){
          ps->NewPage();
        }

        c2->cd();
        c2->Clear();

        TPad *pad_text   = new TPad("pad_text", "Text",        0.00,0.90,1.00,1.00,33);
        TPad *pad_plot   = new TPad("pad_plot", "Plots",       0.00,0.00,0.60,0.90,33);
        TPad *pad_proj   = new TPad("pad_proj", "Projections", 0.60,0.00,1.00,0.90,33);

        pad_text->SetBit(kCanDelete);
        pad_plot->SetBit(kCanDelete);
        pad_proj->SetBit(kCanDelete);

        pad_text->Draw();
        pad_plot->Draw();
        pad_plot->Divide(1,4,0,0);
        pad_proj->Draw();
        pad_proj->Divide(3,8,0,0);

        c2->Update();
        pad_text->cd();

        TPaveText *t1 = new TPaveText(0.0,0.0,1.0,1.0);
        t1->SetBit(kCanDelete);

        t1->SetFillColor(10);
        t1->SetTextAlign(13);
        t1->SetTextSize(0.3);
        t1->SetTextColor(4);

        // ATLAS SCT Module Test:  Response vs. Channel  -  time  -  place
        sprintf(line,"ATLAS SCT Module Test:  Response vs. Channel");
        if(time)  sprintf(line,"%s  -  %s",line,time->GetName());
        if(place) sprintf(line,"%s  -  %s",line,place->GetName());
        t1->AddText(line);

        // Page %d   Run %d   Scan %d   Module %d (%s) - Type %s
        sprintf(line,"Page %d   Run %d   Start Scan %d   Stream %d   Module %d (%s)", 
                ps_page, run, scan[0], link, module, mname[module]);
        if(strlen(devtype[module]) > 0) 
          sprintf(line, "%s - Type %s", line, devtype[module]);
        t1->AddText(line);
        t1->Draw();

        pad_plot->cd(1);
	h_vt50[module][link]->SetMaximum(250);
	h_vt50[module][link]->SetMinimum(0);
        h_vt50[module][link]->Draw();
        pad_plot->cd(2);
	h_gain[module][link]->SetMaximum(70);
	h_gain[module][link]->SetMinimum(20);
        h_gain[module][link]->Draw();
        pad_plot->cd(3);
	h_offset[module][link]->SetMaximum(150);
	h_offset[module][link]->SetMinimum(0);
        h_offset[module][link]->Draw();
        pad_plot->cd(4);
	h_innse[module][link]->SetMaximum(3200);
	h_innse[module][link]->SetMinimum(200);
        h_innse[module][link]->Draw();

        for(chip=0;chip<6;chip++){
          pad_proj->cd(1+chip);
          hp_vt50[module][link][chip]->Draw();
          hp_vt50[module][link][chip]->Fit("gaus","LQ");
          pad_proj->cd(7+chip);
          hp_gain[module][link][chip]->Draw();
          hp_gain[module][link][chip]->Fit("gaus","LQ");
          pad_proj->cd(13+chip);
          hp_offset[module][link][chip]->Draw();
          hp_offset[module][link][chip]->Fit("gaus","LQ");
          pad_proj->cd(19+chip);
          hp_innse[module][link][chip]->Draw();
          hp_innse[module][link][chip]->Fit("gaus","LQ");
        }

        c2->Update();

        if(dowait){
          printf("Enter a string to continue\n");
          scanf("%s",line);
        }

        ps_page++;
        screen_page++;
        if(ps){
          ps->NewPage();
        }

        c2->cd();
        c2->Clear();

        pad_text   = new TPad("pad_text", "Text",        0.00,0.90,1.00,1.00,33);
        pad_proj   = new TPad("pad_proj", "Projections", 0.00,0.00,1.00,0.90,33);

        pad_text->SetBit(kCanDelete);
        pad_proj->SetBit(kCanDelete);

        pad_text->Draw();
        pad_proj->Draw();
        pad_proj->Divide(6,4,0,0);

        for(i=1;i<25;i++){
          pad_proj->cd(i);
          gPad->SetLeftMargin(0.15);
          gPad->SetBottomMargin(0.15);
        }

        c2->Update();
        pad_text->cd();

        t1 = new TPaveText(0.0,0.0,1.0,1.0);
        t1->SetBit(kCanDelete);

        t1->SetFillColor(10);
        t1->SetTextAlign(13);
        t1->SetTextSize(0.3);
        t1->SetTextColor(4);

        // ATLAS SCT Module Test:  Response Curve  -  time  -  place
        sprintf(line,"ATLAS SCT Module Test:  Response Curve");
        if(time)  sprintf(line,"%s  -  %s",line,time->GetName());
        if(place) sprintf(line,"%s  -  %s",line,place->GetName());
        t1->AddText(line);

        // Page %d   Run %d   Scan %d   Module %d (%s) - Type %s
        sprintf(line,"Page %d   Run %d   Start Scan %d   Stream %d   Module %d (%s)", 
                ps_page, run, scan[0], link, module, mname[module]);
        if(strlen(devtype[module]) > 0) 
          sprintf(line, "%s - Type %s", line, devtype[module]);
        t1->AddText(line);
        t1->Draw();

        for(chip=0;chip<6;chip++){
          
          pad_proj->cd(1+chip);
          hf_rc[link][chip]->Draw(); 
          gr_rc2[module][link][chip]->Draw("P");

          // Set initial guesses

          switch(fit_type){
            case 3:   // exponential
              rc[module][link][chip]->SetParameters(900,6,-400);
              break;
            case 2:   // grillo
              rc[module][link][chip]->SetParameters(70,40,600);
              break;
            default:  // polynomial
              rc[module][link][chip]->SetParameters(70,50,0);
              break;
          }

          sprintf(h_name,"rc_%d_%d_%d",module,link,chip);
          gr_rc2[module][link][chip]->Fit(h_name,"NRQ");
          Double_t* the_rc2 = ch_rcfit[module][link][chip];
          the_rc2[0]=rc[module][link][chip]->GetParameter(0);
          the_rc2[1]=rc[module][link][chip]->GetParameter(1);
          the_rc2[2]=rc[module][link][chip]->GetParameter(2); 
          rc[module][link][chip]->Draw("same");

          ch_offset[module][link][chip] = rc[module][link][chip]->Eval(0);

          // Calculate gain and input noise as f(charge)

          for(i=0;i<n_points;i++){
            switch(fit_type){
              case 3:   // exponential
                if(the_rc2[1]>0){
                  ch_gain[module][link][chip][i]= (the_rc2[0]*exp(-charge[i]/the_rc2[1]))/
                    (the_rc2[1] * pow( (1.0+(exp(-charge[i]/the_rc2[1]))),2));
                }
                break;
              case 2:   // grillo
                Float_t d1 = 1.0 +  pow( (the_rc[2]/(the_rc2[1]*charge[i])),2);
                Float_t d2 = pow(the_rc2[1],2) * pow(charge[i],3) * pow(d1,1.5);
                if(d2>0) ch_gain[module][link][chip][i]= pow(the_rc2[2],3)/d2;
                break;
              default:  // polynomial
                ch_gain[module][link][chip][i]=  the_rc2[1]+(charge[i]*2*the_rc2[2]);
                break;
            }
            if(ch_gain[module][link][chip][i]>0){
              ch_innse[module][link][chip][i]=6250.0 * ch_sigma[module][link][chip][i]
                                                     / ch_gain[module][link][chip][i];
              ch_innse_err[module][link][chip][i]=6250.0 * ch_sigma_err[module][link][chip][i]
		                                     / ch_gain[module][link][chip][i];
            }

          } // end for(i=0;i<n_points
	  
          pad_proj->cd(7+chip);
          hf_gain[link][chip]->Draw();
          gr_gain[module][link][chip]=new TGraph(n_points,charge,ch_gain[module][link][chip]);
          gr_gain[module][link][chip]->SetMarkerColor(kBlue);
          gr_gain[module][link][chip]->Draw("P");

          pad_proj->cd(13+chip);
          hf_outnse[link][chip]->Draw(); 
          gr_outnse[module][link][chip]->Draw("P");

          pad_proj->cd(19+chip);
          hf_innse[link][chip]->Draw(); 
          gr_innse[module][link][chip]=new TGraphErrors(n_points,charge,ch_innse[module][link][chip],
							charge_err,ch_innse_err[module][link][chip]);
          gr_innse[module][link][chip]->SetMarkerColor(kBlue);
          gr_innse[module][link][chip]->Draw("P");
        }

        c2->Update();

      } // end for(link)
    } // end if(module_status)
  } // end for(module)

  if(ps) {
    ps->Close();
    delete ps;
  }

  if(time) delete time;
  if(place) delete place;

  //
  // Before opening the DB results file, write a seperate text file 
  // for each module containing gain/offset/noise values as requested 
  // by Janet and Nobu. PWP 04.10.01
  //

  for(module=0;module<MAX_MODULES;module++){  // loop over modules
    if(module_status[module][0]>0){

      FILE* outfile = OpenRaw("RC", strun, module, run, scan[0]); 

      if(outfile){
        fprintf(outfile,"#chan\tcode\tgain\toffset\tinnse\tcomment\n");

        for(link=0;link<MAX_LINKS;link++){
          for(chan=0;chan<MAX_CHANS;chan++){
   
            the_code  = code[module][link][chan][special_point];
    
            // Generate comment appropriate to this channel
  
            sprintf(comment,"unknown");

            if      (the_code==0)             sprintf(comment,"OK");
            else if (the_code&CH_DEAD)        sprintf(comment,"dead");
            else if (the_code&CH_STUCK)       sprintf(comment,"stuck"); 
            else if (the_code&CH_LO_GAIN)     sprintf(comment,"low gain");
            else if (the_code&CH_HI_GAIN)     sprintf(comment,"high gain");
            else if (the_code&CH_LO_OFFSET)   sprintf(comment,"low offset");
            else if (the_code&CH_HI_OFFSET)   sprintf(comment,"high offset");
            else if (the_code&CH_UNBONDED)    sprintf(comment,"unbonded");
            else if (the_code&CH_PARTBONDED)  sprintf(comment,"partbonded");
            else if (the_code&CH_HI_NOISE)    sprintf(comment,"high noise");
            else if (the_code&CH_INEFFICIENT) sprintf(comment,"inefficient");

            fprintf(outfile,"%4d\t%4hx\t%2.1f\t%2.1f\t%4.0f\t%s\n",
                    (768*link)+chan,
                    code[module][link][chan][special_point],
                    h_gain[module][link]->GetBinContent(chan+1),
                    h_offset[module][link]->GetBinContent(chan+1),
                    h_innse[module][link]->GetBinContent(chan+1),
                    comment);
          }
        }
        fclose(outfile);
      }
    } // end if(module_status)
  } // end for(module)


  //
  // 7. Write the results to an ascii file, one per module, suited to the database
  //



  for(module=0;module<MAX_MODULES;module++){
    if(module_status[module][0]>0){

	  // 7.1 Classify chip by noise, gain etc. Added LE 31/5 2001

      for(link=0;link<MAX_LINKS;link++){
	for(chip=0;chip<MAX_CHIPS;chip++){
	  the_code=CH_DEAD; // all channels dead
	  for(chan=(chip*128);chan<((chip+1)*128);chan++){
	    if((code[module][link][chan][special_point]&CH_DEAD)==0) the_code=0;
	  }
	  if((the_code&CH_DEAD)==1) ch_code[module][link][chip]+=CH_DEAD;

	  the_code=CH_STUCK; // all channels stuck
	  for(chan=(chip*128);chan<((chip+1)*128);chan++){
	    if((code[module][link][chan][special_point]&CH_STUCK)==0) the_code=0;
	  }
	  if((the_code&CH_STUCK)==1) ch_code[module][link][chip]+=CH_STUCK;

	  the_code=CH_INEFFICIENT; // all channels inefficient
	  for(chan=(chip*128);chan<((chip+1)*128);chan++){
	    if((code[module][link][chan][special_point]&CH_INEFFICIENT)==0) the_code=0;
	  }
	  if((the_code&CH_INEFFICIENT)==1) ch_code[module][link][chip]+=CH_INEFFICIENT;

	  if(ch_gain[module][link][chip][special_point]<MIN_GAIN){
	    ch_code[module][link][chip]+=CH_LO_GAIN;
	  }else if(ch_gain[module][link][chip][special_point]>MAX_GAIN){   
	    ch_code[module][link][chip]+=CH_HI_GAIN;
	  }
	  
	  if(ch_offset[module][link][chip]<MIN_OFFSET){
	    ch_code[module][link][chip]+=CH_LO_OFFSET;
	  }else if(ch_offset[module][link][chip]>MAX_OFFSET){
	    ch_code[module][link][chip]+=CH_HI_OFFSET;
	  }
	  
	  if(ch_innse[module][link][chip][special_point]<BONDED_NOISE){
	    ch_code[module][link][chip]+=CH_UNBONDED;
	  }else if(ch_innse[module][link][chip][special_point]<MIN_INNSE){
	    ch_code[module][link][chip]+=CH_PARTBONDED;
	  }else if(ch_innse[module][link][chip][special_point]>MAX_INNSE){
	    ch_code[module][link][chip]+=CH_HI_NOISE;
	  }
	}
      }



      // Open today's output file
      // Output from a series of tests will be added to this file

      FILE* outfile = OpenResults("ResponseCurve", strun, module, run, scan[0], 1,0); 

      if(outfile){

        ListScanInfo(outfile,"QCAL (fC)",n_points,charge);

        fprintf(outfile,"#\n");
        if(n_points==3){
          fprintf(outfile,"%%ThreePointGain\n");
        }else{
          fprintf(outfile,"%%ResponseCurve\n");
        }

        fprintf(outfile,"#\n");
        fprintf(outfile,"#Loop A - Fit\n");
        fprintf(outfile,"#\tfunc\tp0\tp1\tp2\n");

        for(link=0;link<MAX_LINKS;link++){
          for(chip=0;chip<MAX_CHIPS;chip++){
            fprintf(outfile,"#%s\n",MseLabel(link,chip));
            fprintf(outfile,"\t%d \t%3.2f \t%3.2f \t%3.2f\n",
              fit_type,
              ch_rcfit[module][link][chip][0],
              ch_rcfit[module][link][chip][1],
              ch_rcfit[module][link][chip][2]);
          }
        }

        fprintf(outfile,"#\n");
        fprintf(outfile,"#Loop B - Gain, Offset, Noise at %2.2ffC\n",charge[special_point]);
        fprintf(outfile,"#\tvt50\trms\tgain\trms\toffset\trms\toutnse\tinnse\trms\n");

        for(link=0;link<MAX_LINKS;link++){
          for(chip=0;chip<MAX_CHIPS;chip++){
            fprintf(outfile,"#%s\n",MseLabel(link,chip));
            fprintf(outfile,
		    "\t%5.1f \t%5.2f \t%4.1f \t%4.2f \t%5.1f \t%5.2f \t%4.2f \t%5.0f \t%4.0f\n",
		     vt50_mean[module][link][chip],
		     vt50_rms[module][link][chip],
		     gain_mean[module][link][chip],
		     gain_rms[module][link][chip],
		     offset_mean[module][link][chip],
		     offset_rms[module][link][chip],
		     ch_sigma[module][link][chip][special_point],
		     innse_mean[module][link][chip],
		     innse_rms[module][link][chip]);
	  }
	}

        fprintf(outfile,"#\n");
        fprintf(outfile,"#Loop C - Comment\n");

        for(link=0;link<MAX_LINKS;link++){
          if(link==0){
            fprintf(outfile,"#M0 S1  S2  S3  S4  E5\n");
          }else{
            fprintf(outfile,"#M8 S9  S10 S11 S12 E13\n");
          }
          for(chip=0;chip<MAX_CHIPS;chip++){
	    sprintf(comment,"OK");
	    if     (ch_code[module][link][chip]&CH_DEAD)       sprintf(comment,"dead");
	    else if(ch_code[module][link][chip]&CH_STUCK)      sprintf(comment,"stuck"); 
	    else if(ch_code[module][link][chip]&CH_LO_GAIN)    sprintf(comment,"low gain");
	    else if(ch_code[module][link][chip]&CH_HI_GAIN)    sprintf(comment,"high gain");
	    else if(ch_code[module][link][chip]&CH_LO_OFFSET)  sprintf(comment,"low offset");
	    else if(ch_code[module][link][chip]&CH_HI_OFFSET)  sprintf(comment,"high offset");
	    else if(ch_code[module][link][chip]&CH_UNBONDED)   sprintf(comment,"unbonded");
	    else if(ch_code[module][link][chip]&CH_PARTBONDED) sprintf(comment,"partbonded");
	    else if(ch_code[module][link][chip]&CH_HI_NOISE)   sprintf(comment,"high noise");
	    else if(ch_code[module][link][chip]&CH_INEFFICIENT)sprintf(comment,"inefficient");
            fprintf(outfile,"\"%s\" \t",comment);
	  }
          fprintf(outfile,"\n");
	}

        fprintf(outfile,"#BadChannelSummary - not for the database\n");
        fprintf(outfile,"#  at %2.2ffC\n",charge[special_point]);
        fprintf(outfile,"#\t\tlost\tdodgy\tdead\tstuck\tineff");
        fprintf(outfile,"\tunbon\tlo_gn\thi_gn\tlo_off\thi_off\tpartbon\thi_nse\n");
  
        for(link=0;link<MAX_LINKS;link++){
          for(chip=0;chip<MAX_CHIPS;chip++){
             fprintf(outfile,"#Chip %2d:\t%3d\t%3d\t%3d\t%3d\t%3d\t%3d\t%3d\t%3d\t%3d\t%3d\t%3d\t%3d\n",
               (6*link)+chip,
               ch_bad[module][link][chip][LOST],
               ch_bad[module][link][chip][DODGY],
               ch_bad[module][link][chip][DEAD],
               ch_bad[module][link][chip][STUCK],
               ch_bad[module][link][chip][INEFFICIENT],
               ch_bad[module][link][chip][UNBONDED],
               ch_bad[module][link][chip][LO_GAIN],
               ch_bad[module][link][chip][HI_GAIN],
               ch_bad[module][link][chip][LO_OFFSET],
               ch_bad[module][link][chip][HI_OFFSET],
               ch_bad[module][link][chip][PARTBONDED],
               ch_bad[module][link][chip][HI_NOISE]);
          }
        }

        for(link=0;link<MAX_LINKS+1;link++){
           fprintf(outfile,"#Link %2d:\t%3d\t%3d\t%3d\t%3d\t%3d\t%3d\t%3d\t%3d\t%3d\t%3d\t%3d\t%3d\n",
             link,
             lk_bad[module][link][LOST],
             lk_bad[module][link][DODGY],
             lk_bad[module][link][DEAD],
             lk_bad[module][link][STUCK],
             lk_bad[module][link][INEFFICIENT],
             lk_bad[module][link][UNBONDED],
             lk_bad[module][link][LO_GAIN],
             lk_bad[module][link][HI_GAIN],
             lk_bad[module][link][LO_OFFSET],
             lk_bad[module][link][HI_OFFSET],
             lk_bad[module][link][PARTBONDED],
             lk_bad[module][link][HI_NOISE]);
        }


        // Now we list the defects in a manner compatible with the SCT databse.
        // All old options have now been removed.  PWP 14.12.01

        Int_t first, last;
        Int_t old_defect, new_defect;

        Int_t n_defects=0;
        for(link=0;link<MAX_LINKS;link++){

          first=-1;
          last=-1;
          old_defect=0;

          for(chan=0;chan<MAX_CHANS;chan++){

            new_defect = 0;
            the_code  = code[module][link][chan][special_point];
            chip_code = ch_code[module][link][chan/128];

            // Is there a defect on this strip?

            if(the_code&BAD_TEST){
              if(   ((the_code&CH_UNBONDED)   && (chip_code&CH_UNBONDED))
                 || ((the_code&CH_PARTBONDED) && (chip_code&CH_PARTBONDED)) ){

                  // We consider all bad channels to be defective EXCEPT 
                  // unbonded or part bonded channels 
                  // of unbonded or partbonded chips
                  //  - so this channel is not defective

              }else{ // This channel is defective 

                if(the_code&CH_DEAD){
                  new_defect=CH_DEAD;
                }else if(the_code&CH_STUCK){
                  new_defect=CH_STUCK;
                }else if(the_code&CH_LO_GAIN){
                  new_defect=CH_LO_GAIN;
                }else if(the_code&CH_HI_GAIN){
                  new_defect=CH_HI_GAIN;
                }else if(the_code&CH_LO_OFFSET){
                  new_defect=CH_LO_OFFSET;
                }else if(the_code&CH_HI_OFFSET){
                  new_defect=CH_HI_OFFSET;
                }else if(the_code&CH_UNBONDED){
                  new_defect=CH_UNBONDED;
                }else if(the_code&CH_PARTBONDED){
                  new_defect=CH_PARTBONDED;
                }else if(the_code&CH_HI_NOISE){
                  new_defect=CH_HI_NOISE;
                }else if(the_code&CH_INEFFICIENT){
                  new_defect=CH_INEFFICIENT;
                }
              }
            }

            // Is this a continued defect?
            if((old_defect)&&(new_defect==old_defect)){
              last = ((link*MAX_CHANS)+chan);
            }

            // Is this a new defect?
            if((new_defect)&&(old_defect==0)){
              old_defect = new_defect;
              first = ((link*MAX_CHANS)+chan);
              last = first;
            }

            // If a defect is associated with the previous strip and a defect
            // of a different type, or no defect at all, is associated with 
            // this strip then we must record the previous strip's defect

            if((old_defect)&&(new_defect!=old_defect)){

              switch(old_defect){
                case CH_DEAD:
                  ListDefect(outfile,"DEAD",first,last);
                  break;
                case CH_STUCK:
                  ListDefect(outfile,"STUCK",first,last);
                  break;
                case CH_LO_GAIN:
                  ListDefect(outfile,"LO_GAIN",first,last);
                  break;
                case CH_HI_GAIN:
                  ListDefect(outfile,"HI_GAIN",first,last);
                  break;
                case CH_LO_OFFSET:
                  ListDefect(outfile,"LO_OFFSET",first,last);
                  break;
                case CH_HI_OFFSET:
                  ListDefect(outfile,"HI_OFFSET",first,last);
                  break;
                case CH_UNBONDED:
                  ListDefect(outfile,"UNBONDED",first,last);
                  break;
                case CH_PARTBONDED:
                  ListDefect(outfile,"PARTBONDED",first,last);
                  break;
                case CH_HI_NOISE:
                  ListDefect(outfile,"NOISY",first,last);
                  break;
                case CH_INEFFICIENT:
                  ListDefect(outfile,"INEFFICIENT",first,last);
                  break;
                default:
                  break;
              }

              n_defects++;
              if(new_defect==0){ // no defect on this ek;trip
                old_defect = 0;
                first = -1;
                last = -1;
              }else{ // this strip has a defect of a different type
                old_defect = new_defect;
                first = ((link*MAX_CHANS)+chan);
                last = first;
              }
            }

          } // end for(chan)

          // Is there a defect on the last channel of this link?

          if(old_defect){

            switch(old_defect){
              case CH_DEAD:
                ListDefect(outfile,"DEAD",first,last);
                break;
              case CH_STUCK:
                ListDefect(outfile,"STUCK",first,last);
                break;
              case CH_LO_GAIN:
                ListDefect(outfile,"LO_GAIN",first,last);
                break;
              case CH_HI_GAIN:
                ListDefect(outfile,"HI_GAIN",first,last);
                break;
              case CH_LO_OFFSET:
                ListDefect(outfile,"LO_OFFSET",first,last);
                break;
              case CH_HI_OFFSET:
                ListDefect(outfile,"HI_OFFSET",first,last);
                break;
              case CH_UNBONDED:
                ListDefect(outfile,"UNBONDED",first,last);
                break;
              case CH_PARTBONDED:
                ListDefect(outfile,"PARTBONDED",first,last);
                break;
              case CH_HI_NOISE:
                ListDefect(outfile,"NOISY",first,last);
                break;
              case CH_INEFFICIENT:
                ListDefect(outfile,"INEFFICIENT",first,last);
                break;
            }
            n_defects++;

          } // end if(old_defect)

        } // end for(link)

        if(n_defects==0){
          fprintf(outfile,"#No defects found!\n");
        }else{
          fprintf(outfile,"#%d defects found\n",n_defects);
        }

        // And finally we add the link to the raw data file

        fprintf(outfile,"#\n");
        sprintf(line,"%s%s_RC_%d_%d.txt",RESULTS_DIRECTORY,mname[module],run,scan[0]);
        ListRawData(outfile,line);

        // And a URL to the postscript file

  #ifdef WEB_KEY        
        sprintf(f_name,"%s%s_RCPlot_%d_%06d.ps",WEB_KEY,mname[module],date,the_time);
        ListWebLink(outfile,"Plots (postscript)",f_name);
  #endif

        fclose(outfile);

        //
        // upload the RC data to ST
        //

        if(online){
          for(link=0;link<MAX_LINKS;link++){
            for(chip=0;chip<MAX_CHIPS;chip++){
              UploadRC(module,
                (6*link)+chip, 1, fit_type,
                ch_rcfit[module][link][chip][0],
                ch_rcfit[module][link][chip][1],
                ch_rcfit[module][link][chip][2]);
            }
          }
        }
      }
    } // end if(module_status)
  }

  //
  // 8. Delete lots of objects
  //

  printf("Deleting Graph and Function Objects\n");

  for(module=0;module<MAX_MODULES;module++){
    for(link=0;link<MAX_LINKS;link++){
      for(chip=0;chip<MAX_CHIPS;chip++){
        if(gr_rc2[module][link][chip])    gr_rc2[module][link][chip]->Delete();
        if(gr_gain[module][link][chip])   gr_gain[module][link][chip]->Delete();
        if(gr_outnse[module][link][chip]) gr_outnse[module][link][chip]->Delete();
        if(gr_innse[module][link][chip])  gr_innse[module][link][chip]->Delete();
        if(rc[module][link][chip])        rc[module][link][chip]->Delete();
      }
    }
  }

  printf("Deleting Frame Objects\n");

  for(link=0;link<MAX_LINKS;link++){
    for(chip=0;chip<MAX_CHIPS;chip++){
      if(hf_rc[link][chip])     hf_rc[link][chip]->Delete();
      if(hf_gain[link][chip])   hf_gain[link][chip]->Delete();
      if(hf_outnse[link][chip]) hf_outnse[link][chip]->Delete();
      if(hf_innse[link][chip])  hf_innse[link][chip]->Delete();
    }
  }

  printf("Deleting Histogram Objects\n");

  for(module=0;module<MAX_MODULES;module++){
    for(link=0;link<MAX_LINKS;link++){
      for(chip=0;chip<MAX_CHIPS;chip++){
        if(hp_vt50[module][link][chip])   hp_vt50[module][link][chip]->Delete();
        if(hp_gain[module][link][chip])   hp_gain[module][link][chip]->Delete();
        if(hp_offset[module][link][chip]) hp_offset[module][link][chip]->Delete();
        if(hp_innse[module][link][chip])  hp_innse[module][link][chip]->Delete();
      }

      if(h_vt50[module][link])   h_vt50[module][link]->Delete();
      if(h_gain[module][link])   h_gain[module][link]->Delete();
      if(h_offset[module][link]) h_offset[module][link]->Delete();
      if(h_innse[module][link])  h_innse[module][link]->Delete();

    }
  }
  if(c2) c2->Delete();
  if(strun) {
    strun->Close();
    delete strun;
  }
}

void No8fCCurvePlot(Int_t run, Int_t scan, Short_t fit_type){

  Int_t n_points = 9;
  Int_t special_point = 5; // 2fC
  Int_t scans[9];
  Float_t charges[9] = {0.50,0.75,1.00,1.25,1.50,
			 2.00,3.00,4.00,6.00};

  for(Int_t i=0;i<n_points;i++) scans[i] = i+scan;

  ResponseCurvePlot(run,n_points,special_point,scans,charges,fit_type,0,1,0);
}

void ResponseCurvePlot(Int_t run, Int_t scan, Short_t fit_type){

  Int_t n_points = 10;
  Int_t special_point = 5; // 2fC
  Int_t scans[10];
  Float_t charges[10] = {0.50,0.75,1.00,1.25,1.50,
			 2.00,3.00,4.00,6.00,8.00};

  for(Int_t i=0;i<n_points;i++) scans[i] = i+scan;

  ResponseCurvePlot(run,n_points,special_point,scans,charges,fit_type,0,1,0);
}

void ThreePointGainPlot(Int_t run, Int_t scan, Short_t fit_type){

  Int_t n_points = 3;
  Int_t special_point = 1; // 2fC
  Int_t scans[3];
  Float_t charges[3] = {1.50,2.00,2.50};

  for(Int_t i=0;i<n_points;i++) scans[i] = i+scan;

  ResponseCurvePlot(run,n_points,special_point,scans,charges,fit_type,0,1,0);
}

