void ResponseVsDelay(Float_t Qstart, Float_t Qstop, Float_t Qstep) { 

  // version 0.1 PWP 27.09.00
  // version 0.2 PWP 28.09.00
  // version 0.3 PWP 10.12.00
  // version 0.4 PWP 06.06.01 - use parameters.h
  // version 0.41 PWP 05.09.01 - small fix after Zdenek
  // version 0.42 BJG 30.04.02 - small fix

  #ifdef DATA_DIRECTORY
    // do not reload parameters.h
  #else
    // must load parameters.h
  #include <parameters.h>
  #endif

  Int_t delay;
  Float_t charge;
  Int_t module, chip;
  char f_name[128];
 
  // load the plot macro 
  gROOT->ProcessLine(".L stan_ntplot.cpp");
  gROOT->ProcessLine(".L stan_util.cpp");


  // Store settings before the scan
  Int_t do_fits   = e->do_fits;

  Float_t in_charge[MAX_MODULES][12];
  Int_t in_delay[MAX_MODULES][12];

  for (module=0; module<MAX_MODULES; module++){
    if(e->m[module]->present){
      for(chip=0;chip<12;chip++){
        in_charge[module][chip] = e->RetrieveVariable(module,chip,42);
        in_delay[module][chip] = e->RetrieveVariable(module,chip,3);
      }
    }
  }


  // integrated fitting must be turned on!!
  e->abort=0;
  e->do_fits = 1;

  Int_t start_scan = e->scannum+1;

  sprintf(f_name,"%sstnt_%d_%d.dat",DATA_DIRECTORY,e->runnum,start_scan);
  
  FILE* outfile = fopen(f_name,"w");
  if(outfile==NULL){
    printf("Could not open file for ascii ntuple of fit summary data\n - exiting");
    return;
  }

  // write ntuple column names to ascii file
  fprintf(outfile,"scan:module:chip:delay:charge:vt50:outnse:goodfits\n");

  for(delay=0;delay<=60;delay+=4){ 

    e->ConfigureVariable(3,delay);  // 3 - STROBE DELAY (bits)

    for(charge=Qstart;charge<=Qstop;charge+=Qstep){
    
      if(e->abort==0){
        e->ConfigureVariable(42,charge);  // 42 - QCAL (fC)
        e->ConfigureScan(1,0,350,5);

        st_scan(11,-1);

        for (module=0; module<e->GetMaxModules(); module++){
          if(e->m[module]->present){  // only build our ascii ntuple for present modules
  
            for(chip=0;chip<12;chip++){
              fprintf(outfile,"%d %d %d %f %f %f %f %d\n",
                e->scannum, 
                module,
                chip, 
                delay,
                charge,
                e->m[module]->chip_mean[chip/6][chip%6],
                e->m[module]->chip_sigma[chip/6][chip%6],
                e->m[module]->chip_goodfits[chip/6][chip%6]);
            } // end for(chip)  
          }
        } // end for(module)
      }

    } // end for(charge)
  }  // end for(delay)

  fclose(outfile);


  // Reset values after scan
  e->do_fits = do_fits;      

  for (module=0; module<MAX_MODULES; module++){
    if(e->m[module]->present){
      for(chip=0;chip<12;chip++){
        e->ConfigureVariable(module,chip,42,in_charge[module][chip]);
        e->ConfigureVariable(module,chip,3,in_delay[module][chip]);
      }
    }
  }

  // Now generate the root ntuple, plots and postscript file
  //
  // We will see vt50, output noise, n(good fits), 
  // gain (linear fit to all charges), offset and input noise
  // all plotted against var2_name ("delay") with
  // one line on the chart for each setting of var1_name ("charge")

  stan_ntplot(e->runnum,start_scan,"delay","charge");
} 

void ResponseVsDelay() 
{ 
  ResponseVsDelay(1.0,4.2,1.0);
}
