void SetStrobeDelay(Short_t mid,Short_t cid, Float_t var){

  //
  // Q. Why is this code tucked away in a subroutine?
  // A. because CINT objects to the appearance of a reference to the TST object
  //    (e->) even if it is in a section of the main code which is not called 
  //    unless the object is present.  I must admit that I do not understand why,
  //    since it was (and still is) fine within stan_fitscan.  Anyway, the 
  //    "proper" fix for this would most likely have been to include a definition 
  //    of the TST class, but this was easier...  PWP 30.10.00
  //

  if(e->m[mid]->present) e->ConfigureVariable(mid,cid,3,var);
}


void StrobeDelayPlot(Int_t run, Int_t scan, Int_t moi, Float_t fraction, 
                    Int_t online, Int_t graphics, Int_t postscript) {

  //
  // This macro fits the rising and falling edges of the strobe delay plateau,
  // calculates a point at some sepcified fraction between these two edges and,
  // if called from within the ST environment (e.g. online), sets the strobe
  // delay of each chip to be in agreement with the calculated "optimum" point.
  //
  // The suggested "fraction" is 0.25
  //
  // version 0.10 LE 
  // version 0.20 PWP 30.10.00
  // version 0.21 PWP/LE 01.11.00
  // version 0.22 LE/PWP 23.11.00
  // version 0.30 PWP 15.11.01
  // version 0.31 PWP 28.01.02 trivial bugfix re. output format
  // version 0.32 PWP 21.03.02 add web link to postscript plot if WEB_KEY defined
  //                           place leading zero in PS file timestamp
  // version 0.33 BJG 30.04.02 Put DUT type on plots
  //                           Fix some memory leaks
  //

  #ifdef DATA_DIRECTORY
    // do not reload parameters.h
  #else
    // must load parameters.h
  #include <parameters.h>
  #endif

  #define MIN_RISE  0  // NB these limits will change 
  #define MIN_FALL 28  // system timing by 1 BCO....
  #define MAX_RISE 35
  #define MAX_FALL 63

  Char_t h_name[80], f_name[128], s[100];
  Int_t  ichip;


  // Load other files needed by this one
  gROOT->LoadMacro("stan_util.cpp");
  gROOT->LoadMacro("ToolBox.cpp");

  stan_styles();
  gStyle->SetOptStat(0);


  //
  // Sanity checks
  //

  if((fraction<0.0)||(fraction>1.0)){
    printf("StrobeDelayPlot:: fraction(%2.2f) is outside the permitted range of 0.0 to 1.0\n",fraction);
    return;
  }


  //
  // Open the root data file and obtain pointers to the event histogram
  //


  sprintf(f_name,"%sstrun%d_%d.root",DATA_DIRECTORY,run,scan);
  TFile *strun = new TFile(f_name);

  TDatime *dt = new TDatime;
  Int_t date = dt->GetDate();
  Int_t the_time = dt->GetTime();

  //
  // Open the output canvas
  //

  Int_t ps_page=0;
  Int_t screen_page=0;

  TCanvas* StrobeDelay =0;
  TPostScript* ps =0;

  if((graphics!=0)||(postscript!=0)){
    graphics = 1;
    StrobeDelay=new TCanvas ("StrobeDelay","Strobe Delay Scan",1100,750);
  }


  //
  //  loop over modules
  //

  Int_t start = 0;
  Int_t end   = 0;

  if(moi==-1){
    start = 0;
    end   = MAX_MODULES;

  } else {
    start = moi;
    end   = moi+1;
  }


  Int_t dowait=0;

  if((graphics!=0)&&(end-start>1)){
    dowait = 1;
  }

  TH1F* hp_chip[MAX_CHIPS*2];
  TF1* f1[MAX_CHIPS*2];
  TF1* f2[MAX_CHIPS*2];

  for(ichip=0;ichip<MAX_CHIPS*2;ichip++){
    hp_chip[ichip]=0;

    sprintf(h_name,"f1_%d",ichip);
    f1[ichip]= new TF1 (h_name,erfc,  0, 1000, 3);
    f1[ichip]->SetParNames("Constant","Mean","Sigma");

    sprintf(h_name,"f2_%d",ichip);
    f2[ichip]= new TF1 (h_name,erf,  0, 1000, 3);
    f2[ichip]->SetParNames("Constant","Mean","Sigma");
  }

  Float_t chip_fall[MAX_CHIPS*2];
  Float_t chip_rise[MAX_CHIPS*2];
  Float_t chip_optimal[MAX_CHIPS*2];
  Int_t   chip_fitOK[MAX_CHIPS*2];

  TObjString *time  = 0;
  TObjString *place = 0;
  TObjString *mname = 0;
  TObjString *type = 0;

  for(moi=start;moi<end;moi++){

    //
    // Delete unnecessary objects and clear arrays
    //

    for(ichip=0;ichip<MAX_CHIPS*2;ichip++){
      if(hp_chip[ichip]) hp_chip[ichip]->Delete();

      chip_fall[ichip]=-1;
      chip_rise[ichip]=-1;
      chip_optimal[ichip]=-1;
    }

    if((screen_page>0)&&(dowait)){
      printf("Enter a string to continue\n");
      scanf("%s",s);
    }

    ps_page++;
    screen_page++;
    if(ps){
      ps->NewPage();
    }

    if(time) delete time; 
    if(place) delete place; 
    if(mname) delete mname; 
    if(type) delete type;
    time  = (TObjString*)strun->Get("Time");
    place = (TObjString*)strun->Get("Place");
    sprintf(h_name,"ModuleName;%d",moi+1);
    mname = (TObjString*)strun->Get(h_name);

    sprintf(h_name,"DUT;%d",moi+1);
    type = (TObjString*)strun->Get(h_name);

    sprintf(h_name,"h_scan0;%d",moi+1);
    TH2F *h_scan0= (TH2F*)strun->Get(h_name);
    sprintf(h_name,"h_scan1;%d",moi+1);
    TH2F *h_scan1= (TH2F*)strun->Get(h_name);

    //
    // only attempt plotting or fitting if data exists for the module
    //

    Int_t chips_with_data=0;

    if((h_scan0!=0)&&(h_scan1!=0)){

      printf("Fitting data for module %d\n",moi);

      if(postscript){
        ps_page=0;
        sprintf(f_name,"%s%s_StrobeDelayPlot_%d_%06d.ps",PS_DIRECTORY,mname->GetName(),date,the_time);

        ps = new TPostScript(f_name,112);
      }

      TPad *pad_sd = 0;
      
	  if(StrobeDelay){ // the canvas open, therefore the plots are to be drawn

        StrobeDelay->cd();
        StrobeDelay->Clear();

        TPad *pad_t  = new TPad("pad_t","Text",           0.01,0.91,0.99,0.99,33);
        pad_sd = new TPad("pad_sd","Strobe Delays", 0.01,0.01,0.99,0.90,33);
 
        pad_t->Draw();
        pad_sd->Draw();
        pad_sd->Divide(3,4,0,0);
        pad_t->cd();

        TPaveText *t1 = new TPaveText(0.0,0.0,1.0,1.0);
        t1->SetFillColor(10);
        t1->SetTextAlign(13);
        t1->SetTextSize(0.3);
        t1->SetTextColor(4);
  

        sprintf(s,"ATLAS SCT Module Test");
        if(time) {
          sprintf(s,"%s  -  %s",s,time->GetName());
        }
        if(place) {
          sprintf(s,"%s  -  %s",s,place->GetName());
        }
        if(mname) sprintf(s,"%s  -  Module %s",s, mname->GetName());
        t1->AddText(s);
        sprintf(s,"Page %d   Run %d   Scan %d   Module %d  -  Strobe Delay Optimisation", 
                   ps_page, run, scan, moi);
        if(type && strlen(type->GetName()) > 0) 
          sprintf(s, "%s - Type %s", s, type->GetName());
        t1->AddText(s);

        t1->Draw();
      }


      //
      // Loop over the chips and do the calculations
      //


      for(ichip=0;ichip<MAX_CHIPS*2;ichip++) {

        sprintf(h_name,"hp_chip%d",ichip);
        if (ichip<6) h_scan0->ProjectionY(h_name,(ichip*128),((ichip+1)*128)-1);
        else h_scan1->ProjectionY(h_name,((ichip-6)*128),((ichip-5)*128)-1);
   
        hp_chip[ichip] = (TH1F*) gDirectory->GetList()->FindObject(h_name);

        if(hp_chip[ichip]->GetMaximum()>0.0){ // data for this chip

        chips_with_data++;

          sprintf(h_name,"%s chip %d", hp_chip[ichip]->GetTitle(),ichip);
          hp_chip[ichip]->SetTitle(h_name);
          hp_chip[ichip]->SetXTitle("Strobe Delay");
          hp_chip[ichip]->SetYTitle("Efficiency");
          hp_chip[ichip]->Scale(1.0/hp_chip[ichip]->GetMaximum());

          Float_t centre = (find_bin2(hp_chip[ichip],0.99,0,1)+find_bin2(hp_chip[ichip],0.99,1,0))/2;


          // Falling edge

          Float_t p0_guess = hp_chip[ichip]->GetMaximum();
          Float_t p1_guess = find_level2(hp_chip[ichip],0.50,1);          // last bin > 50%
          Float_t p2_guess = find_level2(hp_chip[ichip],0.82,1)-p1_guess;  
          Float_t range_lo = centre;
          Float_t range_hi = hp_chip[ichip]->GetBinCenter(hp_chip[ichip]->GetNbinsX());
          if(p2_guess<0) p2_guess = p2_guess*-1.0;

          f1[ichip]->SetParameters(p0_guess,p1_guess,p2_guess);
          f1[ichip]->SetParLimits(0,p0_guess,p0_guess);
          f1[ichip]->SetRange(range_lo, range_hi);

          sprintf(h_name,"f1_%d",ichip);
          hp_chip[ichip]->Fit(h_name,"NRQ"); // no graphics, quiet mode
          chip_fall[ichip] = f1[ichip]->GetParameter(1); 


          // Rising edge

          p0_guess = hp_chip[ichip]->GetMaximum();
          p1_guess = find_level2(hp_chip[ichip],0.50,0);          // first bin < 50%
          p2_guess = find_level2(hp_chip[ichip],0.82,0)-p1_guess;
          range_lo = hp_chip[ichip]->GetBinCenter(1);
          range_hi = centre;
          if(p2_guess<0) p2_guess = p2_guess*-1.0;
      
          f2[ichip]->SetParameters(p0_guess,p1_guess,p2_guess);
          f2[ichip]->SetParLimits(0,p0_guess,p0_guess);
          f2[ichip]->SetRange(range_lo, range_hi);

          sprintf(h_name,"f2_%d",ichip);
          hp_chip[ichip]->Fit(h_name,"NRQ"); // no graphics, quiet mode
          chip_rise[ichip] = f2[ichip]->GetParameter(1);


          // Optimal setting

          chip_optimal[ichip] = chip_rise[ichip] + 
                               (fraction *(chip_fall[ichip]-chip_rise[ichip]));

          printf("  Chip %2d fall %2.1f  rise %2.1f  optimum %2.1f\n",
            ichip,chip_fall[ichip],chip_rise[ichip],chip_optimal[ichip]); 


          if(StrobeDelay){ // the canvas open, therefore the plots are to be drawn

            pad_sd->cd(ichip+1);
            hp_chip[ichip]->Draw();

            f1[ichip]->SetLineColor(kBlue);
            f1[ichip]->Draw("same");
            f2[ichip]->SetLineColor(kBlue);
            f2[ichip]->Draw("same");

            TMarker *FallMark=new TMarker (chip_fall[ichip],0.5,23); 
            FallMark->SetMarkerColor(kBlue);
            FallMark->SetMarkerSize(1);
            FallMark->SetMarkerStyle(23);
            FallMark->Draw("same");
 
            TMarker *RiseMark=new TMarker (chip_rise[ichip],0.5,23); 
            RiseMark->SetMarkerColor(kBlue);
            RiseMark->SetMarkerSize(1);
            RiseMark->SetMarkerStyle(23);
            RiseMark->Draw("same");
      
            TMarker *OptMark=new TMarker (chip_optimal[ichip],0.5,23); 
            OptMark->SetMarkerColor(kRed);
            OptMark->SetMarkerSize(1);
            OptMark->SetMarkerStyle(29);
            OptMark->Draw("same");

            TText * t = new TText();
            sprintf(h_name,"%2.1f",chip_fall[ichip]);
            t->DrawText(chip_fall[ichip]-1,0.35,h_name);
            sprintf(h_name,"%2.1f",chip_rise[ichip]);
            t->DrawText(chip_rise[ichip]-1,0.35,h_name);
            sprintf(h_name,"%2.1f",chip_optimal[ichip]);
            t->DrawText(chip_optimal[ichip]-1,0.35,h_name);
          }


        } else {
          printf("  Chip %2d NO DATA!!!\n",ichip);

        } // end if (data for this chip)

      } // end for(ichip)

      if(StrobeDelay){
        StrobeDelay->Update();
      }

      if(ps) {
        ps->Close();
        delete ps;
      }


      if(chips_with_data>0){

        //
        // Set the values
        //

	Float_t mean_delay = 0.0;
        Int_t n_good_chips = 0;

        for(ichip=0;ichip<MAX_CHIPS*2;ichip++){
          if((chip_rise[ichip]<MIN_RISE)||(chip_fall[ichip]<MIN_FALL)){
            chip_fitOK[ichip] = 1;
          } else if((chip_rise[ichip]>MAX_RISE)||(chip_fall[ichip]>MAX_FALL)){
            chip_fitOK[ichip] = 2;
          } else if(  (chip_optimal[ichip]<chip_rise[ichip])
                    ||(chip_optimal[ichip]>chip_fall[ichip]) ){
            chip_fitOK[ichip] = 3;
          } else {
            chip_fitOK[ichip] = 0; // both fits are OK
            mean_delay += chip_optimal[ichip];
            n_good_chips++;
          }
        }

        if(n_good_chips>0){
          mean_delay /= n_good_chips;
        }

        if(online){
          for(ichip=0;ichip<MAX_CHIPS*2;ichip++){
            if(chip_fitOK[ichip]==0){
              SetStrobeDelay(moi,ichip,chip_optimal[ichip]);
            } else if(n_good_chips>0){
              // set mean delay if fit fails for an individual chip
              // (probably still up for debate!!) PWP 23.11.00
              SetStrobeDelay(moi,ichip,mean_delay);
            }
          }         
        }

        Short_t pass=1;
        for(ichip=0;ichip<MAX_CHIPS*2;ichip++){
          if(chip_fitOK[ichip]!=0){
            pass=0;
          }         
        }


        //
        // Open today's output file
        // Output from a series of tests will be added to this file
        //

        FILE* outfile = OpenResults("StrobeDelay", strun, moi, run, scan, pass,0); 

        if(outfile){

          fprintf(outfile,"%%StrobeDelay\n");
          fprintf(outfile,"#\n");

          fprintf(outfile,"#DELAY\n");

          fprintf(outfile,"#M0 S1  S2  S3  S4  E5\n");
          for(ichip=0;ichip<MAX_CHIPS;ichip++){
            fprintf(outfile,"%02d ", chip_optimal[ichip]);
          }
          fprintf(outfile,"\n#\n");

          fprintf(outfile,"#M8 S9  S10 S11 S12 E13\n");
          for(ichip=6;ichip<MAX_CHIPS*2;ichip++){
            fprintf(outfile,"%02d ", chip_optimal[ichip]);
          }
          fprintf(outfile,"\n#\n");

          Int_t n_defects=0;
          for(ichip=0;ichip<MAX_CHIPS*2;ichip++){
	    switch (chip_fitOK[ichip]) {
	      case 1: 
                ListDefect(outfile,"SD_LO",(ichip*128),((ichip+1)*128)-1);
                n_defects++;
                break;
              case 2: 
                ListDefect(outfile,"SD_HI",(ichip*128),((ichip+1)*128)-1);
                n_defects++;
                break;
          //    case 3: 
          //      ListDefect(outfile,"SD_OOR",(ichip*128),((ichip+1)*128)-1);
          //      n_defects++;
          //      break;
              default:
                break;
	    }
          }
          if(n_defects==0){
            fprintf(outfile,"#No defects found!\n");
          }else{
            fprintf(outfile,"#%d defects found\n",n_defects);
          }
         
        // And a URL to the postscript file

  #ifdef WEB_KEY        
          if(postscript){
            sprintf(f_name,"%s%s_StrobeDelayPlot_%d_%06d.ps",WEB_KEY,mname->GetName(),date,the_time);
            ListWebLink(outfile,"Plots (postscript)",f_name);
          }
  #endif

          fclose(outfile);
        }
      } else {
        printf("No data for module %d\n",moi);
      }

    } else {
      printf("No histograms for module %d\n",moi);
    }

  } // end for (moi)


  if(StrobeDelay==0){

    // Delete objects and close the ROOT file.
    // We do not do this if graphical display was 
    // selected since the plots will disappear!

    for(ichip=0;ichip<MAX_CHIPS*2;ichip++){
      if(hp_chip[ichip]) hp_chip[ichip]->Delete();
      if(f1[ichip]) f1[ichip]->Delete();
      if(f2[ichip]) f2[ichip]->Delete();
    }

    strun->Close();
    delete strun;
  }    

  if(time) delete time;
  if(place) delete place;
  if(mname) delete mname;
  if(type) delete type;
}


//
// wrappers
//


void StrobeDelayPlot(Int_t run, Int_t scan, Int_t moi){

  //       no graphics and no postscript

  StrobeDelayPlot(run,scan,moi,0.25,0,0,0);
}

void StrobeDelayPlot(Int_t run, Int_t scan, Int_t moi, Float_t fraction){

  //       show graphics but no postscript
  StrobeDelayPlot(run,scan,moi,fraction,0,1,0);
}
 
void StrobeDelayPlot(Int_t run, Int_t scan, Int_t moi, Float_t fraction,
                     Int_t postscript){

  StrobeDelayPlot(run,scan,moi,fraction,0,1,postscript);
} 

