void StrobeSpeed(Int_t mod, Float_t startVdd, Float_t stopVdd, Float_t stepVdd) {

  // version 0.1 LE 
  // version 0.2 PWP 30.10.00
  // version 0.3 PWP 01.11.00
  // version 0.4 PWP 15.11.00
  // version 0.5 LE/PWP 23.11.00
  // version 0.6 PWP 10.12.00
  // version 0.7 LE/HP  02.03.00  - rewrite to use as a measurement of the chip speed
  // version 0.8 PWP 06.06.01 - use parameters.h
  // version 0.81 LE 06.08.01 - use SaveState and reset other parameters after scan.
  // version 0.82 BJG 06.08.01 - minor syntax errors

  #ifdef DATA_DIRECTORY
    // do not reload parameters.h
  #else
    // must load parameters.h
  #include <parameters.h>
  #endif

  Int_t manual=0; // set Vdd manually
  Int_t module, chip;
  Char_t f_name[64];
  Int_t find_two_fc = 1;
  FILE* outfile[MAX_MODULES];

  Float_t in_charge[MAX_MODULES][12];
  Int_t comp_mode[MAX_MODULES][12];
  Int_t edge[MAX_MODULES][12];
  Int_t route_mask[MAX_MODULES][12];
  Char_t key[10];

  e->SaveState();

  for (module=0; module<MAX_MODULES; module++){
    if(e->m[module]->present){
      for(chip=0;chip<12;chip++){
	in_charge[module][chip] = e->RetrieveVariable(module,chip,2);
	comp_mode[module][chip] = e->RetrieveVariable(module,chip,11);
	edge[module][chip] = e->RetrieveVariable(module,chip,13);
	route_mask[module][chip] = e->RetrieveVariable(module,chip,14);
      }
    }
  }


  TDatime *dt = new TDatime;
  Int_t date = dt->GetDate();
  Int_t the_time = dt->GetTime();

  // Configure the scan
  e->StOutScanReset();
  e->ascii_nt=0;
  e->abort=0;
  
  e->do_cal_loop=1;        // cal loop ON 
  e->do_mask_test=0;       // alternating mask patterns OFF 
  e->do_preburst=1;        // do preburs - to suspend evil modules.
  
  e->burst_trtype=3;       // CAL + L1A
  e->burst_ntrigs=1000;
  
  e->ConfigureVariable(14,0);  // Route Mask OFF

  int first, last;

  if(mod==-1){
    first = 0;
    last = e->GetMaxModules();
  }else{
    first = mod;
    last = mod+1;
  }

  //
  // If the calling routine/user has not been able to set 2fC threshold,
  // we can find an approximate value now using the default timing
  //

  if(find_two_fc){
    e->do_fits=1;           // integrated fitting ON

    e->ConfigureVariable(11,0);   // Compression Mode 0.
    e->ConfigureVariable(13,0);   // Edge detect off.
    
    e->ConfigureVariable(2,20);   // 2.00fC input charge
    e->ConfigureScan(1,0,350,5);  // range tbc
    
    st_scan(11,-1);
    
    for (module=first; module<last; module++){
      if(e->m[module]->present){
        for(chip=0;chip<12;chip++){
          e->ConfigureVariable(module,chip,1,e->m[module]->chip_mean[chip/6][chip%6]);
        } // end for(chip)  
      }
    } // end for(module)
  }



  //
  // Make a strobe delay scan with
  //   Edge Detect ON, edge compression
  // and calculate the optimum strobe delay setting
  //

  // load the analysis macro 
  gROOT->ProcessLine(".L StrobeSpeedAnalysis.cpp");

  if(e->abort==0){
    e->do_fits=0;           // integrated fitting OFF

    e->ConfigureVariable(11,2);   // Compression Mode 2.
    e->ConfigureVariable(13,1);   // Edge detect on.
  
    e->ConfigureVariable(2,60);   // 6.00fC input charge.
    e->ConfigureScan(3,0,63,1);
    
    if(startVdd<stopVdd) {
      for(Float_t Vdd=startVdd; Vdd<stopVdd; Vdd+=stepVdd){
	if(e->abort!=0) break;
	
	e->ConfigureVariable(33,Vdd);   // Set Vdd

	// set volages manually
	if (manual)
	  {
	    printf("Set voltage to Vdd=%f and type a string and return to continue\n",Vdd);
	    scanf("%s",key);
	  }	  

	e->ConfigureVariable(27,130);   // Set trigger delay to 130
	st_scan(11,-1);

	for (module=first; module<last; module++){
	  if(e->m[module]->present){
	    sprintf(f_name,"%sspeedlog_%s_%d.txt",RESULTS_DIRECTORY,e->m[module]->name,date);
	    outfile[module] = fopen(f_name,"a+");
	    if(outfile[module]!=NULL){
	      fprintf(outfile[module],"%3.1f 130 %d %d %d\n",Vdd,e->runnum,e->scannum,module);
	      fclose(outfile[module]);
	    }
	  }
	}

	e->ConfigureVariable(27,129);   // Set trigger delay to 129
	st_scan(11,-1);

	for (module=first; module<last; module++){
	  if(e->m[module]->present){
	    sprintf(f_name,"%sspeedlog_%s_%d.txt",RESULTS_DIRECTORY,e->m[module]->name,date);
	    outfile[module] = fopen(f_name,"a+");
	    if(outfile[module]!=NULL){
	      fprintf(outfile[module],"%3.1f 129 %d %d %d\n",Vdd,e->runnum,e->scannum,module);
	      fclose(outfile[module]);
	    }
	  }
	}
	
	//
	// Call the analysis macro.
	//

	if(e->abort==0){
	  StrobeSpeedAnalysis(e->runnum,e->scannum-1,-1,Vdd,130,0,0);
	  StrobeSpeedAnalysis(e->runnum,e->scannum,  -1,Vdd,129,0,0);
	}


      }

    }else{
      for(Float_t Vdd=startVdd; Vdd>stopVdd; Vdd-=stepVdd){
	if(e->abort!=0) break;
	
	e->ConfigureVariable(33,Vdd);   // Set Vdd

	// set volages manually
	if (manual)
	  {
	    printf("Set voltage to Vdd=%f and type a string and return to continue\n",Vdd);
	    scanf("%s",key);
	  }	  

	e->ConfigureVariable(27,130);   // Set trigger delay to 130
	st_scan(11,-1);

	for (module=first; module<last; module++){
	  if(e->m[module]->present){
            sprintf(f_name,"%sspeedlog_%s_%d.txt",RESULTS_DIRECTORY,e->m[module]->name,date);
	    outfile[module] = fopen(f_name,"a+");
	    if(outfile[module]!=NULL){
	      fprintf(outfile[module],"%3.1f 130 %d %d %d\n",Vdd,e->runnum,e->scannum,module);
	      fclose(outfile[module]);
	    }
	  }
	}
	
	e->ConfigureVariable(27,129);   // Set trigger delay to 129
	st_scan(11,-1);
	
	for (module=first; module<last; module++){
	  if(e->m[module]->present){	 
            sprintf(f_name,"%sspeedlog_%s_%d.txt",RESULTS_DIRECTORY,e->m[module]->name,date);
	    outfile[module] = fopen(f_name,"a+");
	    if(outfile[module]!=NULL){
	      fprintf(outfile[module],"%3.1f 129 %d %d %d\n",Vdd,e->runnum,e->scannum,module);
	      fclose(outfile[module]);
	    }
	  }
	}
	//
	// Call the analysis macro.
	//

	if(e->abort==0){
	  StrobeSpeedAnalysis(e->runnum,e->scannum-1,-1,Vdd,130,0,0);
	  StrobeSpeedAnalysis(e->runnum,e->scannum,  -1,Vdd,129,0,0);
	}
      }
    }
  }
  
  e->RestoreState();
  for (module=0; module<MAX_MODULES; module++){
    if(e->m[module]->present){
      for(chip=0;chip<12;chip++){
	e->ConfigureVariable(module,chip,2,in_charge[module][chip]);
	e->ConfigureVariable(module,chip,11,comp_mode[module][chip]);
	e->ConfigureVariable(module,chip,13,edge[module][chip]);
	e->ConfigureVariable(module,chip,14,route_mask[module][chip]);
      }
    }
  }


}

void StrobeSpeed() {

  StrobeSpeed(-1,5.0,3.8,0.1);
}




