void T7RadContTest(Double_t small_interval, Double_t big_interval){

  //
  // hit STOP to move to the next scan
  // hit ABORT to end the test
  // PWP 13.12.00
  // HP modified for T7 test 3.4.01
  // mods PWP 11.04.01

  Char_t fname[64];
  Double_t small_loop_end, big_loop_end;
  Int_t n;
  Int_t const NVPoints=7;
  Float_t vdd[NVPoints] = {3.5, 3.75, 4.0, 4.25, 4.5, 4.75, 5.0};
  Float_t vdd_set[10];
  Float_t vcc_set[10];
  Float_t vcc[NVPoints] = {3.3, 3.5, 3.7, 3.9 ,4.1, 4.3, 4.5};


  // do PV test (increase scan number)
  gROOT->ProcessLine(".L PVTest.cpp");
  PVTest(3.3,5.0,0.1,0);
  // RecoverTrips, HardReset and ExcuteConfigs, just to make sure that 
  // the chips are in a well defined state.
  e->HardReset();
  e->RecoverTrips();
  e->ExecuteConfigs();

  
  sprintf(fname,"D:\\sctvar\\data\\DCSntuple_%d_%d.root",e->runnum,e->scannum);

  TFile* outfile = new TFile(fname,"recreate");
  if (!outfile) {
    printf("Can't open DCS output file - will stop now");
    return;
  }

  TModuleDCS* dcs;

  TTree* DCSTree = new TTree("DCSTree","DCSTree");
  DCSTree->Branch("DCSBranch","TModuleDCS",&dcs,8000,1);
  DCSTree->Write();
  TStopwatch* timer = new TStopwatch();
  Double_t start_time = timer->GetRealTime();


  Int_t stop_loop = 0;
  Int_t big_loop = 0;
  Int_t small_loop =0;

  e->abort=0;
  small_interval /=2;

  //
  // These are irradiated modules:
  // currents will be high, so allow up to 500 LV trips each
  //

  for(n=0;n<e->GetMaxModules();n++){
    if(e->m[n]->present){
      e->m[n]->DCS->maxLVtrips =500;
      e->m[n]->DCS->t_source=1;
    }
  }


  // Main loop


  while(e->abort==0){


    // Monitor currents for up to "big_loop_end" seconds

    st_start_stopper();
    small_loop = 0;
    big_loop_end = big_interval + timer->GetRealTime();

    while((timer->GetRealTime()<big_loop_end)&&(stop_loop==0)){

      printf("Time since start: %ld seconds\n",timer->GetRealTime()-start_time);
      printf("Loop %d_%d: Entering trigger phase\n",big_loop,small_loop);
      e->DCSAction();
      e->ExecuteConfigs();

      e->RunFree(); // start triggers

      // wait small interval

      small_loop_end = small_interval + timer->GetRealTime();
      while(timer->GetRealTime()<small_loop_end){
        // DO NOWT!!
      }

      // do short loop actions
      //========================================================================

      // RecoverTrips, HardReset and ExcuteConfigs, just to make sure that 
      // the chips are in a well defined state.
      e->RecoverTrips();
      e->ExecuteConfigs();

      e->DCSAction();
      printf("Filling tree...\n");
      for(n=0;n<e->GetMaxModules();n++){
        if(e->m[n]->present){
          DCSTree->SetBranchAddress("DCSBranch",&e->m[n]->DCS);
          DCSTree->Fill();
	  vdd_set[n] = e->m[n]->DCS->vdd_set;
	  vcc_set[n] = e->m[n]->DCS->vcc_set;
	  printf("T7==> Module %s was Vdd=%2.2f and Vcc=%2.2f; Idd=%2.2f, Icc=%2.2f T1=%d T2=%d\n",
		 e->m[n]->name,vdd_set[n],vcc_set[n],e->m[n]->DCS->idd,e->m[n]->DCS->icc,
		 e->m[n]->DCS->T1,e->m[n]->DCS->T2);
        }
      }

      if (!outfile) {
	TFile* outfile = new TFile(fname,"recreate");
	printf("Reopening output file");
      } else outfile->cd();
      DCSTree->Write("DCSTree",TNamed::kOverwrite);

      printf("Loop %d_%d: Entering idle phase\n",big_loop,small_loop);
      e->RunStop(); // stop triggers



      // wait small interval
      //      small_loop_end = small_interval + timer->GetRealTime();
      //      while(timer->GetRealTime()<small_loop_end){
      // DO NOWT!!
      //    }

      stop_loop=st_do_stop();
      small_loop++;
    }

    st_stop_stopper();

    if(e->abort==0){

      // RecoverTrips, HardReset and ExcuteConfigs, just to make sure that 
      // the chips are in a well defined state.
      e->HardReset();
      e->RecoverTrips();
      e->ExecuteConfigs();

      st_start_stopper();

      printf("Time since start: %ld seconds\n",timer->GetRealTime()-start_time);
      printf("Loop %d: Entering Measurement phase\n",big_loop);

      //
      // Make Measurements Here....
      //============================================================================

      // test different voltages
      // VDD:

      // first set lower bound and recover trips

      printf("T7==> Initialising Vdd Scan\n");
      e->ConfigureVariable(33,vdd[0]);
      gSystem->Sleep(5000); // Sleep 5000mS
      e->DCSAction();

      // Make the scan

      for (Int_t i=0;i<NVPoints;i++) {

        printf("T7==> Current Scan at Vdd=%2.2f\n",vdd[i]);
        e->ConfigureVariable(33,vdd[i]);

	// RecoverTrips, HardReset and ExcuteConfigs, just to make sure that 
	// the chips are in a well defined state.
	//	e->HardReset();
	e->RecoverTrips();
	e->ExecuteConfigs();

        gSystem->Sleep(5000); // Sleep 5000mS
        e->DCSAction();

        for(n=0;n<e->GetMaxModules();n++){
          if(e->m[n]->present){
            DCSTree->SetBranchAddress("DCSBranch",&e->m[n]->DCS);
            DCSTree->Fill();
          }
        }
      }

      printf("T7==> Restoring Vdd Settings\n");
      for(n=0;n<e->GetMaxModules();n++){
	if(e->m[n]->present){
	  e->ConfigureVariable(n,33,vdd_set[n]);
	}
      }
      e->LVStatus();

      // VCC:

      // first set lower bound and recover trips
      printf("T7==> Initialising Vcc Scan\n");
      e->ConfigureVariable(31,vcc[0]);

      // RecoverTrips, HardReset and ExcuteConfigs, just to make sure that 
      // the chips are in a well defined state.
      e->HardReset();
      e->RecoverTrips();
      e->ExecuteConfigs();

      gSystem->Sleep(5000); // Sleep 5000mS
      e->DCSAction();

      for (Int_t i=0;i<NVPoints;i++) {

        printf("T7==> Current Scan at Vcc=%2.2f\n",vcc[i]);
        e->ConfigureVariable(31,vcc[i]);

	// RecoverTrips, HardReset and ExcuteConfigs, just to make sure that 
	// the chips are in a well defined state.
	e->HardReset();
	e->RecoverTrips();
	e->ExecuteConfigs();

        gSystem->Sleep(5000); // Sleep 5000mS
        e->DCSAction();

        for(n=0;n<e->GetMaxModules();n++){
          if(e->m[n]->present){
            DCSTree->SetBranchAddress("DCSBranch",&e->m[n]->DCS);
            DCSTree->Fill();
          }
        }
      }

      printf("T7==> Restoring Vcc Settings\n");
      for(n=0;n<e->GetMaxModules();n++){
	if(e->m[n]->present){
	  e->ConfigureVariable(n,31,vcc_set[n]);
	}
      }

      // RecoverTrips, HardReset and ExcuteConfigs, just to make sure that 
      // the chips are in a well defined state.
      e->HardReset();
      e->RecoverTrips();
      e->ExecuteConfigs();

      e->LVStatus();

      gSystem->Sleep(5000);
      e->DCSAction();
      e->LVStatus();

      if (!outfile) {
	TFile* outfile = new TFile(fname,"recreate");
	printf("Reopening output file");
      } else outfile->cd();
      DCSTree->Write("DCSTree",TNamed::kOverwrite);

      // do PVTest
      PVTest(3.3,5.0,0.1,0);

      st_stop_stopper();
    }

    stop_loop=st_do_stop();
    big_loop++;

  } // end of main loop

  // Save file and Power Off

  //  e->LVOff();
  outfile->Close();
}


void T7RadContTest(){
  //
  // default is to make DCS measurements every 5 minutes
  // with module tests every 12 hours
  //

  T7RadContTest(300,43200);
}

