void TestFit(double p0, double p1, double p2, double nbins, double lo, double hi, double step, short log){

  // p0 - ntrigs
  // p1 - mean
  // p2 - sigma


  //
  //  A routine to fit st scan histograms - FAST!!!
  //  The fit routine used is based upon the integrated fitting within
  //  stdll, but compiled into a standalone DLL (TestFit.dll)
  //
  // The fitted data is saved back into the root file to be compatible
  // with the on line fitting.  If online fit data is present in the
  // file, the exisiting fit data is used instead!!
  //
  //  A reduced set of cuts have been implemented as follows:
  //  o Channels with <1% occupancy in any bin are marked as "DEAD"
  //  o Channels with >98% occupancy in every bin are marked "STUCK"
  //  o Channels which do not reach 100% efficiency are marked "UNDER"
  //  o Channels which reach > 100% efficiency are marked "OVER"
  //    (Such a condition would imply decoder errors, inviting the
  //     user to check the digital integrity of their system)
  //
  //  Version 0.1, PWP 26.10.00
  //  Version 0.2, LE  23.05.01
  //  Version 0.21 BJG 30.04.02 - Variable declaration changes
  //


  #ifdef DATA_DIRECTORY
    // do not reload parameters.h
  #else
    // must load parameters.h
  #include <parameters.h>
  #endif

  char  h_title[64];
  char  scan_name[64];
  char  s[256];

  //
  // Load other files needed by this one
  //

  gSystem->Load("KwikFit.dll");
  gROOT->LoadMacro("stan_util.cpp");

  // Global style variations
  stan_styles();
  gStyle->SetOptStat(0);



  //
  // Open the output canvas (and postscript file if requested)
  //


  TCanvas* c1 =0;

  c1=new TCanvas ("c1","TestFit",1100,750);


//  for(moi=start;moi<end;moi++){

  // Create histogram objects

  TH1F* h_nevent;
  TH2F* h_scan0;
  TH2F* h_scan1;


  sprintf(h_title,"Mustard %d Events Decoded",1);
  h_nevent = new TH1F("h_nevent", h_title, nbins, lo-(step/2), hi+(step/2));

  sprintf(h_title, "Stream 0 Scan Data");
  h_scan0 = new TH2F("h_scan0",h_title, 768,0.5,768.5, nbins, lo-(step/2), hi+(step/2));
  h_scan0->SetXTitle("Channel Number");

  sprintf(h_title, "Stream 1 Scan Data");
  h_scan1 = new TH2F("h_scan1",h_title, 768,0.5,768.5, nbins, lo-(step/2), hi+(step/2));
  h_scan1->SetXTitle("Channel Number");


  // Create new ERFC with specified parameters

  TF1 *f1 = new TF1 ("f1",erfc,  0, 1000, 3);
  f1->SetParNames("Constant","Mean","Sigma");
  f1->SetParameters(p0,p1,p2);


  // fill histograms


  Int_t x,y;
  TAxis* axis;

  axis = (TAxis*) h_scan0->GetYaxis();

  for(y=0;y<h_scan0->GetNbinsY()+1;y++){
    double contentf = f1->Eval(axis->GetBinCenter(y));
    int content = (int) (contentf+0.5);

    for(x=0;x<h_scan0->GetNbinsX()+1;x++){
      h_scan0->SetCellContent(x,y,content);
    }
  }

  axis = (TAxis*) h_scan1->GetYaxis();

  for(y=0;y<h_scan1->GetNbinsY()+1;y++){
    double contentf = f1->Eval(axis->GetBinCenter(y));
    int content = (int) (contentf+0.5);

    for(x=0;x<h_scan1->GetNbinsX()+1;x++){
      h_scan1->SetCellContent(x,y,content);
    }
  }

  for(x=0;x<h_nevent->GetNbinsX()+1;x++){
    h_nevent->SetBinContent(x,p0);
  }

  TPad *pad_p = 0;

    //
    // only attempt plotting or fitting if data exists for the module
    //

    if((h_scan0!=0)||(h_scan1!=0)){

      if(c1){
        c1->cd();
        c1->Clear();
        TPad *pad_t = new TPad("pad_t","Text",   0.00,0.90,1.00,1.00,33);
        pad_p = new TPad("pad_p","Plots",  0.00,0.00,1.00,0.90,33);

        pad_t->Draw();
        pad_p->Draw();
        pad_p->Divide(2,4,0,0);

        pad_t->cd();

        TPaveText *t1 = new TPaveText(0.0,0.0,1.0,1.0);
        t1->SetFillColor(10);
        t1->SetTextAlign(13);
        t1->SetTextSize(0.3);
        t1->SetTextColor(4);
  
        sprintf(s,"Simulated ATLAS SCT Module Test Data");
        t1->AddText(s);
        sprintf(s,"Parameters: p0 %3.2f p1 %3.2f p2 %3.2f",p0,p1,p2);
        t1->AddText(s);

        t1->Draw();

        pad_p->cd(1);
        h_scan0->Draw("COLZ");
        h_scan0->SetXTitle("Channel Number");
        pad_p->cd(2);
        h_scan1->Draw("COLZ");
        h_scan1->SetXTitle("Channel Number");

        c1->Update();
      }

      //
      // Fit the data if it wasn't already done
      //

      TKwikFit *kf = new TKwikFit(0);
      kf->likelihood=log;

      kf->LoadScan(h_scan0,h_scan1,h_nevent);
      kf->PrintScan();

      printf("\nFitting Data\n");

      TStopwatch* patek_phillipe = new TStopwatch();
      kf->FitScan(1,-1);
      patek_phillipe->Print();
      kf->PrintScan();

      sprintf(h_title,"Sigma (%s)",scan_name);
      kf->h_sigma0->SetMarkerColor(kBlue);
      kf->h_sigma0->SetXTitle("Channel Number");
      kf->h_sigma0->SetYTitle(h_title);
      kf->h_sigma1->SetMarkerColor(kBlue);
      kf->h_sigma1->SetXTitle("Channel Number");
      kf->h_sigma1->SetYTitle(h_title);

      sprintf(h_title,"Fit Code");
      kf->h_code0->SetMarkerColor(kBlue);
      kf->h_code0->SetXTitle("Channel Number");
      kf->h_code0->SetYTitle(h_title);
      kf->h_code1->SetMarkerColor(kBlue);
      kf->h_code1->SetXTitle("Channel Number");
      kf->h_code1->SetYTitle(h_title);

      sprintf(h_title,"Chisq/NDF");
      kf->h_chisq0->SetMarkerColor(kBlue);
      kf->h_chisq0->SetXTitle("Channel Number");
      kf->h_chisq0->SetYTitle(h_title);
      kf->h_chisq1->SetMarkerColor(kBlue);
      kf->h_chisq1->SetXTitle("Channel Number");
      kf->h_chisq1->SetYTitle(h_title);


      if(c1){

        pad_p->cd(1);
        kf->h_mean0->Draw("same");
        pad_p->cd(2);
        kf->h_mean1->Draw("same");

        pad_p->cd(3);
        kf->h_sigma0->Draw("P");
        pad_p->cd(4);
        kf->h_sigma1->Draw("P");

        pad_p->cd(5);
        kf->h_chisq0->Draw("P");
        pad_p->cd(6);
        kf->h_chisq1->Draw("P");

        pad_p->cd(7);
        kf->h_fom0->Draw("P");
        pad_p->cd(8);
        kf->h_fom1->Draw("P");

        c1->Update();
      }


    }
//  }


}


