void Timewalk(Int_t find_one_fc) {

  // version 0.2 PWP 30.11.00
  // version 0.3 PWP 10.12.00
  // version 0.4 PWP 08.04.02 Reset strobe delay settings after the test
  // version 0.5 PWP 17.10.02 comment out change of trig delay
  // version 0.6 PWP 24.10.02 bugfix re. restoration of original SD settings -
  //                          previous code was broken for multiple modules

  #ifdef DATA_DIRECTORY
    // do not reload parameters.h
  #else
    // must load parameters.h
  #include <parameters.h>
  #endif

  Int_t module, chip;

  Int_t n_points = 10;
  Int_t scans[10];
  Float_t charges[10] = {1.25,1.50,1.75,2.00, 3.00,
                        4.00,5.00,6.00,8.00,10.00};

  Float_t sd[MAX_MODULES][MAX_LINKS][MAX_CHIPS];
  Float_t td;

  // Store settings before the scan
  e->SaveState();

  // Record the settings of the strobe delay register
  // so we can restore these values after the scan

  for(module=0; module<MAX_MODULES; module++){
    if(module<e->GetMaxModules()){
      if(e->m[module]->present){
        for(link=0;link<2;link++){
          for(chip=0;chip<6;chip++){
            sd[module][link][chip]=e->RetrieveVariable(module,((link*6)+chip),3);
          }
        }
      }
    }
  }

  // Record the setting of trig delay
  td=e->RetrieveVariable(0,27);

  // Configure the scan
  e->StOutScanReset();
  e->ascii_nt=0;
  e->abort=0;

  e->do_cal_loop=1;        // cal loop ON 
  e->do_mask_test=0;       // alternating mask patterns OFF 

  e->burst_trtype=3;       // CAL + L1A
  e->burst_ntrigs=1000;

  e->ConfigureVariable(14,0);  // Route Mask OFF


  //
  // If the calling routine/user has not been able to set 
  // 1fC threshold, we can do it now
  //
  // N.B. THIS METHOD NOT ADVISED FOR ABCD2T due to nonlinear
  // response of the calibration circuit at low thresholds
  //

  if(find_one_fc){
  e->do_fits=1;           // integrated fitting ON

    e->ConfigureVariable(11,0);   // Compression Mode 0.
    e->ConfigureVariable(13,0);   // Edge detect off.

    e->ConfigureVariable(2,10);   // 1.00fC input charge
    e->ConfigureScan(1,0,200,5);  // range tbc

    st_scan(11,-1);

    for (module=0; module<e->GetMaxModules(); module++){
      if(e->m[module]->present){

        for(chip=0;chip<12;chip++){
          e->ConfigureVariable(module,chip,1,e->m[module]->chip_mean[chip/6][chip%6]);
        } // end for(chip)  
      }
    } // end for(module)
  }



  //
  // Take a series of strobe delay scans with
  //   Edge Detect ON, edge compression
  //

  // load the analysis macro 
  gROOT->ProcessLine(".L TimewalkPlot.cpp");

  e->do_fits=1;           // integrated fitting ON (???)

  // maximum occupancy in edge mode with small charges may not be 100%:
  // need to fit function to the actual maximum occupancy (???)

  for (module=0; module<e->GetMaxModules(); module++){
    if(e->m[module]->present){
      e->m[module]->use_ch_maxima=1;
    }
  }

  e->ConfigureVariable(11,2);   // Compression Mode 2.
  e->ConfigureVariable(13,1);   // Edge detect on.
  
  e->ConfigureVariable(2,12.5); // 1.25fC input charge.

// No need for next line: if pulse is not already centred
// the strobe delay test would have failed.  PWP 17.10.02
//  e->ConfigureVariable(27,129); // trig_delay to 129 to centre pulse 

  Int_t i;
  for(i=0;i<n_points;i++){

    if(e->abort==0){
      e->ConfigureVariable(42,charges[i]);  // 42 - QCAL (fC)
      e->ConfigureScan(3,0,63,1);

      st_scan(11,-1);

      scans[i]=e->scannum;
    }
  }

  // Reset values after scan
  e->RestoreState();

  // Restore the settings of the strobe delay register
  for(module=0; module<MAX_MODULES; module++){
    if(module<e->GetMaxModules()){
      if(e->m[module]->present){
        for(link=0;link<2;link++){
          for(chip=0;chip<6;chip++){
            e->ConfigureVariable(module,((link*6)+chip),3,sd[module][link][chip]);
          }
        }
      }
    }
  }

  // Restore the previous setting of trig_delay
  e->ConfigureVariable(27,td);

  for (module=0; module<e->GetMaxModules(); module++){
    if(e->m[module]->present){
      e->m[module]->use_ch_maxima=0;
    }
  }

  //
  // Call the analysis macro.
  // Arguments are:
  //  run, n_points, scans, charges,
  //  graphics(=0), postscript(=1)
  //

  if(e->abort==0){
    TimewalkPlot(e->runnum, n_points, scans, charges, 0,1);
  }
}

void Timewalk() {

  Timewalk(0);
}


