void TimewalkPlot(Int_t run, Int_t n_points, Int_t* scan, Float_t* charge,
                    Int_t graphics, Int_t postscript) {

  // version 0.1 PWP 24.11.00
  // version 0.2 PWP 06.12.00
  // version 0.3 PWP 06.06.01 - use parameter.h and Toolbox.cpp
  // version 0.31 PWP 21.08.01 - database compliance
  // version 0.32 PWP 12.09.01 - further DB compliance; lower MAX_TX from 20 to 16
  // version 0.33 PWP 28.01.02 - use 12 names for chip histograms, not 6; delete when done
  // version 0.34 PWP 21.03.02 - add web link to postscript plot if WEB_KEY defined
  //                             place leading zero in PS file timestamp
  // version 0.35 BJG 30.04.02 - Put DUT type on plots
  //                             Variable type fixes
  //                             Fix some memory leaks
  //
  // Plot the timewalk curves for all channels of each module.
  // Can be used with between 2 and 16 delay scans taken with
  // differing values of injected charge.  The scale is calibrated
  // from the width of the strobe delay peak of the final scan -
  // hence this macro should only be run with EDGE DETECT ON and
  // EDGE COMPRESSION.
  //
  // STILL UNDER DEVELOPMENT!!!

  #ifdef DATA_DIRECTORY
    // do not reload parameters.h
  #else
    // must load parameters.h
  #include <parameters.h>
  #endif

  #define MAX_TWSCANS    16

  #define MIN_TW  5
  #define MAX_TW 16 

  Int_t module, link, chip, chan, i;
  char f_name[128], h_name[64], h_title[64], line[1024];
  char mname[MAX_MODULES][32];
  char devtype[MAX_MODULES][32];

  Short_t code[MAX_MODULES][MAX_LINKS][MAX_CHANS][MAX_TWSCANS];
  Float_t mean[MAX_MODULES][MAX_LINKS][MAX_CHANS][MAX_TWSCANS];
  Float_t sigma[MAX_MODULES][MAX_LINKS][MAX_CHANS][MAX_TWSCANS];

  Short_t module_status[MAX_MODULES][MAX_LINKS];

  Int_t xchan[MAX_CHANS];
  Float_t timewalk[MAX_LINKS][MAX_CHANS];
  Float_t timewalk2[MAX_LINKS][MAX_CHANS];
  Float_t chip_fall[MAX_LINKS][MAX_CHIPS];
  Float_t chip_rise[MAX_LINKS][MAX_CHIPS];
  Float_t chip_cal_factor[MAX_LINKS][MAX_CHIPS];

  Float_t mean_timewalk[MAX_LINKS][MAX_CHANS];
  Float_t rms_timewalk[MAX_LINKS][MAX_CHANS];
  Float_t mean_timewalk2[MAX_LINKS][MAX_CHANS];
  Float_t rms_timewalk2[MAX_LINKS][MAX_CHANS];

  // Load other files needed by this one
  gROOT->LoadMacro("stan_util.cpp");
  gROOT->LoadMacro("KwikFit.cpp");
  gROOT->LoadMacro("ToolBox.cpp");
  stan_styles();
  gStyle->SetOptFit(10);
  gStyle->SetOptStat(10);

  // 
  // 1. Ensure that the main storage arrays are cleared to start
  //

  for(module=0;module<MAX_MODULES;module++){
    for(link=0;link<MAX_LINKS;link++){
      for(chan=0;chan<MAX_CHANS;chan++){
        for(i=0;i<MAX_TWSCANS;i++){
          code[module][link][chan][i]=0;
          mean[module][link][chan][i]=0;
          sigma[module][link][chan][i]=0;
        }
      }
      module_status[module][link] = 0;
    }
    sprintf(mname[module]," ");
    sprintf(devtype[module],"");
  }

  TObjString* time  = 0;
  TObjString* place = 0;

  TFile *strun = 0;

  Float_t start =  0.0;
  Float_t stop =  12.0;

  for(i=0;i<MAX_CHANS;i++){
    xchan[i]=i;
  }

  // 
  // 2. Load the Fitted Scan Data
  //

  // 2.1 Find today's date - needed for file output

  TDatime *dt = new TDatime;
  Int_t date = dt->GetDate();
  Int_t the_time = dt->GetTime();
  delete dt;

  // 2.2 Some simple validity tests and helpful (!) messages

  if(n_points<0){
    printf("You requested that %d scans be processed.\n");
    printf("You must be having a laugh!!\n");
    printf("Now calm down and try again!!!\n");
  }

  if(n_points>MAX_TWSCANS){
    printf("WARNING: requested number of scans (%d) exceeds maximum\n",n_points,MAX_TWSCANS);
    printf("         only %d scans will be processed\n");
    n_points = MAX_TWSCANS;
  }


  // 2.3 Loop to load files

  for(i=0;i<n_points;i++){

    // 2.3.1 Open file

    sprintf(f_name,"%sstrun%d_%d.root",DATA_DIRECTORY,run,scan[i]);
    TFile *strun = new TFile(f_name);

    if(strun==0){
      printf("Failed to open file %s - EXITING!!\n",f_name);
      return;
    }

    // 2.3.2 Find histograms and fill arrays

    for (module=0; module<MAX_MODULES; module++){
      if(i==0){
        if(time) delete time;
        if(place) delete place;

        TObjString *name = 0;
        TObjString *type = 0;
        time  = (TObjString*)strun->Get("Time");
        place = (TObjString*)strun->Get("Place");

        sprintf(h_title,"ModuleName;%d",module+1);
        name = (TObjString*)strun->Get(h_title);

        sprintf(h_title,"DUT;%d",module+1);
        type = (TObjString*)strun->Get(h_title);

        if(name) {
          sprintf(mname[module],"%s",name->GetName());
          delete name;
        }
        if(type) {
          sprintf(devtype[module],"%s",type->GetName());
          delete type;
        }
      }

      for(link=0;link<MAX_LINKS;link++){

        sprintf(h_name,"h_scan%d;%d",link,module+1);
        TH2F *h_scan= (TH2F*)strun->Get(h_name);

        sprintf(h_name,"h_code%d;%d",link,module+1);
        TH1S *h_code= (TH1S*)strun->Get(h_name);
        sprintf(h_name,"h_mean%d;%d",link,module+1);
        TH1F *h_mean= (TH1F*)strun->Get(h_name);
        sprintf(h_name,"h_sigma%d;%d",link,module+1);
        TH1F *h_sigma= (TH1F*)strun->Get(h_name);

        if(h_mean==0){ // fitted data does not exist

          if(h_scan->GetMean()>0){ // raw data exists: call KwikFit to fit the data
            strun->Close();
            delete strun;

            KwikFit(run,scan[i],module,1,0,0);
	  
            sprintf(f_name,"%sstrun%d_%d.root",DATA_DIRECTORY,run,scan[i]);
            TFile *strun = new TFile(f_name);

            sprintf(h_name,"h_code%d;%d",link,module+1);
            TH1S *h_code= (TH1S*)strun->Get(h_name);
            sprintf(h_name,"h_mean%d;%d",link,module+1);
            TH1F *h_mean= (TH1F*)strun->Get(h_name);
            sprintf(h_name,"h_sigma%d;%d",link,module+1);
            TH1F *h_sigma= (TH1F*)strun->Get(h_name);
          }
        }
	  
        if(h_mean){ // fitted data now exists
          if(h_mean->GetMean()>0){
            printf("Filling arrays for scan %d charge %f module %d link %d\n",
                    scan[i], charge[i], module, link);
            module_status[module][link] = 1;
	    
            for(chan=0;chan<768;chan++){
              code[module][link][chan][i] = h_code->GetBinContent(chan);
              mean[module][link][chan][i] = h_mean->GetBinContent(chan);
              sigma[module][link][chan][i]= h_sigma->GetBinContent(chan);
            }
          } 
        }

      } // end for(link)  
    } // end for(module)

    if((i+1)<n_points){
      strun->Close();
      delete strun;
    }

  } // end for(scan)

  //
  // 3. Creation of various objects
  //

  TH2F* h_tc[MAX_LINKS][MAX_CHIPS];
  TH1F* h_tw[MAX_LINKS][MAX_CHIPS];
  TH1F* h_cal[MAX_LINKS][MAX_CHIPS];
  TH1F* h_tw2[MAX_LINKS][MAX_CHIPS];

  TF1* f1[MAX_LINKS][MAX_CHIPS];
  TF1* f2[MAX_LINKS][MAX_CHIPS];

  TH1F* h_tw_vch[MAX_LINKS];
  TH1F* h_tw2_vch[MAX_LINKS];

  for(link=0;link<MAX_LINKS;link++){
    for(chip=0;chip<MAX_CHIPS;chip++){

      sprintf(h_name, "h_tc_%d_%d",link,chip);
      sprintf(h_title,"Timewalk Curve chip %d",chip);
      h_tc[link][chip] = new TH2F(h_name,h_title,2,start,stop,2,-0.5,63.5);
      h_tc[link][chip]->SetXTitle("Charge (fC)");
      h_tc[link][chip]->SetYTitle("Strobe Delay (units)");

      sprintf(h_name, "h_tw_%d_%d",link,chip);
      sprintf(h_title,"Timewalk chip %d",chip);
      h_tw[link][chip] = new TH1F(h_name,h_title,128,5,15);
      h_tw[link][chip]->SetXTitle("Timewalk (units)");
      h_tw[link][chip]->SetYTitle("n(channels)");
      h_tw[link][chip]->SetLineColor(kRed);
      h_tw[link][chip]->SetMarkerColor(kRed);

      sprintf(h_name, "h_cal_%d_%d",link,chip);
      sprintf(h_title,"Calibration Factor chip %d",chip);
      h_cal[link][chip] = new TH1F(h_name,h_title,128,0,32);
      h_cal[link][chip]->SetXTitle("Calibration Factor (units)");
      h_cal[link][chip]->SetYTitle("n(channels)");
      h_cal[link][chip]->SetLineColor(kRed);
      h_cal[link][chip]->SetMarkerColor(kRed);

      sprintf(h_name, "h_tw2_%d_%d",link,chip);
      sprintf(h_title,"Timewalk chip %d",chip);
      h_tw2[link][chip] = new TH1F(h_name,h_title,128,5,15);
      h_tw2[link][chip]->SetXTitle("Timewalk (nS)");
      h_tw2[link][chip]->SetYTitle("n(channels)");
      h_tw2[link][chip]->SetLineColor(kRed);
      h_tw2[link][chip]->SetMarkerColor(kRed);

      sprintf(h_name,"f1_%d_%d",link,chip);
      f1[link][chip]= new TF1 (h_name,erfc,  0, 1000, 3);
      f1[link][chip]->SetParNames("Constant","Mean","Sigma");

      sprintf(h_name,"f2_%d_%d",link,chip);
      f2[link][chip]= new TF1 (h_name,erf,  0, 1000, 3);
      f2[link][chip]->SetParNames("Constant","Mean","Sigma");

    } // end for(chip)

    sprintf(h_name, "h_tw_vch_%d",link);
    sprintf(h_title,"Timewalk link %d (units)",link);
    h_tw_vch[link] = new TH1F(h_name,h_title,768,-0.50,767.5);
    h_tw_vch[link]->SetXTitle("Channel Number");
    h_tw_vch[link]->SetYTitle("Timewalk (units)");
    h_tw_vch[link]->SetLineColor(kRed);
    h_tw_vch[link]->SetMarkerColor(kRed);

    sprintf(h_name, "h_tw2_vch_%d",link);
    sprintf(h_title,"Timewalk link %d (nS)",link);
    h_tw2_vch[link] = new TH1F(h_name,h_title,768,-0.50,767.5);
    h_tw2_vch[link]->SetXTitle("Channel Number");
    h_tw2_vch[link]->SetYTitle("Timewalk (nS)");
    h_tw2_vch[link]->SetLineColor(kRed);
    h_tw2_vch[link]->SetMarkerColor(kRed);

  } // end for(link)

  //
  // Open the output canvas
  //

  Int_t dowait=0;
  Int_t ps_page=0;
  Int_t screen_page=0;

  TCanvas* c3 =0;
  TPostScript* ps =0;

  if(graphics!=0){
    dowait=1;
  }

  if((graphics!=0)||(postscript!=0)){
    graphics = 1;
    c3= new TCanvas ("c3","TC",1100,750);
  }


  //
  // 4. Loop over modules
  //


  TGraph* TimewalkGraph[MAX_LINKS][MAX_CHANS];
  TH1F* hp_chip[MAX_LINKS][MAX_CHIPS];
//  TMarker* FallMark;
//  TMarker* RiseMark;

  Float_t lx[MAX_TWSCANS];
  Float_t ly[MAX_TWSCANS];

  for(module=0;module<MAX_MODULES;module++){

    // Initialise pointers to graph and histogram objects

    for(link=0;link<MAX_LINKS;link++){
      for(chan=0;chan<MAX_CHANS;chan++){
        TimewalkGraph[link][chan]=0;
      }
    }

    for(link=0;link<MAX_LINKS;link++){
      for(chip=0;chip<MAX_CHIPS;chip++){
        hp_chip[link][chip]=0;
      }
    }

//    FallMark = 0;
//    RiseMark = 0;

    for(link=0;link<MAX_LINKS;link++){
      if(module_status[module][link]>0){

        // Reset the arrays and objects used for storage

        for(chip=0;chip<MAX_CHIPS;chip++){
          h_tw[link][chip]->Reset();
          h_tw2[link][chip]->Reset();
          chip_fall[link][chip]=-1;
          chip_rise[link][chip]=-1;
          chip_cal_factor[link][chip]=-1;
        }
        h_tw_vch[link]->Reset();
        h_tw2_vch[link]->Reset();

        printf("Calculating Calibration Factor for module %d link %d\n",module,link);

        sprintf(h_name,"h_scan%d;%d",link,module+1);
        TH2F *h_scan= (TH2F*)strun->Get(h_name);

        for(chip=0;chip<MAX_CHIPS;chip++){

          sprintf(h_name,"hp_chip_%d_%d",link,chip);
          h_scan->ProjectionY(h_name,(chip*128),((chip+1)*128)-1);
   
          hp_chip[link][chip] = (TH1F*) gDirectory->GetList()->FindObject(h_name);

          if(hp_chip[link][chip]->GetMaximum()>0.0){ // data for this chip

            sprintf(h_name,"%s chip %d", hp_chip[link][chip]->GetTitle(),chip);
            hp_chip[link][chip]->SetTitle(h_name);
            hp_chip[link][chip]->SetXTitle("Strobe Delay");
            hp_chip[link][chip]->SetYTitle("Efficiency");
            hp_chip[link][chip]->Scale(1.0/hp_chip[link][chip]->GetMaximum());

            Float_t centre = (find_bin2(hp_chip[link][chip],0.99,0,1)+find_bin2(hp_chip[link][chip],0.99,1,0))/2;


            // Falling edge

            Float_t p0_guess = hp_chip[link][chip]->GetMaximum();
            Float_t p1_guess = find_level2(hp_chip[link][chip],0.50,1);          // last bin > 50%
            Float_t p2_guess = find_level2(hp_chip[link][chip],0.82,1)-p1_guess;  
            Float_t range_lo = centre;
            Float_t range_hi = hp_chip[link][chip]->GetBinCenter(hp_chip[link][chip]->GetNbinsX());
            if(p2_guess<0) p2_guess = p2_guess*-1.0;

            f1[link][chip]->SetParameters(p0_guess,p1_guess,p2_guess);
            f1[link][chip]->SetParLimits(0,p0_guess,p0_guess);
            f1[link][chip]->SetRange(range_lo, range_hi);

            sprintf(h_name,"f1_%d_%d",link,chip);
            hp_chip[link][chip]->Fit(h_name,"NRQ"); // no graphics, quiet mode
            chip_fall[link][chip] = f1[link][chip]->GetParameter(1); 


            // Rising edge

            p0_guess = hp_chip[link][chip]->GetMaximum();
            p1_guess = find_level2(hp_chip[link][chip],0.50,0);          // first bin < 50%
            p2_guess = find_level2(hp_chip[link][chip],0.82,0)-p1_guess;
            range_lo = hp_chip[link][chip]->GetBinCenter(1);
            range_hi = centre;
            if(p2_guess<0) p2_guess = p2_guess*-1.0;
      
            f2[link][chip]->SetParameters(p0_guess,p1_guess,p2_guess);
            f2[link][chip]->SetParLimits(0,p0_guess,p0_guess);
            f2[link][chip]->SetRange(range_lo, range_hi);

            sprintf(h_name,"f2_%d_%d",link,chip);
            hp_chip[link][chip]->Fit(h_name,"NRQ"); // no graphics, quiet mode
            chip_rise[link][chip] = f2[link][chip]->GetParameter(1);

            // Calibration Factor
            chip_cal_factor[link][chip]=chip_fall[link][chip]-chip_rise[link][chip];

          } // end if data for chip
        } // end for(chip) 

        printf("Calculating Timewalk for module %d link %d\n",module,link);

        Int_t lp = 0;
        for(chan=0;chan<MAX_CHANS;chan++){
          chip = chan/128;

          // Exclude bad vt50 points (fit errors) from the graph 
	  
	  lp = 0;
          for(i=0;i<n_points;i++){
            if((code[module][link][chan][i]&BAD_TEST)==0){
              lx[lp]=charge[i];
              ly[lp]=mean[module][link][chan][i];
              lp++;
            }
          }

          if(lp>1){

            TimewalkGraph[link][chan] = new TGraph(lp,lx,ly);
            TimewalkGraph[link][chan]->SetLineColor(chan%128);
            TimewalkGraph[link][chan]->SetMarkerColor(chan%128);
          }else{
            TimewalkGraph[link][chan]=0;
          }

          timewalk[link][chan] = (float)(mean[module][link][chan][n_points-1]-mean[module][link][chan][0]);
          timewalk2[link][chan] = (timewalk[link][chan]*25.0)/chip_cal_factor[link][chip];

          if(timewalk[link][chan]>0){
            h_tw[link][chip]->Fill(timewalk[link][chan]);
            h_tw2[link][chip]->Fill(timewalk2[link][chan]);

            h_tw_vch[link]->Fill(chan,timewalk[link][chan]);
            h_tw2_vch[link]->Fill(chan,timewalk2[link][chan]);
          }
        } // end for(chan)

      }
    } // end for(link)

      //
      // 6. Plot the Results if the canvas has been opened
      //

    if((module_status[module][0]>0)||(module_status[module][1]>0)){
      if(c3){
        if(postscript){
          ps_page=0;
          sprintf(f_name,"%s%s_TimewalkPlot_%d_%06d.ps",PS_DIRECTORY,mname[module],date,the_time);
          ps = new TPostScript(f_name,112);
        }

        for(link=0;link<MAX_LINKS;link++){   // loop over links

          if((screen_page>0)&&(dowait)){
            printf("Enter a string to continue\n");
            scanf("%s",line);
          }

          printf("Generating Plots for module %d link %d\n",module,link);

          ps_page++;
          screen_page++;
          if(ps){
            ps->NewPage();
          }

          c3->cd();
          c3->Clear();

          TPad* pad_text   = new TPad("pad_text", "Text",        0.00,0.90,1.00,1.00,33);
          TPad* pad_plot   = new TPad("pad_plot", "Plots",       0.00,0.00,1.00,0.90,33);

          pad_text->Draw();
          pad_plot->Draw();
          pad_plot->Divide(6,4,0,0);

          c3->Update();
          pad_text->cd();

          TPaveText* t1 = new TPaveText(0.0,0.0,1.0,1.0);
          t1->SetFillColor(10);
          t1->SetTextAlign(13);
          t1->SetTextSize(0.3);
          t1->SetTextColor(4);

          // ATLAS SCT Module Test:  Timewalk Curve  -  time  -  place
          sprintf(line,"ATLAS SCT Module Test:  Timewalk Curve");
          if(time)  sprintf(line,"%s  -  %s",line,time->GetName());
          if(place) sprintf(line,"%s  -  %s",line,place->GetName());
          t1->AddText(line);

          // Page %d   Run %d   Start Scan %d   Module %d (%s)  Stream %d - Type %s
          sprintf(line,"Page %d   Run %d   Start Scan %d   Stream %d  Module %d (%s)", 
                  ps_page,run, scan[0],link,module,mname[module]);
          if(strlen(devtype[module]) > 0) 
            sprintf(line, "%s - Type %s", line, devtype[module]);
          t1->AddText(line);
          t1->Draw();
          t1->SetBit(kCanDelete);
  
          for(chip=0;chip<6;chip++){
            pad_plot->cd(1+chip);
            h_tc[link][chip]->Draw();
            for(chan=(chip*128);chan<((chip+1)*128);chan++){
              if(TimewalkGraph[link][chan]){
                TimewalkGraph[link][chan]->Draw("LP");
              }
            }

            if(hp_chip[link][chip]){
              pad_plot->cd(7+chip);
              hp_chip[link][chip]->Draw();

              f1[link][chip]->SetLineColor(kBlue);
              f1[link][chip]->Draw("same");
              f2[link][chip]->SetLineColor(kBlue);
              f2[link][chip]->Draw("same");

              TMarker* FallMark=new TMarker (chip_fall[link][chip],0.5,23); 
              FallMark->SetMarkerColor(kBlue);
              FallMark->SetMarkerSize(1);
              FallMark->SetMarkerStyle(23);
              FallMark->Draw("same");
              FallMark->SetBit(kCanDelete);
 
              TMarker* RiseMark=new TMarker (chip_rise[link][chip],0.5,23); 
              RiseMark->SetMarkerColor(kBlue);
              RiseMark->SetMarkerSize(1);
              RiseMark->SetMarkerStyle(23);
              RiseMark->Draw("same");
              RiseMark->SetBit(kCanDelete);
     
              TText* t = new TText();
              sprintf(h_name,"%2.1f",chip_fall[link][chip]);
              t->DrawText(chip_fall[link][chip]-1,0.35,h_name);
              sprintf(h_name,"%2.1f",chip_rise[link][chip]);
              t->DrawText(chip_rise[link][chip]-1,0.35,h_name);
              sprintf(h_name,"%2.1f",chip_cal_factor[link][chip]);
              t->DrawText((chip_fall[link][chip]-chip_rise[link][chip])-1,0.35,h_name); 
              t->SetBit(kCanDelete);
            }
 
            pad_plot->cd(13+chip);
            h_tw[link][chip]->Draw();
            mean_timewalk[link][chip]=h_tw[link][chip]->GetMean();
            rms_timewalk[link][chip]=h_tw[link][chip]->GetRMS();

            pad_plot->cd(19+chip);
            h_tw2[link][chip]->Draw();
            mean_timewalk2[link][chip]=h_tw2[link][chip]->GetMean();
            rms_timewalk2[link][chip]=h_tw2[link][chip]->GetRMS();
            printf("  link %d chip %d tw %2.1f(%2.1f) units \ttw2 %2.1f(%2.1f) nS\n",link,chip,
            mean_timewalk[link][chip],rms_timewalk[link][chip],
            mean_timewalk2[link][chip],rms_timewalk2[link][chip]);
          }

          c3->Update();

          if((screen_page>0)&&(dowait)){
            printf("Enter a string to continue\n");
            scanf("%s",line);
          }

          printf("Generating Summary Plots for module %d link %d\n",module,link);

          ps_page++;
          screen_page++;
          if(ps){
            ps->NewPage();
          }

          c3->cd();
          c3->Clear();

          pad_text = new TPad("pad_text", "Text",        0.00,0.90,1.00,1.00,33);
          pad_plot = new TPad("pad_plot", "Plots",       0.00,0.00,1.00,0.90,33);

          pad_text->Draw();
          pad_plot->Draw();
          pad_plot->Divide(1,2,0,0);

          c3->Update();
          pad_text->cd();

          t1 = new TPaveText(0.0,0.0,1.0,1.0);
          t1->SetFillColor(10);
          t1->SetTextAlign(13);
          t1->SetTextSize(0.3);
          t1->SetTextColor(4);

          // ATLAS SCT Module Test:  Timewalk Curve  -  time  -  place
          sprintf(line,"ATLAS SCT Module Test:  Timewalk");
          if(time)  sprintf(line,"%s  -  %s",line,time->GetName());
          if(place) sprintf(line,"%s  -  %s",line,place->GetName());
          t1->AddText(line);

          // Page %d   Run %d   Start Scan %d   Module %d (%s)  Stream %d - Type %s
          sprintf(line,"Page %d   Run %d   Start Scan %d   Stream %d  Module %d (%s)", 
                  ps_page,run, scan[0],link,module,mname[module]);
          if(strlen(devtype[module]) > 0) 
            sprintf(line, "%s - Type %s", line, devtype[module]);
          t1->AddText(line);

          t1->Draw();
          t1->SetBit(kCanDelete);

          pad_plot->cd(1);
          sprintf(h_name,"Timewalk (units) module %d link %d",module,link);
          h_tw_vch[link]->SetTitle(h_name);
          h_tw_vch[link]->Draw("P");

          pad_plot->cd(2);
          sprintf(h_name,"Timewalk (nS) module %d link %d",module,link);
          h_tw2_vch[link]->SetTitle(h_name);
          h_tw2_vch[link]->Draw("P");

          c3->Update();

        } //end for(link)

        if(ps) {
          ps->Close();
          delete ps;
        }
      } // end if(c3)

      Int_t pass=1;
      for(link=0;link<MAX_LINKS;link++){
        for(chip=0;chip<MAX_CHIPS;chip++){
          if(mean_timewalk2[link][chip]<MIN_TW) pass=0;
          if(mean_timewalk2[link][chip]>MAX_TW) pass=0;
        }
      }

      //
      // 7. Write the results to an ascii file, one per module, suited to the database
      //


      // Open today's output file
      // Output from a series of tests will be added to this file

      FILE* outfile = OpenResults("Timewalk", strun, module, run, scan[0], pass,0); 

      if(outfile==NULL){
        printf("Could not open result file %s\n",f_name);
        return;
      } else {

        ListScanInfo(outfile,"QCAL (fC)",n_points,charge);

        fprintf(outfile,"#\n");
        fprintf(outfile,"%%Timewalk\n");
        fprintf(outfile,"#\n");

        fprintf(outfile,"#TW\n");
        for(link=0;link<MAX_LINKS;link++){
          fprintf(outfile,"#");
          for(chip=0;chip<MAX_CHIPS;chip++){
            fprintf(outfile,"%s\t",MseLabel(link,chip));
          }
          fprintf(outfile,"\n");
          for(chip=0;chip<MAX_CHIPS;chip++){
            fprintf(outfile,"%2.1f \t", mean_timewalk2[link][chip]);
          }
          fprintf(outfile,"\n#\n");
        }

        fprintf(outfile,"#TCAL\n");
        for(link=0;link<MAX_LINKS;link++){
          fprintf(outfile,"#");
          for(chip=0;chip<MAX_CHIPS;chip++){
            fprintf(outfile,"%s\t",MseLabel(link,chip));
          }
          fprintf(outfile,"\n");
          for(chip=0;chip<MAX_CHIPS;chip++){
            fprintf(outfile,"%2.1f \t", chip_cal_factor[link][chip]);
          }
          fprintf(outfile,"\n#\n");
        }


        Int_t n_defects=0;
        for(link=0;link<MAX_LINKS;link++){
          for(chip=0;chip<MAX_CHIPS;chip++){
            if(mean_timewalk2[link][chip]<MIN_TW){
              ListDefect(outfile,"TW_LO",(((6*link)+chip)*128),(((6*link)+chip+1)*128)-1);
              n_defects++; 
            }
            if(mean_timewalk2[link][chip]>MAX_TW){
              ListDefect(outfile,"TW_HI",(((6*link)+chip)*128),(((6*link)+chip+1)*128)-1);
              n_defects++; 
            }
          }
        }
        if(n_defects==0){
          fprintf(outfile,"#No defects found!\n");
        }else{
          fprintf(outfile,"#%d defects found\n",n_defects);
        }

        // Add a URL to the postscript file

  #ifdef WEB_KEY        
        if(postscript){
          sprintf(f_name,"%s%s_TimewalkPlot_%d_%06d.ps",WEB_KEY,mname[module],date,the_time);
          ListWebLink(outfile,"Plots (postscript)",f_name);
        }
  #endif

        fclose(outfile);
      }

    } else {
      printf("No data for module %d\n",module);
    }

    printf("Deleting Graph and Histogram Objects\n");

    for(link=0;link<MAX_LINKS;link++){
      for(chan=0;chan<MAX_CHANS;chan++){
        if(TimewalkGraph[link][chan]) TimewalkGraph[link][chan]->Delete();
      }
    }

    for(link=0;link<MAX_LINKS;link++){
      for(chip=0;chip<MAX_CHIPS;chip++){
        if(hp_chip[link][chip]) hp_chip[link][chip]->Delete();
      }
    }

  } // end for(module)


  printf("Deleting Remaining Objects\n");

  for(link=0;link<MAX_LINKS;link++){
    for(chip=0;chip<MAX_CHIPS;chip++){
      f1[link][chip]->Delete();
      f2[link][chip]->Delete();
      h_tc[link][chip]->Delete();
      h_tw[link][chip]->Delete();
      h_cal[link][chip]->Delete();
      h_tw2[link][chip]->Delete();
    }
    h_tw_vch[link]->Delete();
    h_tw2_vch[link]->Delete();
  }

  if(time) delete time;
  if(place) delete place;

  if(c3) c3->Delete();
  if(strun) {
    strun->Close();
    delete strun;
  }
}

void TimewalkPlot(Int_t run, Int_t start_scan) {

  Int_t n_points = 10;
  Int_t scans[10];
  Float_t charges[10] = {1.25,1.50,1.75,2.00, 3.00,
                        4.00,5.00,6.00,8.00,10.00};
  
  for(int i=0;i<n_points;i++) scans[i]=i+start_scan;

  TimewalkPlot(run, n_points, scans, charges, 0, 1);
} 

