// Macro containing common functionality used by the SCTDAQ
// module testing macros. 

// version 0.1 LE  14.05.01
// version 0.2 PWP 23.05.01
// version 0.21 PWP 14.06.01 add one printf #....
// version 0.22 PWP 30.07.01 modify to better match datatbase requirements;
//                           write both temp values to file if they exist
// version 0.23 PWP 06.07.01 make FindTP, FindRC return -1 if not found
// version 0.24 PWP 21.08.01 revised for database compatibility
// version 0.25 PWP 12.09.01 and again
// version 0.26 PWP 12.09.01 and again, also add routine OpenRaw
// version 0.30 PWP 14.11.01 Revised to suit new DB format
// version 0.31 PWP 28.01.02 Trivial bugfix to FindTP
// version 0.32 PWP 29.01.02 Fix memory leaks (TDaTime objects)
// version 0.33 PWP 21.03.02 add ListComment, ListTestImage and ListWebLink
// version 0.34 BJG 30.04.02 More memory leaks, (TObjStrings from TFile not deleted)
// version 0.35 PWP 20.08.02 Fix SCAN_INFO table to include dots in lieu of unused fields for DB upload
// version 0.36 PWP 24.09.02 Bugfix re. "DB dots"

FILE* OpenRaw(Char_t *TestName, TFile *strun, 
              Short_t module, Int_t run, Int_t scan){
  
  Char_t f_name[64]; // was 32
  Char_t h_name[32];

  FILE* outfile;
  TObjString* name;

  // Extract parameters from outfile

  sprintf(h_name,"ModuleName;%d",module+1);
  name  = (TObjString*)strun->Get(h_name);

  sprintf(f_name,"%s%s_%s_%d_%d.txt",RESULTS_DIRECTORY,name->GetName(),TestName,run,scan);
  outfile = fopen(f_name,"w");

  delete name;

  if(outfile==NULL){
    printf("Could not open result outfile %s\n",f_name);
    return NULL;
  } else {
    printf("File %s opened to write results\n",f_name);
  }
  return outfile;
}

FILE* OpenResults(Char_t *TestName, TFile *strun, 
                  Short_t module, Int_t run, Int_t scan, Short_t Pass, Short_t Problem){ 
  
  Char_t f_name[64]; // was 32
  Char_t h_name[32];

  FILE* outfile;

  // Find today's date - needed for outfile output
  TDatime *dt = new TDatime;
  Int_t date = dt->GetDate();
  delete dt;

  Int_t hour,min,sec,year;

  // Extract parameters from outfile

  TObjString* time  = 0;
  TObjString* place = 0;
  TObjString* host  = 0;
  TObjString* user  = 0;

  TObjString* dut   = 0;
  TObjString* name  = 0;

  time  = (TObjString*)strun->Get("Time");
  place = (TObjString*)strun->Get("Place");
  host  = (TObjString*)strun->Get("Host");
  user  = (TObjString*)strun->Get("User");

  sprintf(h_name,"DUT;%d",module+1);
  dut  = (TObjString*)strun->Get(h_name);

  sprintf(h_name,"ModuleName;%d",module+1);
  name  = (TObjString*)strun->Get(h_name);

  // Get items for DCS table

  TObjString* t0  = 0;
  TObjString* t1  = 0;
  TObjString* vdet = 0;
  TObjString* idet = 0;
  TObjString* vcc = 0;
  TObjString* icc = 0;
  TObjString* vdd = 0;
  TObjString* idd = 0;
  TObjString* tp  = 0;

  sprintf(h_name,"T0;%d",module+1);
  t0  = (TObjString*)strun->Get(h_name);

  sprintf(h_name,"T1;%d",module+1);
  t1  = (TObjString*)strun->Get(h_name);

  sprintf(h_name,"VDET;%d",module+1);
  vdet  = (TObjString*)strun->Get(h_name);

  sprintf(h_name,"IDET;%d",module+1);
  idet  = (TObjString*)strun->Get(h_name);

  sprintf(h_name,"VCC;%d",module+1);
  vcc  = (TObjString*)strun->Get(h_name);

  sprintf(h_name,"ICC;%d",module+1);
  icc  = (TObjString*)strun->Get(h_name);

  sprintf(h_name,"VDD;%d",module+1);
  vdd  = (TObjString*)strun->Get(h_name);

  sprintf(h_name,"IDD;%d",module+1);
  idd  = (TObjString*)strun->Get(h_name);

  sprintf(h_name,"TP;%d",module+1);
  tp  = (TObjString*)strun->Get(h_name);

  sprintf(f_name,"%s%s_%d.txt",RESULTS_DIRECTORY,name->GetName(),date);
  outfile = fopen(f_name,"a+");
  
  if(outfile==NULL){
    printf("Could not open result outfile %s\n",f_name);
    return NULL;
  } else {
    printf("File %s opened to append results\n",f_name);
    printf("     for %s test\n\n",TestName);

    fprintf(outfile,"#\n");
    fprintf(outfile,"%%NewTest\n");
    fprintf(outfile,"#\n");

    fprintf(outfile,"SERIAL NUMBER  : %s\n",name->GetName());

    if(user){
      fprintf(outfile,"TEST MADE BY   : %s\n",user->GetName());
    }else{
      fprintf(outfile,"TEST MADE BY   : unknown1\n");
    }

    if(place){
      fprintf(outfile,"LOCATION NAME  : %s\n",place->GetName());
    }else{
      fprintf(outfile,"LOCATION NAME  : %s\n",LOCATION_NAME);
    }

    fprintf(outfile,"Run number     : %d-%d\n",run,scan);

    if(time){
      Char_t s_day[4];
      Char_t s_mon[4];

      sscanf(time->GetName(),"%s %s %d %d:%d:%d %d",s_day,s_mon,&date,&hour,&min,&sec,&year);

      Int_t mon=0;
      if(strcmp(s_mon,"Jan")==0) mon=1;
      if(strcmp(s_mon,"Feb")==0) mon=2;
      if(strcmp(s_mon,"Mar")==0) mon=3;
      if(strcmp(s_mon,"Apr")==0) mon=4;
      if(strcmp(s_mon,"May")==0) mon=5;
      if(strcmp(s_mon,"Jun")==0) mon=6;
      if(strcmp(s_mon,"Jul")==0) mon=7;
      if(strcmp(s_mon,"Aug")==0) mon=8;
      if(strcmp(s_mon,"Sep")==0) mon=9;
      if(strcmp(s_mon,"Oct")==0) mon=10;
      if(strcmp(s_mon,"Nov")==0) mon=11;
      if(strcmp(s_mon,"Dec")==0) mon=12;

      fprintf(outfile,"TEST_DATE      : %02d/%02d/%04d\n",date,mon,year);
    }

    fprintf(outfile,"PASSED         : ");
    if(Pass==1){
      fprintf(outfile,"YES\n");
    }else{
      fprintf(outfile,"NO\n");
    }

    fprintf(outfile,"PROBLEM        : ");
    if(Problem==1){
      fprintf(outfile,"YES\n");
    }else{
      fprintf(outfile,"NO\n");
    }


    fprintf(outfile,"#\n");
    fprintf(outfile,"%%DAQ_INFO\n");
    fprintf(outfile,"#\n");

    fprintf(outfile,"#HOST\n");
    if(host){fprintf(outfile,"\"%s\"\n",host->GetName());}
    else{fprintf(outfile,".\n");}
    fflush(outfile);

    fprintf(outfile,"#VERSION\n");
    fprintf(outfile,"\"%s\"\n",ANALYSIS_VERSION);

    fprintf(outfile,"#DUT\n");
    if(dut){fprintf(outfile,"\"%s\"\n",dut->GetName());}
    else{fprintf(outfile,".\n");}

    fprintf(outfile,"#TIME\n");
    fprintf(outfile,"\"%02d:%02d:%02d\"\n",hour,min,sec);


    fprintf(outfile,"#\n");
    fprintf(outfile,"%%DCS_INFO\n");
    fprintf(outfile,"#\n");

    fprintf(outfile,"#T0\tT1\n");
    if(t0){fprintf(outfile,"%s \t",t0->GetName());}
    else{fprintf(outfile,". \t");}
    if(t1){fprintf(outfile,"%s \t",t1->GetName());}
    else{fprintf(outfile,". \t");}

    fprintf(outfile,"\n#VDET\tIDET\n");
    if(vdet){fprintf(outfile,"%s \t",vdet->GetName());}
    else{fprintf(outfile,". \t");}
    if(idet){fprintf(outfile,"%s \t",idet->GetName());}
    else{fprintf(outfile,". \t");}

    fprintf(outfile,"\n#VCC\tICC\n");
    if(vcc){fprintf(outfile,"%s \t",vcc->GetName());}
    else{fprintf(outfile,". \t");}
    if(icc){fprintf(outfile,"%s \t",icc->GetName());}
    else{fprintf(outfile,". \t");}

    fprintf(outfile,"\n#VDD\tIDD\n");
    if(vdd){fprintf(outfile,"%s \t",vdd->GetName());}
    else{fprintf(outfile,". \t");}
    if(idd){fprintf(outfile,"%s \t",idd->GetName());}
    else{fprintf(outfile,". \t");}

    fprintf(outfile,"\n#TIME_POWERED\n");
    if(tp){fprintf(outfile,"%s \n",tp->GetName());}
    else{fprintf(outfile,". \n");}


    fprintf(outfile,"#\n");
    fflush(outfile);
  }

  // Delete these now we've used them
  if(time) { delete time; }
  if(place) { delete place; }
  if(host) { delete host; }
  if(user) { delete user; }
  if(dut) { delete dut; }
  if(name) { delete name; }

  if(t0) { delete t0; }
  if(t1) { delete t1; }
  if(vdet) { delete vdet; }
  if(idet) { delete idet; }
  if(vcc) { delete vcc; }
  if(icc) { delete icc; }
  if(vdd) { delete vdd; }
  if(idd) { delete idd; }
  if(tp) { delete tp; }

  return outfile;
}


Int_t FindRCData(Char_t* TestName, TFile *strun, Short_t module, 
                 Short_t RCFunction[12], Float_t RCData[12][3]){ 

  // Objective: find the most recent ModuleResponseCurve for this module
  // (most recent occurence in any result file)

  Char_t f_name[64]; // was 32
  Char_t h_name[32];


  // Find today's date - needed for outfile output
  TDatime *dt = new TDatime;
  Int_t date = dt->GetDate();
  Int_t time = dt->GetDate();
  Int_t length = strlen(TestName);
  delete dt;

  // Extract parameters from outfile
  sprintf(h_name,"ModuleName;%d",module+1);
  TObjString* name = (TObjString*)strun->Get(h_name);

  FILE* infile = NULL;
  Char_t line[256];

  Int_t found=0;
  Int_t loop=0;

  Int_t chip, func;
  Float_t p0,p1,p2;

  while((found==0)&&(loop<30)){

    sprintf(f_name,"%s%s_%d.txt",RESULTS_DIRECTORY,name->GetName(),date);
    infile = fopen(f_name,"r");
  
    if(infile==NULL){
      printf("Could not open result infile %s\n",f_name);
      printf("Try one day previous....\n");
    } else {
      printf("File %s opened to read results\n",f_name);
      printf("Checking for %s test\n\n",TestName);

      while(fgets(line,160,infile)!=NULL){
        if(strncmp(line,"%ResponseCurve",14)==0){
          printf("%s found in NEW format\n",TestName);
          found=1;

          // skip four lines to begin
          for(chip=0;chip<4;chip++){
            fgets(line,160,infile);
          }

          // Now read the fit data
          for(chip=0;chip<12;chip++){
            fgets(line,160,infile);
            sscanf(line,"%d %f %f %f", &func,&p0,&p1,&p2);
            printf("Chip %d %d %3.2f %3.2f %3.2f\n",chip,func,p0,p1,p2);
              
            RCFunction[chip]=func;
            RCData[chip][0]=p0;
            RCData[chip][1]=p1;
            RCData[chip][2]=p2;

            // Skip next line
            fgets(line,160,infile);
          }
        }else if(strncmp(line,"%ModuleResponseCurve",20)==0){
          printf("%s found in OLD format\n",TestName);
          found=1;

          Int_t row=0;
          while((fgets(line,160,infile)!=NULL)&&(row<12)){
            if(strncmp(line,"Chip",4)==0){
              sscanf(line,"%s %d: %d %f %f %f", &h_name,&chip,&func,&p0,&p1,&p2);
              printf("%s %d %d %3.2f %3.2f %3.2f\n", h_name,chip,func,p0,p1,p2);
              
              if((chip>=0)&&(chip<12)){
                RCFunction[chip]=func;
                RCData[chip][0]=p0;
                RCData[chip][1]=p1;
                RCData[chip][2]=p2;
              }
              row++;
            }
          }
        }
      }
      fclose(infile);
      infile = NULL;
    }

    if(found==0){
      if((date%100)==1){
        date -= 70;
      } else {
        date--;
      }
      loop++;
    }
  }

  if(name) delete name;

  if(found==0){
    return -1;
  }else{
    return date;
  }
}


Int_t FindTP(Char_t *TableName, TFile *strun, Short_t module, 
             Float_t TrimTarget[12]){ 

  // Objective: find the most recent result file for this module
  // containing a result from result table "TableName"

  Char_t f_name[64]; // was 32
  Char_t h_name[32];


  // Find today's date - needed for outfile output
  TDatime *dt = new TDatime;
  Int_t date = dt->GetDate();
  Int_t time = dt->GetDate();
  delete dt;

  // Extract parameters from outfile
  sprintf(h_name,"ModuleName;%d",module+1);
  TObjString* name = (TObjString*)strun->Get(h_name);

  FILE* infile = NULL;
  Char_t line[256];
  Int_t length = strlen(TableName);

  Int_t found=0;
  Int_t loop=0;

  Int_t chip, range;
  Float_t target;

  while((found==0)&&(loop<30)){

    sprintf(f_name,"%s%s_%d.txt",RESULTS_DIRECTORY,name->GetName(),date);
    infile = fopen(f_name,"r");
  
    if(infile==NULL){
      printf("Could not open result infile %s\n",f_name);
      printf("Try one day previous....\n");
    } else {
      printf("File %s opened to read results\n",f_name);
      printf("Checking for %s test\n\n",TableName);

      while(fgets(line,160,infile)!=NULL){
        if(strncmp(line,"%TrimSummary",12)==0){
          printf("%s found in OLD format\n",TableName);
          found=1;

          Int_t row=0;
          while((fgets(line,160,infile)!=NULL)&&(row<12)){
            if(strncmp(line,"Chip",4)==0){
              sscanf(line,"%s %d: %d %f", &h_name,&chip,&range,&target);
              printf("%s %d: %d %3.2f\n", h_name,chip,range,target);
              if((chip>=0)&&(chip<12)){
                TrimTarget[chip]=target;
              } 
              row++;
            }
          }
        }else if(strncmp(line,"%Trim",5)==0){
          printf("%s found in NEW format\n",TableName);
          found=1;

          // skip seven lines to begin
          for(chip=0;chip<7;chip++){
            fgets(line,160,infile);
          }

          // Now read the fit data
          for(chip=0;chip<12;chip++){
            fgets(line,160,infile);
            sscanf(line,"%d %f", &range,&target);
            printf("Chip %d %d %f\n",chip,range,target);
              
            TrimTarget[chip]=target;

            // Skip next line
            fgets(line,160,infile);
          }
        }
      }
      fclose(infile);
      infile = NULL;
    }

    if(found==0){
      if((date%100)==1){
        date -= 70;
      } else {
        date--;
      }
      loop++;
    }
  }

  if(name) delete name;
  
  if (found==0){
    return -1;
  }else{
    return date;
  }
}


void ListComment(FILE* outfile, char* comment){
  // Append a comment to the results file
  // PWP 21.03.02

  if(outfile==NULL){
    printf("ERROR: ListComment called for NULL file - returning\n");
    return;
  }

  fprintf(outfile,"%%Comment\n");
  fprintf(outfile,"COMMENT        : %s\n",comment);
  fprintf(outfile,"#\n");
}

void ListDefect(FILE* outfile, char* name, Int_t first, Int_t last){
  // Append a defect record to the results file
  // PWP 15.11.01

  if(outfile==NULL){
    printf("ERROR: ListDefect called for NULL file - returning\n");
    return;
  }

  if(first>last){
    printf("ERROR: ListDefect called with first(%d)>last(%d)\n",first,last);
    return;
  }

  fprintf(outfile,"%%Defect\n");
  fprintf(outfile,"DEFECT NAME    : %s\n",name);
  fprintf(outfile,"FIRST CHANNEL  : %d\n",first);
  fprintf(outfile,"LAST CHANNEL   : %d\n",last);
  fprintf(outfile,"#\n");
}

void ListRawData(FILE* outfile, char* name){
  // Append a RawData record to the results file
  // PWP 14.12.01

  if(outfile==NULL){
    printf("ERROR: ListRawData called for NULL file - returning\n");
    return;
  }

  fprintf(outfile,"%%TEST Rawdata\n");
  fprintf(outfile,"FILENAME       : %s\n",name);
  fprintf(outfile,"#\n");
}

void ListTestImage(FILE* outfile, char* filename, char* url, char* title){
  // Append a Test Image to the results file
  // PWP 21.03.02

  if(outfile==NULL){
    printf("ERROR: ListTestImage called for NULL file - returning\n");
    return;
  }

  fprintf(outfile,"%%TEST Image\n");
  fprintf(outfile,"Filename       : %s\n",filename);
  fprintf(outfile,"Url            : %s\n",url);
  fprintf(outfile,"Title          : %s\n",title);
  fprintf(outfile,"#\n");
}

void ListWebLink(FILE* outfile, char* description, char* url){
  // Append a WebLink record to the results file
  // PWP 21.03.02

  if(outfile==NULL){
    printf("ERROR: ListWebLink called for NULL file - returning\n");
    return;
  }

  fprintf(outfile,"%%Web link\n");
  fprintf(outfile,"DESCRIPTION    : %s\n",description);
  fprintf(outfile,"URL            : %s\n",url);
  fprintf(outfile,"#\n");
}

void ListScanInfo(FILE* outfile, char* type, Int_t n_points, Float_t* points){

  if(outfile==NULL){
    printf("ERROR: ListScanInfo called for NULL file - returning\n");
    return;
  }

  if(n_points>16){
    printf("ERROR: ListScanInfo called for too many points (%d)\n",n_points);
    printf("       can list only 16...\n");
    n_points=16;
  }

  fprintf(outfile,"#\n");
  fprintf(outfile,"%%SCAN_INFO\n");
  fprintf(outfile,"#\n");

  fprintf(outfile,"#POINT_TYPE\n");
  fprintf(outfile,"\"%s\"\n",type);
  
  fprintf(outfile,"#N_POINTS\n");
  fprintf(outfile,"%d\n",n_points);
  
  fprintf(outfile,"#POINTS\n");
  for(Int_t i=0;i<n_points;i++){
    fprintf(outfile,"%1.2f \t",points[i]);
    if(i==7){ // PWP bugfix 24.09.02
      fprintf(outfile,"\n");
    }
  }

  // Fix for DB upload
  // insert dots to pad up to 16 fields
  // PWP 20.08.02

  for(i=n_points;i<16;i++){
    fprintf(outfile,". \t");
    if(i==7){ // PWP bugfix 24.09.02
      fprintf(outfile,"\n");
    }
  }

  fprintf(outfile,"\n#\n");
}

char* MseLabel(Int_t link, Int_t chip){
  // Return "M0" type label for given link/chip
  // PWP 15.11.01

  return MseLabel((link*6)+chip);
}

char* MseLabel(Int_t chip){
  // Return "M0" type label for given chip
  // PWP 15.11.01

  switch(chip){
    case 0:
      return "M0 ";
      break;
    case 1:
      return "S1 ";
      break;
    case 2:
      return "S2 ";
      break;
    case 3:
      return "S3 ";
      break;
    case 4:
      return "S4 ";
      break;
    case 5:
      return "E5 ";
      break;
    case 6:
      return "M8 ";
      break;
    case 7:
      return "S9 ";
      break;
    case 8:
      return "S10";
      break;
    case 9:
      return "S11";
      break;
    case 10:
      return "S12";
      break;
    case 11:
      return "E13";
      break;
    default:
      return "???";
      break;
  }
}
