static void SetMask(Short_t mid,Short_t cid){
  e->ConfigureMask(mid,cid);
}

static void SetTrim(Short_t mid,Short_t cid, Short_t var){
  e->ConfigureTrim(mid,cid,var);
}


void TrimScanPlot(Int_t run, Int_t n_points, Int_t method, Int_t trimrange, Float_t charge, Int_t* scan, Float_t* trim,
                    Int_t online, Int_t graphics, Int_t postscript) {

  // version 0.21 PWP 01.11.00
  // version 0.22 PWP 14.11.00 - changes to plotted ranges
  // version 0.23 LE  19.11.00 - automatic compiled fitting if not already done.
  // version 0.24 LE  19.11.00 - chip by chip optimisation.
  // version 0.25 PWP 21.11.00 - various fixes
  // version 0.26 PWP 24.11.00 - add plots for TrimDAC slope and offset
  // version 0.27 PWP 28.11.00 - fixes; hists instead of graphs for par vs channel
  // version 0.28 PWP 22.01.01 - also mask DEAD or STUCK channels
  // version 0.29 LE  16.04.01 - ouput results to file, give trimrange and charge as arguments.
  // version 0.30 PWP 17.05.01 - mask noisy channel (crude); write trimrange settings into trimfile
  // version 0.31 PWP 06.06.01 - use parameter.h and Toolbox.cpp
  // version 0.31 PWP 08.11.01 - small bugfix re. Null RMSTrimGraph
  // version 0.32 PWP 21.03.02 - place leading zero in PS file timestamp
  // version 0.33 PWP 30.04.02 - Put DUT type on plots
  //                             Some memory leak fixes
  //
  // STILL UNDER DEVELOPMENT!!!

  #ifdef DATA_DIRECTORY
    // do not reload parameters.h
  #else
    // must load parameters.h
  #include <parameters.h>
  #endif

  Int_t module, link, chip, chan, i;
  char f_name[128], h_name[64], h_title[64], line[1024];
  char mname[MAX_MODULES][32];
  char mdut[MAX_MODULES][32];
  char s_method[20];

  Short_t code[MAX_MODULES][MAX_LINKS][MAX_CHANS][MAX_POINTS];
  Float_t mean[MAX_MODULES][MAX_LINKS][MAX_CHANS][MAX_POINTS];
  Float_t sigma[MAX_MODULES][MAX_LINKS][MAX_CHANS][MAX_POINTS];
  Float_t mean_noise[MAX_MODULES][MAX_LINKS][MAX_CHANS];
  Float_t noise_cut[MAX_MODULES][MAX_LINKS][MAX_CHIPS];

  Short_t module_status[MAX_MODULES][MAX_LINKS];

  Float_t ntrim[MAX_TARGETS];
  Float_t target[MAX_TARGETS];
  Float_t mean_trim[MAX_LINKS][MAX_CHIPS][MAX_TARGETS];
  Float_t trim_rms[MAX_LINKS][MAX_CHIPS][MAX_TARGETS];

  Float_t trim_setting[MAX_LINKS][MAX_CHANS];
  Float_t vt50_setting[MAX_LINKS][MAX_CHANS];
  Float_t trim_offset[MAX_LINKS][MAX_CHANS];
  Float_t trim_slope[MAX_LINKS][MAX_CHANS];

  Int_t xchan[MAX_CHANS];

  TObjString* time  = 0;
  TObjString* place = 0;

  // Load other files needed by this one
  gROOT->LoadMacro("stan_util.cpp");
  gROOT->LoadMacro("KwikFit.cpp");
  gROOT->LoadMacro("ToolBox.cpp");
  stan_styles();
  gStyle->SetOptFit(10);

  // 
  // 1. Ensure that the main storage arrays are cleared to start
  //

  for(module=0;module<MAX_MODULES;module++){
    for(link=0;link<MAX_LINKS;link++){
      for(chan=0;chan<MAX_CHANS;chan++){
        for(i=0;i<MAX_POINTS;i++){
          code[module][link][chan][i]=0;
          mean[module][link][chan][i]=0;
          sigma[module][link][chan][i]=0;
        }
        mean_noise[module][link][chan]=0;
      }
      for(chip=0;chip<MAX_CHIPS;chip++){
        noise_cut[module][link][chip]=0;
      }
      module_status[module][link] = 0;
    }
    sprintf(mname[module]," ");
    sprintf(mdut[module],"");
  }

  for(i=0;i<MAX_TARGETS;i++){
    ntrim[i] = 0;
    target[i] = 0;
  }

  // Initialise target array
 
  Float_t start = 2.5;
  Float_t stop = 300;  // was 200, but that is not good enough for
                       // all trim ranges PWP 08.12.00 
  Float_t step = 2.5;

  Float_t f_val;
  Int_t n_targets=0;
  for(f_val=start;f_val<=stop;f_val+=step){

    target[n_targets]=f_val;
    n_targets++;
  }

  for(i=0;i<MAX_CHANS;i++){
    xchan[i]=i;
  }

  // 
  // 2. Load the Fitted Scan Data
  //

  // 2.1 Find today's date - needed for file output

  TDatime *dt = new TDatime;
  Int_t date = dt->GetDate();
  Int_t the_time = dt->GetTime();

  // 2.2 Some simple validity tests and helpful (!) messages

  if(n_points<0){
    printf("You requested that %d scans be processed.\n");
    printf("You must be having a laugh!!\n");
    printf("Now calm down and try again!!!\n");
  }

  if(n_points>MAX_POINTS){
    printf("WARNING: requested number of scans (%d) exceeds maximum\n",n_points,MAX_POINTS);
    printf("         only %d scans will be processed\n");
    n_points = MAX_POINTS;
  }


  // 2.3 Loop to load files
  TFile *strun = 0;

  for(i=0;i<n_points;i++){

    // 2.3.1 Open file

    sprintf(f_name,"%sstrun%d_%d.root",DATA_DIRECTORY,run,scan[i]);
    strun = new TFile(f_name);

    if(strun==0){
      printf("Failed to open file %s - EXITING!!\n",f_name);
      return;
    }

    // 2.3.2 Find histograms and fill arrays

    for (module=0; module<MAX_MODULES; module++){
      if(i==0){
        TObjString *name = 0;
        TObjString *dut = 0;

        if(time) delete time;
        if(place) delete place;
		time  = (TObjString*)strun->Get("Time");
        place = (TObjString*)strun->Get("Place");

        sprintf(h_title,"ModuleName;%d",module+1);
        name = (TObjString*)strun->Get(h_title);
        
        sprintf(h_title,"DUT;%d",module+1);
        dut = (TObjString*)strun->Get(h_title);

        if(name) {
          sprintf(mname[module],"%s",name->GetName());
          delete name;
        }
        if(dut) {
          sprintf(mdut[module],"%s",dut->GetName());
          delete dut;
        }
      }

      for(link=0;link<MAX_LINKS;link++){

        sprintf(h_name,"h_scan%d;%d",link,module+1);
        TH2F *h_scan= (TH2F*)strun->Get(h_name);

        sprintf(h_name,"h_code%d;%d",link,module+1);
        TH1S *h_code= (TH1S*)strun->Get(h_name);
        sprintf(h_name,"h_mean%d;%d",link,module+1);
        TH1F *h_mean= (TH1F*)strun->Get(h_name);
        sprintf(h_name,"h_sigma%d;%d",link,module+1);
        TH1F *h_sigma= (TH1F*)strun->Get(h_name);

        if(h_mean==0){ // fitted data does not exist

          if(h_scan->GetMean()>0){ // raw data exists: call KwikFit to fit the data
            strun->Close();
            delete strun;

            KwikFit(run,scan[i],module,1,0,0);
	  
            sprintf(f_name,"%sstrun%d_%d.root",DATA_DIRECTORY,run,scan[i]);
            TFile *strun = new TFile(f_name);

            sprintf(h_name,"h_code%d;%d",link,module+1);
            TH1S *h_code= (TH1S*)strun->Get(h_name);
            sprintf(h_name,"h_mean%d;%d",link,module+1);
            TH1F *h_mean= (TH1F*)strun->Get(h_name);
            sprintf(h_name,"h_sigma%d;%d",link,module+1);
            TH1F *h_sigma= (TH1F*)strun->Get(h_name);
          }
        }
	  
        if(h_mean){ // fitted data now exists
          if(h_mean->GetSum()>0){
            printf("Filling arrays for scan %d trim %f module %d link %d\n",
                    scan[i], trim[i], module, link);
            module_status[module][link] = 1;
	    
            for(chan=0;chan<768;chan++){
              code[module][link][chan][i] = h_code->GetBinContent(chan);
              mean[module][link][chan][i] = h_mean->GetBinContent(chan);
              sigma[module][link][chan][i]= h_sigma->GetBinContent(chan);

              mean_noise[module][link][chan]+=h_sigma->GetBinContent(chan);
              noise_cut[module][link][chan/128]+=h_sigma->GetBinContent(chan);
            }
          } 
        }
      } // end for(link)  
    }

    if(i<(n_points-1)){
      strun->Close();
      delete strun;
    }
  } // end for(module)

  // Noise cuts added PWP 17.05.01

  for(module=0;module<MAX_MODULES;module++){
    for(link=0;link<MAX_LINKS;link++){
      for(chan=0;chan<768;chan++){
        mean_noise[module][link][chan]/=n_points;
      }
      for(chip=0;chip<MAX_CHIPS;chip++){
        noise_cut[module][link][chip]*=1.25;
        noise_cut[module][link][chip]/=(128*n_points);
      }
    }
  }     

  //
  // 3. Creation of various objects
  //

  TF1* f3[MAX_LINKS][MAX_CHANS];

  TH2F* h_trim[MAX_LINKS][MAX_CHIPS];
  TH1S* h_ntrim[MAX_LINKS][MAX_CHIPS];
  TH2F* h_mtrim[MAX_LINKS][MAX_CHIPS];
  TH2F* h_trms[MAX_LINKS][MAX_CHIPS];
  TH1F* h_tp[MAX_LINKS][MAX_CHIPS];
  TH1F* h_ts[MAX_LINKS][MAX_CHIPS];
  TH1F* h_toff[MAX_LINKS][MAX_CHIPS];
  TH1F* h_toss[MAX_LINKS][MAX_CHIPS];

  TH1F* h_ts_vch[MAX_LINKS];
  TH1F* h_tp_vch[MAX_LINKS];
  TH1F* h_toff_vch[MAX_LINKS];
  TH1F* h_toss_vch[MAX_LINKS];

  for(link=0;link<MAX_LINKS;link++){
    for(chip=0;chip<MAX_CHIPS;chip++){

      sprintf(h_name, "h_trim_%d_%d",link,chip);
      sprintf(h_title,"TrimDAC Curve chip %d",chip);
      h_trim[link][chip] = new TH2F(h_name,h_title,2,start,stop,2,-0.5,15.5);
      h_trim[link][chip]->SetXTitle("Threshold (mV)");
      h_trim[link][chip]->SetYTitle("TrimDAC setting (bits)");

      sprintf(h_name, "h_ntrim_%d_%d",link,chip);
      sprintf(h_title,"n(trimmable) chip %d",chip);
      h_ntrim[link][chip] = new TH1S(h_name,h_title,n_targets,start,stop);
      h_ntrim[link][chip]->SetXTitle("Target Threshold (mV)");
      h_ntrim[link][chip]->SetYTitle("n(trimmable)");
      h_ntrim[link][chip]->SetLineColor(kRed);
      h_ntrim[link][chip]->SetMarkerColor(kRed);

      sprintf(h_name, "h_mtrim_%d_%d",link,chip);
      sprintf(h_title,"Mean Trim chip %d",chip);
      h_mtrim[link][chip] = new TH2F(h_name,h_title,2,start,stop,2,-0.5,15.5);
      h_mtrim[link][chip]->SetXTitle("Target Threshold (mV)");
      h_mtrim[link][chip]->SetYTitle("Mean Trim");

      sprintf(h_name, "h_trms_%d_%d",link,chip);
      sprintf(h_title,"RMS(trimmed vt50) chip %d",chip);
      h_trms[link][chip] = new TH2F(h_name,h_title,2,start,stop,2,0,10);
      h_trms[link][chip]->SetXTitle("Target Threshold (mV)");
      h_trms[link][chip]->SetYTitle("RMS(vt50)");

      sprintf(h_name, "h_tp_%d_%d",link,chip);
      sprintf(h_title,"Threshold Distribution chip %d",chip);
      h_tp[link][chip] = new TH1F(h_name,h_title,10*((stop-start)/step),(start-10),(stop-10));
      h_tp[link][chip]->SetXTitle("Threshold (mV)");
      h_tp[link][chip]->SetYTitle("n(channels)");
      h_tp[link][chip]->SetLineColor(kRed);
      h_tp[link][chip]->SetMarkerColor(kRed);

      sprintf(h_name, "h_ts_%d_%d",link,chip);
      sprintf(h_title,"Trim Distribution chip %d",chip);
      h_ts[link][chip] = new TH1F(h_name,h_title,16,-0.50,15.5);
      h_ts[link][chip]->SetXTitle("Trim Setting");
      h_ts[link][chip]->SetYTitle("n(channels)");
      h_ts[link][chip]->SetLineColor(kRed);
      h_ts[link][chip]->SetMarkerColor(kRed);

      sprintf(h_name, "h_toff_%d_%d",link,chip);
      sprintf(h_title,"TrimDAC Offset chip %d",chip);
      h_toff[link][chip] = new TH1F(h_name,h_title,40,-20,20);
      h_toff[link][chip]->SetXTitle("TrimDAC Offset");
      h_toff[link][chip]->SetYTitle("n(channels)");
      h_toff[link][chip]->SetLineColor(kRed);
      h_toff[link][chip]->SetMarkerColor(kRed);

      sprintf(h_name, "h_toss_%d_%d",link,chip);
      sprintf(h_title,"TrimDAC Slope chip %d",chip);
      h_toss[link][chip] = new TH1F(h_name,h_title,40,-20,20);
      h_toss[link][chip]->SetXTitle("TrimDAC Slope");
      h_toss[link][chip]->SetYTitle("n(channels)");
      h_toss[link][chip]->SetLineColor(kRed);
      h_toss[link][chip]->SetMarkerColor(kRed);

    } // end for(chip)

    for(chan=0;chan<MAX_CHANS;chan++){

      sprintf(h_name,"f3_%d_%d",link,chan);
      f3[link][chan]= new TF1 (h_name,"pol1",start,stop);
      f3[link][chan]->SetLineColor(chan%32);
      f3[link][chan]->SetMarkerColor(chan%32);

    } // end for(chan)

    sprintf(h_name, "h_ts_vch_%d",link);
    sprintf(h_title,"Trim Setting link %d",link);
    h_ts_vch[link] = new TH1F(h_name,h_title,768,-0.50,767.5);
    h_ts_vch[link]->SetXTitle("Channel Number");
    h_ts_vch[link]->SetYTitle("Trim Setting");
    h_ts_vch[link]->SetLineColor(kRed);
    h_ts_vch[link]->SetMarkerColor(kRed);

    sprintf(h_name, "h_tp_vch_%d",link);
    sprintf(h_title,"Trimmed Threshold link %d",link);
    h_tp_vch[link] = new TH1F(h_name,h_title,768,-0.50,767.5);
    h_tp_vch[link]->SetXTitle("Channel Number");
    h_tp_vch[link]->SetYTitle("Trimmed Threshold (mV)");
    h_tp_vch[link]->SetLineColor(kRed);
    h_tp_vch[link]->SetMarkerColor(kRed);

    sprintf(h_name, "h_toff_vch_%d",link);
    sprintf(h_title,"TrimDAC Offset link %d",link);
    h_toff_vch[link] = new TH1F(h_name,h_title,768,-0.50,767.5);
    h_toff_vch[link]->SetXTitle("Channel Number");
    h_toff_vch[link]->SetYTitle("TrimDAC Offset (mV)");
    h_toff_vch[link]->SetLineColor(kRed);
    h_toff_vch[link]->SetMarkerColor(kRed);

    sprintf(h_name, "h_toss_vch_%d",link);
    sprintf(h_title,"TrimDAC Slope link %d",link);
    h_toss_vch[link] = new TH1F(h_name,h_title,768,-0.50,767.5);
    h_toss_vch[link]->SetXTitle("Channel Number");
    h_toss_vch[link]->SetYTitle("TrimDAC Slope");
    h_toss_vch[link]->SetLineColor(kRed);
    h_toss_vch[link]->SetMarkerColor(kRed);

  } // end for(link)

  //
  // Open the output canvas
  //

  Int_t dowait=0;
  Int_t ps_page=0;
  Int_t screen_page=0;

  TCanvas* c3 =0;
  TPostScript* ps =0;

  if(graphics!=0){
    dowait=1;
  }

  if((graphics!=0)||(postscript!=0)){
    graphics = 1;
    c3= new TCanvas ("c3","TC",1100,750);
  }


  //
  // 4. Loop over modules
  //


  TGraph* TrimGraph[MAX_LINKS][MAX_CHANS];
  TGraph* MeanTrimGraph[MAX_LINKS][MAX_CHIPS];
  TGraph* RMSGraph[MAX_LINKS][MAX_CHIPS];

  Float_t lx[MAX_TARGETS];
  Float_t ly[MAX_TARGETS];

  for(module=0;module<MAX_MODULES;module++){
    for(link=0;link<MAX_LINKS;link++){
      if(module_status[module][link]>0){

        // Reset the arrays and objects used for storage

        for(chip=0;chip<MAX_CHIPS;chip++){
          h_ntrim[link][chip]->Reset();
          h_tp[link][chip]->Reset();

          for(i=0;i<MAX_TARGETS;i++){
            mean_trim[link][chip][i]=0;
            trim_rms[link][chip][i]=0;
          }
        }

        for(chan=0;chan<MAX_CHANS;chan++){
          trim_setting[link][chan]=-1;
          vt50_setting[link][chan]=-1;
          trim_offset[link][chan] = 0;
          trim_slope[link][chan]  = 0;
        }

        h_ts_vch[link]->Reset();
        h_tp_vch[link]->Reset();
        h_toff_vch[link]->Reset();
        h_toss_vch[link]->Reset();

        printf("Calculating TrimDAC Response Curves for module %d link %d\n",module,link);

        Int_t lp = 0;
        for(chan=0;chan<MAX_CHANS;chan++){

          // Exclude bad vt50 points (fit errors) from the graph 
	  
	  lp = 0;
          for(i=0;i<n_points;i++){
            if((code[module][link][chan][i]&BAD_TEST)==0){
              lx[lp]=mean[module][link][chan][i];
              ly[lp]=trim[i];
              lp++;
            }
          }

          if(lp>1){

            sprintf(h_name,"f3_%d_%d",link,chan);
            TrimGraph[link][chan] = new TGraph(lp,lx,ly);
            TrimGraph[link][chan]->Fit(h_name,"NRQ");
            TrimGraph[link][chan]->SetLineColor(chan%32);
            TrimGraph[link][chan]->SetMarkerColor(chan%32);
          } else {
            TrimGraph[link][chan]=0;
            printf("No Graph for link %d ch %d\n",link,chan);
          }
        } // end for(chan)


        //
        // Solve the function for each target value:
        // can the channel can be trimmed to that value?
        //

        printf("Studying targets for module %d link %d\n",module,link);

        for(i=0;i<n_targets;i++){

          for(chip=0;chip<MAX_CHIPS;chip++){
            h_tp[link][chip]->Reset();
            h_tp[link][chip]->SetBins(40,target[i]-(10*step),target[i]+(10*step));
          }

          for(chan=0;chan<MAX_CHANS;chan++){
            chip=chan/128;

            Float_t p0 = f3[link][chan]->GetParameter(0);
            Float_t p1 = f3[link][chan]->GetParameter(1);

            Int_t i_val = -1;

            f_val = f3[link][chan]->Eval(target[i]);
            i_val =(int)((f_val+0.5)/1.0);

            if((i_val>=0)&&(i_val<=15)){ // trimmable

              // if vt50 data is available for all 16 trim scans, 
              // use the data - otherwise calculate from the fit

              if(n_points==16){
                f_val = mean[module][link][chan][i_val];
              } else {
                f_val = 0;
                if(p1!=0){
                  f_val = ((float)i_val-p0) / p1;
                }
              }

              h_tp[link][chip]->Fill(f_val);
              h_ntrim[link][chip]->Fill(target[i]);

              mean_trim[link][chip][i]+=i_val;
            }
          } // end for(chan)

          //
          // Evaluate the RMS of the distribution of trimmed points for this target
          //

          for(chip=0;chip<MAX_CHIPS;chip++){

            Short_t l_ntrim = h_ntrim[link][chip]->GetBinContent(i);
            if(l_ntrim>0){ 
              mean_trim[link][chip][i]  /= l_ntrim;
            }

            trim_rms[link][chip][i] = h_tp[link][chip]->GetRMS();

          } // end for(chip)
        } // end for(target)


        for(chip=0;chip<MAX_CHIPS;chip++){
          MeanTrimGraph[link][chip] = new TGraph(n_targets,target,mean_trim[link][chip]);
          MeanTrimGraph[link][chip]->SetLineColor(kRed);
          MeanTrimGraph[link][chip]->SetMarkerColor(kRed);

          Int_t lp = 0;
          for(i=0;i<n_targets;i++){
            if(trim_rms[link][chip][i]>0){
              lx[lp]=target[i];
              ly[lp]=trim_rms[link][chip][i];
              lp++;
            }
          }

          if(lp>0){
            RMSGraph[link][chip] = new TGraph(lp,lx,ly);
            RMSGraph[link][chip]->SetLineColor(kRed);
            RMSGraph[link][chip]->SetMarkerColor(kRed);
          } else {
            RMSGraph[link][chip] = 0;
          }
            
        } // end for(chip)
      }
    } // end for(link)


    //
    // 5. Pick the best target and calculate the TrimDAC settings
    //

    if((module_status[module][0]>0)||(module_status[module][1]>0)){
      switch(method){

      case 1:   // Maximisation of ntrim chip by  chip 
        
        Int_t   low_target[MAX_LINKS][MAX_CHIPS];
        Int_t   high_target[MAX_LINKS][MAX_CHIPS];
        Int_t   best_target[MAX_LINKS][MAX_CHIPS];
        Short_t best_ntrim[MAX_LINKS][MAX_CHIPS];

        for(link=0;link<MAX_LINKS;link++){
          for(chip=0;chip<MAX_CHIPS;chip++) {
            low_target[link][chip]=0;
            high_target[link][chip]=0;
            best_target[link][chip]=0;
            best_ntrim[link][chip]=0;
	    
            for(i=0;i<n_targets;i++){
              ntrim[i] = h_ntrim[link][chip]->GetBinContent(i);
            }
	    
            for(i=0;i<n_targets;i++){
              if(ntrim[i]==best_ntrim[link][chip]){
        	high_target[link][chip] = i;
              }
              if(ntrim[i]>best_ntrim[link][chip]){
        	low_target[link][chip] = i;
        	high_target[link][chip] = i;
        	best_ntrim[link][chip] = ntrim[i];
              }
            } 
	    
            best_target[link][chip] = (low_target[link][chip] + high_target[link][chip])/2;
            printf("Mod%d Lk%d Ch%d low target %3.2f high target %3.2f -> best target %3.2f giving %d trimmable ch\n",
                   module,link,chip,target[low_target[link][chip]], target[high_target[link][chip]], 
                   target[best_target[link][chip]], ntrim[best_target[link][chip]]);
	    
            h_tp[link][chip]->Reset();
            h_tp[link][chip]->SetBins(40,target[best_target[link][chip]]-
                                      (10*step),target[best_target[link][chip]]+(10*step));
            h_ts[link][chip]->Reset();
            h_toff[link][chip]->Reset();
            h_toss[link][chip]->Reset();
          }
	  
          for(chan=0;chan<MAX_CHANS;chan++){
            chip=chan/128;
            Float_t p0=0.0;
            Float_t p1=0.0;
            Int_t i_val = -1;
            f_val = -1;
	    
            if(f3[link][chan]!=0){
              p0 = f3[link][chan]->GetParameter(0);
              p1 = f3[link][chan]->GetParameter(1);
		
              f_val = f3[link][chan]->Eval(target[best_target[link][chip]]);
              i_val =(int)((f_val+0.5)/1.0);
	      
              // if vt50 data is available for all 16 trim scans, 
              // use the data - otherwise calculate from the fit
	      
              if((i_val>=0)&&(i_val<=15)){ // trimmable
                if(n_points==16){
                  f_val = mean[module][link][chan][i_val];
                } else {
                  if(p1!=0){
                    f_val = ((float)i_val-p0) / p1;
                  }
                }
              } else {
        	f_val = -1;
        	i_val = -1;
              }
            }
	      
            trim_setting[link][chan] = i_val;
            vt50_setting[link][chan] = f_val;
            trim_offset[link][chan] = p0;
            trim_slope[link][chan]  = p1;
	    
            if((i_val>=0)&&(i_val<=15)){ // fill trim and vt50 histograms
                                         // only if channel is trimmable
              h_ts[link][chip]->Fill(i_val);
              h_tp[link][chip]->Fill(f_val);
              h_ts_vch[link]->Fill(chan,i_val);
              h_tp_vch[link]->Fill(chan,f_val);
            }

            if(f3[link][chan]!=0){ // fill slope and offset histograms
                                   // only if channel is basically alive
              h_toff_vch[link]->Fill(chan,p0);
              h_toss_vch[link]->Fill(chan,p1);
            }
           
          }

          // new section PWP 24.11.00
          // form projections of TrimDAC Slope and Offset

          for(chip=0;chip<MAX_CHIPS;chip++){
            Float_t mean_offset = 0;
            Float_t mean_slope = 0;
            Int_t mean_points = 0;

            for(chan=(chip*128);chan<((chip+1)*128);chan++){
              if((f3[link][chan]!=0)&&(trim_slope[link][chan]!=0)&&
                 (trim_offset[link][chan]>-640)&&(trim_offset[link][chan]<640)){
                mean_offset += trim_offset[link][chan];
                mean_slope  += trim_slope[link][chan];
                mean_points++;
              }
            }
            if(mean_points<1) mean_points=1;
            mean_offset/=mean_points;
            mean_slope/=mean_points;

            Float_t rms_offset = 0;
            Float_t rms_slope = 0;
            Int_t rms_points = 0;

            for(chan=(chip*128);chan<((chip+1)*128);chan++){
              if((f3[link][chan]!=0)&&(trim_slope[link][chan]!=0)&&
                 (trim_offset[link][chan]>-640)&&(trim_offset[link][chan]<640)){
                rms_offset +=(float)TMath::Power((trim_offset[link][chan]-mean_offset),2);
                rms_slope +=(float)TMath::Power((trim_slope[link][chan]-mean_slope),2);
                rms_points++;
              }
            }
            if(rms_points<1) rms_points=1;
            Float_t rms_offset2 = (float)TMath::Sqrt(rms_offset)/rms_points;
            Float_t rms_slope2 = (float)TMath::Sqrt(rms_slope)/rms_points;

            printf("link %d chip %d offset mean %3.2f rms %3.2f np %d\n", link, chip, mean_offset, rms_offset2, mean_points);
            printf("link %d chip %d slope mean %3.2f rms %3.2f\n", link, chip, mean_slope, rms_slope2);
 
            h_toff[link][chip]->SetBins(32,mean_offset-(50*rms_offset2),mean_offset+(50*rms_offset2));
            h_toss[link][chip]->SetBins(32,mean_slope-(50*rms_slope2),mean_slope+(50*rms_slope2));

            for(chan=(chip*128);chan<((chip+1)*128);chan++){
              if((f3[link][chan]!=0)&&(trim_slope[link][chan]!=0)){
                h_toff[link][chip]->Fill(trim_offset[link][chan]);
                h_toss[link][chip]->Fill(trim_slope[link][chan]);
              }
            }
          }
        }
        break;
	
      default:  // Maximisation of ntrim over the whole module 

        for(i=0;i<n_targets;i++){
          ntrim[i]=0; // PWP bugfix 21.11.00

          for(link=0;link<MAX_LINKS;link++){
            for(chip=0;chip<MAX_CHIPS;chip++){
              ntrim[i] += h_ntrim[link][chip]->GetBinContent(i);
            }    
          }
        }

        Int_t   low_target = 0;
        Int_t   high_target= 0;
        Int_t   best_target= 0;

        Short_t best_ntrim = 0;

        for(i=0;i<n_targets;i++){
          if(ntrim[i]==best_ntrim){
            high_target = i;
          }
          if(ntrim[i]>best_ntrim){
            low_target = i;
            high_target = i;
            best_ntrim = ntrim[i];
          }
        } 

        best_target = (low_target + high_target)/2;
        printf("  Module %d low target %3.2f high target %3.2f -> best target %3.2f giving %d trimmable ch\n",
          module, target[low_target], target[high_target], target[best_target], ntrim[best_target]);

        for(link=0;link<MAX_LINKS;link++){

          for(chip=0;chip<MAX_CHIPS;chip++){
            h_tp[link][chip]->Reset();
            h_tp[link][chip]->SetBins(40,target[best_target]-(10*step),target[best_target]+(10*step));
            h_ts[link][chip]->Reset();
            h_toff[link][chip]->Reset();
            h_toss[link][chip]->Reset();
          }

          for(chan=0;chan<MAX_CHANS;chan++){
            chip=chan/128;
            Float_t p0=0.0;
            Float_t p1=0.0;
            Int_t i_val = -1;
            f_val = -1;

            if(f3[link][chan]!=0){
              p0 = f3[link][chan]->GetParameter(0);
              p1 = f3[link][chan]->GetParameter(1);

              f_val = f3[link][chan]->Eval(target[best_target]);
              i_val =(int)((f_val+0.5)/1.0);
 
              // if vt50 data is available for all 16 trim scans, 
              // use the data - otherwise calculate from the fit

              if((i_val>=0)&&(i_val<=15)){ // trimmable
                if(n_points==16){
                  f_val = mean[module][link][chan][i_val];
                } else {
                  if(p1!=0){
                    f_val = ((float)i_val-p0) / p1;
                  }
                }
              } else {
                f_val = -1;
                i_val = -1;
              }
            }

            trim_setting[link][chan] = i_val;
            vt50_setting[link][chan] = f_val;
            trim_offset[link][chan] = p0;
            trim_slope[link][chan]  = p1;
            
            if((i_val>=0)&&(i_val<=15)){ // fill trim and vt50 histograms
                                         // only if channel is trimmable
              h_ts[link][chip]->Fill(i_val);
              h_tp[link][chip]->Fill(f_val);
              h_ts_vch[link]->Fill(chan,i_val);
              h_tp_vch[link]->Fill(chan,f_val);
            }

            if(f3[link][chan]!=0){ // fill slope and offset histograms
                                   // only if channel is basically alive
              h_toff_vch[link]->Fill(chan,p0);
              h_toss_vch[link]->Fill(chan,p1);
            }
          }

          // new section PWP 24.11.00
          // form projections of TrimDAC Slope and Offset

          for(chip=0;chip<MAX_CHIPS;chip++){

            Float_t mean_offset = 0;
            Float_t mean_slope = 0;
            Int_t mean_points = 0;

            for(chan=(chip*128);chan<((chip+1)*128);chan++){
              if((f3[link][chan]!=0)&&(trim_slope[link][chan]!=0)&&
                 (trim_offset[link][chan]>-640)&&(trim_offset[link][chan]<640)){
                mean_offset += trim_offset[link][chan];
                mean_slope  += trim_slope[link][chan];
                mean_points++;
              }
            }
            if(mean_points<1) mean_points=1;
            mean_offset/=mean_points;
            mean_slope/=mean_points;

            Float_t rms_offset = 0;
            Float_t rms_slope = 0;
            Int_t rms_points = 0;

            for(chan=(chip*128);chan<((chip+1)*128);chan++){
              if((f3[link][chan]!=0)&&(trim_slope[link][chan]!=0)&&
                 (trim_offset[link][chan]>-640)&&(trim_offset[link][chan]<640)){
                rms_offset +=(float)TMath::Power((trim_offset[link][chan]-mean_offset),2);
                rms_slope +=(float)TMath::Power((trim_slope[link][chan]-mean_slope),2);
                rms_points++;
              }
            }
            if(rms_points<1) rms_points=1;
            Float_t rms_offset2 = (float)TMath::Sqrt(rms_offset)/rms_points;
            Float_t rms_slope2 = (float)TMath::Sqrt(rms_slope)/rms_points;

            printf("link %d chip %d offset mean %3.2f rms %3.2f np %d\n", link, chip, mean_offset, rms_offset2, mean_points);
            printf("link %d chip %d slope mean %3.2f rms %3.2f\n", link, chip, mean_slope, rms_slope2);

            h_toff[link][chip]->SetBins(32,mean_offset-(50*rms_offset2),mean_offset+(50*rms_offset2));
            h_toss[link][chip]->SetBins(32,mean_slope-(50*rms_slope2),mean_slope+(50*rms_slope2));

            for(chan=(chip*128);chan<((chip+1)*128);chan++){
              if((f3[link][chan]!=0)&&(trim_slope[link][chan]!=0)){
                h_toff[link][chip]->Fill(trim_offset[link][chan]);
                h_toss[link][chip]->Fill(trim_slope[link][chan]);
              }
            }
          }
        }
        break;
      } // end switch

      // Fit gaussians to the distributions of trim values and trimmed vt50 

      for(link=0;link<MAX_LINKS;link++){
        for(chip=0;chip<MAX_CHIPS;chip++){
          h_ts[link][chip]->Fit("gaus","Q");
          h_tp[link][chip]->Fit("gaus","Q");
          h_toff[link][chip]->Fit("gaus","Q");
          h_toss[link][chip]->Fit("gaus","Q");
        }
      }


      //
      // 6. Plot the Results if the canvas has been opened
      //

      if(c3){
        if(postscript){
          ps_page=0;
          sprintf(f_name,"%s%s_TrimScanPlot_%d_%06d.ps",PS_DIRECTORY,mname[module],date,the_time);
          ps = new TPostScript(f_name,112);
        }

        for(link=0;link<MAX_LINKS;link++){   // loop over links
          if(module_status[module][link]){

            if((screen_page>0)&&(dowait)){
              printf("Enter a string to continue\n");
              scanf("%s",line);
            }

            printf("Generating Plots for module %d link %d\n",module,link);

            ps_page++;
            screen_page++;
            if(ps){
              ps->NewPage();
            }

            c3->cd();
            c3->Clear();

            TPad *pad_text   = new TPad("pad_text", "Text",        0.00,0.90,1.00,1.00,33);
            TPad *pad_plot   = new TPad("pad_plot", "Plots",       0.00,0.00,1.00,0.90,33);

            pad_text->Draw();
            pad_plot->Draw();
            pad_plot->Divide(6,4,0,0);

            c3->Update();
            pad_text->cd();

            TPaveText *t1 = new TPaveText(0.0,0.0,1.0,1.0);
            t1->SetFillColor(10);
            t1->SetTextAlign(13);
            t1->SetTextSize(0.3);
            t1->SetTextColor(4);
            sprintf(line,"ATLAS SCT Module Test:  TrimDac Response");
            if(time)  sprintf(line,"%s  -  %s",line,time->GetName());
            if(place) sprintf(line,"%s  -  %s",line,place->GetName());
            sprintf(line,"%s  -  Module %s",line,mname[module]);
            t1->AddText(line);
            sprintf(line,"Page %d   Run %d   Start Scan %d   Module %d   Stream %d", 
                    ps_page,run, scan[0],module,link);
            if(strlen(mdut[module]) > 0) 
              sprintf(line, "%s - Type %s", line, mdut[module]);
            t1->AddText(line);
            t1->Draw();

            for(chip=0;chip<6;chip++){
              pad_plot->cd(1+chip);
              h_trim[link][chip]->Draw();
              for(chan=(chip*128);chan<((chip+1)*128);chan++){
               if(TrimGraph[link][chan]!=0){
                 TrimGraph[link][chan]->Draw("P");
                 f3[link][chan]->Draw("same");
                }
              }
  
              pad_plot->cd(7+chip);
              h_ntrim[link][chip]->Draw();

              pad_plot->cd(13+chip);
              h_mtrim[link][chip]->Draw();
              MeanTrimGraph[link][chip]->Draw("LP");
 
              pad_plot->cd(19+chip);
              h_trms[link][chip]->Draw();
              if(RMSGraph[link][chip]) RMSGraph[link][chip]->Draw("LP");
            }

            c3->Update();

            if((screen_page>0)&&(dowait)){
              printf("Enter a string to continue\n");
              scanf("%s",line);
            }

            printf("Generating Summary Plots for module %d link %d\n",module,link);

            ps_page++;
            screen_page++;
            if(ps){
              ps->NewPage();
            }

            c3->cd();
            c3->Clear();

            pad_text   = new TPad("pad_text", "Text",        0.00,0.90,1.00,1.00,33);
            pad_plot   = new TPad("pad_plot", "Plots",       0.00,0.00,0.60,0.90,33);
            TPad *pad_proj   = new TPad("pad_proj", "Projections", 0.60,0.00,1.00,0.90,33);

            pad_text->Draw();
            pad_plot->Draw();
            pad_plot->Divide(1,4,0,0);
            pad_proj->Draw();
            pad_proj->Divide(3,8,0,0);
 
            c3->Update();
            pad_text->cd();

            t1 = new TPaveText(0.0,0.0,1.0,1.0);
            t1->SetFillColor(10);
            t1->SetTextAlign(13);
            t1->SetTextSize(0.3);
            t1->SetTextColor(4);
            sprintf(line,"ATLAS SCT Module Test:  TrimDac Response");
            if(time)  sprintf(line,"%s  -  %s",line,time->GetName());
            if(place) sprintf(line,"%s  -  %s",line,place->GetName());
            sprintf(line,"%s  -  Module %s",line,mname[module]);
            t1->AddText(line);
            sprintf(line,"Page %d   Run %d   Start Scan %d   Module %d   Stream %d", 
                    ps_page,run, scan[0],module,link);
            if(strlen(mdut[module]) > 0) 
              sprintf(line, "%s - Type %s", line, mdut[module]);
            t1->AddText(line);
            t1->Draw();

            pad_plot->cd(1);
            sprintf(h_name,"Trim Settings module %d link %d",module,link);
            h_ts_vch[link]->SetTitle(h_name);
            h_ts_vch[link]->Draw("P");

            pad_plot->cd(2);
            sprintf(h_name,"Trimmed Threshold module %d link %d",module,link);
            h_tp_vch[link]->SetTitle(h_name);
            h_tp_vch[link]->Draw("P");

            pad_plot->cd(3);
            sprintf(h_name,"TrimDAC Offset module %d link %d",module,link);
            h_toff_vch[link]->SetTitle(h_name);
            h_toff_vch[link]->Draw("P");

            pad_plot->cd(4);
            sprintf(h_name,"TrimDAC Slope module %d link %d",module,link);
            h_toss_vch[link]->SetTitle(h_name);
            h_toss_vch[link]->Draw("P");

            for(chip=0;chip<MAX_CHIPS;chip++){
              pad_proj->cd(1+chip);
              h_ts[link][chip]->Draw();
  
              pad_proj->cd(7+chip);
              h_tp[link][chip]->Draw();

              pad_proj->cd(13+chip);
              h_toff[link][chip]->Draw();

              pad_proj->cd(19+chip);
              h_toss[link][chip]->Draw();
            }

            c3->Update();
          }
        } //end for(link)

        if(ps) {
          ps->Close();
          delete ps;
        }
      } // end if(c3)


      //
      // 7. Write the results to an ascii file, one per module, suited to the database
      //

      sprintf(f_name,"%s%s_tr%d_%d.trim",RESULTS_DIRECTORY,mname[module],trimrange,date);
      FILE* trimfile = fopen(f_name,"w");

      if(trimfile==NULL){
        printf("Could not open trim file %s\n",f_name);
        return;
      } else {
        printf("File %s opened \n",f_name);
      }

      sprintf(f_name,"%s%s_tr%d_%d.mask",RESULTS_DIRECTORY,mname[module],trimrange,date);
      FILE* maskfile = fopen(f_name,"w");

      if(maskfile==NULL){
        printf("Could not open mask file %s\n",f_name);
        return;
      } else {
        printf("File %s opened \n",f_name);
      }

      // New section, PWP, 17.05.01
      // write TrimRange settings into the trim file

      for(chip=0;chip<MAX_CHIPS*2;chip++){
        fprintf(trimfile,"Chip %d %d\n",chip,trimrange);
      }

      // end new

      for(link=0; link<MAX_LINKS;link++){
        for(chan=0;chan<MAX_CHANS;chan++){
          fprintf(trimfile,"%f\n",(trim_setting[link][chan]/15.0));

          // Mask channels which cannot be trimmed including
          // channels which are DEAD or STUCK PWP 22.01.01
          if(TrimGraph[link][chan]==0){
            fprintf(maskfile,"%d  #DEAD or STUCK\n",(link*768)+chan);
          }
          else if(mean_noise[module][link][chan]>noise_cut[module][link][chan/128]){
            fprintf(maskfile,"%d  #NOISY\n",(link*768)+chan);
          }
          else if((trim_setting[link][chan]<0.0)||(trim_setting[link][chan]>15.0)){
            fprintf(maskfile,"%d  #UNTRIMMABLE\n",(link*768)+chan);
          }
        }
      }
      fclose(maskfile);
      fclose(trimfile);

      //
      // 8.1 TODO: write data to new style (database) results file 
      //

      FILE* outfile = OpenResults("TrimScan", strun, module, run, scan[0], 1, 0); 

      if(outfile==NULL){
        printf("Could not open result file %s\n",f_name);
        return;
      } else {

	if(method==1){
	  sprintf(s_method,"chip by chip");
	}else{
	  sprintf(s_method,"whole module");
	}
        fprintf(outfile,"# TrimType: charge %3.1f, range %d, method: %s\n",
           charge,trimrange,s_method);

        fprintf(outfile,"#\n");
        fprintf(outfile,"# run\tscan\ttrimDAC\n");
        for(i=0;i<n_points;i++){
          fprintf(outfile,"# %d\t%d\t%d\n",
            run, scan[i],trim[i]); 
        }

        fprintf(outfile,"#\n");
        fprintf(outfile,"%%ModuleTrimSummary\n");
        fprintf(outfile,"#");

        fprintf(outfile,"#\t\ttarget\tntrim\tvt50\tvt50rms\ttr_off\ttr_slope\n");
	if(method==1){
	  for(link=0;link<MAX_LINKS;link++){
	    for(chip=0;chip<MAX_CHIPS;chip++){
	      fprintf(outfile,"Chip %2d:\t%3.1f\t%d\t%5.1f\t%5.2f\t%4.1f\t%5.2f\n",
		      (6*link)+chip,
		      target[best_target[link][chip]],
		      best_ntrim[link][chip]);
		      h_tp[link][chip]->GetMean(),
		      h_tp[link][chip]->GetRMS(),
		      h_toff[link][chip]->GetMean(),
		      h_toss[link][chip]->GetMean());
	    }
	  }
	}else{
	  for(link=0;link<MAX_LINKS;link++){
	    for(chip=0;chip<MAX_CHIPS;chip++){
	      fprintf(outfile,"Chip %2d:\t%3.1f\t%d\t%5.1f\t%5.2f\t%4.1f\t%5.2f\n",
		      (6*link)+chip,
		      target[best_target],
		      best_ntrim,
		      h_tp[link][chip]->GetMean(),
		      h_tp[link][chip]->GetRMS(),
		      h_toff[link][chip]->GetMean(),
		      h_toss[link][chip]->GetMean());
	    }
	  }
	}
        fclose(outfile);
      }
      //
      // 8.2 Set the trim values and masked channels if running online
      //

      if(online){
        printf("Setting the trim values and masked channels for module %d\n",module);

        for(link=0;link<MAX_LINKS;link++){
          for(chan=0;chan<MAX_CHANS;chan++){
            SetTrim(module,(link*768)+chan,trim_setting[link][chan]);

            if((trim_setting[link][chan]<0.0)||(trim_setting[link][chan]>15.0)||(TrimGraph[link][chan]==0)
                                             ||(mean_noise[module][link][chan]>noise_cut[module][link][chan/128])){
              SetMask(module,(link*768)+chan);
            }
          }
        }
      }

    } else {
      printf("No data for module %d\n",module);
    }

  } // end for(module)


  //
  // 9. Delete lots of objects
  //

  if(time) delete time;
  if(place) delete place;

  if(strun) {
    strun->Close();
    delete strun;
  }

  printf("Deleting Graph Objects\n");

  for(link=0;link<MAX_LINKS;link++){
    for(chan=0;chan<MAX_CHANS;chan++){
      if(TrimGraph[link][chan]!=0) TrimGraph[link][chan]->Delete();
    }

    for(chip=0;chip<MAX_CHIPS;chip++){
      if(MeanTrimGraph[link][chip]!=0) MeanTrimGraph[link][chip]->Delete();
      if(RMSGraph[link][chip]!=0) RMSGraph[link][chip]->Delete();
    }
  }

  printf("Deleting Function Objects\n");

  for(link=0;link<MAX_LINKS;link++){
    for(chan=0;chan<MAX_CHANS;chan++){
      if(f3[link][chan]!=0) f3[link][chan]->Delete();
    }
  }

  printf("Deleting Histogram Objects\n");

  for(link=0;link<MAX_LINKS;link++){
    for(chip=0;chip<MAX_CHIPS;chip++){
      h_trim[link][chip]->Delete();
      h_ntrim[link][chip]->Delete();
      h_mtrim[link][chip]->Delete();
      h_trms[link][chip]->Delete();
      h_tp[link][chip]->Delete();
      h_ts[link][chip]->Delete();
      h_toff[link][chip]->Delete();
      h_toss[link][chip]->Delete();
    }
    h_tp_vch[link]->Delete();
    h_ts_vch[link]->Delete();
    h_toff_vch[link]->Delete();
    h_toss_vch[link]->Delete();
  }

  printf("Finished!!\n");
}


void TrimScanPlot(Int_t run, Int_t start_scan, Int_t trimrange, Float_t charge) {

  Int_t n_points = 16;
  Int_t scans[16];
  Float_t trims[16] = {0, 1, 2, 3, 4, 5, 6,7,
                       8,9,10,11,12,13,14,15};
  
  Short_t method = 0;   // whole module

  for(int i=0;i<n_points;i++) scans[i]=i+start_scan;

  TrimScanPlot(run,n_points,method,trimrange,charge, scans,trims,0,1,1);
} 

void KwikTrimScanPlot(Int_t run, Int_t start_scan, Int_t trimrange, Float_t charge) {

  Int_t n_points = 4;
  Int_t scans[4];
  Float_t trims[4] = {3,7,10,15};
  
  Short_t method = 0;   // whole module

  for(int i=0;i<n_points;i++) scans[i]=i+start_scan;

  TrimScanPlot(run,n_points,method,trimrange,charge, scans,trims,0,1,1);
} 

