// Some macros
// Created by Pavel Reznicek (reznicek@ipnp.troja.mff.cuni.cz)
// Last update 29/07/2002

void DelCanvas(TCanvas *c,TPad *p=(TPad*)NULL){
// procedure for removing canvas 'c' and all its primitives (recursively call to all subpads)
// commented code only for testing purpose

  static Long_t deep;//
  TList *l;
  TObject *o;
  Long_t i;//
  if(c!=(TCanvas*)NULL){
    deep=1;//
    printf("TCanvas* %s:\n",c->GetName());//
    l=c->GetListOfPrimitives();
    o=l->First();
    while(o!=(TObject*)NULL){
      printf("  %s (%s)\n",o->GetName(),o->ClassName());//
      if(!strcmp(o->ClassName(),"TPad")) DelCanvas((TCanvas*)NULL,(TPad*)o);
      o=l->After(o);
    }
    l->Delete();
    printf("Canvas '%s'('%s') deleted !\n",c->GetName(),c->GetTitle());
    delete c;
  }
  
  if(p!=(TPad*)NULL){
    l=p->GetListOfPrimitives();
    o=l->First();
    while(o!=(TObject*)NULL){
      for(i=0;i<deep;i++) printf("  ");//
      printf("  %s (%s)\n",o->GetName(),o->ClassName());//
      if(!strcmp(o->ClassName(),"TPad")){//
        deep++;//
        DelCanvas((TCanvas*)NULL,(TPad*)o);
        deep--;//
        }//
      o=l->After(o);
    }
    l->Delete();
  }
}

void DelCanvas(Char_t *cname){
// finds canvas 'cname' and removes it
  TCanvas *c=(TCanvas*) gROOT->FindObject(cname);
  if(c!=NULL) DelCanvas(c);
}

TObject* UnderCanvas(Char_t *name,TCanvas *c,TPad *p=(TPad*)NULL){
// finds object 'name' under canvas 'c' (recursively call to all subpads)
// commented code only for testing purpose

  static Long_t deep;//
  static TObject *ret;
  TList *l;
  TObject *o;
  Long_t i;//
  if(c!=(TCanvas*)NULL){
    deep=1;//
    printf("TCanvas* %s:",c->GetName());//
    if(!strcmp(c->GetName(),name)){
      ret=c;
      printf(" !!!");//
    }
    printf("\n");//
    l=c->GetListOfPrimitives();
    o=l->First();
    while(o!=(TObject*)NULL){
      printf("  %s (%s)",o->GetName(),o->ClassName());//
      if(!strcmp(o->GetName(),name)){
        ret=o;
        printf(" !!!");//
      }
      printf("\n");//
      if(!strcmp(o->ClassName(),"TPad")) UnderCanvas(name,(TCanvas*)NULL,(TPad*)o);
      o=l->After(o);
    }
    if(ret!=NULL) printf("Object '%s' found in canvas '%s'('%s').\n",name,c->GetName(),c->GetTitle());
    else printf("Object '%s' NOT found in canvas '%s'('%s') !\n",name,c->GetName(),c->GetTitle());
  }
  
  if(p!=(TPad*)NULL){
    l=p->GetListOfPrimitives();
    o=l->First();
    while(o!=(TObject*)NULL){
      for(i=0;i<deep;i++) printf("  ");//
      printf("  %s (%s)",o->GetName(),o->ClassName());//
      if(!strcmp(o->GetName(),name)){
        ret=o;
        printf(" !!!");//
      }
      printf("\n");//
      if(!strcmp(o->ClassName(),"TPad")){//
        deep++;//
        UnderCanvas(name,(TCanvas*)NULL,(TPad*)o);
        deep--;//
        }//
      o=l->After(o);
    }
  }
  
  return ret;
}

Int_t MaxNonZeroBinTH1(TH1 *h){
// finds maximal bin of histogram 'h' with non-zero content
  Int_t i,last;
  for(i=1;i<=h->GetNbinsX();i++)
    if(h->GetBinContent(i)!=0) last=i;
  return last;
}

Int_t MinNonZeroBinTH1(TH1 *h){
// finds minimal bin of histogram 'h' with non-zero content
  Int_t i,last;
  for(i=1;i<=h->GetNbinsX();i++)
    if(h->GetBinContent(i)!=0) return i;
  return 0;
}

void ConnectGraphs(TGraph *dest,TGraph *add){
// connects points of graph 'add' to graph 'dest'
  Double_t *xadd,*yadd,*xdest,*ydest;
  if(add==NULL || dest==NULL) return;
  Int_t nadd=add->GetN();
  Int_t ndest=dest->GetN();
  dest->Set(ndest+nadd);
  xadd=add->GetX();
  yadd=add->GetY();
  xdest=dest->GetX();
  ydest=dest->GetY();
  for(Int_t i=0;i<nadd;i++){
    xdest[i+ndest]=xadd[i];
    ydest[i+ndest]=yadd[i];
  }
  return;
}