void CorrelNoisePlot(Int_t run, Int_t scan, Float_t Qthr){

  #ifdef DATA_DIRECTORY
    // do not reload parameters.h
  #else
    // must load parameters.h
  #include <parameters.h>
  #endif

  #include "Evtree.h"

  Int_t moi,i,j,nevbins,nmods, events, offset;
  Char_t mname[32], hname[32], htitle[64],line[128], fname[128],info[256];
  TH2F* hit_evt0[MAX_MODULES];
  TH2F* hit_evt1[MAX_MODULES];
  Evtree* m[MAX_MODULES];

  sprintf(fname,"%sstrun%d_%d.root",DATA_DIRECTORY,run,scan);
  TFile *f=new TFile(fname);
  
  // fill histos
  nmods = 0;
  for(moi=0;moi<MAX_MODULES;moi++){

    sprintf(mname,"ModuleName;%d",moi+1);
    TObjString* name  = (TObjString*) f->Get(mname);
    if(name){
      sprintf(mname,"%s",name->GetName());
      delete name;
      if(strcmp(mname,"missing")){
        nmods++;
        printf("processing module %d %s...  \n",moi,mname);
      }
      else
	break;
    }
    else
      break;

    sprintf(htitle,"EvTree_m%d",moi);
    TTree* tree = (TTree*)gDirectory->Get(htitle);
    tree->Print();

    // take too long to plot too many events, restrict to max. 5000
    events = tree->GetEntries();
    if(events>5000) {
      sprintf(info,"(first 5000 events out of %d)",events);
      events=5000;
    }
    else
      sprintf(info,"");

    // adjust nevt-binning to be int-fraction of nevt
    for(nevbins=120;nevbins<250;nevbins++)
      if(events%nevbins==0) break;

    m[moi] = new Evtree(tree);

    sprintf(hname,"hit_evt0_%d",moi);
    sprintf(htitle,"Module %s Stream 0, Hitmap",mname);
    hit_evt0[moi] = new TH2F(hname,htitle,128,0,768,128,0,(Float_t)events);
    sprintf(hname,"hit_evt1_%d",moi);
    sprintf(htitle,"Module %s Stream 1, Hitmap",mname);
    hit_evt1[moi] = new TH2F(hname,htitle,128,0,768,128,0,(Float_t)events);
    // fill
    for(i=0;i<events;i++){
      m[moi]->GetEntry(i);
      for(j=0;j<768;j++){
	if(m[moi]->strm0[j])
	  hit_evt0[moi]->Fill((Float_t)j,(Float_t)i);
	if(m[moi]->strm1[j])
	  hit_evt1[moi]->Fill((Float_t)j,(Float_t)i);
      }
    }
  }

  // open canvas
  TCanvas* cn[4];
  TPad *pad_text[4], *pad_plot[4];
  int npan,ipan,nmodppan;

  nmodppan = e->num_panelmods;
  npan = (nmods-1)/nmodppan+1;
  if(npan > 4) npan = 4;

  for(ipan=0;ipan<npan;ipan++){
    sprintf(info,"CorrNoise Modules %d-%d",ipan*nmodppan,(ipan+1)*nmodppan-1);
    sprintf(hname,"cn%d",ipan);
    cn[ipan] = new TCanvas (hname,info,1100,1000);
    cn[ipan]->cd();
    cn[ipan]->Clear();
    gStyle->SetOptStat(0);

    sprintf(hname,"ptxt%d",ipan);
    pad_text[ipan]   = new TPad(hname, "Text",        0.00,0.93,1.00,1.00);
    sprintf(hname,"pplt%d",ipan);
    pad_plot[ipan]   = new TPad(hname, "Plots",       0.00,0.00,1.00,0.93,17);
  
    pad_text[ipan]->Draw();
    pad_plot[ipan]->Draw();
    pad_plot[ipan]->Divide(2,nmodppan,0,0);
    
    cn[ipan]->Update();
  
    pad_text[ipan]->cd();
    TPaveText *t1 = new TPaveText(0.0,0.0,1.0,1.0);
    t1->SetFillColor(10);
    t1->SetTextAlign(13);
    t1->SetTextSize(0.3);
    t1->SetTextColor(4);
    sprintf(line,"ATLAS SCT:  Correlated Noise Analysis - Hit pattern, events vs strips %s",info);
    t1->AddText(line);
    sprintf(line,"Run %d, scan %d -- Qthr = %.1ffC",run,scan, Qthr);
    t1->AddText(line);
    t1->Draw();
  
    for (moi=ipan*nmodppan; moi<nmods && moi<(ipan+1)*nmodppan; moi++){
      //for(moi=0;moi<nmods;moi++){
      pad_plot[ipan]->cd(2*(moi-ipan*nmodppan)+1);
      hit_evt0[moi]->SetXTitle("Strip");
      hit_evt0[moi]->SetYTitle("Event");
      hit_evt0[moi]->Draw("COLZ");
      //hit_evt0[moi]->Draw();
      pad_plot[ipan]->cd(2*(moi-ipan*nmodppan)+2);
      hit_evt1[moi]->SetXTitle("Strip");
      hit_evt1[moi]->SetYTitle("Event");
      hit_evt1[moi]->Draw("COLZ");
      //hit_evt1[moi]->Draw();
    }

    cn[ipan]->Update();
  }

  return;
}
