void ENCsum(Int_t run, Int_t firstscan, Int_t comp=1){

  #ifdef DATA_DIRECTORY
    // do not reload parameters.h
  #else
    // must load parameters.h
  #include <parameters.h>
  #endif

  Int_t nread=1,i=0,j,round,chipno,goodchip,nmod,scans[3],newlog;
  //Int_t tsec=0,tfirst=0;
  Float_t gain[MAX_MODULES][MAX_LINKS][MAX_CHIPS],offset[MAX_MODULES][MAX_LINKS][MAX_CHIPS];
  Float_t noise[MAX_MODULES][MAX_LINKS][MAX_CHIPS],dif[MAX_CHIPS*MAX_LINKS],chipid[MAX_CHIPS*MAX_LINKS];
  Float_t elec_n[MAX_MODULES][MAX_CHIPS*MAX_LINKS], sec_n[MAX_MODULES][MAX_CHIPS*MAX_LINKS],data,maxy,miny;
  char mname[MAX_MODULES][32],a[10],b[10],c[10],d[10];
  FILE* input;
  TDatime *dt = new TDatime;
  Int_t date = dt->GetDate();
  Float_t max_n=2000, min_n=1000, dummy;
  Color_t col;
  Char_t line[1024],gtitle[64],h_title[64],f_name[128];
  Char_t dbasedir[128];
  Float_t charges[3] = {1.50,2.00,2.50};
  Float_t allnse[MAX_MODULES*MAX_LINKS*MAX_CHIPS], id[MAX_MODULES*MAX_LINKS*MAX_CHIPS];
  Float_t allele[MAX_MODULES*MAX_LINKS*MAX_CHIPS];
  TPostScript *ps;

  // load the 3pt-gain plot macro 
  gROOT->ProcessLine(".L ScurveFit.cpp");
  gROOT->ProcessLine(".L estand_secind_data.cpp");

  for(j=0;j<3;j++)
    scans[j] = firstscan+j;

  for(j=0;j<MAX_CHIPS*MAX_LINKS;j++)
    chipid[j] = (Float_t) j;

  if(strncmp(gSystem->GetName(),"Unix",4))
    sprintf(dbasedir,"J:\\public\\refdat\\");
  else
    sprintf(dbasedir,"~/public/refdat/");

  sprintf(f_name,"%sstrun%d_%d.root",DATA_DIRECTORY,run,scans[0]);
  printf("opening file %s\n",f_name);
  TFile *strun = new TFile(f_name);
  nmod=0;
  for (i=0; i<MAX_MODULES; i++){
    if(i==0){
      TObjString* time  = (TObjString*)strun->Get("Time");
      TObjString* place = (TObjString*)strun->Get("Place");
    }
    sprintf(h_title,"ModuleName;%d",i+1);
    TObjString* name = (TObjString*)strun->Get(h_title);
    if(name){
      if(strcmp(name->GetName(),"missing")){
	sprintf(mname[nmod],"%s",name->GetName());
	nmod++;
      }
    }
  }
  strun->Close();
    
  // do the fitting
  ScurveFit(run,3,1,scans,charges,4,gain,offset,noise);

  for(i=0;i<nmod;i++){
    // delete txt files from above
    if(strncmp(gSystem->GetName(),"Unix",4))
      sprintf(f_name,"del %s%s_stfit.txt",RESULTS_DIRECTORY,mname[i]);
    else
      sprintf(f_name,"rm %s%s_stfit.txt",RESULTS_DIRECTORY,mname[i]);
    gSystem->Exec(f_name);
    if(comp){
      // read estand results
      sprintf(f_name,"%s%s.dat",dbasedir,mname[i]);
      input=fopen(f_name,"r");
      if(!input){
	printf("cannot open file %s\n",f_name);
	for(chipno=0;chipno<12;chipno++)
	  elec_n[i][chipno] = 0.;
      } else{
	nread=2;
	for(chipno=0;chipno<12 && nread==2;chipno++){
	  nread = fscanf(input,"%f %f",&data,&dummy);
	  if(nread!=2) data = 0.; 
	  elec_n[i][chipno] = data;
	}
	fclose(input);
      }
    }
  }


  // write data to file

  sprintf(f_name,"%sENCsum_%d_%d-%d.txt",RESULTS_DIRECTORY,run,firstscan,firstscan+2);
  FILE* output=fopen(f_name,"w");

  fprintf(output,"absolute noise in ENC\n");
  fprintf(output,"module\t chip\tnoise\n");
  for(i=0;i<nmod;i++){
    for(j=0;j<MAX_LINKS;j++){
      for(round=0;round<MAX_CHIPS;round++){
	chipno = j*MAX_CHIPS+round;
	fprintf(output,"%s\t%d\t%.1f\n",mname[i],chipno,noise[i][j][round]);
      }
    }
  }

  // open canvas
  TCanvas* c6= new TCanvas ("c6","ENC",1100,750);

  //plot
  
  sprintf(f_name,"%sENCsum_%d_%d-%d.ps",PS_DIRECTORY,run,firstscan,firstscan+2);
  ps = new TPostScript(f_name,112);
  ps->NewPage();

  c6->cd();
  c6->Clear();
  pad_text   = new TPad("pad_text", "Text",        0.00,0.90,1.00,1.00);
  pad_plot   = new TPad("pad_plot", "Plots",       0.00,0.00,1.00,0.90);
  pad_text->Draw();
  pad_plot->Draw();
  c6->Update();
  
  pad_text->cd();
  TPaveText *t1 = new TPaveText(0.0,0.0,1.0,1.0);
  t1->SetFillColor(10);
  t1->SetTextAlign(13);
  t1->SetTextSize(0.3);
  t1->SetTextColor(4);
  sprintf(line,"ATLAS SCT: 3-pt gain scans");
  if(time)  sprintf(line,"%s  -  %s",line,time->GetName());
  if(place) sprintf(line,"%s  -  %s",line,place->GetName());
  t1->AddText(line);
  sprintf(line,"Run %d   Scans %d-%d", run,firstscan,firstscan+2);
  t1->AddText(line);
  sprintf(line,"Input-noise at 2fC from 3point gain scans");
  t1->AddText(line);
  t1->Draw();
  c6->Update();
  
  pad_plot->cd();
  leg = new TLegend(.9,.4,.98,.95);
  leg->SetTextSize(0.03);
  goodchip=0;
  for(i=0;i<nmod;i++){
    for(j=0;j<MAX_LINKS;j++){
      for(round=0;round<MAX_CHIPS;round++){
	chipno = j*MAX_CHIPS+round;
	if(noise[i][j][round]>1000){
	  id[goodchip]=0.5+(Float_t)chipno + MAX_LINKS*MAX_CHIPS*i;
	  allnse[goodchip] = noise[i][j][round];
	  allele[goodchip] = elec_n[i][chipno];
	  goodchip++;
	}
      }
    }
  }
  leg = new TLegend(.8,.9,1.,1.);
  leg->SetTextSize(0.03);
  grm = new TGraph(goodchip,id,allnse);
  if(comp) gre = new TGraph(goodchip,id,allele);
  sprintf(gtitle,"Noise");
  grm->SetTitle(gtitle);
  grm->SetMarkerStyle(20);
  grm->SetMarkerSize(1);
  grm->SetMarkerColor((Color_t) 1);
  grm->SetLineColor((Color_t) 1);
  grm->SetMaximum((Double_t)2000.);
  grm->SetMinimum((Double_t)1100.);
  if(comp){
    gre->SetMarkerStyle(20);
    gre->SetMarkerSize(1);
    gre->SetMarkerColor((Color_t) 2);
    gre->SetLineColor((Color_t) 2);
  }
  // draw graph
  pad_plot->Range(-15,1000,nmod*12+10,2100);
  TGaxis *xax = new TGaxis(0,1100,nmod*12,1100,0,nmod,6*100+nmod,"U");
  xax->SetName("xaxis");
  xax->SetTitle("Module");
  xax->Draw();
  TGaxis *yax = new TGaxis(0,1100,0,2000,1100,2000,808,"");
  yax->SetName("yaxis");
  yax->SetTitle("Input-noise (ENC)");
  yax->Draw();
  for(i=0;i<=nmod;i++){
    l = new TLine(i*12,1100,i*12,2000);
    l->Draw();
  }
  for(i=0;i<nmod;i++){
    TText t;
    t.SetTextSize(0.04);
    t.DrawText((Double_t)(i*12+1),1060.,mname[i]);
  }
  l = new TLine(0,2000,nmod*12,2000);
  l->Draw();
  grm->Draw("PL");
  if(comp) gre->Draw("PL");
  leg->AddEntry(grm,"Multimodule","p");
  if(comp) leg->AddEntry(gre,"Elec. Stand","p");
  leg->Draw();
  c6->Update();

  fclose(output);
  ps->Close();

  return;
}
