// Macro for use with external source
// triggering is external
// version 0.10 22/04/2002 JGK,PB 
//
// Arguments:
// n_events  : number of events to be taken
// Qthr      : threshold in fC


void ExternalSource (Int_t n_events, Float_t Qthr, Int_t standalone=1) {

  #ifdef DATA_DIRECTORY
    // do not reload parameters.h
  #else
    // must load parameters.h
  #include <parameters.h>
  #endif

  Int_t burst, moi, stop_loop;
  Int_t n_bursts, bst_rem;
  char fname[128];

  Int_t MAX_NTRIGS=2500;
  if(Qthr>0.8) MAX_NTRIGS = 25000;

  // load DCS communication macro
  gROOT->ProcessLine(".L systest/DCSSafety.cpp");

  // Save settings
  e->SaveState();

  e->abort=0;              // perhaps unnecessary?

  e->do_cal_loop=0;        // cal loop OFF
  e->do_autostop=0;        // autostop OFF
  e->do_fits=0;            // integrated fitting OFF
  e->do_mask_test=0;       // alternating mask patterns OFF 
  e->do_fill_evtree=1;     // build ntuple of individual events
  e->config_frequency = 0; // do NOT send full configuration before each event

  e->burst_type   = 21;    // 21 - ST_DEBURST

  e->burst_trtype = 0;     // 0 - L1A
 
  e->ConfigureScan(0,0,1,1);      // 0 - SCAN_ELAPSED
  if(Qthr>-99) e->ConfigureVariable(41,Qthr);  // 41 - QTHR (fC)

  e->ConfigureVariable(11,1);  // X1X compression 
  e->ConfigureVariable(13,0);  // EDGE DETECT OFF
  e->ConfigureVariable(14,0);  // Route Mask OFF

  e->ConfigureVariable(27,116);  // Trigger Delay

  e->ExecuteConfigs();

  e->scannum++;

  // open output file, needed here to have a buffer for the ntuples
  sprintf(fname,"%sstrun%d_%d.root",e->sct_data,e->runnum,e->scannum);
  TFile* out = new TFile(fname,"RECREATE");

  // clear all histogram, ntuple and data storage
  for(moi=0;moi<e->GetMaxModules();moi++){
    e->m[moi]->ResetData();
    e->m[moi]->CreateTree(); // added PWP 08.08.01
  }

  // trigger settings - split into several rounds to allow for an abort
  if(n_events>MAX_NTRIGS){
    e->burst_ntrigs = MAX_NTRIGS;
    n_bursts = n_events/MAX_NTRIGS;
    bst_rem = n_events-n_bursts*MAX_NTRIGS;
  }
  else{
    e->burst_ntrigs = n_events;
    n_bursts = 1;
    bst_rem = 0;
  }

  st_start_stopper();

  // execute burst

  burst=0;
  e->burst_count = 0;
  stop_loop=0;

  while((stop_loop==0)&&(burst<n_bursts)){

    if(standalone)
      if(DCSSafety(kTRUE)) return;

    st_dcs_log();

    printf("requesting burst %d\n",burst);
    e->ExecuteBurst();

    st_display_burst();
    st_display_burst2();
    for (moi=0; moi<e->num_showmods; moi++){
      if(e->m[moi]->present){
        e->m[moi]->PrintBurst();
      }
    }
    stop_loop = st_do_stop();
    burst++;

  }

  if((stop_loop==0) && bst_rem){
    st_dcs_log();
    printf("requesting burst %d\n",burst);
    e->burst_ntrigs = bst_rem;
    e->ExecuteBurst();

    st_display_burst();
    st_display_burst2();
    for (moi=0; moi<e->num_showmods; moi++){
      if(e->m[moi]->present){
        e->m[moi]->PrintBurst();
      }
    }
    stop_loop = st_do_stop();
    burst++;
  }

  st_stop_stopper();

  if(!e->abort){
    // print ntuple(s) for present modules
    for(moi=0;moi<MAX_MODULES;moi++){
      if(e->m[moi]->present){
        e->m[moi]->evtree->Print();
      }
    }

    st_save_scan(out);
  }

  // Restore settings
  e->RestoreState();

  // some plots for first check of data
  gROOT->ProcessLine(".L CorrelNoisePlot.cpp");
  CorrelNoisePlot(e->runnum,e->scannum,Qthr);

  if(standalone)
    DCSSafety(kFALSE);

  return;
}
