void FastInjPlot(Int_t NDAT, Int_t *run, Int_t *scans, Char_t *in_label,
		 Char_t *exttit=0, Int_t doout=1, Int_t dosqu=0){

  #ifdef DATA_DIRECTORY
    // do not reload parameters.h
  #else
    // must load parameters.h
  #include <parameters.h>
  #endif

  #define MAXNDAT 30

  Int_t i,j,k,chip,goodchip, nread=1, nmod, refer=-1, chan,link;
  Float_t noise[MAXNDAT][12][12],diff[144],maxnse=0.,tmp;
  Char_t fname[128],mname[30][MAX_MODULES],text[128],line[1024];
  Char_t label[MAXNDAT][64],clock[MAXNDAT][10],a[30],b[30],c[30];
  Float_t allnse[MAX_MODULES*MAX_LINKS*MAX_CHIPS], id[MAX_MODULES*MAX_LINKS*MAX_CHIPS];
  TGraph *grm;
  TPostScript *ps;

  FILE *output;
  TFile *strun;
  TH1F* h_sigma;

  if(NDAT>MAXNDAT){
    printf("Too many data points, maximum is %d\n",MAXNDAT);
    return;
  }

  j=0;
  k=0;
  for(i=0;i<strlen(in_label);i++){
    if(in_label[i]==':') {
      label[j][k]='\0';
      j++;
      k=0;
    }
    else{
      label[j][k]=in_label[i];
      k++;
    }
  }
  label[j][k]='\0';

  // try to find zero-current reference measurement
  for(i=0;i<NDAT;i++){
    if(!strcmp(label[i],"refer")){
      refer = i;
      break;
    }
  }
  if(refer<0 || refer>NDAT-1){
    printf("Couldn't find any measurement labelled \"refer\", no reference -> QUIT\n");
    return;
  }
  else
    printf("Using scan %d as zero-reference\n",scans[refer]);
  
  // read from data file
  for(i=0;i<NDAT;i++){
    sprintf(fname,"%sstrun%d_%d.root",DATA_DIRECTORY,run[i],scans[i]);
    strun = new TFile(fname);
    TObjString* time  = (TObjString*)strun->Get("Time");
    sscanf(time->GetName(),"%s %s %s %s",a,b,c,clock[i],text);
    TObjString* place = (TObjString*)strun->Get("Place");
    for(j=0;j<MAX_MODULES;j++){
      sprintf(text,"ModuleName;%d",j+1);
      TObjString* name = (TObjString*)strun->Get(text);
      if(name){
	if(strcmp(name->GetName(),"missing")){
	  sprintf(mname[j],"%s",name->GetName());
	}
	else
	  break;
      }
      else
	break;
      for(link=0;link<MAX_LINKS;link++){
	sprintf(text,"h_sigma%d;%d",link,j+1);
	h_sigma= (TH1F*)strun->Get(text);
	for(chip=0;chip<MAX_CHIPS;chip++){
	  noise[i][j][chip+MAX_CHIPS*link]=0.;
	  for(chan=(chip*128);chan<((chip+1)*128);chan++){
	    noise[i][j][chip+MAX_CHIPS*link] += h_sigma->GetBinContent(chan);
	    }
	  noise[i][j][chip+MAX_CHIPS*link] *= 6240/55./128.;
	}
      }
    }
    strun->Close();
    if(i==0){
      nmod = j;
      printf("found %d modules\n",nmod);
    }
  }
  
  // find maximum noise difference
  for(i=0;i<NDAT;i++){
    for(chip=0;chip<MAX_LINKS*MAX_CHIPS;chip++){
      for(j=0;j<nmod;j++){
	if(dosqu){
	  tmp = pow(noise[i][j][chip],2)-pow(noise[refer][j][chip],2);
	  if(tmp>0) 
	    tmp = sqrt(tmp);
	  else
	    tmp = 0.;
	}
	else
	  tmp = noise[i][j][chip]-noise[refer][j][chip];
	if(tmp>maxnse)
	  maxnse = tmp;
      }
    }
  }
  maxnse*=1.1;
  if(maxnse<100) maxnse = 100.;

  // open canvas
  TCanvas* cinj= new TCanvas ("cinj","Noise Injection",1100,750);
  
  // get time/place info
  
  sprintf(fname,"%sstrun%d_%d.root",DATA_DIRECTORY,run[refer],scans[refer]);

  // plot
  
  sprintf(fname,"%sENCninj_%d_%d-%d.ps",PS_DIRECTORY,run[refer],scans[refer],scans[NDAT-1]);
  if(doout){
    ps = new TPostScript(fname,112); 
    ps->NewPage();
  }

  cinj->cd();
  cinj->Clear();
  pad_text   = new TPad("pad_text", "Text",        0.00,0.90,1.00,1.00);
  pad_plot   = new TPad("pad_plot", "Plots",       0.00,0.00,1.00,0.90);
  pad_text->Draw();
  pad_plot->Draw();
  cinj->Update();

  pad_text->cd();
  TPaveText *t1 = new TPaveText(0.0,0.0,1.0,1.0);
  t1->SetFillColor(10);
  t1->SetTextAlign(13);
  t1->SetTextSize(0.3);
  t1->SetTextColor(4);
  sprintf(line,"ATLAS SCT: pick-up with noise injection");
  if(time)  sprintf(line,"%s  -  %s",line,time->GetName());
  if(place) sprintf(line,"%s  -  %s",line,place->GetName());
  t1->AddText(line);
  sprintf(line,"Input-noise at 2fC from threshold scans - difference to zero-injection (run %d, scan %d)",
	  run[refer],scans[refer]);
  t1->AddText(line);
  if(exttit) t1->AddText(exttit);
  t1->Draw();
  cinj->Update();
  
  pad_plot->cd();

  // draw drawing area
  pad_plot->Range(-2*nmod,-100-.13*maxnse,nmod*18,1.12*maxnse);
  TLegend *leg = new TLegend(.7,1.-.04*(Float_t)(NDAT-1),1.,1.);
  leg->SetTextFont(62);
  leg->SetTextSize(0.03);
  TGaxis *xax = new TGaxis(0,-100,nmod*12,-100,0,nmod,6*100+nmod,"U");
  xax->SetName("xaxis");
  xax->SetTitle("Module");
  xax->Draw();
  TGaxis *yax = new TGaxis(0,-100,0,maxnse,-100,maxnse,808,"");
  yax->SetName("yaxis");
  if(dosqu)
    yax->SetTitle("Quadratic difference in input-noise (ENC)");
  else
    yax->SetTitle("Difference in input-noise (ENC)");
  yax->Draw();
  for(i=0;i<=nmod;i++){
    l = new TLine(i*12,-100,i*12,maxnse);
    l->Draw();
  }
  for(i=0;i<nmod;i++){
    TText t;
    t.SetTextSize(0.04);
    t.DrawText((Double_t)(i*12+1),-100-.05*maxnse,mname[i]);
  }
  l = new TLine(0,maxnse,nmod*12,maxnse);
  l->Draw();
  l = new TLine(0,0,nmod*12,0);
  l->Draw();

  // draw graphs
  for(j=1;j<NDAT;j++){
    if(j!=refer){
      for(i=0;i<nmod;i++){
	goodchip=0;
	for(chip=0;chip<12;chip++){
	  if(noise[j][i][chip]>1000){
	    id[goodchip]=0.5+(Float_t)chip + 12*i;
	    if(dosqu){
	      allnse[goodchip] = pow(noise[j][i][chip],2)-pow(noise[refer][i][chip],2);
	      if(allnse[goodchip]>0.)
		allnse[goodchip] = sqrt(allnse[goodchip]);
	      else
		allnse[goodchip] = 0.;
	    }
	    else
	      allnse[goodchip] = noise[j][i][chip]-noise[refer][i][chip];
	    goodchip++;
	  }
	}
	grm = new TGraph(goodchip,id,allnse);
	grm->SetTitle("");
	grm->SetMarkerSize(1);
	if(j<8){
	  grm->SetMarkerStyle(20);
	  grm->SetMarkerColor((Color_t) j);
	  grm->SetLineColor((Color_t) j);
	} else if (j<15) {
	  grm->SetMarkerStyle(24);
	  grm->SetMarkerColor((Color_t) j-7);
	  grm->SetLineColor((Color_t) j-7);
	} else{
	  grm->SetMarkerStyle(29);
	  grm->SetMarkerColor((Color_t) j-14);
	  grm->SetLineColor((Color_t) j-14);
	}
	sprintf(text,"%s (%d, %d)",label[j],run[j],scans[j]);
	if(i==0) leg->AddEntry(grm,text,"p");
	grm->Draw("PL");
      }
    }
  }

  leg->Draw();
  cinj->Update();

  if(doout){
    ps->Close();
    sprintf(fname,"%sENCninj_%d_%d-%d.txt",RESULTS_DIRECTORY,run[refer],scans[refer],scans[NDAT-1]);
    output = fopen(fname,"w");
    
    fprintf(output,"time:");
    for(j=1;j<NDAT;j++)
      fprintf(output,"\t%s",clock[j]);
    fprintf(output,"\n");

    for(i=0;i<nmod;i++){
      for(chip=0;chip<12;chip++){
	fprintf(output,"%s\t",mname[i]);
	for(j=1;j<NDAT;j++)
	  if(j!=refer) fprintf(output,"%.1f\t",noise[j][i][chip]-noise[refer][i][chip]);
	fprintf(output,"\n");
	//fprintf(output,"%.1f\n",noise[refer][i][chip]);
      }
    }
    fclose(output);
  }

  return;
}

