#include "Evtree.h"                   //***
#include "TMath.h"                    //*** 
  
void GammaAna(Int_t runno, Int_t scanno, Float_t qthr, 
	      Bool_t slow=kFALSE, Bool_t usemask=kFALSE,
	      Int_t tpgrun=-999, Int_t tpgscan=-1) {

  #ifdef DATA_DIRECTORY
    // do not reload parameters.h
  #else
    // must load parameters.h
  #include <parameters.h>
  #endif

  //finds gamma for all chips of chosen module
  
  FILE *in, *txtout;
  Char_t fname[256];
  
  Char_t line[1024],filename[70],module0[30],module1[30], filestore[50];
  Char_t mod_name[50],mname[MAX_MODULES][50],texta[64],textb[64];
  Char_t dbasedir[128];

  Int_t i, mod, nmod, nevt=0, chip, mask_strip, hpe1, hpe2,scans[3],modchip,link,nread=1;
  
  Double_t sumsqd[12][MAX_MODULES], sum[12][MAX_MODULES];
  Double_t Ne, Var, Del2, CMN, dCMN;
  
  Bool_t notmasked[MAX_MODULES][MAX_LINKS*MAX_CHIPS*MAX_TARGETS],encfile=kFALSE;

  Float_t charges[3] = {1.50,2.00,2.50};
  Float_t gain[MAX_MODULES][MAX_LINKS][MAX_CHIPS],offset[MAX_MODULES][MAX_LINKS][MAX_CHIPS];
  Float_t noise[MAX_MODULES][MAX_LINKS][MAX_CHIPS];
  Float_t data, dummy;

  Evtree *m[MAX_MODULES];
  TTree *tree[MAX_MODULES];
  
  TFile *f;

  if(tpgrun!=-999){
    // get ENC reference
    if(tpgrun==-1)  tpgrun=runno;
    if(tpgscan==-1) tpgscan=scanno-3;
    sprintf(filename,"%sENCsum_%d_%d-%d.txt",RESULTS_DIRECTORY,tpgrun,tpgscan,tpgscan+2);
    in = fopen(filename,"r");
    if(in){  // did the ENC fits already, read from file
      encfile=kTRUE;
      for(i=0;i<7 && nread==1;i++)
	nread = fscanf(in,"%s",texta);
      if(nread!=1){
	printf("no data in file %s\n",fname);
	return;
      }
      for(i=0;i<MAX_MODULES && nread==1;i++){
	for(chip=0;chip<MAX_LINKS*MAX_CHIPS && nread==1;chip++){
	  if(chip)
	    nread = fscanf(in,"%s",texta);
	  else
	    nread = fscanf(in,"%s",mname[i]);
	  nread = fscanf(in,"%s",texta);
	  if(chip<6)
	    nread = fscanf(in,"%f",&(noise[i][0][chip]));
	  else
	    nread = fscanf(in,"%f",&(noise[i][1][chip-6]));
	}
      }
      fclose(in);
    }
    else{  // have to fit the tpg stuff myself...
      for(i=0;i<3;i++)
	scans[i] = tpgscan+i;
      gROOT->ProcessLine(".L ScurveFit.cpp");
      ScurveFit(tpgrun,3,1,scans,charges,4,gain,offset,noise);
    }
  }

  //open data file
  
  sprintf(filename,"%sstrun%d_%d.root",DATA_DIRECTORY,runno,scanno);
  f = new TFile(filename);
    
  TObjString* time  = (TObjString*)f->Get("Time");
  TObjString* place = (TObjString*)f->Get("Place");
  
  for(nmod=0;nmod<MAX_MODULES;nmod++){
    // find modules  
    sprintf(mod_name,"ModuleName;%d",nmod+1);
    TObjString* name = (TObjString*)f->Get(mod_name);
    if(name){
      sprintf(mname[nmod],"%s",name->GetName());
      if(!strcmp(mname[nmod],"missing")) break;
    }
    else
      break;
    // init tree
    sprintf(module0,"EvTree_m%d",nmod);
    tree[nmod] = (TTree*)gDirectory->Get(module0);
    m[nmod] = new Evtree(tree[nmod]);
    i=nevt;
    nevt = tree[nmod]->GetEntries();
    if(nmod>0 && i!= nevt){
      printf("Module %s has %d events, should have been\n",mname[nmod], nevt, i);
      return;
    }

    // find masked strips
  
    for(i=0;i<MAX_LINKS*MAX_CHIPS*MAX_TARGETS;i++) notmasked[nmod][i] = kTRUE;

    if(usemask){
      sprintf(fname,"D:\\sctvar\\config\\%s.mask",mname[nmod]);
      in = fopen(fname,"r");
      if(!in){
	printf("Can't open mask file %s\n",fname);
	return;
      }
      
      printf("Found the following strips masked for %s: ",mname[nmod]);
      i=1;
      strcpy(texta," ");
      while(i==1){
	strcpy(textb,texta);
	i = fscanf(in,"%s",texta);
	if(texta[0]=='#' && i==1){
	  sscanf(textb,"%d",&mask_strip);
	  if(mask_strip<MAX_CHIPS*MAX_LINKS*MAX_TARGETS && mask_strip>0){
	    printf("%d ",mask_strip);
	    notmasked[nmod][mask_strip] = kFALSE;
	  }
	}
      }
      fclose(in);
      printf("\n");
    }
  }

  if(tpgrun==-999){
    printf("Reading ENC reference from archive\n");
    if(strncmp(gSystem->GetName(),"Unix",4))
      sprintf(dbasedir,"J:\\public\\refdat\\");
    else
      sprintf(dbasedir,"~/public/refdat/");
    for(i=0;i<nmod;i++){
      sprintf(fname,"%s%s.dat",dbasedir,mname[i]);
      in=fopen(fname,"r");
      if(!in){
	printf("cannot open file %s\n",fname);
	for(chip=0;chip<6;chip++){
	  noise[i][0][chip] = 0.;
	  noise[i][1][chip] = 0.;
	}
      } else{
	nread=2;
	for(chip=0;chip<12 && nread==2;chip++){
	  nread = fscanf(in,"%f %f",&data,&dummy);
	  if(nread!=2) data = 0.; 
	  if(chip<6)
	    noise[i][0][chip]=data;
	  else
	    noise[i][1][chip-6]=data;
	}
	fclose(in);
      }
    }
  }

  //nevt = 10000;
    
  printf("Processing %d modules with %d events\n",nmod,nevt);

  // zero result arrays

  for(mod=0;mod<nmod;mod++){
    for(chip=0;chip<MAX_LINKS*MAX_CHIPS;chip++){
      sum[chip][mod]=0.0;
      sumsqd[chip][mod]=0.0;
    }
  }

  //start the event looop

  for(Int_t ev=0;ev<nevt;ev++){ 
    
    //load arrays    
    for(mod=0;mod<nmod;mod++){
      m[mod]->GetEntry(ev);
      //get number of hits per chip in this event      
      for(chip=0;chip<MAX_CHIPS;chip++){
	if(slow){
	  hpe1=0;
	  hpe2=0;
	  for(Int_t strip=0;strip<MAX_TARGETS;strip++){
	    if(notmasked[mod][strip+chip*MAX_TARGETS] && 
	                 m[mod]->strm0[strip+chip*MAX_TARGETS]) hpe1++;
	    if(notmasked[mod][strip+(chip+MAX_CHIPS)*MAX_TARGETS] && 
	                 m[mod]->strm1[strip+chip*MAX_TARGETS]) hpe2++;
	  }
	  // top side
	  sum[chip][mod]              += (Double_t) hpe1;
	  sumsqd[chip][mod]           += (Double_t) (hpe1*hpe1);
	  // bottom side
	  sum[chip+MAX_CHIPS][mod]    += (Double_t) hpe2;
	  sumsqd[chip+MAX_CHIPS][mod] += (Double_t) (hpe2*hpe2);
	}

	else{
	  // top side
	  sum[chip][mod]              += (Double_t)  m[mod]->hits0[chip];
	  sumsqd[chip][mod]           += (Double_t) (m[mod]->hits0[chip]*m[mod]->hits0[chip]);
	  // bottom side
	  sum[chip+MAX_CHIPS][mod]    += (Double_t)  m[mod]->hits1[chip];
	  sumsqd[chip+MAX_CHIPS][mod] += (Double_t) (m[mod]->hits1[chip]*m[mod]->hits1[chip]);	
	}

      }
    }

    if((ev+1)%100000==0) printf("finished event %d \n",ev+1);
  }
  
  f->Close();
  
  // open TXT output file
  sprintf(fname,"%sGammas_preana_%d_%d.txt",RESULTS_DIRECTORY,runno,scanno);
  txtout = fopen(fname,"w");
  if(!txtout){
    printf("ERROR opening output file %s!\n",fname);
    return;
  }
  
  fprintf(txtout,"Gamma for %d events at Qthr=%.2f\nModule\tChip\tENC\tNe\tVar\n",nevt,qthr);

  for(mod=0;mod<nmod;mod++){
    
    //delete text summary from tpg
    sprintf(fname,"del %s%s_stfit.txt",RESULTS_DIRECTORY,mname[mod]);
    if(tpgrun>0 && !encfile) gSystem->Exec(fname);
    //pre-analysis    
    for(chip=0;chip<MAX_CHIPS*MAX_LINKS;chip++){  
      link=0;
      modchip=chip;
      if(chip>5){
	link=1;
	modchip-=6;
      }
      
      Ne=sum[chip][mod]/(Double_t)nevt;
      Var=sumsqd[chip][mod]/(Double_t)nevt-(Ne*Ne);
      fprintf(txtout,"%s\t%d\t%.1f\t%le\t%le\n",mname[mod],chip,noise[mod][link][modchip],Ne,Var);
      
    }

  }  
  fclose(txtout);

  // plot result
  gROOT->ProcessLine(".L GammaPlot.cpp");
  GammaPlot(runno,scanno);

  return;
}
