void ScurveStability(Int_t runstrobe, Int_t standalone, Int_t interac=1, Int_t ext_nexec=2, Int_t ext_twait=0){

  //
  // Repeatedly run a three-point gain to check stability of result
  // 
  // Version 0.1 JGK 04.04.01
  // Version 1.1 JGK 01.06.01: postpone analysis till after scans -> faster
  // Version 1.2 JGK 31.07.01: added some interplay with DCS for stand-alone 
  //                           running
  //

#define MAX_EXECS  100

  Int_t n_exec=0,i,j,nmod,round,delt,towait, response, t_HVupdate;
  Int_t scans[MAX_EXECS][3];
  Float_t charges[3] = {1.50,2.00,2.50};
  Char_t answer[1];
  TDatime* dt = new TDatime(); // MEMORY LEAK???

  Short_t fit_type = 4;   // linear fit


  gROOT->ProcessLine(".L StrobeDelay.cpp");
  gROOT->ProcessLine(".L sleep.cpp");
  gROOT->ProcessLine(".L DCSSafety.cpp");

  //Store settings before the scan
  e->SaveState();

  // Clear abort flag to start
  e->abort=0;

  if(interac){
    while(n_exec<1 || n_exec > MAX_EXECS){
      printf("How many runs of a three point gain scan do you want?\n");
      scanf("%d",&n_exec);
    }
    printf("Specify the time of the intervals in minutes (int):\n");
    scanf("%d",&delt);
  }
  else{
    n_exec = ext_nexec;
    delt = ext_twait;
  }
  printf("You asked for %d minutes wait.\n",delt);
  sleep(10);

  // HV timer
  dt->Set();
  t_HVupdate = dt->GetTime();
  // repetition loop
  for(round=0;round<n_exec;round++){
    printf("Starting round %d out of %d\n",round+1,n_exec);
    // run strobe delay scan if required
    if(runstrobe){
      if(e->abort==0) StrobeDelay(1);
    }
    // Configure the scan
    e->StOutScanReset();
    e->ascii_nt=0;
    e->abort=0;
    
    e->do_cal_loop=1;        // cal loop ON
    e->do_fits=1;            // integrated fitting ON
    
    e->burst_trtype = 3;     // CAL + L1A
    e->burst_ntrigs = 1000;
    
    e->ConfigureVariable(11,1);  // X1X compression 
    e->ConfigureVariable(13,0);  // EDGE DETECT OFF
    e->ConfigureVariable(14,0);  // Route Mask OFF
    
    
    // query DCS when in stand-alone mode
    if(standalone)
      if(DCSSafety(kTRUE)) return;
    // send ramp command every 10 min to keep SCTHV's busy
    dt->Set();
    if( ((dt->GetTime()-t_HVupdate)>600)
        ||((dt->GetTime()-t_HVupdate)<0)  ){ // past midnight!!
      t_HVupdate = dt->GetTime();
      e->HVRamp(0);
    }
    // run first scan
    if(e->abort==0){
      e->ConfigureVariable(42,charges[0]);  // 42 - QCAL in fC
      e->ConfigureScan(1,0,250,5);
      st_scan(11,-1);
      scans[round][0]=e->scannum;
    }
    
    // query DCS when in stand-alone mode
    if(standalone)
      if(DCSSafety(kTRUE)) return;
    // send ramp command every 3 min to keep SCTHV's busy
    dt->Set();
    if( ((dt->GetTime()-t_HVupdate)>180)
        ||((dt->GetTime()-t_HVupdate)<0)  ){ // past midnight!!
      t_HVupdate = dt->GetTime();
      e->HVRamp(0);
    }
    // run second scan
    if(e->abort==0){
      e->ConfigureVariable(42,charges[1]);
      e->ConfigureScan(1,0,300,5);
      st_scan(11,-1);
      scans[round][1]=e->scannum;
    }
    
    // query DCS when in stand-alone mode
    if(standalone)
      if(DCSSafety(kTRUE)) return;
    // send ramp command every 3 min to keep SCTHV's busy
    dt->Set();
    if( ((dt->GetTime()-t_HVupdate)>180)
        ||((dt->GetTime()-t_HVupdate)<0)  ){ // past midnight!!
      t_HVupdate = dt->GetTime();
      e->HVRamp(0);
    }
    // run third scan
    if(e->abort==0){
      e->ConfigureVariable(42,charges[2]);
      e->ConfigureScan(1,50,350,5);
      st_scan(11,-1);
      scans[round][2]=e->scannum;
    }

    // wait
    if(e->abort==0){
      if(round<n_exec-1){
	for(j=0;j<delt;j++){
	  towait = delt-j;
	  if(towait%5==0){
	    printf("waiting for %d minutes\n",towait);
	    // query DCS when in stand-alone mode
	    if(standalone)
	      if(DCSSafety(kTRUE)) return;
	  }
	  if(e->abort==0) sleep(60);
	}
      }
    }
  }
  
  // Reset values after scan
  e->RestoreState();
  delete dt;

  if(interac){
    // switch LV off when in stand-alone mode
    j=0;
    i=60;
    if(standalone){
      st_start_stopper();
      printf("Will switch off LV in 10 minutes - press ABORT button to leave LV on\n");
      for(j=0;j<i;j++){
	towait = i-j;
	if(towait%10==0) printf("waiting %.2f minutes\n",((Float_t)towait)/6.);
	if(st_do_stop()) break;
	else sleep(10);
      }
      if(j==i) {
	printf("\nwill switch off LV now\n\n");
	e->LVOff();
	e->HVRamp(1);
	DCSSafety(kFALSE);
      }
      st_stop_stopper();
    }
    
    // analyse data
    printf("do you want to run the analysis now? (y/n)\n");
    scanf("%s",answer);
    gROOT->ProcessLine(".L ScurveStabilityPlot.cpp");
    if(!strcmp(answer,"y"))
      ScurveStabilityPlot(e->runnum,scans,n_exec);
    else
      printf("\nPlease run analysis calling: .x ScurveStabilityPlot.cpp(%d,%d,%d)\n\n",e->runnum,scans[0][0],n_exec);
  }
  else{
    gROOT->ProcessLine(".L ScurveStabilityPlot.cpp");
    ScurveStabilityPlot(e->runnum,scans,n_exec);
  }
    
  DCSSafety(kFALSE);
  printf("\nFinished...............\n");
  if(standalone && j==i && interac) 
    printf("\n\nPlease note that LV+HV is off now.\n\n");
  return;
}

void ScurveStability(Int_t runstrobe){
  ScurveStability(runstrobe,0);
  return;
}

void ScurveStability(){
  ScurveStability(0,0);
  return;
}
