void ScurveStabilityPlot(Int_t run, Int_t scans[100][3], Int_t nrounds){

  #ifdef DATA_DIRECTORY
    // do not reload parameters.h
  #else
    // must load parameters.h
  #include <parameters.h>
  #endif

  #define MAX_EXECS  100

  Int_t nread=1,i=0,j,round,chipno,nmod;
  Int_t tsec=0,tfirst=0,scans2[3];
  Float_t gain[MAX_MODULES][MAX_LINKS][MAX_CHIPS],offset[MAX_MODULES][MAX_LINKS][MAX_CHIPS];
  Float_t noise[MAX_MODULES][MAX_LINKS][MAX_CHIPS],dif[MAX_LINKS*MAX_CHIPS],chipid[MAX_LINKS*MAX_CHIPS];
  char texta[1024],textb[1024],timestr[8], smon[32],sday[32],srun[5];
  char mname[MAX_MODULES][32],a[10],b[10],c[10],d[10];
  FILE* input;
  TDatime *dt = new TDatime;
  Int_t date = dt->GetDate();
  Float_t max_g[MAX_MODULES], min_g[MAX_MODULES],max_o[MAX_MODULES], min_o[MAX_MODULES];
  Float_t max_n[MAX_MODULES], min_n[MAX_MODULES];
  Double_t tpoints[MAX_EXECS],ypoints[MAX_EXECS];
  Double_t hga[MAX_MODULES][MAX_LINKS*MAX_CHIPS][MAX_EXECS],hof[MAX_MODULES][MAX_LINKS*MAX_CHIPS][MAX_EXECS];
  Double_t hno[MAX_MODULES][MAX_LINKS*MAX_CHIPS][MAX_EXECS];
  Color_t h_color;
  char line[1024],h_name[64],h_title[64],f_name[128];
  Double_t fitpar;
  Double_t diff_o[MAX_MODULES][MAX_LINKS*MAX_CHIPS], diff_g[MAX_MODULES][MAX_LINKS*MAX_CHIPS], diff_n[MAX_MODULES][MAX_LINKS*MAX_CHIPS];
  Float_t charges[3] = {1.50,2.00,2.50};

  TObjString* time = 0;
  TObjString* place = 0;
  TObjString* name = 0;

  // load the 3pt-gain plot macro 
  gROOT->ProcessLine(".L ScurveFit.cpp");
  gROOT->ProcessLine(".L strtosec.cpp");

  for(round=0;round<nrounds;round++){

    sprintf(f_name,"%sstrun%d_%d.root",DATA_DIRECTORY,run,scans[round][0]);
    printf("opening file %s\n",f_name);
    TFile *strun = new TFile(f_name);
    nmod=0;
    for (i=0; i<MAX_MODULES; i++){
      if(i==0){
	if(time) delete time;
        time = (TObjString*)strun->Get("Time");
        if(place) delete place;
        place = (TObjString*)strun->Get("Place");
	sscanf(time->GetName(), "%s %s %s %s %s",a,b,c,timestr,d);
	tsec = strtosec(timestr);
	if(round==0) tfirst=tsec;
	tpoints[round] = ((Double_t)(tsec-tfirst))/3600.;
      }
      sprintf(h_title,"ModuleName;%d",i+1);
      if(name) delete name;
      name = (TObjString*)strun->Get(h_title);
      if(name){
	if(strcmp(name->GetName(),"missing")){
	  sprintf(mname[nmod],"%s",name->GetName());
	  nmod++;
	}
      }
    }
    strun->Close();
    delete strun;

    for(i=0;i<3;i++)
      scans2[i] = scans[round][i];
    
    // do the fitting
    ScurveFit(run,3,1,scans2,charges,4,gain,offset,noise);

    //reset max/min arrays
    for(j=0;j<nmod;j++){
      min_g[j] = 1000.;
      min_o[j] = 1000.;
      min_n[j] = 1000.;
      max_g[j] = 0.;
      max_o[j] = 0.;
      max_n[j] = 0.;
    }

    // fill arrays with data
    for(j=0;j<nmod;j++){
      for(i=0;i<2;i++){
	for(chipno=0;chipno<MAX_CHIPS;chipno++){
	  hga[j][chipno+i*MAX_CHIPS][round] = (double) gain[j][i][chipno];
	  hof[j][chipno+i*MAX_CHIPS][round] = (double) offset[j][i][chipno];
	  hno[j][chipno+i*MAX_CHIPS][round] = (double) noise[j][i][chipno];
	  if(gain[j][i][chipno]<min_g[j] && gain[j][i][chipno]>0)     min_g[j] = gain[j][i][chipno];
	  if(gain[j][i][chipno]>max_g[j])                             max_g[j] = gain[j][i][chipno];
	  if(offset[j][i][chipno]<min_o[j] && offset[j][i][chipno]>0) min_o[j] = offset[j][i][chipno];
	  if(offset[j][i][chipno]>max_o[j])                           max_o[j] = offset[j][i][chipno];
	  if(noise[j][i][chipno]<min_n[j] && noise[j][i][chipno]>0)   min_n[j] = noise[j][i][chipno];
	  if(noise[j][i][chipno]>max_n[j])                            max_n[j] = noise[j][i][chipno];
	}
      }
    }
  }

  // open canvas
  TCanvas* c4= new TCanvas ("c4","ScT",1100,750);

  // open PS output file
  sprintf(f_name,"%sSStabPlot_%d_%d-%d.ps",PS_DIRECTORY,run,scans[0][0],scans[nrounds-1][2]);
  ps = new TPostScript(f_name,112);

  //loop over modules
  for(i=0;i<nmod;i++){

    // rename txt out file
    sprintf(f_name,"ren %s%s_stfit.txt  %s_%d_%d-%d.txt",RESULTS_DIRECTORY,
	    mname[i],mname[i],run,scans[0][0],scans[nrounds-1][2]);
    gSystem->Exec(f_name);

    //plot the result

    ps->NewPage();

    c4->cd();
    c4->Clear();
    
    pad_text   = new TPad("pad_text", "Text",        0.00,0.90,1.00,1.00);
    pad_plot   = new TPad("pad_plot", "Plots",       0.00,0.00,1.00,0.90);
    
    pad_text->Draw();
    pad_plot->Draw();
    pad_plot->Divide(3,2,0,0);
    
    c4->Update();
    
    gStyle->SetOptStat(0);
    gStyle->SetOptFit(0);
        
    pad_text->cd();
    TPaveText *t1 = new TPaveText(0.0,0.0,1.0,1.0);
    t1->SetFillColor(10);
    t1->SetTextAlign(13);
    t1->SetTextSize(0.3);
    t1->SetTextColor(4);
    sprintf(line,"ATLAS SCT Module Test:  Gain/Offset/Noise vs. Time");
    if(time)  sprintf(line,"%s  -  %s",line,time->GetName());
    if(place) sprintf(line,"%s  -  %s",line,place->GetName());
    t1->AddText(line);
    sprintf(line,"Module %d %s, run %d, scans %d-%d",i,mname[i],run,scans[0][0],scans[nrounds-1][2]);
    t1->AddText(line);
    t1->Draw();
    
    pad_plot->cd(1);
    max_g[i]*=1.1;
    min_g[i]*=0.9;
    for(j=0;j<12;j++){
      // set colour
      if(j<6)
	h_color = (Color_t) j+1;
      else
	h_color = (Color_t)(j-5);
      // fill graph-array
      for(round=0;round<nrounds;round++){
	ypoints[round] = hga[i][j][round];
      }
      TGraph *t = new TGraph(nrounds,tpoints,ypoints);
      t->SetMarkerStyle(20);
      t->SetMarkerSize(1);
      t->SetMarkerColor(h_color);
      t->SetMaximum((Double_t)max_g[i]);
      t->SetMinimum((Double_t)min_g[i]);
      if(j<6) t->SetTitle("Stream 0");
      else t->SetTitle("Stream 1");
      if(j==0){
	t->Draw("AP");
        t->GetXaxis()->SetTitle("Time(h)");
	t->GetYaxis()->SetTitle("Gain");
	t->Draw("AP");
      } else if(j==6){
	pad_plot->cd(4);
	t->Draw("AP");
        t->GetXaxis()->SetTitle("Time(h)");
	t->GetYaxis()->SetTitle("Gain");
	t->Draw("AP");
      } else
	t->Draw("P"); 
      fc= new TF1("fc","pol0",tpoints[0],tpoints[nrounds-1]);
      t->Fit("fc","Q0","");
      fitpar = fc->GetParameter(0);
      diff_g[i][j] = 0.;
      for(round=0;round<nrounds;round++)
	diff_g[i][j] += (fitpar-ypoints[round])*(fitpar-ypoints[round]);
      if(nrounds>1)
        diff_g[i][j] = (Float_t) TMath::Sqrt((Double_t)diff_g[i][j]/(Double_t)(nrounds-1));
      else
        diff_g[i][j] = (Float_t) TMath::Sqrt((Double_t)diff_g[i][j]);
      // write legend
      if(j==0 || j==6) leg = new TLegend(.7,.7,.95,.95);
      sprintf(f_name,"Chip %d",j);
      leg->AddEntry(t,f_name,"p");
      if(j==5 || j==11) leg->Draw();
    }

    pad_plot->cd(2);
    max_o[i]*=1.1;
    min_o[i]*=0.9;
    for(j=0;j<12;j++){
      // set colour
      if(j<6)
	h_color = (Color_t) j+1;
      else
	h_color = (Color_t)(j-5);
      // fill graph-array
      for(round=0;round<nrounds;round++){
	ypoints[round] = hof[i][j][round];
      }
      TGraph *t = new TGraph(nrounds,tpoints,ypoints);
      t->SetMarkerStyle(20);
      t->SetMarkerSize(1);
      t->SetMarkerColor(h_color);
      t->SetMaximum((Double_t)max_o[i]);
      t->SetMinimum((Double_t)min_o[i]);
      if(j<6) t->SetTitle("Stream 0");
      else t->SetTitle("Stream 1");
      if(j==0){
	t->Draw("AP");
        t->GetXaxis()->SetTitle("Time(h)");
	t->GetYaxis()->SetTitle("Offset");
	t->Draw("AP");
      } else if(j==6){
	pad_plot->cd(5);
	t->Draw("AP");
        t->GetXaxis()->SetTitle("Time(h)");
	t->GetYaxis()->SetTitle("Offset");
	t->Draw("AP");
      } else
	t->Draw("P"); 
      fc= new TF1("fc","pol0",tpoints[0],tpoints[nrounds-1]);
      t->Fit("fc","Q0","");
      fitpar = fc->GetParameter(0);
      diff_o[i][j] = 0.;
      for(round=0;round<nrounds;round++)
	diff_o[i][j] += (fitpar-ypoints[round])*(fitpar-ypoints[round]);
      if(nrounds>1)
	diff_o[i][j] = (Float_t) TMath::Sqrt((Double_t)diff_o[i][j]/(Double_t)(nrounds-1));
      else
	diff_o[i][j] = (Float_t) TMath::Sqrt((Double_t)diff_o[i][j]);
    }

    pad_plot->cd(3);
    max_n[i]*=1.1;
    min_n[i]*=0.9;
    for(j=0;j<MAX_LINKS*MAX_CHIPS;j++){
      // set colour
      if(j<6)
	h_color = (Color_t) j+1;
      else
	h_color = (Color_t)(j-5);
      // fill graph-array
      for(round=0;round<nrounds;round++){
	ypoints[round] = hno[i][j][round];
      }
      TGraph *t = new TGraph(nrounds,tpoints,ypoints);
      t->SetMarkerStyle(20);
      t->SetMarkerSize(1);
      t->SetMarkerColor(h_color);
      t->SetMaximum((Double_t)max_n[i]);
      t->SetMinimum((Double_t)min_n[i]);
      if(j<6) t->SetTitle("Stream 0");
      else t->SetTitle("Stream 1");
      if(j==0){
	t->Draw("AP");
        t->GetXaxis()->SetTitle("Time(h)");
	t->GetYaxis()->SetTitle("Noise");
	t->Draw("AP");
      } else if(j==6){
	pad_plot->cd(6);
	t->Draw("AP");
        t->GetXaxis()->SetTitle("Time(h)");
	t->GetYaxis()->SetTitle("Noise");
	t->Draw("AP");
      } else
	t->Draw("P"); 
      fc= new TF1("fc","pol0",tpoints[0],tpoints[nrounds-1]);
      t->Fit("fc","Q0","");
      fitpar = fc->GetParameter(0);
      diff_n[i][j] = 0.;
      for(round=0;round<nrounds;round++)
	diff_n[i][j] += (fitpar-ypoints[round])*(fitpar-ypoints[round]);
      if(nrounds>1)
	diff_n[i][j] = (Float_t) TMath::Sqrt((Double_t)diff_n[i][j]/(Double_t)(nrounds-1));
      else
	diff_n[i][j] = (Float_t) TMath::Sqrt((Double_t)diff_n[i][j]);
    }
    c4->Update();
    
  }

  ps->Close();
  ps->Delete();

  // summary plot

  // open new PS output file
  sprintf(f_name,"%sSStabPlotSum_%d_%d-%d.ps",PS_DIRECTORY,run,scans[0][0],scans[nrounds-1][2]);
  ps = new TPostScript(f_name,112);

  ps->NewPage();
  c4->cd();
  c4->Clear();
  
  pad_text   = new TPad("pad_text", "Text",        0.00,0.90,1.00,1.00);
  pad_plot   = new TPad("pad_plot", "Plots",       0.00,0.00,1.00,0.90);
  
  pad_text->Draw();
  pad_plot->Draw();
  
  c4->Update();
  
  gStyle->SetOptStat(0);
  gStyle->SetOptFit(0);
  
  pad_text->cd();
  TPaveText *t1 = new TPaveText(0.0,0.0,1.0,1.0);
  t1->SetFillColor(10);
  t1->SetTextAlign(13);
  t1->SetTextSize(0.3);
  t1->SetTextColor(4);
  sprintf(line,"ATLAS SCT: S-curve Stability");
  if(time)  sprintf(line,"%s  -  %s",line,time->GetName());
  if(place) sprintf(line,"%s  -  %s",line,place->GetName());
  t1->AddText(line);
  sprintf(line,"Run %d, scans %d-%d",run,scans[0][0],scans[nrounds-1][2]);
  t1->AddText(line);
  t1->Draw();
    
  pad_plot->cd();
  for(i=0;i<nmod;i++){
    for(j=0;j<MAX_LINKS*MAX_CHIPS;j++){
      dif[j] = diff_n[i][j];
      chipid[j] = (Float_t) j;
    }

    if(i<8)
      h_color = (Color_t) i+1;
    else
      h_color = (Color_t) i-7;
    TGraph *t = new TGraph(MAX_LINKS*MAX_CHIPS,chipid,dif);
    if(i<8)
      t->SetMarkerStyle(20);
    else
      t->SetMarkerStyle(21);
    t->SetMarkerSize(1);
    t->SetMarkerColor(h_color);
    t->SetLineColor(h_color);
    t->SetMaximum(100.);
    t->SetMinimum(0.);
    t->SetTitle("Summary");
    if(i==0){
      t->Draw("APL");
      t->GetXaxis()->SetTitle("Chip");
      t->GetYaxis()->SetTitle("rms-Spread in Input-Noise");
      t->Draw("APL");
    } else
      t->Draw("PL"); 
    if(i==0) leg = new TLegend(.88,.2,1.0,.95);
    sprintf(line,"Module %s",mname[i]);
    leg->AddEntry(t,line,"p");
    if(i==nmod-1) leg->Draw();
  }

  ps->Close();
  ps->Delete();

  // open results text file
  sprintf(f_name,"%sSStab_%d_%d-%d.txt",RESULTS_DIRECTORY,run,scans[0][0],scans[nrounds-1][2]);
  FILE* outfile = fopen(f_name,"a+");
  if(outfile==NULL){
    printf("Could not open result file %s\n",f_name);
    return;
  } else 
    printf("File %s opened to append results \n",f_name);

  for(j=0;j<nmod;j++){
    for(i=0;i<3;i++){
      fprintf(outfile,"%s",mname[j]);
      if(i<2) fprintf(outfile,"\t");
    }
    if(j<nmod-1) 
      fprintf(outfile,"\t");
    else
      fprintf(outfile,"\n");
  }
  for(j=0;j<nmod;j++){
    fprintf(outfile,"gain\toffs.\tnoise");
    if(j<nmod-1) 
      fprintf(outfile,"\t");
    else
      fprintf(outfile,"\n");
  }
  for(i=0;i<MAX_LINKS*MAX_CHIPS;i++){
    for(j=0;j<nmod;j++){
      fprintf(outfile,"%f\t%f\t%f",diff_g[j][i],diff_o[j][i],diff_n[j][i]);
      if(j<nmod-1) 
	fprintf(outfile,"\t");
      else
	fprintf(outfile,"\n");
    }
  }
  fclose(outfile);

  if(time) delete time;
  if(place) delete place;
  if(name) delete name;

  return;
}

void ScurveStabilityPlot(Int_t run, Int_t firstscan, Int_t nrounds){
  Int_t scans[100][3],i,j;
  for(i=0;i<nrounds;i++){
    for(j=0;j<3;j++){
      scans[i][j] = j+i*3+firstscan;
    }
  }
  ScurveStabilityPlot(run,scans,nrounds);
  return;
}
