/* optif_menu.c: Call OPTIF functions from a menu - useful
 * for simple testing.
 * Version 1.1. J.C.Hill. 16 August 2000.
 * V1.0. 11 August 2000. JCH. Initial version.
 * V1.1. 16 August 2000. JCH. Bug fixes in handling string input.
 */

#include <stdio.h>

#include "optif_lib.h"
#include "optif_menu.h"

/* Two functions with single character commands. 
 * Parameters need to be included in the command line.
 * BPM chip interactions are in a separate routine*/

int optif_menu(id)
int id;
{
  char c, cm[128], command[10];
  int status;
  short v1, v2, v3, v4;
  int go = 1;

  while (go) {
	  status = 0;
	  printf("\n\nOPTIF: e(x)it, set (c)lock, co(m)mand, (r)eceiver polarity\n");
	  printf("OPTIF: read cloc(k), comman(d), r(e)eceiver polarity\n");
	  printf("OPTIF: set (t)hresholds, (l)asers, re(s)et bpm0/1\n");
	  printf("OPTIF: (b)pm chip functions\n");
	  printf("OPTIF: (i)nitialise, (v)ersion info, (p)rint status\n");
	  printf("==> ");
/* */
	  gets(cm);
	  sscanf(cm,"%1s %d %d",command,&v1,&v2);
	  c = command[0];
	  
	  if (c == 'x') go = 0;
	  else if (c == 'b') optif_menu_bpm(id);
	  else if (c == 'c') status = optif_set_clock_invert( id, v1);
	  else if (c == 'd') {
		  status = optif_read_command_invert( id, &v1);
		  printf("command invert %d\n",v1);
	  }
	  else if (c == 'e') {
		  status = optif_read_receiver_invert( id, v1, &v2);
		  printf("receiver invert stream %d, %d\n",v1, v2);
	  }
	  else if (c == 'i') status = optif_init( id );
	  else if (c == 'k') {
		  status = optif_read_clock_invert( id, &v1);
		  printf("clock invert %d\n",v1);
	  }
	  else if (c == 'l') status = optif_set_laser( id, v1, v2);
	  else if (c == 'm') status = optif_set_command_invert( id, v1);
	  else if (c == 'p') status = optif_status( id );
	  else if (c == 'r') status = optif_set_receiver_invert( id, v1, v2);
	  else if (c == 's') status = optif_reset_bpm( id, v1);
	  else if (c == 't') status = optif_set_threshold( id, v1, v2);
	  else if (c == 'v') {
		  status = optif_read_info( id, &v1, &v2, &v3, &v4);
		  printf("Version %d type %d firmware %d manufacturer %hx\n",
			  v3, v4, v1, v2);
	  }
	  else {
		  printf("OPTIF: unknown command %c\n",c);
	  }

	  if( status!=0 ) printf("Command failed with status %d\n",status);
  }
  return 0;
}
/* */
void optif_menu_bpm(id)
int id;
{
char c, cm[128],command[10];
  int status;
  short v1, v2;
  int go = 1;

  while (go) {
	  status = 0;
	  printf("\n\nOPTIF BPM: e(x)it this menu, (p)rint status\n");
	  printf("OPTIF BPM: set (c)hannelmask, c(o)arsedelay, (f)inedelay\n");
	  printf("OPTIF BPM: set (m)k/sp negative edge, m(k)/sp positive edge\n");
	  printf("OPTIF BPM: p(u)lsewidth, mo(d)e, (r)andom/fixed, (t)est data\n");
	  printf("==> ");
/* */
	  gets(cm);
	  sscanf(cm,"%s %d %d",command,&v1,&v2);
	  c = command[0];

	  if (c == 'x') go = 0;
	  else if (c == 'c') status = optif_bpm_set_channelmask( id, v1, v2);
	  else if (c == 'd') status = optif_bpm_set_mode( id, v1, v2);
	  else if (c == 'f') status = optif_bpm_set_finedelay( id, v1, v2);
	  else if (c == 'k') status = optif_bpm_set_mksppos( id, v1, v2);
	  else if (c == 'm') status = optif_bpm_set_mkspneg( id, v1, v2);
	  else if (c == 'o') status = optif_bpm_set_coarsedelay( id, v1, v2);
	  else if (c == 'p') status = optif_status( id );
	  else if (c == 'r') status = optif_bpm_set_random( id, v1, v2);
	  else if (c == 't') status = optif_bpm_set_testdata( id, v1, v2);
	  else if (c == 'u') status = optif_bpm_set_pulsewidth( id, v1, v2);
	  else {
		  printf("OPTIF: unknown command %c\n",c);
	  }

	  if( status!=0 ) printf("Command failed with status %d\n",status);
  }
}
