
/*
 * scthv_lib.h
 *
 * Version 0.1 PWP 20.07.00
 * Version 0.2 PWP 06.12.00 - revised voltage step size
 * Version 0.3 PWP 12.07.01 - changes to suit version 0.30 of scthv_lib.c
 * Version 0.4 PWP 23.01.02 - changes to suit version 0.40 of scthv_lib.c
 * Version 0.5 PWP 10.06.02 - changes to suit version 0.50 of scthv_lib.c
 * Version 0.51 JCH 24.06.02 - support for ATLAS RCC library (for SBCs).
 */

#include <time.h>

#define HV_RAMPING     0x10
#define HV_BADCOMMAND  0x20
#define HV_OVERCURRENT 0x40
#define HV_OVERVOLTAGE 0x80

#define HV_MAX_LOOPS   200

#define CH_OFF          0
#define CH_ON           1
#define CH_RAMPING      2
#define CH_STEADY       3
#define CH_TRIP_PENDING 4
#define CH_TRIPPED      5

#define LO 0
#define HI 1
#define V_STEP     0.125

#define HV_NSAME    3 
#define HV_TSTUCK 300   // seconds = 5 minutes 

static float i_step[4] = {(float)97.565, (float)19.565, (float)3.965, (float)0.676}; 


static char states[6][10]={ "OFF      ", "ON       ", "RAMPING  ",
                            "STEADY   ", "TRIP_PEND", "TRIPPED  " };


/*
 * function prototypes - __STDC__ was causing problems, so now it's gone
 */

int scthv_exit();

int scthv_ison(short card, short chan);
int scthv_off(short card, short chan);
int scthv_on(short card, short chan);

int scthv_get_status(short card, short chan, 
                     float* vmon, float* imon, short* status);
int scthv_monitor(short card, short chan, 
                  float* vmon, float* imon, short* status);
void scthv_print_error(int rcode, short card, short chan);
int scthv_print_status(short card, short chan);
int scthv_repeat_request(short card, short chan);
int scthv_request(short card, short chan, 
                  float vset, float ilim, short ramp);

int scthv_vmemap(unsigned long address, short *card);


/* Useful internal structure for SCTHV info*/

struct scthv_info_structure
{
#if defined (NIVXI) || defined(RCC)
  int window;
#endif
  unsigned long address;
  short missing;

  int vmon[4];   /* local copy of last status info */
  int imon[4];
  char status[4];

  float vset[4]; /* local copy of last request */
  float ilim[4];
  short ramp[4];

  char state[4];

  short nerror[4];  /* n corrupt data errors */
  short nrepeat[4]; /* n times ramp command has been repeated */
  short nsame[4];   /* n times same data has been returned    */
  short nstuck[4];  /* n times channel has been deemed stuck  */

  clock_t last_update[4];  /* time of last monitor request  */
  clock_t diff_update[4];  /* time of last change in v or i */
};

#define scthv_info_size sizeof(struct scthv_info_structure)

