// 
//  TCloac.cpp
//  Version 0.1   PWP 07.08.00
//  Version 0.2   GFM 20010321
//  Version 0,3   GFM 20020320 
//

//  In order to use VBURST sensibly, I needed to write a lot of new
//  code to facilitate the selection of frequencies, modes etc in a
//  reasonably user friendly way. I decided that I might as well do
//  this by writing a new class, TCloac, rather than by adding new
//  routines at the stlib level.  This has the advantage that the
//  following cloac functions can now be called directly from RINT.
//
//  For the time being this class just serves as an interface to
//  stlib layer: eventually the complete functionality could be moved
//  to this layer.
//
//  PWP 11.07.00


extern "C" {
  #include "../CLOAC/cloac_lib.h"
  #include "../CLOAC/cloac_menu.h"
}

#include "../stlib/sct_hardware.h"

#include <iostream.h>

#include "TCloac.h"


ClassImp(TCloac)

TCloac::TCloac(short new_id){

  id = new_id;

  pulse = 0;

  trig_delay = TRIG_DELAY;
  pulse_delay = PULSE_DELAY;
  delay_offset = CLOAC_CAL_OFFSET;

  randomiser = 0;
  periodic_sr = 0;
  periodic_bcr = 0;

  periodic_bits  = 0;
  random_bits    = 0;
  softreset_bits = 0;
 
  periodic_freq  = 600.0; // kHz
  random_freq    = 150.0; // kHz
  softreset_freq =  60.0; //  Hz
}

  
TCloac::~TCloac(){

}

// 
// Print Status - real and imaginary
//

void TCloac::PrintStatus(){

  unsigned short val;

  cout << endl;
  cout << "TCloac::PrintStatus" << endl;
  cout << "SOFTWARE:" << endl;

  cout << "  Trigger Delay = " << trig_delay << endl;
  cout << "  Pulse   Delay = " << pulse_delay << endl;
  cout << "  CLOAC Offset  = " << delay_offset << endl;
  cout << "  Number of triggers  = " << n_trigs << endl;

  cout << "  Presently the ";
  if(pulse) cout << "pulse input register";
  else      cout << "calibration"; 
  cout << " command is selected" << endl << endl;

  cout << "  Randomiser is ";
  if(randomiser) cout << "on" << endl;
  else           cout << "off" << endl; 
  cout << "  Periodic Soft Resets are ";
  if(periodic_sr) cout << "on" << endl;
  else            cout << "off" << endl; 
  cout << "  Periodic BC Resets are ";
  if(periodic_bcr) cout << "on" << endl;
  else             cout << "off" << endl; 


  cout << "  Periodic Trigger Frequency " << periodic_freq << "kHz";
  cout << " or " << periodic_bits << endl;

  cout << "  Random Trigger Frequency " << random_freq << "kHz";
  cout << " or " << random_bits << endl;

  cout << "  Soft Reset Frequency " << softreset_freq << "kHz";
  cout << " or " << softreset_bits << endl << endl;

  cout << "ACTUAL HARDWARE SETTINGS:" << endl;

  cc_read_command(&val);
  cout << "  Command Register:   0x" << hex << val << dec << endl;
  cc_read_delay(&val);
  cout << "  Delay Register:     0x" << hex << val << dec << endl;
  cc_read_enable(&val);
  cout << "  Enable Register:    0x" << hex << val << dec << endl;
  cc_read_frequency(&val);
  cout << "  Frequency Register: 0x" << hex << val << dec << endl;
  cc_read_state(&val);
  cout << "  State Register:     0x" << hex << val << dec << endl;
  cc_read_window(&val);
  cout << "  Window Register:    0x" << hex << val << dec << endl;

  cout << endl;
}

//
// Single Triggers
//

void TCloac::SingleTrigger(){

  cc_load_command(0);
  cc_load_command(VSINGLETRIG);
  cc_load_command(0);
}

void TCloac::SingleSoftReset(){

  cc_load_command(0);
  cc_load_command(VSRST);
  cc_load_command(0);
}

void TCloac::SingleBCReset(){

  cc_load_command(0);
  cc_load_command(VBCRST);
  cc_load_command(0);
}

void TCloac::SingleCalTrig(){

  if (pulse){
    cc_load_cal();   // load cal command and set delay
    cc_load_delay((unsigned short)(trig_delay-delay_offset));
    pulse = 0;
  }

  cc_load_command(0);
  cc_load_command(VCAL);
  cc_load_command(0);
}

void TCloac::SinglePulseTrig(){

  if (!pulse){
    cc_load_pulse(); // load "pulse input register" command
    cc_load_delay((unsigned short)(pulse_delay-delay_offset));
    pulse = 1;
  }

  cc_load_command(0);
  cc_load_command(VCAL);
  cc_load_command(0);
}

void TCloac::OverAllReset(){

  cc_load_command(0);
  cc_load_command(VOARST);
  cc_load_command(0);
}


//
//  The setting of Clock Sources
//

short TCloac::GetExternalClock(){
  unsigned short prev;

  cc_read_enable(&prev);
  if (prev&0x0040) return 1;
  else return 0;
}

short TCloac::GetInternalClock(){
  unsigned short prev;

  cc_read_enable(&prev);
  if (prev&0x0001) return 1;
  else return 0;
}

void TCloac::SetExternalClock(){
  unsigned short prev,next;

  cc_read_enable(&prev);
  next = (prev&0xffbe)|0x0040;
  cc_load_enable(next);
}

void TCloac::SetInternalClock(){
  unsigned short prev,next;

  cc_read_enable(&prev);
  next = (prev&0xffbe)|0x0001;
  cc_load_enable(next);
}


//
// Other Register 0 enables / disables affecting signal
// paths which can be enabled simultaneously.   GFM 21Mar2001
//

void TCloac::EnableInternalBusy(){
  unsigned short prev,next;

  cc_read_enable(&prev);
  next = prev | 0x0020;
  cc_load_enable(next);
}

void TCloac::DisableInternalBusy(){
  unsigned short prev,next;

  cc_read_enable(&prev);
  next = prev & ~0x0020;
  cc_load_enable(next);
}


void TCloac::EnableExternalTrigger(){
  unsigned short prev,next;

  cc_read_enable(&prev);
  next = prev | 0x0080;
  cc_load_enable(next);
}

void TCloac::DisableExternalTrigger(){
  unsigned short prev,next;

  cc_read_enable(&prev);
  next = prev & ~0x0080;
  cc_load_enable(next);
}

void TCloac::EnableExternalSoftReset(){
  unsigned short prev,next;

  cc_read_enable(&prev);
  next = prev | 0x0100;
  cc_load_enable(next);
}

void TCloac::DisableExternalSoftReset(){
  unsigned short prev,next;

  cc_read_enable(&prev);
  next = prev & ~0x0100;
  cc_load_enable(next);
}

void TCloac::EnableExternalBCReset(){
  unsigned short prev,next;

  cc_read_enable(&prev);
  next = prev | 0x0200;
  cc_load_enable(next);
}

void TCloac::DisableExternalBCReset(){
  unsigned short prev,next;

  cc_read_enable(&prev);
  next = prev & ~0x0200;
  cc_load_enable(next);
}

void TCloac::EnableExternalCalibrate(){
  unsigned short prev,next;

  cc_read_enable(&prev);
  next = prev | 0x0400;
  cc_load_enable(next);
}

void TCloac::DisableExternalCalibrate(){
  unsigned short prev,next;

  cc_read_enable(&prev);
  next = prev & ~0x0400;
  cc_load_enable(next);
}

void TCloac::EnableTriggerWindow(){
  unsigned short prev,next;

  cc_read_enable(&prev);
  next = prev | 0x8000;
  cc_load_enable(next);
}

void TCloac::DisableTriggerWindow(){
  unsigned short prev,next;

  cc_read_enable(&prev);
  next = prev & ~0x8000;
  cc_load_enable(next);
}


//
// Free Running and VBURST
//


void TCloac::Run(){

  if(randomiser)  cc_menu_trfreq(random_bits);
  else            cc_menu_trfreq(periodic_bits);
  if(periodic_sr) cc_menu_srfreq(softreset_bits);

  if(randomiser)   cc_menu_enable(ENRANDOMISER);
  if(periodic_sr)  cc_menu_enable(ENINTSRST);
  if(periodic_bcr) cc_menu_enable(ENINTBCRST);

  cc_menu_enable(ENINTRPTTRIG);
}

void TCloac::Stop(){

  // Stop repetitive triggers, unset randomiser
  cc_menu_disable(ENINTRPTTRIG);
  cc_menu_disable(ENRANDOMISER);
  cc_menu_disable(ENINTSRST);
  cc_menu_disable(ENINTBCRST);

}

void TCloac::Burst(){

  if(randomiser)  cc_menu_trfreq(random_bits);
  else            cc_menu_trfreq(periodic_bits);
  if(periodic_sr) cc_menu_srfreq(softreset_bits);

  if(randomiser)   cc_menu_enable(ENRANDOMISER);
  if(periodic_sr)  cc_menu_enable(ENINTSRST);
  if(periodic_bcr) cc_menu_enable(ENINTBCRST);

  cc_menu_burst(n_trigs);
  cc_menu_enable(ENINTRPTTRIG);
}

void TCloac::CalBurst(){

  if (pulse){
    cc_load_cal();   // load cal command and set delay
    cc_load_delay((unsigned short)(trig_delay-delay_offset));
    pulse = 0;
  }

  cc_menu_calburst(n_trigs);
}

void TCloac::PulseBurst(){

  if (!pulse){
    cc_load_pulse(); // load "pulse input register" command
    cc_load_delay((unsigned short)(pulse_delay-delay_offset));
    pulse = 1;
  }

  cc_menu_calburst(n_trigs);
}


//
// Other Setters and Getters
//


void TCloac::SetPeriodicFreq(double khz){

  int i = 0;

  while ((i<27)&&(p_freq[i]<khz)) i++;

  if (i==0){
    periodic_freq = p_freq[i];
    periodic_bits =   bits[i];
  }else{
    if (khz > ((p_freq[i-1]+p_freq[i])/2)){
      periodic_freq = p_freq[i];
      periodic_bits =   bits[i];
    }else{
      periodic_freq = p_freq[i-1];
      periodic_bits =   bits[i-1];
    }
  }
}

void TCloac::SetRandomFreq(double khz){

  int i = 0;

  while ((i<27)&&(r_freq[i]<khz)) i++;

  if (i==0){
    random_freq = r_freq[i];
    random_bits =   bits[i];
  }else{
    if (khz > ((r_freq[i-1]+r_freq[i])/2)){
      random_freq = r_freq[i];
      random_bits =   bits[i];
    }else{
      random_freq = r_freq[i-1];
      random_bits =   bits[i-1];
    }
  }
}

void TCloac::SetSoftResetFreq(double khz){

  int i = 0;

  while ((i<27)&&(sr_freq[i]<khz)) i++;

  if (i==0){
    softreset_freq =sr_freq[i];
    softreset_bits =   bits[i];
  }else{
    if (khz > ((sr_freq[i-1]+sr_freq[i])/2)){
      softreset_freq =sr_freq[i];
      softreset_bits =   bits[i];
    }else{
      softreset_freq =sr_freq[i-1];
      softreset_bits =   bits[i-1];
    }
  }
}

void TCloac::SetPeriodicFreq(short fbits){

  int i;
  double freq = -1.0;

  periodic_bits = fbits&0x1f;

  for(i=0;i<28;i++){
    if(bits[i]== periodic_bits)
      freq = p_freq[i];
  }

  periodic_freq = freq;
}

void TCloac::SetRandomFreq(short fbits){

  int i;
  double freq = -1.0;

  random_bits = fbits&0x1f;

  for(i=0;i<28;i++){
    if(bits[i]== random_bits)
      freq = r_freq[i];
  }

  random_freq = freq;
}

void TCloac::SetSoftResetFreq(short fbits){

  int i;
  double freq = -1.0;

  softreset_bits = fbits&0x1f;

  for(i=0;i<28;i++){
    if(bits[i]== softreset_bits)
      freq = sr_freq[i];
  }

  softreset_freq = freq;
}

double TCloac::GetPeriodicFreq(){  return periodic_freq; }
double TCloac::GetRandomFreq(){    return random_freq;   }
double TCloac::GetSoftResetFreq(){ return softreset_freq;}

short TCloac::GetTrigDelay(){   return trig_delay;  } 
short TCloac::GetPulseDelay(){  return pulse_delay; } 
short TCloac::GetDelayOffset(){ return delay_offset;} 

void TCloac::SetTrigDelay(unsigned short delay){
  trig_delay = delay;
  if(!pulse) cc_load_delay(trig_delay-delay_offset);
}

void TCloac::SetPulseDelay(unsigned short delay){
  pulse_delay = delay;
  if(!pulse) cc_load_delay(pulse_delay-delay_offset);
}

void TCloac::SetDelayOffset(unsigned short delay){
  delay_offset = delay;
}

int TCloac::GetTriggerPeriod(){
  // Give estimate of the interval between triggers
  // use safety factor of 2 for pseudo-random case

  if(randomiser) return (1+(int)(2000/random_freq));    // in microS
  else           return (1+(int)(1000/periodic_freq));  // in microS
}

void TCloac::SetWindowLength(unsigned short winlength){
  unsigned short prev,next;

  if (winlength > 31) winlength = 31;

  cc_read_window(&prev);
  next = (prev & 0x1f00) | winlength;   // preserve window delay in the high order byte
  cc_load_window(next);
}

void TCloac::SetWindowDelay(unsigned short windelay){
  unsigned short prev,next;

  if (windelay > 31) windelay = 31;

  cc_read_window(&prev);
  next = (prev & 0x001f) | (windelay<<8);   // preserve window length in the low order byte
  cc_load_window(next);
}

short TCloac::GetWindowLength(){
  unsigned short window,winlength;
  cc_read_window(&window);
  winlength = window & 0x001f;
  return winlength;
}

short TCloac::GetWindowDelay(){
  unsigned short window,windelay;
  cc_read_window(&window);
  windelay = (window & 0x1f00)>>8;
  return windelay;
}
