#ifndef TCLOAC_h
#define TCLOAC_h

/*
    TCloac.h

    version 0.1   PWP 11.07.00
    version 0.2   GFM 20010321 Add more enables/disables
    version 0.3   GFM 20020320 Add trigger window functions
*/


#if _MSC_VER >= 1000
#pragma once
#endif

  static double p_freq[28]=  { 0.05,  0.10,  0.12,  0.15,  0.20,  0.30,  0.50,
                               0.60,  1.00,  1.20,  1.50,  2.00,  3.00,  5.00,
                               6.00,  10.0,  12.0,  15.0,  20.0,  30.0,  50.0,
                              60.00, 100.0, 120.0, 150.0, 200.0, 300.0, 600.0};

  static double r_freq[28]=  { 0.0125, 0.0250, 0.0300, 0.0375, 0.0500, 0.0750,  0.1250,
                               0.1500, 0.2500, 0.3000, 0.3750, 0.5000, 0.7500,  1.2500,
                               1.5000, 2.5000, 3.0000, 3.7500, 5.0000, 7.5000, 12.5000,
                              15.0000,25.0000,30.0000,37.5000,50.0000,75.0000,150.0000};

  static double sr_freq[28]= {0.005, 0.010, 0.012, 0.015, 0.020, 0.030, 0.050,
                              0.060, 0.100, 0.120, 0.150, 0.200, 0.300, 0.500,
                              0.600, 1.000, 1.200, 1.500, 2.000, 3.000, 5.000,
                              6.000, 10.00, 12.00, 15.00, 20.00, 30.00, 60.00};

  static unsigned short bits[28] = {0x1f, 0x1e, 0x1d, 0x1c, 0x1b, 0x1a, 0x17, 
                                    0x18, 0x16, 0x15, 0x14, 0x13, 0x12, 0x0f, 
                                    0x10, 0x0e, 0x0d, 0x0c, 0x0b, 0x0a, 0x07, 
                                    0x08, 0x06, 0x05, 0x04, 0x03, 0x02, 0x00}; 


#include "TObject.h"

#include "../CLOAC/cloac_lib.h"
#include "../CLOAC/cloac_menu.h"

class TCloac : public TObject {

  public:

    TCloac(short new_id);
    ~TCloac();

    void TCloac::PrintStatus();

    //
    // Single Triggers
    //

    void TCloac::SingleTrigger();
    void TCloac::SingleSoftReset();
    void TCloac::SingleBCReset();
    void TCloac::SingleCalTrig();
    void TCloac::SinglePulseTrig();
    void TCloac::OverAllReset();

    //
    // Free Running and VBURST
    //

    void TCloac::Run();
    void TCloac::Stop();

    void TCloac::Burst();
    void TCloac::CalBurst();
    void TCloac::PulseBurst();

    //
    //  The setting of Clock Sources
    //

    short TCloac::GetExternalClock();
    short TCloac::GetInternalClock();
    void TCloac::SetExternalClock();
    void TCloac::SetInternalClock();

    //
    // Other Register 0 enables / disables affecting signal
    // paths which can be enabled simultaneously
    //

    void TCloac::EnableInternalBusy();
    void TCloac::DisableInternalBusy();
    void TCloac::EnableExternalTrigger();
    void TCloac::DisableExternalTrigger();
    void TCloac::EnableExternalSoftReset();
    void TCloac::DisableExternalSoftReset();
    void TCloac::EnableExternalBCReset();
    void TCloac::DisableExternalBCReset();
    void TCloac::EnableExternalCalibrate();
    void TCloac::DisableExternalCalibrate();
    void TCloac::EnableTriggerWindow();
    void TCloac::DisableTriggerWindow();

    //
    // Other Setters and Getters
    //

    void TCloac::SetPeriodicFreq(short fbits);
    void TCloac::SetPeriodicFreq(double khz);

    void TCloac::SetRandomFreq(short fbits);
    void TCloac::SetRandomFreq(double khz);

    void TCloac::SetSoftResetFreq(short fbits);
    void TCloac::SetSoftResetFreq(double hz);

    double TCloac::GetPeriodicFreq();
    double TCloac::GetRandomFreq();
    double TCloac::GetSoftResetFreq();

    short TCloac::GetTrigDelay();
    short TCloac::GetPulseDelay();
    short TCloac::GetDelayOffset();

    void TCloac::SetTrigDelay(unsigned short delay);
    void TCloac::SetPulseDelay(unsigned short delay);
    void TCloac::SetDelayOffset(unsigned short delay);

    int TCloac::GetTriggerPeriod();

    void TCloac::SetWindowLength(unsigned short winlength);
    void TCloac::SetWindowDelay(unsigned short windelay);
    short TCloac::GetWindowLength();
    short TCloac::GetWindowDelay();

    //
    // Simple flags etc. used during VBURST
    //

    short periodic_sr;
    short periodic_bcr;
    short randomiser;

    unsigned short n_trigs;

  private:

    short id;
    short pulse;

    short trig_delay;
    short pulse_delay;
    short delay_offset;

    short periodic_bits;
    short random_bits;
    short softreset_bits;

    double periodic_freq;
    double random_freq;
    double softreset_freq;

    ClassDef(TCloac,1)
};

#endif
