
/* 
    TModule.cpp

    GFM April 1999
    Version 0.20 PWP 18.08.00
    revised 0.21 PWP 11.09.00 - fix divide by zero errors in FitScan 
    revised 0.22 PWP 25.09.00 - fix silly bug in PrintScan
    revised 0.23 LE/PWP 10.10.00 - additions for LV graphs
    revised 0.24 PWP 16.10.00 - additions for HV graph
    revised 0.25 PWP 27.10.00 - addition of chisq histograms; some tidying up
    revised 0.26 PWP 08.12.00 - graphs of counter values; add ChansOverTarget;
                                integrated fitting for normalised (noise occupancy) scans
    revised 0.32 PWP 08.05.01 - additions for Noise Occupancy without overflow errors
    revised 0.32 PWP 09.05.01 - additions for AutoStop
    revised 0.40 JGK/PWP 18.07.01 - additions for EvTree
    revised 0.41 JGK/PWP 08.08.01 - modification re. EvTree
    revised 0.42 PWP 14.03.02 - add GetDCSPaveTExt 
    Version 0.50 PWP/BJG 03.04.02 - Support for Likelihood fits of ERFC and ERF data
    version 0.51 PWP/BJG 05.04.02 - bugfixes
    version 0.52 BJG 22.05.02 - aesthetic fix
*/


#include <iostream.h>
#include <iomanip.h>

extern "C" {

#include "../stlib/stlib.h"

}

#include "TModule.h"

#if ROOT_VERSION_CODE <= ROOT_VERSION(3,02,0)
class TF1xx : public TF1 {
   public:

      TF1xx() : TF1() {}
      TF1xx(const char *name, const char *formula, Double_t xmin=0, Double_t xmax=1)
	 : TF1(name,formula,xmin,xmax) {}
      TF1xx(const char *name, Double_t xmin, Double_t xmax, Int_t npar)
	 : TF1(name,xmin,xmax,npar) {}
      TF1xx(const char *name, void *fcn, Double_t xmin, Double_t xmax, Int_t npar)
	 : TF1(name,fcn,xmin,xmax,npar) {}
      TF1xx(const char *name, Double_t (*fcn)(Double_t *, Double_t *), Double_t xmin=0, Double_t xmax=1, Int_t npar=0)
	 : TF1(name,fcn,xmin,xmax,npar) {}
      TF1xx(const TF1xx &f1) : TF1( f1 ) {} 
      Bool_t IsInside(const Double_t* x)  {
	 double Xmin,Xmax;
	 GetRange(Xmin,Xmax);
	 if (x[0] < Xmin || x[0] > Xmax) return kFALSE;
	 return kTRUE;
      }
};

#define TF1 TF1xx
#endif

// The next section seems messy, but it is necessary because the
// LikelihoodFit functions must be implemented as simple static functions

TF1* ff;                      // pointer to fit function
TGraph* ch_data;              // pointer Graph to be fitted
double n[ST_MAX_SCANPOINTS];  // n_trig data array

// End of horrible bit!

ClassImp(TModule)

TModule::TModule(short new_id){

  id = new_id;
  present = 0; // missing by default
  use_ch_maxima = 0;
  normalised = 0;

  sprintf(name,"  ");

  DCS = new TModuleDCS();
  DCS->SetID(id);

  sprintf(longname,"Module %d %s", id, name);

  // Burst Histograms

  sprintf(htitle, "%s  Stream 0 Burst Data", longname);
  sprintf(hname,"h_burst0_m%d",id);
  h_burst0 = new TH1F(hname,htitle,768,0.0,768.0);
  h_burst0->SetFillColor(46);
  h_burst0->SetXTitle("Channel Number");
  h_burst0->SetYTitle("Hits");

  sprintf(htitle, "%s  Stream 1 Burst Data", longname);
  sprintf(hname,"h_burst1_m%d",id);
  h_burst1 = new TH1F(hname,htitle,768,0.0,768.0);
  h_burst1->SetFillColor(46);
  h_burst1->SetXTitle("Channel Number");
  h_burst1->SetYTitle("Hits");

  sprintf(htitle, "%s  Stream 0 Accumulated Burst Data", longname);
  sprintf(hname,"h_accbst0_%d",id);
  h_accbst0 = new TH1F(hname,htitle,768,0.0,768.0);
  h_accbst0->SetFillColor(46);
  h_accbst0->SetXTitle("Channel Number");
  h_accbst0->SetYTitle("Hits");

  sprintf(htitle, "%s  Stream 1 Accumulated Burst Data", longname);
  sprintf(hname,"h_accbst1_%d",id);
  h_accbst1 = new TH1F(hname,htitle,768,0.0,768.0);
  h_accbst1->SetFillColor(46);
  h_accbst1->SetXTitle("Channel Number");
  h_accbst1->SetYTitle("Hits");

  sprintf(htitle, "%s  Stream 0 Cluster Size", longname);
  sprintf(hname,"h_cluster0_m%d",id);
  h_cluster0 = new TH1F(hname,htitle,768,0.0,768.0);
  h_cluster0->SetFillColor(46);
  h_cluster0->SetXTitle("Cluster Size");
  h_cluster0->SetYTitle("n(Clusters)");
  
  sprintf(htitle, "%s  Stream 1 Cluster Size", longname);
  sprintf(hname,"h_cluster1_m%d",id);
  h_cluster1 = new TH1F(hname,htitle,768,0.0,768.0);
  h_cluster1->SetFillColor(46);
  h_cluster1->SetXTitle("Cluster Size");
  h_cluster1->SetYTitle("n(Clusters)");
  
  sprintf(htitle, "%s  Correlated Hits", longname);
  sprintf(hname,"h_corr_m%d",id);
  h_corr = new TH2F(hname,htitle,128,0.0,768,128,0.0,768);
  h_corr->SetFillColor(46);
  h_corr->SetXTitle("Hits Stream 0");
  h_corr->SetYTitle("Hits Stream 1");
  
  // Scan Histograms

  sprintf(htitle, "%s Stream 0 Scan Data", longname);
  sprintf(hname,"h_scan0_m%d",id);
  h_scan0 = new TH2F(hname,htitle, 768,0.5,768.5, ST_MAX_SCANPOINTS, 0.0, (float)ST_MAX_SCANPOINTS);
  h_scan0->SetXTitle("Channel Number");

  sprintf(htitle, "%s Stream 1 Scan Data", longname);
  sprintf(hname,"h_scan1_m%d",id);
  h_scan1 = new TH2F(hname,htitle, 768,0.5,768.5, ST_MAX_SCANPOINTS, 0.0, (float)ST_MAX_SCANPOINTS);
  h_scan1->SetXTitle("Channel Number");

  sprintf(htitle, "%s Stream 0 Fit Code", longname);
  sprintf(hname,"h_code0_m%d",id);
  h_code0 = new TH1S(hname, htitle, 768,0.5,768.5);
  h_code0->SetXTitle("Channel Number");

  sprintf(htitle, "%s Stream 1 Fit Code", longname);
  sprintf(hname,"h_code1_m%d",id);
  h_code1 = new TH1S(hname, htitle, 768,0.5,768.5);
  h_code1->SetXTitle("Channel Number");

  sprintf(htitle, "%s Stream 0 Fitted Mean", longname);
  sprintf(hname,"h_mean0_m%d",id);
  h_mean0 = new TH1F(hname, htitle, 768,0.5,768.5);
  h_mean0->SetXTitle("Channel Number");

  sprintf(htitle, "%s Stream 1 Fitted Mean", longname);
  sprintf(hname,"h_mean1_m%d",id);
  h_mean1 = new TH1F(hname, htitle, 768,0.5,768.5);
  h_mean1->SetXTitle("Channel Number");

  sprintf(htitle, "%s Stream 0 Fitted Sigma", longname);
  sprintf(hname,"h_sigma0_m%d",id);
  h_sigma0 = new TH1F(hname, htitle, 768,0.5,768.5);
  h_sigma0->SetXTitle("Channel Number");

  sprintf(htitle, "%s Stream 1 Fitted Sigma", longname);
  sprintf(hname,"h_sigma1_m%d",id);
  h_sigma1 = new TH1F(hname, htitle, 768,0.5,768.5);
  h_sigma1->SetXTitle("Channel Number");

  sprintf(htitle, "%s Stream 0 Chisq/NDF", longname);
  sprintf(hname,"h_chisq0_m%d",id);
  h_chisq0 = new TH1F(hname, htitle, 768,0.5,768.5);
  h_chisq0->SetXTitle("Channel Number");

  sprintf(htitle, "%s Stream 1 Chisq/NDF", longname);
  sprintf(hname,"h_chisq1_m%d",id);
  h_chisq1 = new TH1F(hname, htitle, 768,0.5,768.5);
  h_chisq1->SetXTitle("Channel Number");

  sprintf(htitle, "%s Stream 0 Figure of Merit", longname);
  sprintf(hname,"h_fom0_m%d",id);
  h_fom0 = new TH1F(hname, htitle, 768,0.5,768.5);
  h_fom0->SetXTitle("Channel Number");

  sprintf(htitle, "%s Stream 1 Figure of Merit", longname);
  sprintf(hname,"h_fom1_m%d",id);
  h_fom1 = new TH1F(hname, htitle, 768,0.5,768.5);
  h_fom1->SetXTitle("Channel Number");

  evtree = (TTree*) 0;

  DCSBurstText=0;
  DCSScanText=0;

  T1Graph = 0;
  T2Graph = 0;
  VdetGraph = 0;
  IdetGraph = 0;
  VccGraph = 0;
  IccGraph = 0;
  VddGraph = 0;
  IddGraph = 0;

  n_active_channels = 1536;

  ff=0;
  ch_data=0;
  likelihood=0; // default: use chisq minimisation
  use_range=1;  // default: trim fitted range to include only one bin
                //          at 100% occupancy and one bin 0% occupancy

  ResetData();
}

  
TModule::~TModule(){
  // Destructor
  // PWP 15.07.02

  // Delete Burst Histograms
  delete h_burst0;
  delete h_burst1;
  delete h_accbst0;
  delete h_accbst1;
  delete h_cluster0;
  delete h_cluster1;
  delete h_corr;

  // Delete Scan Histograms
  delete h_scan0;
  delete h_scan1;
  delete h_code0;
  delete h_code1;
  delete h_mean0;
  delete h_mean1;
  delete h_sigma0;
  delete h_sigma1;
  delete h_chisq0;
  delete h_chisq1;
  delete h_fom0;
  delete h_fom1;

  // delete event tree
  if(evtree){
    delete evtree;
    evtree = 0;
  }

  // delete TText objects
  if(DCSBurstText){
    delete DCSBurstText;
    DCSBurstText=0;
  }
  if(DCSScanText){
    delete DCSScanText;
    DCSScanText=0;
  }

  // delete TGraph objects
  if(T1Graph){
    delete T1Graph;
    T1Graph=0;
  }
  if(T2Graph){
    delete T2Graph;
    T2Graph=0;
  }
  if(VdetGraph){
    delete IdetGraph;
    VdetGraph=0;
  }
  if(IdetGraph){
    delete IdetGraph;
    IdetGraph=0;
  }
  if(VccGraph){
    delete VccGraph;
    VccGraph=0;
  }
  if(IccGraph){
    delete IccGraph;
    IccGraph=0;
  }
  if(VddGraph){
    delete VddGraph;
    VddGraph=0;
  }
  if(IddGraph){
    delete IddGraph;
    IddGraph=0;
  }
}


void TModule::ResetData(){
  short nch, nsp, stream;

  for (nsp=0; nsp<ST_MAX_SCANPOINTS; nsp++){

    // DCS variables
    idet[nsp]= 0;
    idd[nsp] = 0;
    icc[nsp] = 0;
    temp1[nsp] = 0;
    temp2[nsp] = 0;

    // scan data etc.
    scan_points[nsp] = 0.0;
    scan_trigs[nsp] = 0;

    for (stream=0; stream<2; stream++){
      for (nch=0; nch<768; nch++){
        scan[nsp][stream][nch] = 0;
      }
      scan_evcnt[stream][nsp] = 0;
      scan_ercnt[stream][nsp] = 0;
      scan_tocnt[stream][nsp] = 0;
      scan_nescnt[stream][nsp] = 0;
      scan_oshcnt[stream][nsp] = 0;
      scan_occupancy[stream][nsp] = 0;
      scan_overflows[stream][nsp] = 0;

      scan_bc_ercnt[stream][nsp] = 0;
      scan_l1_ercnt[stream][nsp] = 0;
    }
  }

  for (stream=0;stream<2;stream++){
    for(nch=0;nch<6;nch++){
      chip_mean[stream][nch]=0;
      chip_sigma[stream][nch]=0;
      chip_chisq[stream][nch]=0;
      chip_fom[stream][nch]=0;
      chip_prob[stream][nch]=0;
      chip_badfits[stream][nch]=0;
      chip_goodfits[stream][nch]=0;
    }
    side_mean[stream]=0;
    side_sigma[stream]=0;
    side_chisq[stream]=0;
    side_fom[stream]=0;
    side_prob[stream]=0;
    side_badfits[stream]=0;
    side_goodfits[stream]=0;
  }

  overall_mean=0;
  overall_sigma=0;
  overall_chisq=0;
  overall_fom=0;
  overall_prob=0;
  overall_badfits=0;
  overall_goodfits=0;

  h_burst0->Reset();
  h_burst1->Reset();
  h_cluster0->Reset();
  h_cluster1->Reset();
  h_corr->Reset();

  h_scan0->Reset();
  h_scan1->Reset();

  h_code0->Reset();
  h_code1->Reset();
  h_mean0->Reset();
  h_mean1->Reset();
  h_sigma0->Reset();
  h_sigma1->Reset();
  h_chisq0->Reset();
  h_chisq1->Reset();
  h_fom0->Reset();
  h_fom1->Reset();

  //CreateTree(); commented out by PWP 08.08.01

  burst=0;
  scan_hi=0;
  scan_lo=0;
  scan_start=0;
  scan_stop=0;
  scan_step=0;
  scan_max_points=0;
  scan_fitted=0;

  last_burst_with_hi_occ=-1;
  last_burst_with_lo_occ=-1;

  if(DCSBurstText){
    delete DCSBurstText;
    DCSBurstText=0;
  }
  if(DCSScanText){
    delete DCSScanText;
    DCSScanText=0;
  }

  if(T1Graph){
    delete T1Graph;
    T1Graph=0;
  }
  if(T2Graph){
    delete T2Graph;
    T2Graph=0;
  }
  if(VdetGraph){
    delete IdetGraph;
    VdetGraph=0;
  }
  if(IdetGraph){
    delete IdetGraph;
    IdetGraph=0;
  }
  if(VccGraph){
    delete VccGraph;
    VccGraph=0;
  }
  if(IccGraph){
    delete IccGraph;
    IccGraph=0;
  }
  if(VddGraph){
    delete VddGraph;
    VddGraph=0;
  }
  if(IddGraph){
    delete IddGraph;
    IddGraph=0;
  }
}


void TModule::PrintStatus(){
  // revised PWP 20.12.01

  cout << endl;
  cout << "TModule::PrintStatus()" << endl;
  cout << endl;
  PrintScan();
  PrintBurst();
}


void TModule::GetModuleData(short mid){
  id = mid;
}


void TModule::GetBurstData(){
}


void TModule::SetModuleName(char* mname){
  //char htitle[80];

  sprintf(name,"%s",mname);
  sprintf(longname,"Module %d %s", id, name);

  // Burst histograms

  sprintf(htitle, "%s  Stream 0 Burst Data", longname);
  h_burst0->SetTitle(htitle);
  sprintf(htitle, "%s  Stream 1 Burst Data", longname);
  h_burst1->SetTitle(htitle);
  
  sprintf(htitle, "%s  Stream 0 Accumulated Burst Data", longname);
  h_accbst0->SetTitle(htitle);
  sprintf(htitle, "%s  Stream 1 Accumulated Burst Data", longname);
  h_accbst1->SetTitle(htitle);
  
  sprintf(htitle, "%s  Stream 0 Cluster Size", longname);
  h_cluster0->SetTitle(htitle);
  sprintf(htitle, "%s  Stream 1 Cluster Size", longname);
  h_cluster1->SetTitle(htitle);

  sprintf(htitle, "%s  Correlated Hits", longname);
  h_corr->SetTitle(htitle);

  // Scan histograms

  sprintf(htitle, "%s Stream 0 Fit Code", longname);
  h_code0->SetTitle(htitle);
  sprintf(htitle, "%s Stream 1 Fit Code", longname);
  h_code1->SetTitle(htitle);

  sprintf(htitle, "%s Stream 0 Fitted Mean", longname);
  h_mean0->SetTitle(htitle);
  sprintf(htitle, "%s Stream 1 Fitted Mean", longname);
  h_mean1->SetTitle(htitle);

  sprintf(htitle, "%s Stream 0 Fitted Sigma", longname);
  h_sigma0->SetTitle(htitle);
  sprintf(htitle, "%s Stream 1 Fitted Sigma", longname);
  h_sigma1->SetTitle(htitle);

  sprintf(htitle, "%s Stream 0 Chisq/NDF", longname);
  h_chisq0 ->SetTitle(htitle);
  sprintf(htitle, "%s Stream 1 Chisq/NDF", longname);
  h_chisq1 ->SetTitle(htitle);

  sprintf(htitle, "%s Stream 0 Figure of Merit", longname);
  h_fom0 ->SetTitle(htitle);
  sprintf(htitle, "%s Stream 0 Figure of Merit", longname);
  h_fom1 ->SetTitle(htitle);
}

//
// Event functions
//

void TModule::ResetEvent(){
  int n;

  // reset arrays used to fill EvTree
  for (n=0; n<768; n++) {
    evtr_strm0[n]=kFALSE;
    evtr_strm1[n]=kFALSE;
  }
  for(n=0;n<7;n++){
    hits_strm0[n] = 0;
    hits_strm1[n] = 0;
  }

}

//
// Burst functions
//

void TModule::ResetBurstHist(){

  scan_occupancy[0][burst] = 0;
  scan_occupancy[1][burst] = 0;
  scan_overflows[0][burst] = 0;
  scan_overflows[1][burst] = 0;

  scan_trigs[burst]  = 0;
  scan_evcnt[0][burst] = 0;
  scan_evcnt[1][burst] = 0;
  scan_ercnt[0][burst] = 0;
  scan_ercnt[1][burst] = 0;
  scan_tocnt[0][burst] = 0;
  scan_tocnt[1][burst] = 0;
  scan_nescnt[0][burst] = 0;
  scan_nescnt[1][burst] = 0;
  scan_oshcnt[0][burst] = 0;
  scan_oshcnt[1][burst] = 0;

  scan_bc_ercnt[0][burst] = 0;
  scan_bc_ercnt[1][burst] = 0;
  scan_l1_ercnt[0][burst] = 0;
  scan_l1_ercnt[1][burst] = 0;

  h_burst0->Reset();
  h_burst1->Reset();
  h_cluster0->Reset();
  h_cluster1->Reset();
  h_corr->Reset();
}


void TModule::FillBurstHist(){

  FillBurstHist(-1);
}


void TModule::FillBurstHist(short cal_mode){
  short n, start, step, target;

  // Revised PWP 15.08.00
  // count the number of channels which have overflowed
  // and exclude these channels from occupancy calculations

  if (burst >= ST_MAX_SCANPOINTS) return;
  
  if ((cal_mode<0) || (cal_mode>3)){  // all cal lines
    start = 0;
    step = 1;
  } else {             // only the designated cal line
    start = cal_mode;
    step = 4;
  }

  for (n=start; n<768; n+=step){

    if(scan[burst][0][n]>=0)
      scan_occupancy[0][burst] += (int)scan[burst][0][n];
    else
      scan_overflows[0][burst]++;

    if(scan[burst][1][n]>=0)
      scan_occupancy[1][burst] += (int)scan[burst][1][n];
    else
      scan_overflows[1][burst]++;

    h_burst0->Fill((float)n,(float)scan[burst][0][n]);
    h_burst1->Fill((float)n,(float)scan[burst][1][n]);
  }

  // PWP additions 09.05.01 to determine if all channels 
  // have exceeded 50% occupancy and/or have had zero occupancy
  // (part of AutoStop feature)
  // PWP bugfix 04.10.01 - only make this test if 
  // all callines have been tested

  if((cal_mode==-1)||(cal_mode==3)){
    target = (short)scan_trigs[burst]/2;
    if((unsigned short)target>0x7fff) target=0x7fff;

    if(ChansOverTarget(target)>=n_active_channels){
      last_burst_with_hi_occ=burst;
    }
    if(ChansUnderTarget(1)==1536){
      last_burst_with_lo_occ=burst;
    }
  }
}


void TModule::FillClusterHist(short stream, short cluster){

  // What to do here wrt calmode??
  // Suggest scale resulting histogram by factor 1/4 at EOB...

  if (stream) h_cluster0->Fill((float)cluster);
  else        h_cluster1->Fill((float)cluster);
}


void TModule::FillCorrHist(short hits0, short hits1){

  // What to do here wrt calmode??
  // Suggest scale resulting histogram by factor 1/4 at EOB...

  h_corr->Fill((float)hits0,(float)hits1);
}

short TModule::ChansOverTarget(short target){

  int j,stream;
  short n_chans_over_target=0;

  for(stream=0;stream<2;stream++){
    for(j=0;j<768;j++){
      // include overflows
      if((scan[burst][stream][j]<0)||(scan[burst][stream][j]>=target)){
        n_chans_over_target++;
      }
    }
  }
  return n_chans_over_target;
}

short TModule::ChansUnderTarget(short target){

  int j,stream;
  short n_chans_under_target=0;

  for(stream=0;stream<2;stream++){
    for(j=0;j<768;j++){
      // exclude overflows
      if((scan[burst][stream][j]>=0)&&(scan[burst][stream][j]<=target)){
        n_chans_under_target++;
      }
    }
  }
  return n_chans_under_target;
}

short TModule::OKtoStop(short scan_type){
  // Simple routine which returns 1 if both the low and
  // high occupancy regions have been adequately covered
  // during the present scan PWP 09.05.01

  short direction;

  // cout << "  Module " << id << " Last scanpoint";
  // cout << " with hi occ " << last_burst_with_hi_occ;
  // cout << " with lo occ " << last_burst_with_lo_occ << endl;
  // cout << " ChansOver " << ChansOverTarget(scan_trigs[burst]/2) << endl;
  // cout << " ChansUnder " << ChansUnderTarget(1) << endl;

  switch(scan_type){
    case 1:   // ST_VTHR
    case 41:  // ST_QTHR
      if(scan_start>scan_stop){
        direction = 1; // low  occ at low  scanpoint,
                       // high occ at high scanpoint
      }else{
        direction = 0; // high occ at low  scanpoint,
                       // low  occ at high scanpoint
      }
      break;
    case 2:   // ST_VCAL
    case 42:  // ST_QCAL
      if(scan_start>scan_stop){
        direction = 0; // high occ at low  scanpoint,
                       // low  occ at high scanpoint
      }else{
        direction = 1; // low  occ at low  scanpoint,
                       // high occ at high scanpoint
      }
      break;
    default:
      return 0;
      break;
  }

  if((last_burst_with_hi_occ>=0)&&(last_burst_with_lo_occ>=0)){
    if(direction==1){ // high occ at low  scanpoint,
                      // low  occ at high scanpoint
      if(last_burst_with_lo_occ<last_burst_with_hi_occ) return 1;

    } else {          // low  occ at low  scanpoint,
                      // high occ at high scanpoint 
      if(last_burst_with_lo_occ>last_burst_with_hi_occ) return 1;
    }
  }
  return 0;
}


float TModule::MeanOccupancy(){
  float occ = 0;
  float occ0 = MeanOccupancy(0);
  float occ1 = MeanOccupancy(1);

  if(occ0>0)   occ+=occ0;
  if(occ1>0)   occ+=occ1;
  if(occ>occ1) occ/=2;

  return occ;
}


float TModule::MeanOccupancy(short stream){
  float hits_per_ch;

  if((stream<0)||(stream>1)) return 0;

  if(scan_overflows[stream][burst]<768) {
    hits_per_ch = (float)scan_occupancy[stream][burst]/(float)(768-scan_overflows[stream][burst]);
  } else {
    hits_per_ch = -1; // error
  }
  return hits_per_ch;
}


float TModule::FractionalOccupancy(){
  float occ = 0;
  float occ0 = FractionalOccupancy(0);
  float occ1 = FractionalOccupancy(1);

  if(occ0>0)   occ+=occ0;
  if(occ1>0)   occ+=occ1;
  if(occ>occ1) occ/=2;

  return occ;
}


float TModule::FractionalOccupancy(short stream){
  float hits_per_ch_per_event;

  if((stream<0)||(stream>1)) return 0;

  if(scan_evcnt[stream][burst]>0) {
    hits_per_ch_per_event = MeanOccupancy(stream) / (float)scan_evcnt[stream][burst];
  } else {
    hits_per_ch_per_event = MeanOccupancy(stream); // can only be 0 or -1
  }
  return hits_per_ch_per_event;
}


void TModule::PrintBurst(){

  int stream;

  cout << longname;
  cout << "   tot. triggers:" << scan_trigs[burst] << endl;

  for(stream=0;stream<2;stream++){

    cout << "  Stream " << stream << " tot. hits:" << scan_occupancy[stream][burst];
    cout << "   tot. overflows:" << scan_overflows[stream][burst] << endl;

    cout << "   mean hits/ch:"    << MeanOccupancy(stream);
    cout << "   fractional occ.:" << FractionalOccupancy(stream) << endl;

    cout << "   evcnt  " << scan_evcnt[stream][burst];
    cout << "   ercnt  " << scan_ercnt[stream][burst];
    cout << "   tocnt  " << scan_tocnt[stream][burst];
    cout << "   nescnt " << scan_nescnt[stream][burst];
    cout << "   oshcnt " << scan_oshcnt[stream][burst] << endl;

    cout << "   bc_ercnt  " << scan_bc_ercnt[stream][burst];
    cout << "   l1_ercnt  " << scan_l1_ercnt[stream][burst] << endl;
  }
}


void TModule::SetBurstCount(int var){
  int v2;

  v2 = var;
  if (var>ST_MAX_SCANPOINTS) v2 = ST_MAX_SCANPOINTS;
  else if (var<0) v2 = 0;
  burst = v2;
}


//
// Scan functions
//


void TModule::InitScan(int new_max_points, float new_start, float new_stop, float new_step, char *new_scan_name){
  //char htitle[128];
  float half_step;

  // All that tedious "reset data" stuff has now been moved to 
  // the ResetData function, where it should have been all along...
  // PWP 27.10.00

  ResetData();

  scan_start = new_start;
  scan_stop = new_stop;
  scan_step = new_step;
  scan_max_points = new_max_points;
  strcpy(scan_name, new_scan_name);

  half_step = scan_step/2;
  if(half_step<0) half_step = half_step*-1;

  printf("Module::InitScan:  %d %f %f %s\n", scan_max_points, scan_start, scan_stop, scan_name);

  if (scan_stop > scan_start){ 
    scan_lo = scan_start;
    scan_hi = scan_stop;
  }else{
    scan_lo = scan_stop;
    scan_hi = scan_start;
  }

  sprintf(htitle, "%s Stream 0 %s Scan", longname, scan_name);
  h_scan0->SetBins(768,0.5,768.5, scan_max_points, scan_lo-half_step, scan_hi+half_step);
  h_scan0->SetTitle(htitle);

  sprintf(htitle, "%s Stream 1 %s Scan", longname, scan_name);
  h_scan1->SetBins(768,0.5,768.5, scan_max_points, scan_lo-half_step, scan_hi+half_step);
  h_scan1->SetTitle(htitle);
}
 

void TModule::FillScanHist(){
  short n;
  normalised = 0;

  for (n=0; n<ST_MAX_SCANPOINTS; n++)  FillScanHist(n);
}


void TModule::FillScanHist(short burst_count){
  short n;

  if (burst_count >= ST_MAX_SCANPOINTS) return;

  for (n=0; n<768; n++) {
    h_scan0->Fill((float)n, (float)scan_points[burst_count], (float)scan[burst_count][0][n]);
    h_scan1->Fill((float)n, (float)scan_points[burst_count], (float)scan[burst_count][1][n]);
  }
}


void TModule::FillNormalisedScanHist(){
  short n;
  normalised = 1;

  for (n=0; n<ST_MAX_SCANPOINTS; n++)  FillNormalisedScanHist(n);
}


void TModule::FillNormalisedScanHist(short burst_count){
  double n0, n1, z0, z1;
  short i;

  if (burst_count >= ST_MAX_SCANPOINTS) return;

  n0 = (double) scan_evcnt[0][burst_count];
  n1 = (double) scan_evcnt[1][burst_count];

  for (i=0; i<768; i++) {
    z0 = (double) scan[burst_count][0][i];
    z1 = (double) scan[burst_count][1][i];

    h_scan0->Fill((float)i, (float)scan_points[burst_count], (float)(z0/n0));
    h_scan1->Fill((float)i, (float)scan_points[burst_count], (float)(z1/n1));
  }
}

void TModule::FillNOScanHist(short burst_count){
  // Designed for use during Noise Occupancy scans
  // If a channel has overflowed, the scan histograms are not filled 
  // such that the normalised value from the last burst is retained
  // PWP 27.04.01

  float n0, n1, z0, z1;
  short i;
  Int_t bin;
  Stat_t entries0, entries1;

  if (burst_count >= ST_MAX_SCANPOINTS) return;

  normalised = 1; // PWP 20.03.02

  entries0 = h_scan0->GetEntries();
  entries1 = h_scan1->GetEntries();

  n0 = (float) scan_evcnt[0][burst_count];
  n1 = (float) scan_evcnt[1][burst_count];

  for (i=0; i<768; i++) {
    z0 = scan[burst_count][0][i];
    z1 = scan[burst_count][1][i];

    bin=h_scan0->FindBin((float)i, scan_points[burst_count]);

    if(z0>=0){
      entries0++;
      h_scan0->SetBinContent(bin, (z0/n0));
    }
    if(z1>=0){
      entries1++;
      h_scan1->SetBinContent(bin, (z1/n1));
    }
  }

  h_scan0->SetEntries(entries0);
  h_scan1->SetEntries(entries1);
}


void TModule::FitScan(short fit_type, short cal_mode){

  // PWP 18.08.00
  // revised PWP 04.04.02

  double ch_max, ch_min;
  double err2;
  double level_1, level_50, level_82, level_98, level_100;
  double p0_guess, p1_guess, p2_guess;
  double range_lo, range_hi;

  double x[ST_MAX_SCANPOINTS];
  double y[ST_MAX_SCANPOINTS];
  double xe[ST_MAX_SCANPOINTS];
  double ye[ST_MAX_SCANPOINTS];

  unsigned short ch_code;
  int i,j,stream,start,step,chip;

  // 1   Initialisation

  // 1-1 clear variables

  for (stream=0;stream<2;stream++){
    for(chip=0;chip<6;chip++){
      chip_mean[stream][chip]=0;
      chip_sigma[stream][chip]=0;
      chip_chisq[stream][chip]=0;
      chip_prob[stream][chip]=0;
      chip_fom[stream][chip]=0;
      chip_badfits[stream][chip]=0;
      chip_goodfits[stream][chip]=0;
    }
    side_mean[stream]=0;
    side_sigma[stream]=0;
    side_chisq[stream]=0;
    side_fom[stream]=0;
    side_prob[stream]=0;
    side_badfits[stream]=0;
    side_goodfits[stream]=0;
  }

  overall_mean=0;
  overall_sigma=0;
  overall_chisq=0;
  overall_fom=0;
  overall_prob=0;
  overall_badfits=0;
  overall_goodfits=0;

  h_mean0->Reset();
  h_mean1->Reset();
  h_sigma0->Reset();
  h_sigma1->Reset();
  h_code0->Reset();
  h_code1->Reset();
  h_chisq0->Reset();
  h_chisq1->Reset();
  h_fom0->Reset();
  h_fom1->Reset();


  // 1-3 Select fit function appropriate to scan

  switch (fit_type){
  case ST_ERFC:
    ff = new TF1("ff", erfc, 0, 1000, 3);
    ff->SetParNames("Constant","Mean","Sigma");
    break;
  case ST_ERF:
    ff = new TF1("ff", erf,  0, 1000, 3);
    ff->SetParNames("Constant","Mean","Sigma");
    break;
  case ST_GAUS:
    ff = new TF1("ff", gaus,  0, 1000, 3); // not "gaus", PWP bugfix 27.10.00
    ff->SetParNames("Constant","Mean","Sigma");
    break;
  default:
    return;
    break;
  }

  // 1-3 Initialise x, xe and n (n_trigs)  arrays 
  // values of xe are somewhat arbitrary for now (1% of bin centre)
  // simultanesouly, calculate the largest expected event count for this scan

  int target_constant = 0;

  if(scan_stop>scan_start){
    for(j=0;j<=burst;j++){
      n[j] = scan_trigs[j];
      x[j] = scan_points[j];
      xe[j]= 0.01*x[j];

      if(n[j]==0) n[j]=1;  // give protection against divide by zero
                           // THIS SHOULD NOT HAPPEN!
      if(scan_trigs[j]>target_constant) target_constant = scan_trigs[j];
    }
  } else {
    for(j=0;j<=burst;j++){
      n[j] = scan_trigs[burst-j];
      x[j] = scan_points[burst-j];
      xe[j]= 0.01*x[j];

      if(n[j]==0) n[j]=1;  // give protection against divide by zero
                           // THIS SHOULD NOT HAPPEN!
      if(scan_trigs[j]>target_constant) target_constant = scan_trigs[j];
    }
  }


  if(normalised){
    target_constant = 1;
  }

  // 1-4 Default is to use target_constant as basis for target levels

  level_1 =   (float)0.01 * target_constant;
  level_50 =  (float)0.50 * target_constant;
  level_82 =  (float)0.82 * target_constant;
  level_98 =  (float)0.98 * target_constant;
  level_100 = (float)target_constant;
  p0_guess =  (float)target_constant;

  // 1-5 default values of p1_guess, p2_guess

  p1_guess = (scan_start+scan_stop)/2;
  p2_guess = 2* scan_step;


  // 2  Loop over channels
 
  if ((cal_mode<0) || (cal_mode>3)){  // all cal lines
    start = 0;
    step = 1;
  } else {             // only the designated cal line
    start = cal_mode;
    step = 4;
  }

  for(stream=0;stream<2;stream++){
    for (i=start; i<768; i+=step){

      // 2-1 Calculation of which chip hosts the current channel of interest

      chip = i/128;

      // 2-2 Form y array needed for the graph 
      //     and find max and min for this channel

      if(normalised){

        // Here we use the normalised data from the scan histogram

        ch_max = 0.0;
        ch_min = 1.0;

        if(scan_stop>scan_start){
          for(j=0;j<=burst;j++){
            if(stream){
              y[j]=h_scan1->GetBinContent((770*(j+1))+i);
            }else{
              y[j]=h_scan0->GetBinContent((770*(j+1))+i);
            }
            if(y[j]>ch_max) ch_max=y[j];
            if(y[j]<ch_min) ch_min=y[j];
          }
        } else {
          for(j=0;j<=burst;j++){
            if(stream){
              y[j]=h_scan1->GetBinContent((770*(burst+1-j))+i);
            }else{
              y[j]=h_scan0->GetBinContent((770*(burst+1-j))+i);
            }
            if(y[j]>ch_max) ch_max=y[j];
            if(y[j]<ch_min) ch_min=y[j];
          }
        }

      } else {

        // Here we use directly the raw data

        ch_max = 0.0;
        ch_min = 1000000.0;

        if(scan_stop>scan_start){
          for(j=0;j<=burst;j++){
            y[j] = scan[j][stream][i];
            if(y[j]>ch_max) ch_max=y[j];
            if(y[j]<ch_min) ch_min=y[j];
          }
        } else {
          for(j=0;j<=burst;j++){
            y[j] = scan[burst-j][stream][i];
            if(y[j]>ch_max) ch_max=y[j];
            if(y[j]<ch_min) ch_min=y[j];
          }
        }

      }

      // 2-3 Test channels for good data - 
      //     overtly bad channels will be skipped

      ch_code  = 0;

      if     (ch_max<level_1)   ch_code = CH_DEAD;  // max < 1%
      else if(ch_min>level_98)  ch_code = CH_STUCK; // min > 98% 
      else if(ch_max>target_constant) ch_code = CH_OVER;  // max > 100%
      else if(ch_max<target_constant) ch_code = CH_UNDER; // max < 100%


      if(ch_code&(unsigned short)BAD_TEST){
          chip_badfits[stream][chip]++;  // tally bad channels

      } else { // do the fit!!

        // 2-4-1 Calculate errors appropriate to binomial statistics 

        if(normalised){

          // Here, y is a number between zero and 1. We multiply this by the 
          // number of triggers to calculate the binomial error on the data

          for(j=0;j<=burst;j++){
            err2 = ((n[j]*y[j])+1) * (1+n[j]-(n[j]*y[j])) / n[j];
            ye[j]= sqrt(err2);
            ye[j]/=n[j];
          }

        } else {

          for(j=0;j<=burst;j++){
            err2 = (y[j]+1) * (1+n[j]-y[j]) / n[j];
            ye[j]= sqrt(err2);
          }
        }

        // 2-4-2 Use channel's own maxima as basis for target levels
        //       if option "use_ch_maxima" has been selected

        if(use_ch_maxima){
          level_50 = (float)0.50 * ch_max;
          level_82 = (float)0.82 * ch_max;
          level_100 = ch_max;
          p0_guess = ch_max;
        }

        // 2-4-3 Calculation of initial guess 
        //       and range limits to be used during fitting

        range_lo = x[0];
        range_hi = x[burst];

        switch (fit_type){
          case ST_ERFC: // fit back edge
                        // start high and work down
            j=burst;
            while((j>=0)&&(y[j]==0)){
              range_hi = x[j];
              j--;
            }
            while((j>=0)&&(y[j]<level_100)){
              if (y[j]<level_50) p1_guess = x[j];
              if (y[j]<level_82) p2_guess = x[j];
              j--;
            }
            if(j>0) range_lo = x[j];

	    p2_guess = p1_guess - p2_guess;
            break;
          case ST_ERF: // fit front edge
                       // start low and work up
            j=0;
            while((j<=burst)&&(y[j]==0)){
              range_lo = x[j];
              j++;
            }
            while((j<=burst)&&(y[j]<level_100)){
              if (y[j]<level_50) p1_guess = x[j];
              if (y[j]<level_82) p2_guess = x[j];
              j++;
            }
            if(j<burst) range_hi = x[j];

	    p2_guess = p2_guess - p1_guess;
            break;
          default:
            break;
        }


        // 2-4-4 Protection against failure of section 2-4-3!!

        if(p1_guess==0) p1_guess = scan_start+scan_stop/2;
        if(p2_guess==0) p2_guess = scan_step * 2;
        if(p2_guess<0) p2_guess*=-1;

        // 2-4-5 Creation of graph and minimisation
 
        ch_data = new TGraphErrors((burst+1),x,y,xe,ye);

        ff->SetParameters(p0_guess,p1_guess,p2_guess);
        ff->SetParLimits(0,p0_guess,p0_guess);
        if(use_range){
          ff->SetRange(range_lo, range_hi);
        }

        if(likelihood){ // log likelihood fit

          switch(fit_type){
            case ST_ERFC:
              if(normalised){
                TVirtualFitter::Fitter(ch_data)->SetFCN(NErfcLikelihoodFit);
              } else {
                TVirtualFitter::Fitter(ch_data)->SetFCN(ErfcLikelihoodFit);
              }
              ch_data->Fit("ff","UQR"); // quiet, use range, user supplied FCN
              break;
            case ST_ERF:
              if(normalised){
                TVirtualFitter::Fitter(ch_data)->SetFCN(NErfLikelihoodFit);
              } else {
                TVirtualFitter::Fitter(ch_data)->SetFCN(ErfLikelihoodFit);
              }
              ch_data->Fit("ff","UQR"); // quiet, use range, user supplied FCN
              break;
            default:
              // Sorry, no more likelihood fits! Use chisq
              ch_data->Fit("ff","QR"); // quiet mode, use range
              break;
          }

        }else{ // minimisation of chisquare
          ch_data->Fit("ff","QR"); // quiet mode, use range
        }

        // 2-4-6 Validity checking on fit data:
        //       mean value must be within scanned range
        //       sigma must be greater than zero

        // if(ff->GetParameter(1)<scan_lo) ch_code+= FIT_UNDER;
        // if(ff->GetParameter(1)>scan_hi) ch_code+= FIT_OVER;
        if(ff->GetParameter(2)<0)       ch_code+= SIG_UNDER;


        // 2-4-7 Fill arrays and histograms if fit is good

        chip = i/128;

        if(ch_code&BAD_TEST){
          chip_badfits[stream][chip]++;

          if(stream){
            h_mean1->SetBinError(i,0); 
            h_sigma1->SetBinError(i,0); 
          }else{
            h_mean0->SetBinError(i,0); 
            h_sigma0->SetBinError(i,0); 
          }
        } else {

          // Perform chisq goodness of fit test
          // WARNING: this may not yet be entirely correct!!
          // PWP 04.04.02

          double y_fit;
          double residual;
          double divisor;
          double chisq = 0;
          double fom   = 0;
          int ndf = 0;

          if(normalised){
            for(j=0;j<burst;j++){
              if(ff->IsInside(&x[j])){ // restrict calculation of chisq
                                       // to the fitted range
                y_fit = ff->Eval(x[j])*n[j];
                residual = y_fit-(y[j]*n[j]);

                // calculate chisq
                // using binomial errors based on the fit
                if(n[j]>0){  
                  divisor = (y_fit+1) * (1+n[j]-y_fit) / n[j];
                  chisq += ( (residual*residual) / divisor);
                  ndf++;
                }

                // calculate FOM
                divisor  = n[j] * y_fit;
                if(divisor>0){
                  fom += TMath::Abs((residual*scan_step)/divisor);
                }
              }
            }
          } else {
            for(j=0;j<burst;j++){
              if(ff->IsInside(&x[j])){ // restrict calculation of chisq
                                       // to the fitted range
                y_fit = ff->Eval(x[j]);
                residual = y_fit-y[j];

                // calculate chisq
                // using binomial errors based on the fit
                if(n[j]>0){  
                  divisor = (y_fit+1) * (1+n[j]-y_fit) / n[j];
                  chisq += ( (residual*residual) / divisor);
                  ndf++;
                }

                // calculate FOM
                divisor  = n[j] * y_fit;
                if(divisor>0){
                  fom += TMath::Abs((residual*scan_step)/divisor);
                }
              }
            }
          }

          // Evaluate probability that, for the given number of
          // degrees of freedom and the calculated value of chisq,
          // the data is described by the fit

          chip_prob[stream][chip] += (float)TMath::Prob(chisq,ndf);

          // Normalise chisq by number of degrees of freedom
          chisq /=ndf;


          chip_mean[stream][chip] += (float)ff->GetParameter(1);
          chip_sigma[stream][chip] += (float)ff->GetParameter(2);
          chip_chisq[stream][chip] += (float)chisq;
          chip_fom[stream][chip]  += (float)fom;
          chip_goodfits[stream][chip]++;

          if(stream){
            h_mean1->Fill(i,ff->GetParameter(1)); 
            h_sigma1->Fill(i,ff->GetParameter(2)); 
            h_chisq1->Fill(i,chisq);
            h_fom1->Fill(i,fom); 
            h_mean1->SetBinError(i,ff->GetParError(1)); 
            h_sigma1->SetBinError(i,ff->GetParError(2)); 
          }else{
            h_mean0->Fill(i,ff->GetParameter(1)); 
            h_sigma0->Fill(i,ff->GetParameter(2)); 
            h_chisq0->Fill(i,chisq);
            h_fom0->Fill(i,fom); 
            h_mean0->SetBinError(i,ff->GetParError(1)); 
            h_sigma0->SetBinError(i,ff->GetParError(2)); 
          }
        }
        ch_data->Delete();
        ch_data=0;
      }

      // 2-5 Fill code histogram

      if (stream) h_code1->Fill(i,ch_code); 
      else        h_code0->Fill(i,ch_code); 
    }

    // 3 divide summed mean and sigma by number of good fits
    //   to give the mean values

    for(chip=0;chip<6;chip++){
      side_mean[stream]+= chip_mean[stream][chip];
      side_sigma[stream]+= chip_sigma[stream][chip];
      side_chisq[stream]+= chip_chisq[stream][chip];
      side_fom[stream]+= chip_fom[stream][chip];
      side_prob[stream]+= chip_prob[stream][chip];
      side_badfits[stream]+= chip_badfits[stream][chip];
      side_goodfits[stream]+= chip_goodfits[stream][chip];

      if(chip_goodfits[stream][chip]>0){
        chip_mean[stream][chip]/=chip_goodfits[stream][chip];
        chip_sigma[stream][chip]/=chip_goodfits[stream][chip];
        chip_chisq[stream][chip]/=chip_goodfits[stream][chip];
        chip_fom[stream][chip]/=chip_goodfits[stream][chip];
        chip_prob[stream][chip]/=chip_goodfits[stream][chip];
      }
    }
    overall_mean+= side_mean[stream];
    overall_sigma+= side_sigma[stream];
    overall_chisq+= side_chisq[stream];
    overall_fom+= side_fom[stream];
    overall_prob+= side_prob[stream];
    overall_badfits+= side_badfits[stream];
    overall_goodfits+= side_goodfits[stream];

    if(side_goodfits[stream]>0){
      side_mean[stream]/=side_goodfits[stream];
      side_sigma[stream]/=side_goodfits[stream];
      side_chisq[stream]/=side_goodfits[stream];
      side_fom[stream]/=side_goodfits[stream];
      side_prob[stream]/=side_goodfits[stream];
     }
  }

  if(overall_goodfits>0){
    overall_mean/=overall_goodfits;
    overall_sigma/=overall_goodfits;
    overall_chisq/=overall_goodfits;
    overall_fom/=overall_goodfits;
    overall_prob/=overall_goodfits;
  }

  scan_fitted =1;
  ff->Delete();
  ff=0;
}


void TModule::PrintScan(){

  short chip, chip2, stream;

  cout << longname   << " ";
  cout << scan_name  << " scan from ";
  cout << scan_start << " to ";
  cout << scan_stop  << " in steps of ";
  cout << scan_step << endl;

  for(stream=0;stream<2;stream++){

    cout << "  Stream " << stream << endl;

    cout << "   tot_evcnt  " << tot_evcnt[stream];
    cout << "   tot_ercnt  " << tot_ercnt[stream];
    cout << "   tot_tocnt  " << tot_tocnt[stream] << endl;

    cout << "   tot_nescnt " << tot_nescnt[stream];
    cout << "   tot_oshcnt " << tot_oshcnt[stream] << endl;

  }

  if(scan_fitted){
    if(likelihood){
      cout << "  Fit Results for likelihood fit" << endl;
    }else{
      cout << "  Fit Results for chisq fit" << endl;
    }
    cout << "\t\tMean\tSigma\tChisq\tProb\tnGood\tnBad" << endl;

    for(stream=0;stream<2;stream++){

      for(chip=0;chip<6;chip++){
        chip2 = chip;
        if(stream>0) chip2+=6;
 
        cout << "  Chip " << chip2  << "\t";
        cout << setprecision(5) << chip_mean[stream][chip]   << "\t";
        cout << setprecision(4) << chip_sigma[stream][chip]   << "\t";
        cout << chip_chisq[stream][chip]   << "\t";
        cout << chip_fom[stream][chip]   << "\t";
        cout << chip_prob[stream][chip]   << "\t";
        cout << chip_goodfits[stream][chip] << "\t";
        cout << chip_badfits[stream][chip]  << endl;
      }

      cout << "  Link " << stream << "\t";
      cout << setprecision(5) << side_mean[stream]   << "\t";
      cout << setprecision(4) << side_sigma[stream]   << "\t";
      cout << side_chisq[stream]   << "\t";
      cout << side_fom[stream]   << "\t";
      cout << side_prob[stream]   << "\t";
      cout << side_goodfits[stream] << "\t";
      cout << side_badfits[stream]  << endl;
    }

    cout << "  Overall "  << "\t";
    cout << setprecision(5) << overall_mean   << "\t";
    cout << setprecision(4) << overall_sigma   << "\t";
    cout << overall_chisq   << "\t";
    cout << overall_fom   << "\t";
    cout << overall_prob   << "\t";
    cout << overall_goodfits << "\t";
    cout << overall_badfits  << endl;
  }
}


//
// DCS fuctionality LE 5/10-2000, expanded PWP 27.10.00
// and again PWP 10.12.00
// Memory leak fixes to GetGraph functions PWP 21.12.01
//

void TModule::ReadDCS(int burst_count) {
  
  if(burst_count<ST_MAX_SCANPOINTS) {
    temp1[burst_count] =DCS->T1;
    temp2[burst_count] =DCS->T2;
    idet[burst_count] = DCS->idet;
    vdet[burst_count] = DCS->vdet;
    idd[burst_count]  = DCS->idd;
    vdd[burst_count]  = DCS->vdd;
    icc[burst_count]  = DCS->icc;
    vcc[burst_count]  = DCS->vcc;
  }
  return;
}

TPaveText*  TModule::GetDCSBurstText(){
  char s[64];

  if(DCSBurstText){
    DCSBurstText->Delete();
    DCSBurstText=0;
  }

  DCSBurstText = new TPaveText(0.0,0.0,1.0,1.0);
  DCSBurstText->SetFillColor(10);
  DCSBurstText->SetTextAlign(12);
  DCSBurstText->SetTextSize((float)0.10);
  DCSBurstText->SetTextColor(4);

  DCSBurstText->AddText(0.0,0.99,"T0");
  sprintf(s,"%2.1f C",DCS->T1);
  TText* t0_text = DCSBurstText->AddText(0.3,0.99,s);

  DCSBurstText->AddText(0.0,0.9,"T1");
  sprintf(s,"%2.1f C",DCS->T2);
  TText* t1_text = DCSBurstText->AddText(0.3,0.9,s);

  if(DCS->MTtrip){  // Module over or under temperature
    t0_text->SetTextColor(kRed);
    t1_text->SetTextColor(kRed);
  }

  DCSBurstText->AddText(0.0,0.8,"Vdet");
  sprintf(s,"%3.1f V",DCS->vdet);
  TText* line2 = DCSBurstText->AddText(0.3,0.8,s);

  DCSBurstText->AddText(0.0,0.7,"Idet");
  sprintf(s,"%1.2f #muA",DCS->idet);
  TText* line3 = DCSBurstText->AddText(0.3,0.7,s);

  DCSBurstText->AddText(0.0,0.6,"Vcc");
  sprintf(s,"%1.2f V",DCS->vcc);
  DCSBurstText->AddText(0.3,0.6,s);

  DCSBurstText->AddText(0.0,0.5,"Icc");
  sprintf(s,"%4.0f mA",DCS->icc);
  TText* icc_text = DCSBurstText->AddText(0.3,0.5,s);
  if(DCS->icc>DCS->icc_lim){
    icc_text->SetTextColor(kRed);
  }

  DCSBurstText->AddText(0.0,0.4,"Vdd");
  sprintf(s,"%1.2f V",DCS->vdd);
  DCSBurstText->AddText(0.3,0.4,s);

  DCSBurstText->AddText(0.0,0.3,"Idd");
  sprintf(s,"%4.0f mA",DCS->idd);
  TText* idd_text = DCSBurstText->AddText(0.3,0.3,s);
  if(DCS->idd>DCS->idd_lim){
    idd_text->SetTextColor(kRed);
  }

  sprintf(s,"%d HV Trips",DCS->nHVtrips);
  TText* hvt_text = DCSBurstText->AddText(0.0,0.2,s);
  if(DCS->HVtrip){
    hvt_text->SetTextColor(kRed);
  }

  sprintf(s,"%d LV Trips",DCS->nLVtrips);
  TText* lvt_text = DCSBurstText->AddText(0.0,0.1,s);
  if(DCS->LVtrip){
    lvt_text->SetTextColor(kRed);
  }

  return DCSBurstText;
}


TPaveText*  TModule::GetDCSScanText(){
  char s[64];
  int i;

  float t1_max=-999;
  float t1_min= 999;
  float t2_max=-999;
  float t2_min= 999;
  float vdet_max=-999;
  float vdet_min= 999;
  float idet_max=-999;
  float idet_min= 999;
  float vcc_max=-999;
  float vcc_min= 999;
  float icc_max=-999;
  float icc_min= 999;
  float vdd_max=-999;
  float vdd_min= 999;
  float idd_max=-999;
  float idd_min= 999;

  if(DCSScanText){
    DCSScanText->Delete();
    DCSScanText=0;
  }

  for(i=0;i<=burst;i++){
    if(temp1[i]>t1_max){  t1_max=temp1[i]; }
    if(temp1[i]<t1_min){  t1_min=temp1[i]; }
    if(temp2[i]>t2_max){  t2_max=temp2[i]; }
    if(temp2[i]<t2_min){  t2_min=temp2[i]; }
    if(vdet[i]>vdet_max){ vdet_max=vdet[i]; }
    if(vdet[i]<vdet_min){ vdet_min=vdet[i]; }
    if(idet[i]>idet_max){ idet_max=idet[i]; }
    if(idet[i]<idet_min){ idet_min=idet[i]; }
    if(vcc[i]>vcc_max){   vcc_max=vcc[i];   }
    if(vcc[i]<vcc_min){   vcc_min=vcc[i];   }
    if(icc[i]>icc_max){   icc_max=icc[i];   }
    if(icc[i]<icc_min){   icc_min=icc[i];   }
    if(vdd[i]>vdd_max){   vdd_max=vdd[i];   }
    if(vdd[i]<vdd_min){   vdd_min=vdd[i];   }
    if(idd[i]>idd_max){   idd_max=idd[i];   }
    if(idd[i]<idd_min){   idd_min=idd[i];   }
  }

  DCSScanText = new TPaveText(0.0,0.0,1.0,1.0);
  DCSScanText->SetFillColor(10);
  DCSScanText->SetTextAlign(12);
  DCSScanText->SetTextSize((float)0.10);
  DCSScanText->SetTextColor(4);

  DCSScanText->AddText(0.0,0.99,"T0");
  sprintf(s,"%2.1f-%2.1f C",t1_min,t1_max);
  DCSScanText->AddText(0.3,0.99,s);

  DCSScanText->AddText(0.0,0.9,"T1");
  sprintf(s,"%2.1f-%2.1f C",t2_min,t2_max);
  DCSScanText->AddText(0.3,0.9,s);

  DCSScanText->AddText(0.0,0.8,"Vdet");
  sprintf(s,"%3.1f-%3.1f V",vdet_min,vdet_max);
  DCSScanText->AddText(0.3,0.8,s);

  DCSScanText->AddText(0.0,0.7,"Idet");
  sprintf(s,"%1.2f-%1.2f #muA",idet_min,idet_max);
  DCSScanText->AddText(0.3,0.7,s);

  DCSScanText->AddText(0.0,0.6,"Vcc");
  sprintf(s,"%1.2f-%1.2f V",vcc_min,vcc_max);
  DCSScanText->AddText(0.3,0.6,s);

  DCSScanText->AddText(0.0,0.5,"Icc");
  sprintf(s,"%4.0f-%4.0f mA",icc_min,icc_max);
  DCSScanText->AddText(0.3,0.5,s);

  DCSScanText->AddText(0.0,0.4,"Vdd");
  sprintf(s,"%1.2f-%1.2f V",vdd_min,vdd_max);
  DCSScanText->AddText(0.3,0.4,s);

  DCSScanText->AddText(0.0,0.3,"Idd");
  sprintf(s,"%4.0f-%4.0f mA",idd_min,idd_max);
  DCSScanText->AddText(0.3,0.3,s);

  return DCSScanText;
}

TGraph* TModule::GetT1Graph(){
  if(T1Graph==0){
    T1Graph = new TGraph(burst+1,scan_points,temp1);
    sprintf(htitle, "%s T1", longname);
    T1Graph->SetTitle(htitle);
  }
  return T1Graph;
} 

TGraph* TModule::GetT2Graph(){
  if(T1Graph==0){
    T2Graph = new TGraph(burst+1,scan_points,temp2);
    sprintf(htitle, "%s T2", longname);
    T2Graph->SetTitle(htitle);
  }
  return T2Graph;
} 

TGraph* TModule::GetVdetGraph(){
  if(VdetGraph==0){
    VdetGraph = new TGraph(burst+1,scan_points,vdet);
    sprintf(htitle, "%s Vdet", longname);
    VdetGraph->SetTitle(htitle);
  }
  return VdetGraph;
} 

TGraph* TModule::GetIdetGraph(){
  if(IdetGraph==0){
    IdetGraph = new TGraph(burst+1,scan_points,idet);
    sprintf(htitle, "%s Idet", longname);
    IdetGraph->SetTitle(htitle);
  }
  return IdetGraph;
} 

TGraph* TModule::GetVccGraph(){
  if(VccGraph==0){
    VccGraph = new TGraph(burst+1,scan_points,vcc);
    sprintf(htitle, "%s Vcc", longname);
    VccGraph->SetTitle(htitle);
  }
  return VccGraph;
} 

TGraph* TModule::GetIccGraph(){
  if(IccGraph==0){
    IccGraph = new TGraph(burst+1,scan_points,icc);
    sprintf(htitle, "%s Icc", longname);
    IccGraph->SetTitle(htitle);
  }
  return IccGraph;
} 

TGraph* TModule::GetVddGraph(){
  if(VddGraph==0){
    VddGraph = new TGraph(burst+1,scan_points,vdd);
    sprintf(htitle, "%s Vdd", longname);
    VddGraph->SetTitle(htitle);
  }
  return VddGraph;
}
 
TGraph* TModule::GetIddGraph(){
  if(IddGraph==0){
    IddGraph = new TGraph(burst+1,scan_points,idd);
    sprintf(htitle, "%s Idd", longname);
    IddGraph->SetTitle(htitle);
  }
  return IddGraph;
}
 
//
// Graphs of various counters - PWP 08.12.00
// 
// TODO - same memory leak fix as above
// PWP 21.12.01

void TModule::FillDummy(int* data){
  int i;
  for(i=0;i<=burst;i++){
    dummy[i]=(float)data[i];
  }
}

TGraph* TModule::GetBcErcntGraph(short stream){
  if((stream<0)||(stream>1)) return 0;
  
  FillDummy(scan_bc_ercnt[stream]);
  TGraph *gr = new TGraph(burst+1,scan_points,dummy);
  sprintf(htitle, "%s BC Error Count stream %d",longname,stream);
  gr->SetTitle(htitle);
  return gr;
}

TGraph* TModule::GetL1ErcntGraph(short stream){
  if((stream<0)||(stream>1)) return 0;

  FillDummy(scan_l1_ercnt[stream]);
  TGraph *gr = new TGraph(burst+1,scan_points,dummy);
  sprintf(htitle, "%s L1 Error Count stream %d",longname,stream);
  gr->SetTitle(htitle);
  return gr;
}

TGraph* TModule::GetEvcntGraph(short stream){
  if((stream<0)||(stream>1)) return 0;
  
  FillDummy(scan_evcnt[stream]);
  TGraph *gr = new TGraph(burst+1,scan_points,dummy);
  sprintf(htitle, "%s Event Count stream %d",longname,stream);
  gr->SetTitle(htitle);
  return gr;
}

TGraph* TModule::GetErcntGraph(short stream){
  if((stream<0)||(stream>1)) return 0;
  
  FillDummy(scan_ercnt[stream]);
  TGraph *gr = new TGraph(burst+1,scan_points,dummy);
  sprintf(htitle, "%s Error Count stream %d",longname,stream);
  gr->SetTitle(htitle);
  return gr;
}
 
TGraph* TModule::GetTocntGraph(short stream){
  if((stream<0)||(stream>1)) return 0;
  
  FillDummy(scan_tocnt[stream]);
  TGraph *gr = new TGraph(burst+1,scan_points,dummy);
  sprintf(htitle, "%s Timeout Count stream %d",longname,stream);
  gr->SetTitle(htitle);
  return gr;
}
 
TGraph* TModule::GetNesGraph(short stream){
  if((stream<0)||(stream>1)) return 0;
  
  FillDummy(scan_nescnt[stream]);
  TGraph *gr = new TGraph(burst+1,scan_points,dummy);
  sprintf(htitle, "%s Non-existent Strip Count stream %d",longname,stream);
  gr->SetTitle(htitle);
  return gr;
}
 
TGraph* TModule::GetOshGraph(short stream){
  if((stream<0)||(stream>1)) return 0;
  
  FillDummy(scan_oshcnt[stream]);
  TGraph *gr = new TGraph(burst+1,scan_points,dummy);
  sprintf(htitle, "%s Out of Sequence Hit Count stream %d",longname,stream);
  gr->SetTitle(htitle);
  return gr;
} 


//
// Get/Set/Fill arrays which are MODULE specific
//

float  TModule::GetPoint(float var){ return scan_points[burst]; }
int  TModule::GetTrigs(int var)  { return scan_trigs[burst];  }

void TModule::SetPoint(float var){ scan_points[burst] = var; }
void TModule::SetTrigs(int var)  { scan_trigs[burst]  = var; }

void TModule::FillTrigs()        { scan_trigs[burst]++; }
void TModule::FillTrigs(int var) { scan_trigs[burst] += var; }

void TModule::CreateTree(){
  // create EvTree

  evtree = new TTree("EvTree","tree with hit/event");
  evtree->Branch("strm0",&evtr_strm0,"strm0[768]/B");
  evtree->Branch("strm1",&evtr_strm1,"strm1[768]/B");
  evtree->Branch("hits0",&hits_strm0,"hits0[7]/I");
  evtree->Branch("hits1",&hits_strm1,"hits1[7]/I");

  return;
}

void TModule::FillTree(){
  // fill EvTree

  if(evtree!=0) evtree->Fill();

  return;
}

void TModule::DeleteTree(){
  // delete EvTree

  if(evtree!=0) evtree->Delete();
  evtree = (TTree*) 0;

  return;
}


//
// Get/Set/Fill arrays which are STREAM specific
//

int  TModule::GetEvcnt(short stream){ return scan_evcnt[stream][burst]; }
int  TModule::GetErcnt(short stream){ return scan_ercnt[stream][burst]; }
int  TModule::GetTocnt(short stream){ return scan_tocnt[stream][burst]; }
int  TModule::GetOshcnt(short stream){ return scan_oshcnt[stream][burst]; }
int  TModule::GetNescnt(short stream){ return scan_nescnt[stream][burst]; }
int  TModule::GetBcErcnt(short stream){ return scan_bc_ercnt[stream][burst]; }
int  TModule::GetL1Ercnt(short stream){ return scan_l1_ercnt[stream][burst]; }

void TModule::SetEvcnt(short stream, int var){ scan_evcnt[stream][burst] = var; }
void TModule::SetErcnt(short stream, int var){ scan_ercnt[stream][burst] = var; }
void TModule::SetTocnt(short stream, int var){ scan_tocnt[stream][burst] = var; }
void TModule::SetOshcnt(short stream, int var){ scan_oshcnt[stream][burst] = var; }
void TModule::SetNescnt(short stream, int var){ scan_nescnt[stream][burst] = var; }

void TModule::FillEvcnt(short stream){ scan_evcnt[stream][burst]++; }
void TModule::FillErcnt(short stream){ scan_ercnt[stream][burst]++; }
void TModule::FillTocnt(short stream){ scan_tocnt[stream][burst]++; }
void TModule::FillNescnt(short stream){ scan_nescnt[stream][burst]++; }
void TModule::FillOshcnt(short stream){ scan_oshcnt[stream][burst]++; }
void TModule::FillBcErcnt(short stream){ scan_bc_ercnt[stream][burst]++; }
void TModule::FillL1Ercnt(short stream){ scan_l1_ercnt[stream][burst]++; }

void TModule::FillEvcnt(short stream, int var){ scan_evcnt[stream][burst] += var; }
void TModule::FillErcnt(short stream, int var){ scan_ercnt[stream][burst] += var; }
void TModule::FillTocnt(short stream, int var){ scan_tocnt[stream][burst] += var; }
void TModule::FillNescnt(short stream, int var){ scan_nescnt[stream][burst] += var; }
void TModule::FillOshcnt(short stream, int var){ scan_oshcnt[stream][burst] += var; }


void ErfLikelihoodFit(Int_t &npar, Double_t *gin,
                      Double_t &f, Double_t *par, Int_t iflag) {

  // Minimization function for Erf data using a Likelihood method
  // We calculate the likelihood by determining the Binomial probability
  // that given a number of entries in a particular bin, the fit would 
  // predict its value.  This is then done for each bin and the sum of 
  // the logs is taken as the likelihood.

  // based on work by Simon Peeters and Sandra Muijs of NIKHEF
  // with advice from Bill Murray of RAL
  // Version 0.2  PWP/BJG 03.04.02

  // Version for scan data that is NOT NORMALISED

  // Parameters of the fit:
  // par[0] - Constant, fixed equal to the number of triggers (trials)
  // par[1] - Mean
  // par[2] - Sigma

  if(ch_data==0) return;

  Int_t np    = ch_data->GetN();
  Double_t *x = ch_data->GetX();
  Double_t *y = ch_data->GetY();

  Double_t P1;
  Double_t f1;
  Int_t bin;

  f = 0.0;

  // Evaluate the binomial probability that the function describes this data

  if(par[2]>0){ // protect against divide by zero errors

    for(bin=0;bin<np;bin++){

      if(ff->IsInside(&x[bin])){ // restrict fitting to range of function 

        if( (y[bin]>=0) && (y[bin]<=n[bin]) ){

          P1 = 0.5*(1.0+TMath::Erf((x[bin]-par[1])/(par[2]*1.414213562) ));
          f1 = TMath::Log(P1);

          if( (P1>10E-10) && (P1<(1-10E-10)) ){
            f += y[bin]*f1 + (n[bin]-y[bin])*TMath::Log(1.0-P1);
          }
        }
      }
    }
  }

  f *= -2.0; // Scale log likelihood to look like chisq in limit of large stats
}


void ErfcLikelihoodFit(Int_t &npar, Double_t *gin,
                       Double_t &f, Double_t *par, Int_t iflag) {

  // Minimization function for Erfc data using a Likelihood method
  // We calculate the likelihood by determining the Binomial probability
  // that given a number of entries in a particular bin, the fit would 
  // predict its value.  This is then done for each bin and the sum of 
  // the logs is taken as the likelihood.

  // based on work by Simon Peeters and Sandra Muijs of NIKHEF
  // with advice from Bill Murray of RAL
  // Version 0.2  PWP/BJG 03.04.02

  // Version for scan data that is NOT NORMALISED

  // Parameters of the fit:
  // par[0] - Constant, fixed equal to the number of triggers (trials)
  // par[1] - Mean
  // par[2] - Sigma

  if(ch_data==0) return;

  Int_t np    = ch_data->GetN();
  Double_t *x = ch_data->GetX();
  Double_t *y = ch_data->GetY();

  Double_t P1;
  Double_t f1;
  Int_t bin;

  f = 0.0;

  // Evaluate the binomial probability that the function describes this data

  if(par[2]>0){ // protect against divide by zero errors

    for(bin=0;bin<np;bin++){

      if(ff->IsInside(&x[bin])){ // restrict fitting to range of function 

        if( (y[bin]>=0) && (y[bin]<=n[bin]) ){

          P1 = 0.5*(1.0+TMath::Erf((par[1]-x[bin])/(par[2]*1.414213562) ));
          f1 = TMath::Log(P1);

          if( (P1>10E-10) && (P1<(1-10E-10)) ){
            f += y[bin]*f1 + (n[bin]-y[bin])*TMath::Log(1.0-P1);
          }
        }
      }
    }
  }

  f *= -2.0; // Scale log likelihood to look like chisq in limit of large stats
}


void NErfLikelihoodFit(Int_t &npar, Double_t *gin,
                      Double_t &f, Double_t *par, Int_t iflag) {

  // Minimization function for Erf data using a Likelihood method
  // We calculate the likelihood by determining the Binomial probability
  // that given a number of entries in a particular bin, the fit would 
  // predict its value.  This is then done for each bin and the sum of 
  // the logs is taken as the likelihood.

  // based on work by Simon Peeters and Sandra Muijs of NIKHEF
  // with advice from Bill Murray of RAL
  // Version 0.2  PWP/BJG 03.04.02

  // Version for scan data that IS NORMALISED

  // Parameters of the fit:
  // par[0] - Constant, fixed equal to the number of triggers (trials)
  // par[1] - Mean
  // par[2] - Sigma

  if(ch_data==0) return;

  Int_t np    = ch_data->GetN();
  Double_t *x = ch_data->GetX();
  Double_t *y = ch_data->GetY();

  Double_t ly;
  Double_t P1;
  Double_t f1;
  Int_t bin;

  f = 0.0;

  // Evaluate the binomial probability that the function describes this data

  if(par[2]>0){ // protect against divide by zero errors

    for(bin=0;bin<np;bin++){

      if(ff->IsInside(&x[bin])){ // restrict fitting to range of function 

        ly = y[bin]*n[bin];

        if( (ly>=0) && (ly<=n[bin]) ){

          P1 = 0.5*(1.0+TMath::Erf((x[bin]-par[1])/(par[2]*1.414213562) ));
          f1 = TMath::Log(P1);

          if( (P1>10E-10) && (P1<(1-10E-10)) ){
            f += ly*f1 + (n[bin]-ly)*TMath::Log(1.0-P1);
          }
        }
      }
    }
  }

  f *= -2.0; // Scale log likelihood to look like chisq in limit of large stats
}


void NErfcLikelihoodFit(Int_t &npar, Double_t *gin,
                       Double_t &f, Double_t *par, Int_t iflag) {

  // Minimization function for Erfc data using a Likelihood method
  // We calculate the likelihood by determining the Binomial probability
  // that given a number of entries in a particular bin, the fit would 
  // predict its value.  This is then done for each bin and the sum of 
  // the logs is taken as the likelihood.

  // based on work by Simon Peeters and Sandra Muijs of NIKHEF
  // with advice from Bill Murray of RAL
  // Version 0.2  PWP/BJG 03.04.02

  // Version for scan data that IS NORMALISED

  // Parameters of the fit:
  // par[0] - Constant, fixed equal to the number of triggers (trials)
  // par[1] - Mean
  // par[2] - Sigma

  if(ch_data==0) return;

  Int_t np    = ch_data->GetN();
  Double_t *x = ch_data->GetX();
  Double_t *y = ch_data->GetY();

  Double_t ly;
  Double_t P1;
  Double_t f1;
  Int_t bin;

  f = 0.0;

  // Evaluate the binomial probability that the function describes this data

  if(par[2]>0){ // protect against divide by zero errors

    for(bin=0;bin<np;bin++){

      if(ff->IsInside(&x[bin])){ // restrict fitting to range of function 

        ly = y[bin]*n[bin];

        if( (ly>=0) && (ly<=n[bin]) ){

          P1 = 0.5*(1.0+TMath::Erf((par[1]-x[bin])/(par[2]*1.414213562) ));
          f1 = TMath::Log(P1);

          if( (P1>10E-10) && (P1<(1-10E-10)) ){
            f += ly*f1 + (n[bin]-ly)*TMath::Log(1.0-P1);
          }
        }
      }
    }
  }

  f *= -2.0; // Scale log likelihood to look like chisq in limit of large stats
}



