#ifndef TMODULE_h
#define TMODULE_h

/*
    TModule.h

    version 0.3   PWP 27.10.00
    version 0.4   PWP 09.01.01
    version 0.5   JGK/PWP 18.07.01
    version 0.6   PWP/BJG 05.04.02
*/


#if _MSC_VER >= 1000
#pragma once
#endif

#include "TObject.h"
#include "TDatime.h"
#include "TStyle.h"
#include "TH1.h"
#include "TH2.h"
#include "TF1.h"
#include "TNtuple.h"
#include "TGraphErrors.h"
#include "TPaveText.h"
#include "TModuleDefine.h"
#include "TModuleDCS.h"
#include "TFitter.h"

#include "../stlib/sct_scans.h"

#include "functions.h"

class TModule : public TObject {

  public:

    TModule(short new_id);
    ~TModule();

    void TModule::ResetData();
    void TModule::PrintStatus();

    void TModule::GetModuleData(short mid);
    void TModule::GetBurstData();
    void TModule::SetModuleName(char* module_name); 

    //
    // Event functions
    //

    void TModule::ResetEvent();

    //
    // Burst functions
    //

    void TModule::ResetBurstHist();
    void TModule::FillBurstHist();
    void TModule::FillBurstHist(short cal_mode);
    void TModule::FillClusterHist(short stream, short cluster);
    void TModule::FillCorrHist(short hits0, short hits1);

    short TModule::ChansOverTarget(short target);
    short TModule::ChansUnderTarget(short target);
    short TModule::OKtoStop(short scan_type);

    float TModule::MeanOccupancy();
    float TModule::MeanOccupancy(short stream);
    float TModule::FractionalOccupancy();
    float TModule::FractionalOccupancy(short stream);

    void TModule::PrintBurst();
    void TModule::SetBurstCount(int var);

    //
    // Scan Functions
    //

    void TModule::InitScan(int new_max_points, float new_start, float new_stop, float new_step, char *new_scan_name);
    void TModule::FillScanHist();                            // Fill scan histogram wihout normalisation
    void TModule::FillScanHist(short burst_count);           
    void TModule::FillNormalisedScanHist();                  // Fill scan histogram
    void TModule::FillNormalisedScanHist(short burst_count); // normalised such that max occupancy =1
    void TModule::FillNOScanHist(short burst_count);         // normalised, but ignores overflows
    void TModule::FitScan(short fit_type, short cal_mode);   // Fit error function or complementary error function
    void TModule::PrintScan();                               // Show scan error and fit details

    //
    // DCS functionality
    //

    TModuleDCS* DCS;

    void TModule::ReadDCS(int burst_count);

    TPaveText* TModule::GetDCSBurstText();  // PWP 14.03.02
    TPaveText* TModule::GetDCSScanText();   // PWP 14.03.02

    TGraph* TModule::GetT1Graph();
    TGraph* TModule::GetT2Graph();
    TGraph* TModule::GetVdetGraph();
    TGraph* TModule::GetIdetGraph();
    TGraph* TModule::GetVccGraph();
    TGraph* TModule::GetIccGraph();
    TGraph* TModule::GetVddGraph();
    TGraph* TModule::GetIddGraph();

    //
    // Graphs of various counters
    //

    TGraph* TModule::GetBcErcntGraph(short stream);
    TGraph* TModule::GetL1ErcntGraph(short stream);
    TGraph* TModule::GetEvcntGraph(short stream);
    TGraph* TModule::GetErcntGraph(short stream);
    TGraph* TModule::GetTocntGraph(short stream);
    TGraph* TModule::GetNesGraph(short stream);
    TGraph* TModule::GetOshGraph(short stream);

    //
    // Get/Set/Fill arrays which are MODULE specific
    //

    float TModule::GetPoint(float var);
    int  TModule::GetTrigs(int var);

    void TModule::SetPoint(float var);
    void TModule::SetTrigs(int var);

    void TModule::FillTrigs();
    void TModule::FillTrigs(int var);

    void TModule::CreateTree();
    void TModule::FillTree();
    void TModule::DeleteTree();

    //
    // Get/Set/Fill arrays which are STREAM specific
    //

    int  TModule::GetEvcnt(short stream);
    int  TModule::GetErcnt(short stream);
    int  TModule::GetTocnt(short stream);
    int  TModule::GetNescnt(short stream);
    int  TModule::GetOshcnt(short stream);
    int  TModule::GetBcErcnt(short stream);
    int  TModule::GetL1Ercnt(short stream);

    void TModule::SetEvcnt(short stream, int var);
    void TModule::SetErcnt(short stream, int var);
    void TModule::SetTocnt(short stream, int var);
    void TModule::SetNescnt(short stream, int var);
    void TModule::SetOshcnt(short stream, int var);

    void TModule::FillEvcnt(short stream);
    void TModule::FillErcnt(short stream);
    void TModule::FillTocnt(short stream);
    void TModule::FillNescnt(short stream);
    void TModule::FillOshcnt(short stream);
    void TModule::FillBcErcnt(short stream);
    void TModule::FillL1Ercnt(short stream);

    void TModule::FillEvcnt(short stream, int var);
    void TModule::FillErcnt(short stream, int var);
    void TModule::FillTocnt(short stream, int var);
    void TModule::FillNescnt(short stream, int var);
    void TModule::FillOshcnt(short stream, int var);

    char longname[80];
    char name[32];

    short id;
    short present;
    short active;
    short link0;
    short link1;
    short n_active_channels;

    TH1F *h_burst0;   // link0 burst occupancy
    TH1F *h_burst1;   // link1

    TH1F *h_accbst0;   // link0 accumulated burst occupancy
    TH1F *h_accbst1;   // link1

    TH1F *h_cluster0; // link0 cluster size
    TH1F *h_cluster1; // link1

    TH2F *h_corr;     // Correlation in total occupancy between streams

    TTree *evtree;    // event-by-event ntuple for correlated noise analysis
    Bool_t evtr_strm0[768],evtr_strm1[768];
    Int_t hits_strm0[7], hits_strm1[7];

    short scan[ST_MAX_SCANPOINTS][2][768];
    float scan_points[ST_MAX_SCANPOINTS];
    int   scan_trigs[ST_MAX_SCANPOINTS];

    int scan_occupancy[2][ST_MAX_SCANPOINTS];  // total occupancy
    int scan_overflows[2][ST_MAX_SCANPOINTS];  // number of channels which have overflowed

    short scan_count;
    short scan_max_points;
    float scan_start;
    float scan_stop;
    float scan_step;
    float scan_lo;
    float scan_hi;
    char scan_name[128];

    short use_ch_maxima;
    short scan_fitted;

    TH2F *h_scan0;
    TH2F *h_scan1;

    TGraph* gr_icc;

    //
    // Storage for scan fit results
    //

    TH1S *h_code0; 
    TH1S *h_code1;
    TH1F *h_mean0;
    TH1F *h_mean1;
    TH1F *h_sigma0;
    TH1F *h_sigma1;
    TH1F *h_chisq0;
    TH1F *h_chisq1;
    TH1F *h_fom0;
    TH1F *h_fom1;

    float chip_mean[2][6];
    float chip_sigma[2][6];
    float chip_chisq[2][6];
    short chip_badfits[2][6];
    float chip_fom[2][6];
    float chip_prob[2][6];
    short chip_goodfits[2][6];

    float side_mean[2];
    float side_sigma[2];
    float side_chisq[2];
    short side_badfits[2];
    float side_fom[2];
    float side_prob[2];
    short side_goodfits[2];

    float overall_mean;
    float overall_sigma;
    float overall_chisq;
    short overall_badfits;
    float overall_fom;
    float overall_prob;
    short overall_goodfits;

    short likelihood;
    short use_range;

    //
    // DCS variables
    //

    float temp1[ST_MAX_SCANPOINTS];
    float temp2[ST_MAX_SCANPOINTS];
    float vdet[ST_MAX_SCANPOINTS];
    float idet[ST_MAX_SCANPOINTS];
    float vcc[ST_MAX_SCANPOINTS];
    float icc[ST_MAX_SCANPOINTS];
    float vdd[ST_MAX_SCANPOINTS];
    float idd[ST_MAX_SCANPOINTS];

    TPaveText* DCSBurstText;
    TPaveText* DCSScanText;

    TGraph* T1Graph;
    TGraph* T2Graph;
    TGraph* VdetGraph;
    TGraph* IdetGraph;
    TGraph* VccGraph;
    TGraph* IccGraph;
    TGraph* VddGraph;
    TGraph* IddGraph;

  private:

    void TModule::FillDummy(int* data);

    char htitle[128];
    char hname[32];

    float dummy[ST_MAX_SCANPOINTS];

    int burst;
    int normalised;
    int last_burst_with_lo_occ;
    int last_burst_with_hi_occ;

    int scan_bc_ercnt[2][ST_MAX_SCANPOINTS];   // bc error count
    int scan_l1_ercnt[2][ST_MAX_SCANPOINTS];   // l1 error count

    int scan_evcnt[2][ST_MAX_SCANPOINTS];   // event count
    int scan_ercnt[2][ST_MAX_SCANPOINTS];   // stream error count
    int scan_tocnt[2][ST_MAX_SCANPOINTS];   // stream time out count
    int scan_nescnt[2][ST_MAX_SCANPOINTS];  // "non existent strip" error count
    int scan_oshcnt[2][ST_MAX_SCANPOINTS];  // "out of sequence hit" error count

    int tot_evcnt[2];
    int tot_ercnt[2];
    int tot_tocnt[2];
    int tot_nescnt[2];
    int tot_oshcnt[2];

    ClassDef(TModule,2)
};

  void ErfLikelihoodFit(Int_t &npar, Double_t *gin,
                        Double_t &f, Double_t *par, Int_t iflag);
  void ErfcLikelihoodFit(Int_t &npar, Double_t *gin,
                        Double_t &f, Double_t *par, Int_t iflag);
  void NErfLikelihoodFit(Int_t &npar, Double_t *gin,
                        Double_t &f, Double_t *par, Int_t iflag);
  void NErfcLikelihoodFit(Int_t &npar, Double_t *gin,
                        Double_t &f, Double_t *par, Int_t iflag);

#endif
