#ifndef TST_h
#define TST_h

//
//  TST.h
//
//  version 0.1   GFM 990421
//  version 0.2   PWP 010518 - not that there weren't 2^n intermediates!!
//


#if _MSC_VER >= 1000
#pragma once
#endif

#include "TSystem.h"
#include "TObject.h"
#include "TH1.h"
#include "TPaveText.h"

#include "seu.h"

#include "../stlib/abc.h"
#include "../stlib/sct_scans.h"
#include "../stlib/sct_hardware.h"

#define ST_BUF_LENGTH 0x100000

class TST : public TObject {

  public:

    TST();
    ~TST();

    void TST::PrintStatus();
    void TST::RestoreState();
    void TST::SaveState();

    int  TST::UpdateRunInfo();
  
    int  TST::VMETest();
    void TST::Startup();
    void TST::Restart();
    void TST::Shutdown();

    void TST::SysmapMenu();
    void TST::ShowSysmap();
    void TST::ShowModule(int imod);
    void TST::ShowTrims(int imod);
    void TST::ShowRC(int imod);

    int  TST::GetMaxModules();
    void TST::GetModuleName(int imod, char* nam);
    void TST::GetDeviceType(int imod, char *nam);
    void TST::ReadMaskFile(int imod);

    void TST::ModifyEnables(int imod, short present, short active, short link0, short link1);

    void TST::ConfigureMask(int imod, short chan);
    void TST::ConfigureTrim(int imod, short chan, short var);
    void TST::ConfigureRC(int imod, int chip, short method, short function,
                          float p0, float p1, float p2);
    void TST::ConfigureVariable(int imod, int chip, int typ, float var);
    void TST::ConfigureVariable(int imod, int typ, float var);
    void TST::ConfigureVariable(int typ, float var);

    void TST::RetrieveRC(int imod, int chip, short method, short function,
                         float p0, float p1, float p2);
    float TST::RetrieveVariable(int imod, int chip, int chan, int typ);
    float TST::RetrieveVariable(int imod, int chip, int typ);
    float TST::RetrieveVariable(int imod, int typ);
    void TST::ExecuteConfigs();
    void TST::ExecuteConfigs(short dump_level);
    void TST::ExecuteSCConfigs();
    void TST::ExecuteSCConfigs(short hard_reset);

    int TST::AppendEvent();  // Append current event data to output buffer
                             // return total shorts appended (positive) or error (negative)
    int TST::AppendArray(char *id, int length, int count, int *data);
                             // Append arbitrary int array to output buffer
    int TST::WriteBuffer();  // Open file stdat[run].dat, creating it if necessary,
                             // and append current buffer contents

    int  TST::SEUTest();
    int  TST::SEUTest(int inspill);
    void TST::SEUBurst();    // Repeat SEU test [burst_ntrigs] times
    void TST::SEUClear();
    void TST::SEUAppendSummary(int length,int* data, short grand_total);

    int  TST::FindSurvivors(short mid);
    int  TST::FindSurvivors(short mid, short use_link0, short use_link1);

    void TST::ClearSuspense();
    void TST::ClearSuspense(short mid);

    void TST::RunFree();
    void TST::RunStop();
    void TST::Single(short ttype);
    void TST::SingleTrigger();
    void TST::SingleCalTrig();
    void TST::SingleBCReset();
    void TST::SingleSoftReset();

    void TST::ExecuteBurst();
    void TST::ExecuteBurst(short cal_mode);

    void TST::TranslateVariableName(short typ, char *nam);

    void TST::ConfigureScan(short new_type, float new_start, float new_stop, float new_step);
    void TST::ConfigureScanDefaults(short new_type);
    void TST::InitScan();
    void TST::ScanPoint();
    void TST::EndScan();
    void TST::ShowScan();
 
    void TST::OptoTXOn();
    void TST::OptoTXOff();
    void TST::OptifStatus(int oid);
    void TST::HVOff();
    void TST::HVOn();
    void TST::HVRamp(short to_zero);
    void TST::HVRecoverTrips();
    void TST::HVStatus();
    void TST::LVOff();
    void TST::LVOn();
    void TST::LVStatus();
    float TST::LVGetIdd(short imod);
    float TST::LVGetIcc(short imod);
    TGraph* TST::LVGetIddGraph(short imod);
    TGraph* TST::LVGetIccGraph(short imod);
    void TST::FindTrips();
    void TST::RecoverTrips();
    void TST::HardReset();

    void TST::Sleep(int milliseconds);

    void TST::StOutScanReset();

    void TST::DCSAction();
    void TST::DCSQuery();
    void TST::DCSPrint();
    float TST::DCSMessg(char* messg);

    void TST::ClockOff();
    void TST::ClockOn();

    short TST::FindModule(short mid, short sid);
    short TST::FindLink(short mid, short sid);

    void TST::WriteConfigs();
    void TST::WriteRODConfigs();
    void TST::SlogLoadTrig();

    char sct_path[80];  // Sources and executable
    char sct_var[80];   // Variable files (not under CVS)
    char sct_etc[80];   // Run Number and stop files
    char sct_data[80];  // Data files (could be remote disk or otherwise different to var)

    short nCLOAC;       // copies of CLOAC_NUMBER etc. from sct_hardware.h
    short nMuSTARDs;    // to be used later as part of run configuration system
    short nSLOGs;       // PWP 21.08.00
    short nSCTLV;
    short nSCTHV;
    short nOPTIFs;

    short num_modules;
    short num_panelmods;
    short num_showmods;
    short abort;        // 0:1; abort a sequence of tests
    short stopper;      // 0:1; don't use/use external stopper utility

    // Universal counters: these are always incremented by a new run, scan or burst
    // and are to be used for unique ntuple entry identifiers

    int runnum;         // Current run number, saved between program sessions and incremented at startup
    int burstnum;       // Current burst number since start of run, incremented by burst routines
    int scannum;        // Current scan number
    int seu_test_count;
    int seu_bad_count[4];
    int seu_good_count[4];

    long run_tsent;     // Total triggers sent this run
    long run_evcnt;     // Total system-wide events succesfully read this run
    long run_ercnt;     // Total system-wide events with stream errors read this run
    long run_tocnt;     // Total timeouts (any Mustard) this run - no successful event
    long run_xecnt;     // Total other errors this run - no successful event

    //
    // Parameters which control the behaviour of bursts and/or scans
    // and which are saved by SaveState()
    // and restored by RestoreState()
    //

    short ascii_nt;         // toggle output of ascii ntuples

    short config_frequency;      // These control the 
    short soft_reset_frequency;  // repetition rate of
    short bc_reset_frequency;    // interspersed resets

    short do_action_DCS;    // Read currents, temperatures and voltages at EOB
    short do_autostop;      // End scans automatically after the intersting bit
    short do_cal_loop;      // Enable loop over the four cal lines
    short do_fits;          // Toggle automatic fitting of scans
    short do_mask_test;     // Alternating mask patterns on consecuitve events
    short do_preburst;      // Detect and suppress dead links automatically at SOB
    short do_vary_ntrigs;   // Vary requested number of triggers according to occupancy
    short do_fill_evtree;   // Fill event tree

    short burst_type;       // burst type
    short burst_trtype;     // trigger type
    int   burst_ntrigs;     // number of triggers per burst (per cal line)
    int   burst_maxtrigs;   // maximum number of triggers (when do_vary_ntrigs==1)
    short burst_target;     // target no. hits/channel during NO scans

    short paf;              // Programmable Almost Full - see MuSTARD documentation
    short trlim;            // Trigger Limit - see MuSTARD documentation

    short L1A_per_loop;     // Number of L1A per trigger loop, usually 1
    short throw_away;       // If L1A_per_loop>1 we may want to throw some away

    short rate_limit_delay; // Delay in mS, used during laser scans (!!)
    long  seu_duration;     // duration of SEUTest with Software Timer (mS)

    int   debug_level;      // Diagnostic level

    //
    // Other Burst and Scan counters/parameters
    //

    short burst_count;   // Current burst number (reset at start of scan) incremented by scan control

    short scan_running;
    short scan_type;
    float scan_start;
    float scan_stop;
    float scan_step;
    float scan_point;   
    float scan_current; // Actually returned scan point setting
    short scan_count;   // Current scan number
    short scan_max_points;
    char scan_name[32];
    char scan_status_string[128];
    char scan_description_string[128];


    // System and MuSTARD-wide counters reset at start of burst

    int   tsent;                // Triggers sent
    int   evcnt[MUS_NUMBER+1];  // Valid events received
    short ercnt[MUS_NUMBER+1];  // Stream errors (includes ABC/D reported errors and Mustard-detected protocol violations)
    short tocnt[MUS_NUMBER+1];  // Timeouts 
    short xecnt[MUS_NUMBER+1];  // Other errors, eg, incorrect or out of range function arguments

    short ercnt_sum;
    short tocnt_sum;
    short xecnt_sum;

    TH1F *h_merr;                // Histogram for burst counters
    TH1F *h_evlen[MUS_NUMBER+1]; // Histogram of event word multiplicity

    // System-wide counters reset at start of scan: filled each burst from burst counters

    int scan_tsent[ST_MAX_SCANPOINTS];
    int scan_evcnt[ST_MAX_SCANPOINTS];
    short scan_ercnt[ST_MAX_SCANPOINTS];
    short scan_tocnt[ST_MAX_SCANPOINTS];
    short scan_xecnt[ST_MAX_SCANPOINTS];

    TH1F *h_scan_tsent;
    TH1F *h_scan_evcnt[MUS_NUMBER+1];
    TH1F *h_scan_ercnt[MUS_NUMBER+1];
    TH1F *h_scan_tocnt[MUS_NUMBER+1];
    TH1F *h_scan_xecnt[MUS_NUMBER+1];

    char vnam[64][32];   // Scan variable descriptor strings

    // Interfaces to VME modules
    // (Experimental, PWP 11.07.00)

    TCloac *cc;


    // The module data

    TModule *m[SCT_MAX_MODULES];    // Array of module objects including histogram data
    short mp[SCT_MAX_MODULES];      // Array of bits indicating module is present

    TPaveText* GetBurstPaveText();
    TPaveText* GetScanPaveText();

    TPaveText* BurstPaveText;
    TPaveText* ScanPaveText;

  private:

    void TST::TriggerBurst(short cal_mode); 
    void TST::RawBurst();
    void TST::SendIDBurst();
    void TST::NoiseBurst();

    int TST::DecodedBurst(short cal_mode);  // Decoded Trigger Burst, Mustard in DECODED, SAMPLE_ALL modes,
                                            // but histogramming done in software in this routine

    int TST::TestBurst();   // Module 0 only, all cal lines: decoded AND Mustard histograms
    int TST::PreBurst();    // Special burst to suspend streams causing timeout
    int TST::TimeEvBurst(short cal_mode);   // Time evolution over 24 timebins
    int TST::TriggerBurst2(short cal_mode); // Trigger burst, but decode error events
    int TST::VBurst2();       // CLOAC periodic trigger
    int TST::DecodedVBurst(); // CLOAC periodic trigger with software decoding
    int TST::EBurst();        // CLOAC external trigger
    int TST::DecodedEBurst(); // CLOAC external trigger with software decoding
    int TST::DecodedSBurst(); // SLOG periodic  trigger with software decoding

    void TST::AddHistograms(short cal_mode);
    void TST::ReadHistograms(short cal_mode);
    void TST::StreamStatus(short* empty, short* not_empty, short* almost_full, short* overflow);
    int TST::ReadEvent();
    int TST::TestEvent();
    short TST::ResetMustards();

    void TST::DecodeErrors();
    void TST::DecodeEvent(short cal_mode);
    void TST::DecodeEvent3bins(short cal_mode);
    void TST::DumpEvent(FILE* dumpfile);    // for diagnostics
 
    // Single-event Upset study: to be removed to its own class

    // Don't want this to be private just yet....
    // void TST::SEUBurst();    // Repeat SEU test [burst_ntrigs] times 

    // but these can be....
    int  TST::SEUDecodeData(short seu_event);
    int  TST::SEUDecodeEvent(short seu_event);
    int  TST::SEUCompareEvent(short event, short reference);
    void TST::SEUAppendLog(short n_events);
    void TST::SEUAppendLog2(int length,int* data);
    
    int  TST::ChipHunt(short mid, short link, short* chip_status);

   
    // The following variables provide storage space to record
    // the settings of parameters which control burst and scan behaviour.
    // Hence a user may easily Save and Restore the system state.

    short old_ascii_nt;         // toggle output of ascii ntuples

    short old_config_frequency;      // These control the 
    short old_soft_reset_frequency;  // repetition rate of
    short old_bc_reset_frequency;    // interspersed resets

    short old_do_action_DCS;    // Read currents, temperatures and voltages at EOB
    short old_do_autostop;      // End scans automatically after the intersting bit
    short old_do_cal_loop;      // Enable loop over the four cal lines
    short old_do_fits;          // Toggle automatic fitting of scans
    short old_do_mask_test;     // Alternating mask patterns on consecuitve events
    short old_do_preburst;      // Detect and suppress dead links automatically at SOB
    short old_do_vary_ntrigs;   // Vary requested number of triggers according to occupancy
    short old_do_fill_evtree;   // Fill event tree

    short old_burst_type;       // burst type
    short old_burst_trtype;     // trigger type
    int   old_burst_ntrigs;     // number of triggers per burst (per cal line)
    int   old_burst_maxtrigs;   // maximum number of triggers (when do_vary_ntrigs==1)
    short old_burst_target;     // target no. hits/channel during NO scans

    short old_paf;              // Programmable Almost Full - see MuSTARD documentation
    short old_trlim;            // Trigger Limit - see MuSTARD documentation

    short old_L1A_per_loop;     // Number of L1A per trigger loop, usually 1
    short old_throw_away;       // If L1A_per_loop>1 we may want to throw some away

    short old_rate_limit_delay; // Delay in mS, used during laser scans (!!)
    long  old_seu_duration;     // duration of SEUTest with Software Timer (mS)

    int   old_debug_level;      // Diagnostic level


    //


    short  seu_invert;           // 0: look for SEU 0->1; 1: look for SEU 1->0

    short seu_wc[SEU_NEVENTS][SEU_NLINKS];   // SEU word count [event][link]
    short seu_l1[SEU_NEVENTS][SEU_NLINKS];   // SEU L1A count
    short seu_bc[SEU_NEVENTS][SEU_NLINKS];   // SEU BC count
    short seu_data[SEU_NEVENTS][SEU_NLINKS][SEU_BUFSIZE];  // SEU event data

    int   seu_flip[SEU_NEVENTS][SEU_NLINKS][6]; // SEUtest tally of bitflips [event][soft_link][chip]
    int   seu_l1err[SEU_NEVENTS][SEU_NLINKS];   // SEUtest tally of l1 errors [event][hard_link]
    int   seu_wcerr[SEU_NEVENTS][SEU_NLINKS];   // SEUtest tally of wc errors [event][hard_link]

    int   seu_conf[SEU_NLINKS][6];    // SEUtest tally of config bitflips [soft_link][chip]
    int   seu_mask[SEU_NLINKS][6];    // SEUtest tally of mask bitflips
    int   seu_pipe[SEU_NLINKS][6];    // SEUtest tally of pipeline or accumulator bitflips
    int   seu_mystery[SEU_NLINKS][6]; // SEUtest indication of unresolved errors

    int   seu_tot_l1err[4][SEU_NEVENTS][SEU_NLINKS];    // SEUburst tally of l1 errors
    int   seu_tot_wcerr[4][SEU_NEVENTS][SEU_NLINKS];    // SEUburst tally of wc errors

    int   seu_tot_conf[4][SEU_NLINKS][6];    // SEUburst tally of flips in the config register
    int   seu_tot_pipe[4][SEU_NLINKS][6];    // SEUburst tally of flips in the pipeline or accumulator register
    int   seu_tot_mask[4][SEU_NLINKS][6];    // SEUburst tally of flips in the mask register
    int   seu_tot_mystery[4][SEU_NLINKS][6]; // SEUburst tally of unresolved errors

    float version;
    char dedication[80];

    short burst_max_timeouts;

    short evdata[MUS_NUMBER][0x8000];  // Array to store single event from MUS_NUMBER MuSTARDs
                                       // event buffer is size short * 0x8000 
                                       // ### Reserve this space once only?

    int buf[ST_BUF_LENGTH];   // Long buffer for recording binary data
    int bufpnt;               // Offset to next entry in buf
    int bufevt;               // Private event counter to tag recorded events
                              // ### We don't otherwise have a unique event
                              // counter incremented individually - suggestions ?

    int last_BPT;
    int last_SPT;

    ClassDef(TST,1)
};

#endif

