//
// fit functions
//
// PWP 11.10.99 - added expo and "grillo" functions
// PWP 11.09.00 - protection against divide by zero errors
//                to erf, erfc and grillo functions
//


#include "TObject.h"
#include "TMath.h"

double erf(double *x, double *par){
  double xx, y;

  if(par[2] == 0){
    xx = 0;
  }else{
    xx = ( (par[1]-x[0]) / par[2]) / 1.414213562;
  }

  y  = (par[0]/2) * ( 1 - TMath::Erf(xx) );

  return y;
}

double erfc(double *x, double *par){
  double xx, y;

  if(par[2] == 0){
    xx = 0;
  }else{
    xx = ( (par[1]-x[0]) / par[2]) / 1.414213562;
  }

  y  = (par[0]/2) * ( 1 + TMath::Erf(xx) );

  return y;
}

double st_exp(double *x, double *par){

  return par[2] + par[0]/(1 + TMath::Exp(-x[0]/par[1]));
}

double gaus(double *x, double *par){

  return par[0] * TMath::Gaus(x[0],par[1],par[2]); 
}

double grillo(double *x, double *par){

  double xx;

  if((par[1]*x[0]) == 0){
    xx = 0;
  }else{
    xx = par[2]/(par[1]*x[0]);
  }

  return par[0] + par[2]/TMath::Sqrt(1+(xx*xx)); 
}

double pol2(double *x, double *par){

  return par[0] + x[0]*(par[1]+(par[2]*x[0]));
}

double tophat(double *x, double *par){

  if (x[0]< (par[1]-par[2])) return 0;
  if (x[0]> (par[1]+par[2])) return 0;
  return par[0];
}
