Double_t erf(Double_t *x, Double_t *par){
  Double_t xx, y;

  xx = ( (par[1]-x[0]) / par[2]) / 1.414213562; 
  y  = (par[0]/2) * ( 1 - TMath::Erf(xx) );

  return y;
}

Double_t erfc(Double_t *x, Double_t *par){
  Double_t xx, y;

  xx = ( (par[1]-x[0]) / par[2]) / 1.414213562; 
  y  = (par[0]/2) * ( 1 + TMath::Erf(xx) );

  return y;
}

Double_t gaus(Double_t *x, Double_t *par){

  return par[0] * TMath::Gaus(x[0],par[1],par[2]); 
}

Double_t tophat(Double_t *x, Double_t *par){

  if (x[0]< (par[1]-par[2])) return 0;
  if (x[0]> (par[1]+par[2])) return 0;
  return par[0];
}


int find_50(TH1* h_in){

  int bin,i;
  float trip_level = h_in->GetMaximum()/2;

  if(h_in->GetBinContent(0)>h_in->GetBinContent(h_in->GetNbinsX())){
    for(i=0; i<h_in->GetNbinsX(); i++){
      if(h_in->GetBinContent(i)>trip_level) bin = i;
    }
  }else{
    for(i=0; i<h_in->GetNbinsX(); i++){
      if(h_in->GetBinContent(i)<trip_level) bin = i;
    }
  }
  return bin;

}

void st_fit2_h8(int moi, int loi, int run, int scan){

// "Improved" routine to fit st data using FitSlicesY:
// Spaghetti code served in a lovely ROOT based sauce
// by P.W.Phillips, RAL, 15.07.99 revised 20.07.99
// version for ASCII ntuples 29.07.99

  char  f_name[64], h_name[32], h_title[32], scan_name[32];
  char  m_title[32],s_title[32],out_fname[128], s[256];
  int   bin,i,j,k,n,x,y;
  short scan_type;
  float data[3], err2, z;

// Options which change macro output
  int draw_fit_projections = 1;
  int fit_file = 1;
  int postscript = 1;
  int summary_file = 1;

// Global style variations

  gStyle.SetPaperSize(20,26); // I find that the postscript output
                              // is mangled without this line - 
                              // although this size (A4) should in
                              // principle be the default...
  gStyle.SetMarkerStyle(21);
  gStyle.SetMarkerSize(0.3);
  gStyle.SetOptFit(0);
  gStyle.SetOptStat(10);
  gStyle.SetTitleW(0.6);

// Open file, or obtain pointer to histogram in memory

  printf("st_fit2 called for module %d link %d\n",moi,loi);
  
    st_readnt(moi,loi,run,scan);

    n = 1000;
    sprintf(scan_name,"Unknown");
    TString title = h_scan->GetTitle();
    title.ToUpper();
    if (title.Contains("THRESHOLD")){
      scan_type = 1;
      sprintf(scan_name,"THRESHOLD (mV)");
    }
    if (title.Contains("VCAL")){
      scan_type = 2;
      sprintf(scan_name,"VCAL (mV)");
    }
    if (title.Contains("STROBE")){
      scan_type = 3;
      sprintf(scan_name,"Strobe Delay");
    }
    if (title.Contains("PREAMP")){
      scan_type = 4;
      sprintf(scan_name,"Preamp Current (uA)");
    }
    if (title.Contains("SHAPER")){
      scan_type = 5;
      sprintf(scan_name,"Shaper Current (uA)");
    }

// titles for graphs - might as well do this now.

  sprintf(m_title,"Fitted Mean(%s)",scan_name);
  sprintf(s_title,"Fitted Sigma(%s)",scan_name);

  TPaveText *t1 = new TPaveText(0.0,0.0,1.0,1.0);
  t1->SetFillColor(10);
  t1->SetTextAlign(13);
  t1->SetTextSize(0.3);
  t1->SetTextColor(4);
  t1->AddText("ATLAS SCT Module Test");
  sprintf(s,"Run %d   Scan %d   Module %d   Stream %d   Scan type: %s", run, scan, moi, loi, scan_name);
  t1->AddText(s);

// Choose appropriate fit function

  switch (scan_type){
    case 1: // ST_VTHR
      printf("ST_VTHR\n");
      TF1 *f1 = new TF1 ("f1",erfc,  0, 1000, 3);
      f1->SetParNames("Constant","Mean","Sigma");
      break;
    case 2: // ST_VCAL
      printf("ST_VCAL\n");
      TF1 *f1 = new TF1 ("f1", erf,  0, 1000, 3);
      f1->SetParNames("Constant","Mean","Sigma");
      break;
//    case 3: // ST_STROBE_DELAY
//      printf("ST_Strobe\n");
//      TF1 *f1 = new TF1 ("f1", tophat,  0, 1000, 3);
//      f1->SetParNames("Constant","Mean","Sigma");
//      break;
    default: // use gaussian for now 
      TF1 *f1 = new TF1 ("f1", gaus,  0, 1000, 3);
      f1->SetParNames("Constant","Mean","Sigma");
      break;
  }

// Calculate errors appropriate to binomial statistics 

  for(x=0;x<h_scan->GetNbinsX();x++){
    for(y=0;y<h_scan->GetNbinsY();y++){
      z = h_scan->GetCellContent(x,y);
      err2 = (z+1) * (1+n-z) / n;
      h_scan->SetCellError(x,y,sqrt(err2));
    }
  }

// Before we draw the main canvas, open the postscript file
// if the relevant option has been selected 

  if(postscript){
    sprintf(out_fname,"C:\\sctdaq\\data\\st_fit%d_%d_%d_%d.ps",run,scan,moi,loi);
    TPostScript *ps = new TPostScript(out_fname,112);
    ps->NewPage();
  }

// Open and draw the first canvas

  TCanvas* c1=new TCanvas ("c1","ST_fitslices_0",1100,750);
  pad_t = new TPad("pad_t","Text",         0.01,0.91,0.99,0.99,33);
  pad_m = new TPad("pad_m","Fitted Means", 0.01,0.61,0.60,0.90,33);
  pad_s = new TPad("pad_s","Fitted Sigmas",0.01,0.31,0.60,0.60,33);
  pad_c = new TPad("pad_c","Fitted Chisq", 0.01,0.01,0.60,0.30,33);
  pad_p = new TPad("pad_p","Projections",  0.61,0.01,0.99,0.90,33);

  pad_t->Draw();
  pad_m->Draw();
  pad_s->Draw();
  pad_c->Draw();
  pad_p->Draw();
  pad_p->Divide(2,3);

  pad_m->cd();
  h_scan->Draw("COLZ");

  c1->Modify();
  c1->Update();

// In case the macro has been run previously, we must delete
// pre-existing histograms h_dp0 - h_dp1

  TH1F* h_ptr = (TH1F*) gDirectory->GetList()->FindObject("h_dp0");
  if(h_ptr !=0){
    delete h_dp0;
    delete h_dp1;
    delete h_dp2;
    delete h_dp3;
    delete h_dp4;
    delete h_dp5;
  }


// projections to give the initial values for use with FitSlicesY

  h_scan->ProjectionY("h_dp0",  1,128);
  h_scan->ProjectionY("h_dp1",129,256);
  h_scan->ProjectionY("h_dp2",257,384);
  h_scan->ProjectionY("h_dp3",385,512);
  h_scan->ProjectionY("h_dp4",513,640);
  h_scan->ProjectionY("h_dp5",641,768);

  TH1* h_dp[6];
  float i_mean[6];
  float i_sigma[6];
  h_dp[0] = h_dp0;
  h_dp[1] = h_dp1;
  h_dp[2] = h_dp2;
  h_dp[3] = h_dp3;
  h_dp[4] = h_dp4;
  h_dp[5] = h_dp5;

  int n_bins = h_dp[0]->GetNbinsX();
  float scan_lo = h_dp[0]->GetBinCenter(1);
  float scan_hi = h_dp[0]->GetBinCenter(n_bins);
  float half_range = (scan_hi - scan_lo)/4;

// Calculate errors appropriate to binomial statistics 

  for(i=0; i<6; i++){
    int dp_max = h_dp[i]->GetMaximum();
    if (dp_max==0) dp_max = 1;          // (necessary) protection against divide by zero
    for(x=1;x<=h_dp[i]->GetNbinsX();x++){
      z = h_dp[i]->GetBinContent(x);
      err2 = (z+1) * (1+dp_max-z) / dp_max;
      h_dp[i]->SetBinError(x,sqrt(err2));
    }
  }

// Fit the projections - unless the histograms are empty!!!

  for(i=0; i<6; i++){
    sprintf(h_title,"Scan Projection for chip %d",i);
    pad_p->cd(i+1);
    dp_max = h_dp[i]->GetMaximum();
    if(dp_max>0.0){
      f1->SetParameters(dp_max,find_50(h_dp[i]),half_range);
      f1->SetParLimits(0,dp_max,dp_max);
      h_dp[i]->Fit("f1");
//      if (f1->GetChisquare()>10000){
//        printf("\nDid not converge for chip %d - trying equal weights\n",i); 
//        f1->SetParameters(dp_max,find_50(h_dp[i]),half_range);
//        f1->SetParLimits(0,dp_max,dp_max);
//        h_dp[i]->Fit("f1","W");
//      }
      if (f1->GetChisquare()>10000){
        printf("\nStill did not converge for chip %d - trying log likelihood\n",i); 
        f1->SetParameters(dp_max,find_50(h_dp[i]),half_range);
        f1->SetParLimits(0,dp_max,dp_max);
        h_dp[i]->Fit("f1","L");
      }
//      if (f1->GetChisquare()>10000){
//        printf("\nLast try: log likelihood with equal weights\n",i); 
//        f1->SetParameters(dp_max,find_50(h_dp[i]),half_range);
//        f1->SetParLimits(0,dp_max,dp_max);
//        h_dp[i]->Fit("f1","WL");
//      }
    }
    h_dp[i]->SetTitle(h_title);
    h_dp[i]->SetXTitle(scan_name);
    h_dp[i]->SetLineColor(kRed);
    h_dp[i]->SetMarkerColor(kBlack);
    h_dp[i]->Draw("e1p");
    i_mean[i]  = f1->GetParameter(1); 
    i_sigma[i] = f1->GetParameter(2);
  }

  c1->Modify();
  c1->Update();

// Create histograms to hold the (soon to be) fitted scan data.  
// Since we call FitslicesY seperately for each chip, the histograms
// which it creates are overwritten at each call - so we have to
// add the histograms together... 

  TH1S * h_code  = new TH1S("h_code", "Fit Code", 768,0,768);
  TH1F * h_mean  = new TH1F("h_mean", "Fitted Means", 768,0,768);
  TH1F * h_sigma = new TH1F("h_sigma","Fitted Sigma", 768,0,768);
  TH1F * h_chisq = new TH1F("h_chisq", "Fitted Chisq", 768,0,768);

// Fitslices for each chip using initial values from the above projections

  for(i=0; i<6; i++){
    int start = 1+(i*128);
    int stop  = (i+1) * 128;

    printf("  Fitting data for chip %d start %d stop %d\n",i, start, stop);
    f1->SetParameters(n, i_mean[i], i_sigma[i]/2);
    f1->SetParLimits(0,n,n);
    h_scan->FitSlicesY(f1,start,stop,5);

    if(loi==0){
      TH1F* h_scan_1 = (TH1F*) gDirectory->GetList()->FindObject("h_scan0_1");
      TH1F* h_scan_2 = (TH1F*) gDirectory->GetList()->FindObject("h_scan0_2");
      TH1F* h_scan_chi2 = (TH1F*) gDirectory->GetList()->FindObject("h_scan0_chi2");
    }else{
      TH1F* h_scan_1 = (TH1F*) gDirectory->GetList()->FindObject("h_scan1_1");
      TH1F* h_scan_2 = (TH1F*) gDirectory->GetList()->FindObject("h_scan1_2");
      TH1F* h_scan_chi2 = (TH1F*) gDirectory->GetList()->FindObject("h_scan1_chi2");
    }
    h_mean->Add(h_scan_1, 1);
    h_sigma->Add(h_scan_2, 1);
    h_chisq->Add(h_scan_chi2, 1);

  //  NB with root v2.22 for NT, deleting the histograms produced by
  //  FitSlicesY() causes  ROOT to crash...

  }

// Draw the fit parameters using the pads of canvas 1

  pad_m->cd();
  h_mean->Draw("samep");
  pad_s->cd();
  pad_s->SetLogy();
  h_sigma->SetXTitle("Channel Number");
  h_sigma->SetYTitle(s_title);
  h_sigma->SetLineColor(kRed);
  h_sigma->SetMarkerColor(kBlack);
  h_sigma->Draw("ep");
  pad_c->cd();
  pad_c->SetLogy();
  h_chisq->SetXTitle("Channel Number");
  h_chisq->SetYTitle("Chisq");
  h_chisq->SetLineColor(kRed);
  h_chisq->SetMarkerColor(kBlack);
  h_chisq->Draw("ep");

  pad_t->cd();
  t1->Draw();
  c1->Modify();
  c1->Update();

//  Create 18 histograms for projections of Mean, Sigma and Chisq
//   0 to  5  - all channels
//   6 to 11  - odd channels
//  12 to 17  - even channels
//
//  Unfortunately, CINT does not like (for example)
//    TH1F* h_mp[i] = new TH1F...
//  so we cannot initialise the histograms using a loop :-(

  float sigma_lo = 0.5;
  float sigma_hi = (scan_hi-scan_lo)/5;

  TH1F* h_mp[18];
  TH1F* h_sp[18];
  TH1F* h_cp[18];

  TH1F* h_mp[0]  =  new TH1F("h_mp0", "Mean, all channels, chip 0",  50,scan_lo,scan_hi);
  TH1F* h_mp[1]  =  new TH1F("h_mp1", "Mean, all channels, chip 1",  50,scan_lo,scan_hi);
  TH1F* h_mp[2]  =  new TH1F("h_mp2", "Mean, all channels, chip 2",  50,scan_lo,scan_hi);
  TH1F* h_mp[3]  =  new TH1F("h_mp3", "Mean, all channels, chip 3",  50,scan_lo,scan_hi);
  TH1F* h_mp[4]  =  new TH1F("h_mp4", "Mean, all channels, chip 4",  50,scan_lo,scan_hi);
  TH1F* h_mp[5]  =  new TH1F("h_mp5", "Mean, all channels, chip 5",  50,scan_lo,scan_hi);
  TH1F* h_mp[6]  =  new TH1F("h_mp6", "Mean, odd channels, chip 0",  50,scan_lo,scan_hi);
  TH1F* h_mp[7]  =  new TH1F("h_mp7", "Mean, odd channels, chip 1",  50,scan_lo,scan_hi);
  TH1F* h_mp[8]  =  new TH1F("h_mp8", "Mean, odd channels, chip 2",  50,scan_lo,scan_hi);
  TH1F* h_mp[9]  =  new TH1F("h_mp9", "Mean, odd channels, chip 3",  50,scan_lo,scan_hi);
  TH1F* h_mp[10] =  new TH1F("h_mp10","Mean, odd channels, chip 4",  50,scan_lo,scan_hi);
  TH1F* h_mp[11] =  new TH1F("h_mp11","Mean, odd channels, chip 5",  50,scan_lo,scan_hi);
  TH1F* h_mp[12] =  new TH1F("h_mp12","Mean, even channels, chip 0", 50,scan_lo,scan_hi);
  TH1F* h_mp[13] =  new TH1F("h_mp13","Mean, even channels, chip 1", 50,scan_lo,scan_hi);
  TH1F* h_mp[14] =  new TH1F("h_mp14","Mean, even channels, chip 2", 50,scan_lo,scan_hi);
  TH1F* h_mp[15] =  new TH1F("h_mp15","Mean, even channels, chip 3", 50,scan_lo,scan_hi);
  TH1F* h_mp[16] =  new TH1F("h_mp16","Mean, even channels, chip 4", 50,scan_lo,scan_hi);
  TH1F* h_mp[17] =  new TH1F("h_mp17","Mean, even channels, chip 5", 50,scan_lo,scan_hi);

  TH1F* h_sp[0]  =  new TH1F("h_sp0", "Sigma, all channels, chip 0",  50,sigma_lo,sigma_hi);
  TH1F* h_sp[1]  =  new TH1F("h_sp1", "Sigma, all channels, chip 1",  50,sigma_lo,sigma_hi);
  TH1F* h_sp[2]  =  new TH1F("h_sp2", "Sigma, all channels, chip 2",  50,sigma_lo,sigma_hi);
  TH1F* h_sp[3]  =  new TH1F("h_sp3", "Sigma, all channels, chip 3",  50,sigma_lo,sigma_hi);
  TH1F* h_sp[4]  =  new TH1F("h_sp4", "Sigma, all channels, chip 4",  50,sigma_lo,sigma_hi);
  TH1F* h_sp[5]  =  new TH1F("h_sp5", "Sigma, all channels, chip 5",  50,sigma_lo,sigma_hi);
  TH1F* h_sp[6]  =  new TH1F("h_sp6", "Sigma, odd channels, chip 0",  50,sigma_lo,sigma_hi);
  TH1F* h_sp[7]  =  new TH1F("h_sp7", "Sigma, odd channels, chip 1",  50,sigma_lo,sigma_hi);
  TH1F* h_sp[8]  =  new TH1F("h_sp8", "Sigma, odd channels, chip 2",  50,sigma_lo,sigma_hi);
  TH1F* h_sp[9]  =  new TH1F("h_sp9", "Sigma, odd channels, chip 3",  50,sigma_lo,sigma_hi);
  TH1F* h_sp[10] =  new TH1F("h_sp10","Sigma, odd channels, chip 4",  50,sigma_lo,sigma_hi);
  TH1F* h_sp[11] =  new TH1F("h_sp11","Sigma, odd channels, chip 5",  50,sigma_lo,sigma_hi);
  TH1F* h_sp[12] =  new TH1F("h_sp12","Sigma, even channels, chip 0", 50,sigma_lo,sigma_hi);
  TH1F* h_sp[13] =  new TH1F("h_sp13","Sigma, even channels, chip 1", 50,sigma_lo,sigma_hi);
  TH1F* h_sp[14] =  new TH1F("h_sp14","Sigma, even channels, chip 2", 50,sigma_lo,sigma_hi);
  TH1F* h_sp[15] =  new TH1F("h_sp15","Sigma, even channels, chip 3", 50,sigma_lo,sigma_hi);
  TH1F* h_sp[16] =  new TH1F("h_sp16","Sigma, even channels, chip 4", 50,sigma_lo,sigma_hi);
  TH1F* h_sp[17] =  new TH1F("h_sp17","Sigma, even channels, chip 5", 50,sigma_lo,sigma_hi);

  TH1F* h_cp[0]  =  new TH1F("h_cp0", "Chisq, all channels, chip 0",  50,0,10);
  TH1F* h_cp[1]  =  new TH1F("h_cp1", "Chisq, all channels, chip 1",  50,0,10);
  TH1F* h_cp[2]  =  new TH1F("h_cp2", "Chisq, all channels, chip 2",  50,0,10);
  TH1F* h_cp[3]  =  new TH1F("h_cp3", "Chisq, all channels, chip 3",  50,0,10);
  TH1F* h_cp[4]  =  new TH1F("h_cp4", "Chisq, all channels, chip 4",  50,0,10);
  TH1F* h_cp[5]  =  new TH1F("h_cp5", "Chisq, all channels, chip 5",  50,0,10);
  TH1F* h_cp[6]  =  new TH1F("h_cp6", "Chisq, odd channels, chip 0",  50,0,10);
  TH1F* h_cp[7]  =  new TH1F("h_cp7", "Chisq, odd channels, chip 1",  50,0,10);
  TH1F* h_cp[8]  =  new TH1F("h_cp8", "Chisq, odd channels, chip 2",  50,0,10);
  TH1F* h_cp[9]  =  new TH1F("h_cp9", "Chisq, odd channels, chip 3",  50,0,10);
  TH1F* h_cp[10] =  new TH1F("h_cp10","Chisq, odd channels, chip 4",  50,0,10);
  TH1F* h_cp[11] =  new TH1F("h_cp11","Chisq, odd channels, chip 5",  50,0,10);
  TH1F* h_cp[12] =  new TH1F("h_cp12","Chisq, even channels, chip 0", 50,0,10);
  TH1F* h_cp[13] =  new TH1F("h_cp13","Chisq, even channels, chip 1", 50,0,10);
  TH1F* h_cp[14] =  new TH1F("h_cp14","Chisq, even channels, chip 2", 50,0,10);
  TH1F* h_cp[15] =  new TH1F("h_cp15","Chisq, even channels, chip 3", 50,0,10);
  TH1F* h_cp[16] =  new TH1F("h_cp16","Chisq, even channels, chip 4", 50,0,10);
  TH1F* h_cp[17] =  new TH1F("h_cp17","Chisq, even channels, chip 5", 50,0,10);

// Phew!!
// Now fill these histograms...

  for (i=0;i<6;i++){
    for(j=1;j<129;j++){
      bin = j + (128*i);
      float mean  = (float)h_mean->GetBinContent(bin);
      float sigma = (float)h_sigma->GetBinContent(bin);
      float chisq = (float)h_chisq->GetBinContent(bin);
      h_mp[i]->Fill(mean);
      h_sp[i]->Fill(sigma);
      h_cp[i]->Fill(chisq); 
      if(j%2){ // odd channel
        h_mp[i+6]->Fill(mean);
        h_sp[i+6]->Fill(sigma);
        h_cp[i+6]->Fill(chisq); 
      }else{
        h_mp[i+12]->Fill(mean);
        h_sp[i+12]->Fill(sigma);
        h_cp[i+12]->Fill(chisq); 
      }
    }
  }

// ...and fit gaussians to them, storing the results in the arrays

  float chip_fit_par[3][18];
  float chip_fit_err[3][18];

  for(i=0;i<18;i++){
    if(h_mp[i]->GetMaximum()>0.0){
      h_mp[i]->Fit("gaus","0");
      f2=h_mp[i]->GetFunction("gaus");
      chip_fit_par[0][i]=f2->GetParameter(1);
      chip_fit_err[0][i]=f2->GetParameter(2);
    } else {
      chip_fit_par[0][i]=-1;
      chip_fit_err[0][i]=-1;
    }
    if(h_sp[i]->GetMaximum()>0.0){
      h_sp[i]->Fit("gaus","0"); 
      f2=h_sp[i]->GetFunction("gaus");
      chip_fit_par[1][i]=f2->GetParameter(1);
      chip_fit_err[1][i]=f2->GetParameter(2);
    } else {
      chip_fit_par[1][i]=-1;
      chip_fit_err[1][i]=-1;
    }
    if(h_cp[i]->GetMaximum()>0.0){
      h_cp[i]->Fit("expo","0"); 
      f2=h_cp[i]->GetFunction("expo");
      chip_fit_par[2][i]=f2->GetParameter(1);
      chip_fit_err[2][i]=f2->GetParameter(2);
    } else {
      chip_fit_par[2][i]=-1;
      chip_fit_err[2][i]=-1;
    }
  }

// Optionally, three canvases are created to show the projections
// of fit data on a chip by chip basis.  This may be prevented by
// setting draw_fit_projections = 0.

  if(draw_fit_projections){

  // Open and draw new canvas to show the Mean Projections

    if(postscript) ps->NewPage();
    TCanvas* c2=new TCanvas ("c2","ST_fit_mean",1100,750);
    pad_mpt = new TPad("pad_mpt", "Text",                        0.01,0.91,0.99,0.99,33);
    pad_mp0 = new TPad("pad_mp0","Projection of Mean values",   0.01,0.61,0.99,0.90,33);
    pad_mp1 = new TPad("pad_mp1","Projection of Mean values",   0.01,0.31,0.99,0.60,33);
    pad_mp2 = new TPad("pad_mp2","Projection of Mean values",   0.01,0.01,0.99,0.30,33);

    pad_mpt->Draw();
    pad_mp0->Draw();
    pad_mp1->Draw();
    pad_mp2->Draw();
    pad_mp0->Divide(6,1);
    pad_mp1->Divide(6,1);
    pad_mp2->Divide(6,1);
    c2->Modify();
    c2->Update();

    for(i=0; i<6; i++){
      pad_mp0->cd(i+1);
      h_mp[i]->SetXTitle(m_title);
      h_mp[i]->SetYTitle("N(Mean)");
      h_mp[i]->SetLineColor(kRed);
      h_mp[i]->SetMarkerColor(kBlack);
      h_mp[i]->Draw("ep");
    }

    for(i=0; i<6; i++){
      pad_mp1->cd(i+1);
      h_mp[i+6]->SetXTitle(m_title);
      h_mp[i+6]->SetYTitle("N(Mean)");
      h_mp[i+6]->SetLineColor(kRed);
      h_mp[i+6]->SetMarkerColor(kBlack);
      h_mp[i+6]->Draw("ep");
    }

    for(i=0; i<6; i++){
      pad_mp2->cd(i+1);
      h_mp[i+12]->SetXTitle(m_title);
      h_mp[i+12]->SetYTitle("N(Mean)");
      h_mp[i+12]->SetLineColor(kRed);
      h_mp[i+12]->SetMarkerColor(kBlack);
      h_mp[i+12]->Draw("ep");
    }

    pad_mpt->cd();
    t1->Draw();
    c2->Modify();
    c2->Update();

  // Open and draw new canvas to show the Sigma Projections

    if(postscript) ps->NewPage();
    TCanvas* c3=new TCanvas ("c3","ST_fit_sigma",1100,750);
    pad_spt = new TPad("pad_spt", "Text",                        0.01,0.91,0.99,0.99,33);
    pad_sp0 = new TPad("pad_sp0","Projection of Sigma values",   0.01,0.61,0.99,0.90,33);
    pad_sp1 = new TPad("pad_sp1","Projection of Sigma values",   0.01,0.31,0.99,0.60,33);
    pad_sp2 = new TPad("pad_sp2","Projection of Sigma values",   0.01,0.01,0.99,0.30,33);

    pad_spt->Draw();
    pad_sp0->Draw();
    pad_sp1->Draw();
    pad_sp2->Draw();
    pad_sp0->Divide(6,1);
    pad_sp1->Divide(6,1);
    pad_sp2->Divide(6,1);
    c3->Modify();
    c3->Update();

    for(i=0; i<6; i++){
      pad_sp0->cd(i+1);
      h_sp[i]->SetXTitle(s_title);
      h_sp[i]->SetYTitle("N(Sigma)");
      h_sp[i]->SetLineColor(kRed);
      h_sp[i]->SetMarkerColor(kBlack);
      h_sp[i]->Draw("ep");
    }

    for(i=0; i<6; i++){
      pad_sp1->cd(i+1);
      h_sp[i+6]->SetXTitle(s_title);
      h_sp[i+6]->SetYTitle("N(Sigma)");
      h_sp[i+6]->SetLineColor(kRed);
      h_sp[i+6]->SetMarkerColor(kBlack);
      h_sp[i+6]->Draw("ep");
    }

    for(i=0; i<6; i++){
      pad_sp2->cd(i+1);
      h_sp[i+12]->SetXTitle(s_title);
      h_sp[i+12]->SetYTitle("N(Sigma)");
      h_sp[i+12]->SetLineColor(kRed);
      h_sp[i+12]->SetMarkerColor(kBlack);
      h_sp[i+12]->Draw("ep");
    }

    pad_spt->cd();
    t1->Draw();
    c3->Modify();
    c3->Update();

  // Open and draw new canvas to show the chisq Projections

    if(postscript) ps->NewPage();
    TCanvas* c4=new TCanvas ("c4","ST_fit_chisq",1100,750);
    pad_cpt = new TPad("pad_cpt", "Text",                        0.01,0.91,0.99,0.99,33);
    pad_cp0 = new TPad("pad_cp0","Projection of Chisq values",   0.01,0.61,0.99,0.90,33);
    pad_cp1 = new TPad("pad_cp1","Projection of Chisq values",   0.01,0.31,0.99,0.60,33);
    pad_cp2 = new TPad("pad_cp2","Projection of Chisq values",   0.01,0.01,0.99,0.30,33);

    pad_cpt->Draw();
    pad_cp0->Draw();
    pad_cp1->Draw();
    pad_cp2->Draw();
    pad_cp0->Divide(6,1);
    pad_cp1->Divide(6,1);
    pad_cp2->Divide(6,1);
    c4->Modify();
    c4->Update();

    for(i=0; i<6; i++){
      pad_cp0->cd(i+1);
      h_cp[i]->SetXTitle("Chisq");
      h_cp[i]->SetYTitle("N(Chisq)");
      h_cp[i]->SetLineColor(kRed);
      h_cp[i]->SetMarkerColor(kBlack);
      h_cp[i]->Draw("ep");
    }

    for(i=0; i<6; i++){
      pad_cp1->cd(i+1);
      h_cp[i+6]->SetXTitle("Chisq");
      h_cp[i+6]->SetYTitle("N(Chisq)");
      h_cp[i+6]->SetLineColor(kRed);
      h_cp[i+6]->SetMarkerColor(kBlack);
      h_cp[i+6]->Draw("ep");
    }

    for(i=0; i<6; i++){
      pad_cp2->cd(i+1);
      h_cp[i+12]->SetXTitle("Chisq");
      h_cp[i+12]->SetYTitle("N(Chisq)");
      h_cp[i+12]->SetLineColor(kRed);
      h_cp[i+12]->SetMarkerColor(kBlack);
      h_cp[i+12]->Draw("ep");
    }

    pad_cpt->cd();
    t1->Draw();
    c4->Modify();
    c4->Update();

  }  // end if(draw_fit_projections)

  if(postscript) {
    ps->Close();
    delete ps;
  }

// Loop over histograms, applying cuts to determine the number of bad channels.  
// Output fit results as a text file if option "fit_file" selected.

  if(fit_file){
    sprintf(f_name,"C:\\sctdaq\\data\\stfit_%d_%d_%d_%d.fit",run,scan,moi,loi);
    FILE* fitfile = fopen(f_name,"w");
    fprintf(fitfile,"ch code mean err sigma err chi err\n");
  }

  float fdata[6];
  float cuts [2][6];
  short chip_bad_ch[7][7];
  short code;

  cuts[0][0] = scan_lo;    // scan_lo < mean < scan_hi
  cuts[1][0] = scan_hi;    
  cuts[0][1] = 0.0;        // 0.0 < m_err < half_range
  cuts[1][1] = half_range;
  cuts[0][4] = 0;          // 0.0 < chisq < 3.0  
  cuts[1][4] = 3;
  cuts[0][5] = 0;          // 0.0 < c_err < 3.0
  cuts[1][5] = 3;


  switch(scan_type){
    case 1: // ST_VTHR
      cuts[0][2] =  8.0;        // 0.5 < sigma < half range 
      cuts[1][2] = 100.0;
      cuts[0][3] =  0.0;        // 0.0 < s_err < 10.0
      cuts[1][3] = 10.0;
      break;
    case 2: // ST_VCAL
      cuts[0][2] =  0.5;        // 0.5 < sigma < half range 
      cuts[1][2] = half_range;
      cuts[0][3] =  0.0;        // 0.0 < s_err < 1
      cuts[1][3] =  1.0;
      break;
    case 3: // ST_STROBE
      cuts[0][2] =  5.0;        // 1.0 < sigma < half range 
      cuts[1][2] = 20.0;
      cuts[0][3] =  0.0;        // 0.0 < s_err < 1
      cuts[1][3] =  1.0;
      // recall I am (temporarily) fitting a gaussian,
      // hence chisq cuts are not fair!!
      cuts[0][4] = 0;          // 0.0 < chisq < 3.0  
      cuts[1][4] = 100000;
      cuts[0][5] = 0;          // 0.0 < c_err < 3.0
      cuts[1][5] = 10000;
      break;
    default: //
      cuts[0][2] =  1.0;        // 1.0 < sigma < half range 
      cuts[1][2] = half_range;
      cuts[0][3] =  0.0;        // 0.0 < s_err < 1
      cuts[1][3] =  1.0;
      break;
  }


  for(i=0;i<6;i++){
    for(k=0;k<7;k++) {     // clear bad channel count
      chip_bad_ch[i][k]=0;
    } 
    for(j=1;j<129;j++){
      bin = j + (128*i);
      fdata[0]  = (float)h_mean->GetBinContent(bin);
      fdata[1]  = (float)h_mean->GetBinError(bin);
      fdata[2]  = (float)h_sigma->GetBinContent(bin);
      fdata[3]  = (float)h_sigma->GetBinError(bin);
      fdata[4]  = (float)h_chisq->GetBinContent(bin);
      fdata[5]  = (float)h_chisq->GetBinError(bin);

      code = 0;
      for (k=0;k<6;k++){
        if (fdata[k]<cuts[0][k]) code +=(1<<(2*k));
        if (fdata[k]>cuts[1][k]) code +=(2<<(2*k));
        if((fdata[k]<cuts[0][k])||(fdata[k]>cuts[1][k])) chip_bad_ch[i][k]++;
      }
      if(code){
        h_code->SetBinContent(bin,code);
        chip_bad_ch[i][6]++;
      }
      if(fit_file){
        fprintf(fitfile,"%3d 0x%4x %3.2f %3.2f %3.2f %3.2f %3.2f %3.2f\n",
          bin, code, fdata[0], fdata[1], fdata[2], fdata[3], fdata[4], fdata[5]); 
      }
    }
  }
  if(fit_file) fclose(fitfile);

  for(i=0;i<7;i++){ //loop on par cut, then chip
    chip_bad_ch[6][i]=0;
    for(j=0;j<6;j++) chip_bad_ch[6][i] += chip_bad_ch[j][i];
  }

// Summary data to screen

//  Print Summary Data (1)
//  Fitted mean and associated error for all, odd and even channels

  printf("\nChip\tMean\t\t\tm_err\n");
  printf(  "\t(all)\t(odd)\t(even)\t(all)\t(odd)\t(even)\n");
  for(i=0;i<6;i++){
    printf("%d\t%3.2f\t%3.2f\t%3.2f\t%3.2f\t%3.2f\t%3.2f\n", i,
      chip_fit_par[0][i], chip_fit_par[0][i+6], chip_fit_par[0][i+12],
      chip_fit_err[0][i], chip_fit_err[0][i+6], chip_fit_err[0][i+12]);
  }

//  Print Summary Data (2)
//  Fitted sigma and associated error for all, odd and even channels

  printf("\nChip\tSigma\t\t\ts_err\n");
  printf(  "\t(all)\t(odd)\t(even)\t(all)\t(odd)\t(even)\n");
  for(i=0;i<6;i++){
    printf("%d\t%3.2f\t%3.2f\t%3.2f\t%3.2f\t%3.2f\t%3.2f\n", i,
      chip_fit_par[1][i], chip_fit_par[1][i+6], chip_fit_par[1][i+12],
      chip_fit_err[1][i], chip_fit_err[1][i+6], chip_fit_err[1][i+12]);
  }

//  Print Summary Data (3)
//  Is there a difference between odd and even channels??? 

    printf("\nChip\tO_mean-E_mean\tO_sig-E_sig\n");
    printf(      "\t-------------\t-----------\n");
    printf(      "\tO_mean+E_mean\tO_sig+E_sig\n");
  for(i=0;i<6;i++){
    printf("%d\t%03.2f\t\t%03.2f\n",i,
      (chip_fit_par[0][i+6]-chip_fit_par[0][i+12])/(chip_fit_par[0][i+6]+chip_fit_par[0][i+12]),
      (chip_fit_par[1][i+6]-chip_fit_par[1][i+12])/(chip_fit_par[1][i+6]+chip_fit_par[1][i+12]));
  }

//  Print Summary Data (3)
//  Bad Channel Summary

  printf("\nBad Channel Summary\n");
  printf("Chip\ttot.\tmean\tsigma\tchisq\tm_err\ts_err\tc_err\n");
  for(i=0;i<6;i++){
    printf("%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\n",
      i,                 chip_bad_ch[i][6],
      chip_bad_ch[i][0], chip_bad_ch[i][2],
      chip_bad_ch[i][4], chip_bad_ch[i][1],
      chip_bad_ch[i][3], chip_bad_ch[i][5]);
  }
  printf("total\t%d\t%d\t%d\t%d\t%d\t%d\t%d\n",
    chip_bad_ch[6][6],
    chip_bad_ch[6][0], chip_bad_ch[6][2],
    chip_bad_ch[6][4], chip_bad_ch[6][1],
    chip_bad_ch[6][3], chip_bad_ch[6][5]);

// And now to a file - if relevant option selected

  if(summary_file){

    sprintf(f_name,"C:\\sctdaq\\data\\stfit_%d_%d_%d_%d.sum",run,scan,moi,loi);
    FILE* sumfile = fopen(f_name,"w");

  //  Print Summary Data (1)
  //  Fitted mean and associated error for all, odd and even channels
  
    fprintf(sumfile,"\nChip\tMean\t\t\tm_err\n");
    fprintf(sumfile,"\t(all)\t(odd)\t(even)\t(all)\t(odd)\t(even)\n");
    for(i=0;i<6;i++){
      fprintf(sumfile,"%d\t%3.2f\t%3.2f\t%3.2f\t%3.2f\t%3.2f\t%3.2f\n", i,
        chip_fit_par[0][i], chip_fit_par[0][i+6], chip_fit_par[0][i+12],
        chip_fit_err[0][i], chip_fit_err[0][i+6], chip_fit_err[0][i+12]);
    }

  //  Print Summary Data (2)
  //  Fitted sigma and associated error for all, odd and even channels

    fprintf(sumfile,"\nChip\tSigma\t\t\ts_err\n");
    fprintf(sumfile,  "\t(all)\t(odd)\t(even)\t(all)\t(odd)\t(even)\n");
    for(i=0;i<6;i++){
      fprintf(sumfile,"%d\t%3.2f\t%3.2f\t%3.2f\t%3.2f\t%3.2f\t%3.2f\n", i,
        chip_fit_par[1][i], chip_fit_par[1][i+6], chip_fit_par[1][i+12],
        chip_fit_err[1][i], chip_fit_err[1][i+6], chip_fit_err[1][i+12]);
    }

  //  Print Summary Data (3)
  //  Is there a difference between odd and even channels??? 
  
    fprintf(sumfile,"\nChip\tO_mean-E_mean\tO_sig-E_sig\n");
    fprintf(sumfile,      "\t-------------\t-----------\n");
    fprintf(sumfile,      "\tO_mean+E_mean\tO_sig+E_sig\n");
    for(i=0;i<6;i++){
      fprintf(sumfile,"%d\t%03.2f\t\t%03.2f\n",i,
        (chip_fit_par[0][i+6]-chip_fit_par[0][i+12])/(chip_fit_par[0][i+6]+chip_fit_par[0][i+12]),
        (chip_fit_par[1][i+6]-chip_fit_par[1][i+12])/(chip_fit_par[1][i+6]+chip_fit_par[1][i+12]));
    }

  //  Print Summary Data (3)
  //  Bad Channel Summary

    fprintf(sumfile,"\nBad Channel Summary\n");
    fprintf(sumfile,"Chip\ttot.\tmean\tsigma\tchisq\tm_err\ts_err\tc_err\n");
    for(i=0;i<6;i++){
      fprintf(sumfile,"%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\n",
        i,                 chip_bad_ch[i][6],
        chip_bad_ch[i][0], chip_bad_ch[i][2],
        chip_bad_ch[i][4], chip_bad_ch[i][1],
        chip_bad_ch[i][3], chip_bad_ch[i][5]);
    }
    fprintf(sumfile,"total\t%d\t%d\t%d\t%d\t%d\t%d\t%d\n",
      chip_bad_ch[6][6],
      chip_bad_ch[6][0], chip_bad_ch[6][2],
      chip_bad_ch[6][4], chip_bad_ch[6][1],
      chip_bad_ch[6][3], chip_bad_ch[6][5]);

  // List anomalous channels
  
    fprintf(sumfile,"\nBad Channel Data\n");
    fprintf(sumfile,"Chip\tcode\tmean\tm_err\tsigma\ts_err\tchisq\tc_err\n");
    for(j=1;j<769;j++){
      code = h_code->GetBinContent(j);
      if(code){
        fprintf(sumfile,"%d\t0x%x\t%3.2f\t%3.2f\t%3.2f\t%3.2f\t%3.2f\t%3.2f\n",j,code,
          h_mean->GetBinContent(j), h_mean->GetBinError(j),
          h_sigma->GetBinContent(j), h_sigma->GetBinError(j),
          h_chisq->GetBinContent(j), h_chisq->GetBinError(j));
      }
    }
  fclose(sumfile);
  }

  delete f1;
}


