Double_t erf(Double_t *x, Double_t *par){
  Double_t xx, y;

  xx = ( (par[1]-x[0]) / par[2]) / 1.414213562; 
  y  = (par[0]/2) * ( 1 - TMath::Erf(xx) );

  return y;
}

Double_t erfc(Double_t *x, Double_t *par){
  Double_t xx, y;

  xx = ( (par[1]-x[0]) / par[2]) / 1.414213562; 
  y  = (par[0]/2) * ( 1 + TMath::Erf(xx) );

  return y;
}

Double_t gaus(Double_t *x, Double_t *par){

  return par[0] * TMath::Gaus(x[0],par[1],par[2]); 
}


void st_scurves_h8(short moi, short loi, short run, short scan){
  char f_name[64], h_name[32], h_name0[32], h_name1[32], out_fname[64];
  char scan_name[64];
  char title1[32];
  float mean, sigma, chisq;
  float i_mean, i_sigma;
  float scan_start, scan_stop;
  float scan_lo, scan_hi, scan_range;
  int  chip, i, j, n, scan_type, scan_max_points;

// Options which change macro output
  int postscript = 0;

// Global style variations

  gStyle.SetPaperSize(20,26); // I find that the postscript output
                              // is mangled without this line - 
                              // although this size (A4) should in
                              // principle be the default...
  gStyle.SetMarkerStyle(21);
  gStyle.SetMarkerSize(0.3);
  gStyle.SetOptFit(0);
  gStyle.SetOptStat(10);
  gStyle.SetTitleW(0.6);

// Open file, or obtain pointer to histogram in memory

  printf("st_scurves called for module %d link %d\n",moi,loi);
  
    st_readnt(moi,loi,run,scan);

    n = 1000;
    sprintf(scan_name,"Unknown");
    TString title = h_scan->GetTitle();
    title.ToUpper();
    if (title.Contains("THRESHOLD")){
      scan_type = 1;
      sprintf(scan_name,"THRESHOLD (mV)");
    }
    if (title.Contains("VCAL")){
      scan_type = 2;
      sprintf(scan_name,"VCAL (mV)");
    }
    if (title.Contains("STROBE")){
      scan_type = 3;
      sprintf(scan_name,"Strobe Delay");
    }
    if (title.Contains("PREAMP")){
      scan_type = 4;
      sprintf(scan_name,"Preamp Current (uA)");
    }
    if (title.Contains("SHAPER")){
      scan_type = 5;
      sprintf(scan_name,"Shaper Current (uA)");
    }

    scan_start = 0;
    scan_stop = 63;
    scan_max_points = h_scan->GetNbinsY();

// titles for graphs - might as well do this now.

    sprintf(title1,"%s - scurves Module %d Link %d", scan_name, moi, loi);

  float ly0[64];
  float ly1[64];
  float ly2[64];
  float ly3[64];
  float x[64];
  float xe[64];
  float y[768][64];
  float ye[768][64];

  TAxis * axis = h_scan->GetYaxis();
  scan_max_points = axis->GetNbins();
  scan_start = axis->GetBinCenter(1);
  scan_stop  = axis->GetBinCenter(scan_max_points);

printf("scan_start %f, stop %f, npoints %f",scan_start, scan_stop, scan_max_points);

  for(i=0;i<scan_max_points; i++){
    x[i] = axis->GetBinCenter(i+1);
    xe[i] = 1;
  }

  for(i=0;i<768;i++){
    for(j=0;j<scan_max_points; j++){
      y[i][j] = h_scan->GetBinContent((770*(j+1))+i+1);
      ye[i][j] = 1; // TODO - proper binomial error stats
    }
  }


  if(postscript){
    sprintf(out_fname,"C:\\sctdaq\\data\\stscrun%d_%d_%d.ps",run,scan,moi);
    TPostScript *ps = new TPostScript(out_fname,112);
    ps->NewPage();
  }

  // gr1 = new TGraphErrors(23,x,y,xe,ye);

// New canvas for scurves
  TCanvas* csc=new TCanvas ("csc",title1,1100,750);
  csc->SetFillColor(42);

  TH2F *hr0 = new TH2F("cal0","scurves cal0",2,scan_start,scan_stop,2,-100,(n+100));
  TH2F *hr1 = new TH2F("cal1","scurves cal1",2,scan_start,scan_stop,2,-100,(n+100));
  TH2F *hr2 = new TH2F("cal2","scurves cal2",2,scan_start,scan_stop,2,-100,(n+100));
  TH2F *hr3 = new TH2F("cal3","scurves cal3",2,scan_start,scan_stop,2,-100,(n+100));
  hr0->SetXTitle(scan_name);
  hr1->SetXTitle(scan_name);
  hr2->SetXTitle(scan_name);
  hr3->SetXTitle(scan_name);

  csc->Divide(6,4);

  for(chip=0;chip<6;chip++){

    csc->cd( 1+chip);
    hr0->Draw();
    csc->cd( 7+chip);
    hr1->Draw();
    csc->cd(13+chip);
    hr2->Draw();
    csc->cd(19+chip);
    hr3->Draw();
  }


  csc->Modify();
  csc->Update();

  for(chip=0;chip<6;chip++){             // chip
    for(i=0;i<32;i++){                   // channel
      for(j=0;j<scan_max_points;j++){ // scan point
        ly0[j]= y[(128*chip)+(i*4)][j];
        ly1[j]= y[(128*chip)+((i*4)+1)][j];
        ly2[j]= y[(128*chip)+((i*4)+2)][j];
        ly3[j]= y[(128*chip)+((i*4)+3)][j];
      }

      gr0 = new TGraph(scan_max_points,x,ly0);
      gr1 = new TGraph(scan_max_points,x,ly1);
      gr2 = new TGraph(scan_max_points,x,ly2);
      gr3 = new TGraph(scan_max_points,x,ly3);
    
      csc->cd( 1+chip);
      gr0->SetMarkerStyle(6);
      gr0->SetMarkerColor(i);
      gr0->SetLineColor(i);
      gr0->Draw("LP");

      csc->cd( 7+chip);
      gr1->SetMarkerStyle(6);
      gr1->SetMarkerColor(i);
      gr1->SetLineColor(i);
      gr1->Draw("LP");

      csc->cd(13+chip);
      gr2->SetMarkerStyle(6);
      gr2->SetMarkerColor(i);
      gr2->SetLineColor(i);
      gr2->Draw("LP");

      csc->cd(19+chip);
      gr3->SetMarkerStyle(6);
      gr3->SetMarkerColor(i);
      gr3->SetLineColor(i);
      gr3->Draw("LP");
    }
    csc->Modify();
    csc->Update();
  }

  if(postscript) {
    ps->Close();
    delete ps;
  }

}



