
/* abc.h  Definitions and records for ABC(D) control block generation */

/* GFM Nov98
 * PWP 15.08.00 - trim_range takes the place of cal_enable and test_mode bits
 * PWP 26.08.00 - add definition of "AUTO" oddity (follows select line)
 * PWP 11.12.00 - revised
 * PWP 04.07.01 - revised
 * PWP 19.02.02 - add calibration capacitor correction factor
 */

#define ANYHIT     0
#define LEVEL_COMP 1
#define EDGE_COMP  2
#define TEST_COMP  3

#define MISSING    0
#define DEAD       1
#define END        2
#define MASTER     3
#define SLAVE      4
#define LONELY     5
#define PARANOID   6

#define AUTO   	  -1 
#define EVEN       0
#define ODD   	   1

/* PWP additions for config register testing */

#define HEADER     0xebffe000
#define TRAILER    0x80000000

#define HEADER_LENGTH    19
#define TRAILER_LENGTH   15
#define DATABLOCK_LENGTH 28

/* end PWP additions */

#define ABCD       3
#define ABCDT      4
#define ABC_CAFE   5

#define CHIPS_IN_MODULE 12
typedef unsigned int u_int32; 

typedef struct 
{
    short compression; 
    short cal_mode; 
    short trim_range;	
    short edge_detect;
    short mask_readout;
    short accumulate;
    short role;   
    short active;   /* 1=> particip. in scans; 0=>maskbits=0; vthr=50a0; vcal=0 */
    float vcal;
    float vthr;
    float delay;
    float feshaper;   
    float febias;
    short mask[8];  /* ### Why was this int ? */
    short trim[128];
    short rc_method;     /* Method used to determine the response curve     */
    short rc_function;   /* Function used to fit the response curve data    */
    float rc_par[3];     /* Parameters of the functional fit - PWP 12.11.99 */
    float c_factor;      /* Calibration Capacitor correction factor - PWP 19.02.02 */
} chip_cfg;

typedef struct
{
    short chipset;
    short evenodd;
    short dtm;         /* enable data taking mode */
    short updatetrims;

    short bypass;      /* predetermined bypass configuration */
    short token;       /* bitpattern determining token passing scheme */

    short sc_mode;     /* operational mode of Liverpool forward SC */
    short bpm_drive;   /* bpm drive current */
    short vdc_dac0;    /* VDC DAC0 on Liverpool forward SC */
    short vdc_dac1;    /* VDC DAC0 on Liverpool forward SC */

    chip_cfg chips[CHIPS_IN_MODULE];

    /*
     * DCS System:
     * 1. Requested Settings
     */

    float vdet_set;
    float vcc_set;
    float vi1_set;
    float vdd_set;
    float vpin_set;
    float vled0_set;
    float vled1_set;

    short select;/* SCTLV SELECT line */
    short ramp;  /*   HV  Ramp rate   */ 

    /*
     * DCS System: 
     * 2. Requested Limits
     */

    float idet_lim;
    float icc_lim;
    float ivi1_lim;
    float idd_lim;
    float ipin_lim;
    float iled0_lim;
    float iled1_lim;

    float t1_lim; /* maximum allowable temperature on T1 */
    float t2_lim;

    /*
     * DCS System: 
     * 3. Control information
     */ 

    short suspended; /* module removed from DAQ due to failure */
    
    short hv_trip;   /* trip state */
    short lv_trip;
    short mt_trip;

    /*
     * DCS System:
     * 4. Monitored Values
     */

    float vdet;
    float vccout; /* PWP 04.07.01 */
    float vcc;
    float vi1;
    float vddout; /* PWP 04.07.01 */
    float vdd;
    float vpin;
    float vled0;
    float vled1;

    float idet;
    float icc;
    float ivi1;
    float idd;
    float ipin;
    float iled0;
    float iled1;

    float t1;
    float t2;

    short hv_status;
    char  lv_status;
    char  lv_temp;

} module_cfg;


