
/*
   stlib.h 

   GFM 990504 revised PWP 24.08.99
   PWP 12.10.99 - change tmax to type int
   PWP 11.04.01 - add prototypes for new suspense functions
   PWP 17.04.01 - one more suspense function
   PWP 13.08.02 - split st_slog_user into several parts to
                  facilittae simple counter tests
*/
  
#include <time.h>
  
  
#if defined(__STDC__)   
  
/* These functions act on entire system according to configs: */
/* To do: split startup() into separate functions to allow re-initialisations */
int st_startup();
int st_shutdown();

int st_vmemap();             
int st_configure_system();   
int st_configure_modules();  
int st_init();

int st_configure_enable(int det, short present, short active, short link0, short link1);
int st_execute_enables();

int st_coerce_variable(short typ, float var, float *newvar, short is_step);
void st_configure_variable(short mid, short typ, float var);
void st_configure_chip_variable(short mid, short cid, short typ, float var);
void st_configure_trim(short mid, short cid, short chan, short var);
void st_configure_token(int mid, unsigned token, int link0, int link1);
int st_execute_configs(short dump, short full_config);
int st_execute_sc_configs(short dump, short hard_reset);

void st_retrieve_variable(short mid, short cid, short chan, short typ, float* var);

void st_clear_suspense();
void st_clear_1suspense(short mid);
void st_suspend_link(short mid, short lid);
void st_suspend_module(short mid);
void st_suspend_stream(short mid, short sid);


/* These functions orchestrate CLOAC triggers and Mustard recording
   for a burst of multiple triggers */

int st_tburst(int tmax, short ttype,
              short cfgfreq, short srfreq, short bcrfreq, short dump);
int st_nseburst(int tmin, int tmax, short target, short ttype, short dump);
int st_rawburst(int tmax, short ttype,
                short cfgfreq, short srfreq, short bcrfreq, short dump);
int st_compare_sendid(short n_ev, short dump);

/* Start, stop or execute  triggers, calstrobes, soft resets and bc resets: no stream actions */

int st_run(short ttype, short tfreq);
int st_stop();
int st_single(short ttype);
int st_seu_slog_load(short seu_invert);
int st_seu_slog_run();
int st_slog_single(int com1, int com2, int ntrigs, unsigned short delay1, unsigned short delay2, unsigned short delay3);
int st_slog_user(char* fname, unsigned short ncycles);

int st_slog_load(char* fname);
int st_slog_start(short set_cycles, unsigned short ncycles);
int st_slog_stop();

void st_echo_event(int seed);
void st_enable_datataking();
void st_soft_reset();
void st_bc_reset();

void st_clock_off();
void st_clock_on();

/* Slow Controls */

void st_hard_reset();

void st_hv_off();
void st_hv_on();
void st_hv_ramp(short to_zero);
void st_hv_status();
int  st_hv_trips(short recover);
int st_hv_chan(short id);

void st_lv_init();
void st_lv_1off(short id);
void st_lv_off();
void st_lv_1on(short id);
void st_lv_on();
void st_lv_status();
int  st_lv_trips(short recover);

int st_configure_dcs();
void st_dcs_mon(short id);
int st_hv_mon(short id);
int st_lv_mon(short id);
int st_lv_recover(short id);
int st_t_mon(short id);


/* set VCSEL drive currents on BiLED and/or OptIF*/
int st_optorx_init();
int st_optotx_on();
int st_optotx_off();


/* Utility functions: */

/*void sleep(int ms); */
void st_sleep(clock_t wait);

/* Data transmission functions  (required for ROOT/DLL) */
  
short st_give_nmods(short *list);
short st_give_modname(short module,char *name);
short st_give_devtype(short module,char *name);
short st_give_data(short module, short cal_mode, short *hist0, short *hist1);
short st_give_counters(short mus_id, int *tsent, int *evcnt, short *ercnt, short *tocnt, short *xecnt);

/* Functions to set counters used for ntuple generation */

int  st_scan_definition(int dimension, float start, float stop, float step, short points);
void st_scan_increment();
void st_scan_reset();

void st_set_slog_loadtrig(short val);
void st_set_role(short mid, short cid, short role);
short st_used_streams(short mid);

#else
 
int st_startup();
int st_shutdown();

int st_vmemap();             
int st_configure_system();   
int st_configure_modules();  
int st_init();

int st_configure_enable();
int st_execute_enables();

int st_coerce_variable();
void st_configure_variable();
void st_configure_chip_variable();
void st_configure_trim();
void st_configure_token(); /* PWP bugfix 04.11.02 */ 
int st_execute_configs();
int st_execute_sc_configs();
void st_retrieve_variable();

void st_clear_suspense();
void st_clear_1suspense();
void st_suspend_link();
void st_suspend_module();
void st_suspend_stream();

/* These functions orchestrate CLOAC triggers and Mustard recording
   for a burst of multiple triggers */

int st_tburst();
  
int st_rawburst();
int st_compare_sendid();

/* Start, stop or execute  triggers, calstrobes, soft resets and bc resets: no stream actions */

int st_run();
int st_stop();
int st_single();
int st_seu_slog_load();
int st_seu_slog_run();
int st_slog_single();
int st_slog_user();

int st_slog_load();
int st_slog_start();
int st_slog_stop();

void st_echo_event();
void st_enable_datataking();
void st_soft_reset();
void st_bc_reset();

void st_clock_off();
void st_clock_on();

/* Slow Controls */

void st_hard_reset();

void st_hv_off();
void st_hv_on();
void st_hv_ramp();
void st_hv_status();
int  st_hv_trips();

void st_lv_init();
void st_lv_1off();
void st_lv_off();
void st_lv_1on();
void st_lv_on();
void st_lv_status();
int st_lv_trips();

int st_configure_dcs();
void st_dcs_mon();
int st_hv_mon();
int st_lv_mon();
int st_lv_recover();
int st_t_mon();

int st_optorx_init();
int st_optotx_on();
int st_optotx_off();
void st_sleep();
short st_give_nmods();
short st_give_modname();
short st_give_devtype();
short st_give_data();
short st_give_counters();

int  st_scan_definition();
void st_scan_increment();
void st_scan_reset();

void st_set_slog_loadtrig();
void st_set_role();
short st_used_streams();

#endif 
